import {Resp} from "@util/Resp";
import _ from "lodash";

export interface IJSONSchema2MarkdownOptions {
    with_header?: boolean;
    with_properties?: boolean;
    with_example?: boolean;
}

export function jsonschema2md(schema: any, options: IJSONSchema2MarkdownOptions = {
    with_header: true,
    with_properties: true,
    with_example: true
}): string {
    if (!schema || typeof schema !== 'object') {
        throw Resp.gen_err(Resp.InternalServerError, 'schema必须是object');
    }
    let subSchemaTypes = Object.keys(schema.definitions || {}).reduce(function (map, subSchemaTypeName) {
        map['#/definitions/' + subSchemaTypeName] = subSchemaTypeName
        return map
    }, {})
    let text: string[] = [];

    if (options.with_header) {
        if (schema.title) {
            text.push('# ' + schema.title);
        }

        if (schema.description) {
            text.push(schema.description)
        }
        text.push(' ');
    }

    if (options.with_properties) {
        if (schema.type === 'object') {
            text.push('#  参数：')
            let header = [
                `|参数名|类型|必需|取值范围|说明|`,
                `|:--|:--|:-:|:--|:--|`
            ];
            text.push(...header);
            generatePropertySection(1, schema, subSchemaTypes).forEach(function (section) {
                text.push(...section);
            })
        } else {
            text.push(...generateSchemaSectionText(1, '', false, schema, subSchemaTypes));
        }
        text.push(' ');
    }

    if (options.with_example) {
        text.push('# 示例：')
        text.push('```json')
        text.push(JSON.stringify(generateExampleText(schema), undefined, '\t'));
        text.push('```')
        text.push(' ');
    }

    return text.filter(function (line) {
        return !!line
    }).join('\n')
}

function generatePropertyRestrictions(schema) {
    let generate = generateSinglePropertyRestriction(schema)
    return [
        // TODO: 还需要添加其它的约束类型
        generate('minimum', '最小值'),
        generate('maximum', '最大值'),
        generate('pattern', '正则表达式'),
        generate('minItems', '最少元素个数'),
        generate('uniqueItems', '元素唯一')
    ].filter(function (text) {
        return text
    }).join('<br/>')
}

function generateSinglePropertyRestriction(schema) {
    return function (key, text) {
        if (schema[key]) {
            return text + ': `' + schema[key] + '`'
        } else {
            return null
        }
    }
}

function generateSchemaSectionText(level: number, name: string, isRequired: boolean, schema, subSchemas): string[] {
    let schemaType = getActualType(schema, subSchemas)

    let value_range = '';
    if (schema.enum) {
        value_range += '枚举值：<br/>'
        value_range += schema.enum.map(function (enumItem) {
            return '`' + enumItem + '`'
        }).join('<br/>');
    }

    let restrictions = generatePropertyRestrictions(schema)

    if (restrictions) {
        value_range += restrictions;
    }


    if (schemaType === 'array') {

        let itemsType = schema.items && schema.items.type
        if (!itemsType && schema.items['$ref']) {
            itemsType = getActualType(schema.items, subSchemas)
        }

        if (itemsType && name) {
            value_range += '`' + itemsType + '`';
        } else if (itemsType) {
            value_range += '`' + itemsType + '`';
        }
    } else if (schema.oneOf) {
        value_range += '必须是以下类型中的一种：<br/>'
        value_range += schema.oneOf.map(function (oneOf) {
            return '`' + (subSchemas[oneOf['$ref']] || oneOf['const']) + '`' + (oneOf['description'] ? `: ${oneOf['description']}` : '');
        }).join('<br/>');
    }

    if (schema.default !== undefined) {
        if (schema.default === null || ["boolean", "number", "string"].indexOf(typeof schema.default) !== -1) {
            value_range += '默认值: `' + JSON.stringify(schema.default) + '`';
        } else {
            value_range += '默认值: <br/>';
            value_range += '```<br/>' + JSON.stringify(schema.default, null, 2) + '<br/>```';
        }
    }

    let text = [
        `|${'&emsp;'.repeat((level - 1))}${name}|${schemaType}|${isRequired ? '是' : '否'}|${value_range}|${schema.title}${schema.description ? '<br/>' + schema.description : ''}|`
    ]

    if (schemaType === 'object') {
        if (schema.properties) {
            generatePropertySection(level + 1, schema, subSchemas).forEach(function (rows) {
                text = text.concat(rows)
            })
        }
    } else if (schemaType === 'array') {
        let itemsType = schema.items && schema.items.type
        if (!itemsType && schema.items['$ref']) {
            itemsType = getActualType(schema.items, subSchemas)
        }
        if (!itemsType) {
            let validationItems = []

            if (schema.items.allOf) {
                validationItems = schema.items.allOf
            } else if (schema.items.anyOf) {
                validationItems = schema.items.anyOf
            } else if (schema.items.oneOf) {
                validationItems = schema.items.oneOf
            } else if (schema.items.not) {
                validationItems = schema.items.not
            }

            if (validationItems.length > 0) {
                validationItems.forEach(function (item: any) {
                    text = text.concat(generateSchemaSectionText(level + 1, item.title, false, item, subSchemas))
                })
            }
        }

        if (itemsType === 'object') {
            // text.push('The array object has the following properties:')
            generatePropertySection(level + 1, schema.items, subSchemas).forEach(function (section) {
                text = text.concat(section)
            })
        }
    }

    return text
}

function generatePropertySection(level: number, schema: any, subSchemas: any) {
    if (schema.properties) {
        let rows = Object.keys(schema.properties).map(function (propertyKey: string) {
            let propertyIsRequired = schema.required && schema.required.indexOf(propertyKey) >= 0;
            return generateSchemaSectionText(level, propertyKey, propertyIsRequired, schema.properties[propertyKey], subSchemas);
        })
        return [
            ...rows
        ]
    } else {
        return []
    }
}

function getActualType(schema, subSchemas) {
    if (schema.type) {
        return schema.type
    } else if (schema['$ref'] && subSchemas[schema['$ref']]) {
        return subSchemas[schema['$ref']]
    } else {
        return undefined
    }
}

function generateExampleText(schema) {
    if (schema.properties) {
        let result: any = {};
        Object.keys(schema.properties).map(function (propertyKey: string) {
            let subSchemaTypes = Object.keys(schema.definitions || {}).reduce(function (map, subSchemaTypeName) {
                map['#/definitions/' + subSchemaTypeName] = subSchemaTypeName
                return map
            }, {})
            let schemaType = getActualType(schema, subSchemaTypes);
            if (schemaType === 'object') {
                result[propertyKey] = generateExampleText(schema.properties[propertyKey]);
            } else {
                let example;
                let property = schema.properties[propertyKey];
                if (property.examples && _.isArray(property.examples) && property.examples.length > 0) {
                    example = property.examples[0];
                }
                result[propertyKey] = property.default || example || undefined;
            }
        });
        return result;
    } else if (schema.items) {
        let result: string[] = [];
        let itemsType = schema.items.type
        if (itemsType === 'object') {
            result = [generateExampleText(schema.items)];
            return result;
        } else {
            let example;
            if (schema.items.examples && _.isArray(schema.items.examples) && schema.items.examples.length > 0) {
                example = schema.items.examples[0];
            }
            example = schema.default || example;
            return example ? [example] : undefined;
        }
    } else {
        let example;
        if (schema.examples && _.isArray(schema.examples) && schema.examples.length > 0) {
            example = schema.examples[0];
        }
        return schema.default || example || undefined;
    }

}
