import * as crypto from 'crypto';
import dayjs from "dayjs";

// const random = require('string-random');
const intformat = require('biguint-format');
// import FlakeId from "flake-idgen";

// let flakeIdGen = new FlakeId({epoch: 1300000000000});

import {IMobileInfo, IMpOpenIdInfo, IMpUserInfo, MpPlatformBase} from "@classes/MpPlatformBase";
import {Resp} from "@util/Resp";
import {Logger} from "@util/Logger";
import {AcsApplication} from "@core-models/AcsApplication";
import {obj_key_sort} from "@util/JsonToolkit";
import {IdGen} from "@util/IdGen";

const wxErrorCode: any = {
    '-1': '系统繁忙，此时请开发者稍候再试',
    '0': '请求成功',
    '40029': 'code 无效',
    '45011': '频率限制，每个用户每分钟100次',
}

interface IAuthCode2SessionResult {
    openid: string;
    session_key: string;
    errcode?: number;
    errmsg?: string;
}

// https://pay.weixin.qq.com/wiki/doc/api/wxpay_v2/papay/chapter3_7.shtml
interface IWxContractInfo {
    mch_id: string;
    appid: string;
    contract_id: string;
    plan_id: string;
    request_serial: number;
    contract_code: string;
    contract_display_account: string;
    contract_state: number;
    contract_signed_time: string;
    contract_expired_time: string;
    contract_terminated_time: string;
    contract_termination_mode: number;
    sign: string;
    contract_termination_remark: string;
    openid: string;
}

export interface IUnifiedOrderResult {
    timestamp: string;
    nonceStr: string;
    prepayId: string;
    signType: string;
}

export class WxPlatform extends MpPlatformBase {
    constructor(platform: number) {
        super(platform);
    }

    async init() {
    }

    // private decryptData(data: any, sessionKey: string, iv: string): any {
    //     let base64SessionKey = new Buffer(sessionKey, 'base64');
    //     let base64Data = new Buffer(data, 'base64');
    //     let base64Iv = new Buffer(iv, 'base64');
    //     try {
    //         let decipher = crypto.createDecipheriv('aes-128-cbc', base64SessionKey, base64Iv);
    //         decipher.setAutoPadding(true);
    //         let decoded = decipher.update(base64Data.toString('utf8'), 'base64', 'utf8');
    //         decoded += decipher.final('utf8')
    //         decoded = JSON.parse(decoded);
    //         return decoded;
    //     } catch (e) {
    //         throw new Error('wx返回消息解密失败');
    //     }
    //
    // }
    createPrepayReqInfo(orderId: string, appId: string, prepayId: string, privateKey: string): any {
        // 签名，使用字段appId、timeStamp、nonceStr、package计算得出的签名值
        // 签名串一共有四行，每一行为一个参数。行尾以\n（换行符，ASCII编码值为0x0A）结束，包括最后一行。
        // 如果参数本身以\n结束，也需要附加一个\n
        let timestamp: string = parseInt((Date.now() / 1000).toString()).toString();
        let nonceStr = 'payscore' + IdGen.id();//intformat(flakeIdGen.next(), 'dec'); // 随机字符串
        let sign = this.createSign(appId, timestamp, nonceStr, prepayId, privateKey);

        return{
            order_id: orderId,
            status: 1,
            pay_params: {
                timestamp: timestamp,
                nonce_str: nonceStr,
                package: `prepay_id=${prepayId}`,
                sign_type: 'RSA',
                pay_sign: sign
            }
        };
    }
    createSign(appId: string, timestamp: string, nonce: string, prepayId: string, privateKey: string): string {
        let data = `${appId}\n${timestamp}\n${nonce}\nprepay_id=${prepayId}\n`;
        Logger.trace(data);
        let sign = crypto.createSign('RSA-SHA256');

        sign.write(data);
        sign.end();
        let result = sign.sign(privateKey, 'base64');
        return result;
    }

    private decrypt(key: string, encryptedData: any, iv: string): any {
        let bufferedKey = new Buffer(key, 'base64');
        let bufferedData = new Buffer(encryptedData, 'base64');
        let bufferedIv = new Buffer(iv, 'base64');
        try {
            let decipher = crypto.createDecipheriv('aes-128-cbc', bufferedKey, bufferedIv);
            decipher.setAutoPadding(true);
            // let decoded = decipher.update(bufferedData.toString('utf8'), 'base64', 'utf8');
            let decoded = decipher.update(encryptedData, 'base64', 'utf8');
            decoded += decipher.final('utf8');
            let info: any = JSON.parse(decoded);
            return info;
        } catch (e: any) {
            throw Resp.gen_err(Resp.InternalServerError, e.message);
        }
    }

    public decryptUserInfo(key: string, encryptInfo: any): IMpUserInfo {
        let info = this.decrypt(key, encryptInfo.encryptedData, encryptInfo.iv);
        Logger.debug(info);
        return {
            nickname: info.nickName,
            avatarUrl: info.avatarUrl,
            unionId: info.unionId
        };
    }

    public decryptMobile(key: string, encryptInfo: any): IMobileInfo {
        let info = this.decrypt(key, encryptInfo.encryptedData, encryptInfo.iv);
        Logger.debug(info);
        return {
            mobile: info.phoneNumber,
        }
    }

    private getWxParams(appId: string): Promise<any> {
        return new Promise<any>(async (resolve, reject) => {
            try {
                let application = await AcsApplication.findByPk(appId, {raw: true});
                if (application === null) return reject(Resp.gen_err(Resp.ResourceNotFound, `没有找到微信appid: ${appId}`));
                return resolve({
                    key: application.app_key,
                    app_type: application.app_type,
                    pay_key: application.pay_key,
                    private_key: application.private_key,
                    name: application.name,
                    mch_id: application.mch_id,
                    serial_no: application.api_cert_serial_no,
                })
            } catch (e: any) {
                reject(Resp.gen_err(Resp.PostgresqlServerError, e.message));
            }
        })
    }

    // 通过code换取openid
    // https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/login/auth.code2Session.html
    getOpenId(appId: string, code: string, encryptedData?: string, iv?: string): Promise<IMpOpenIdInfo> {
        return new Promise<IMpOpenIdInfo>(async (resolve, reject) => {
            try {
                let wxParams = await this.getWxParams(appId);

                let url = `https://api.weixin.qq.com/sns/jscode2session?appid=${appId}&secret=${wxParams.key}&js_code=${code}&grant_type=authorization_code`;
                let data: IAuthCode2SessionResult = <IAuthCode2SessionResult>await this.apiGet(url);
                console.log(data);
                Logger.debug(data);
                if (data.errcode && data.errcode !== 0) {
                    return reject(Resp.gen_err(Resp.MpPlatformFail, wxErrorCode[`${data.errcode}`]))
                }

                let result: IMpOpenIdInfo = {
                    session_key: data.session_key,
                    open_id: data.openid,
                    app_type: wxParams.app_type
                }
                resolve(result);
            } catch (e) {
                Logger.debug(e);
                return reject(e);
            }


        });
    }

    // https://pay.weixin.qq.com/wiki/doc/api/wxpay_v2/papay/chapter3_7.shtml
    queryContract(): Promise<IWxContractInfo> {
        return new Promise<IWxContractInfo>(async (resolve, reject) => {

        })
    }

    private _sign_sort_unifiedorder(opt: any) {
        let newObj = obj_key_sort(opt);
        let key = Object.keys(newObj);
        let values = Object.values(newObj);
        let str = '';
        for (let i = 0; i < key.length; i++) {
            if (i > 0) {
                str += "&";
            }
            str += key[i] + "=" + (values[i]);  //_percentEncode
        }
        return str;
    }

    // 小程序 JSAPI下单
    // 商户系统先调用该接口在微信支付服务后台生成预支付交易单，返回正确的预支付交易会话标识后再按Native、JSAPI、APP等不同场景生成交易串调起支付。
    // https://pay.weixin.qq.com/wiki/doc/apiv3/apis/chapter3_5_1.shtml
    // 签名：https://pay.weixin.qq.com/wiki/doc/apiv3/wechatpay/wechatpay4_0.shtml
    unifiedorder(appId: string, notifyHost: string, openId: string, total: number, outTradeNo: string, attach?: string): Promise<IUnifiedOrderResult> {
        return new Promise<IUnifiedOrderResult>(async (resolve, reject) => {
            try {
                let wxParams = await this.getWxParams(appId);
                let params: any = {
                    appid: appId,       //由微信生成的应用ID，全局唯一。请求基础下单接口时请注意APPID的应用属性，例如公众号场景下，需使用应用属性为公众号的APPID
                    mchid: wxParams.mch_id,      // 直连商户的商户号，由微信支付生成并下发。
                    time_expire: dayjs().add(2, 'minute'),
                    description: wxParams.name + '充值', // 商品简单描述
                    out_trade_no: outTradeNo,//'PowerWX_' + intformat(flakeIdGen.next(), 'dec'),  // 商户系统内部订单号，要求32个字符内，只能是数字、大小写字母_-|* 且在同一个商户号下唯一。
                    notify_url: notifyHost, //异步接收微信支付结果通知的回调地址，通知url必须为外网可访问的url，不能携带参数。公网域名必须为https，如果是走专线接入，使用专线NAT IP或者私有回调域名可使用http
                    attach: attach,
                    amount: {
                        total: total,
                        currency: 'CNY'
                    },
                    payer: {
                        openid: openId
                    }
                }
                Logger.trace(params);
                let body = JSON.stringify(params);
                let [timestamp, nonceStr, sign, authorization] = await this.buildAuthorization('POST', '/v3/pay/transactions/jsapi', wxParams.mch_id, wxParams.serial_no, wxParams.private_key, body);
                let url = `https://api.mch.weixin.qq.com/v3/pay/transactions/jsapi`;
                let data: any = await this.apiPost(url, body, {
                    "Content-Type": "application/json",
                    Authorization: authorization
                });

                Logger.trace({
                    prepay_id: data.prepay_id,
                    sign: sign,
                    nonce_str: nonceStr

                });
                resolve({
                    timestamp: timestamp,
                    nonceStr: nonceStr,
                    prepayId: data.prepay_id,
                    signType: 'RSA'
                })
            } catch (e: any) {
                Logger.debug(e);
                reject(Resp.gen_err(Resp.InternalServerError, e.message));
            }

        })
    }

    refunds(appId: string, notifyHost: string, openId: string, total: number, refund: number, outTradeNo: string): Promise<any> {
        return new Promise<any>(async (resolve, reject) => {
            try {
                let wxParams = await this.getWxParams(appId);
                let params: any = {
                    // appid: appId,       //由微信生成的应用ID，全局唯一。请求基础下单接口时请注意APPID的应用属性，例如公众号场景下，需使用应用属性为公众号的APPID
                    // mchid: wxParams.mch_id,      // 直连商户的商户号，由微信支付生成并下发。
                    // description: wxParams.name + '余额退回', // 商品简单描述
                    out_trade_no: outTradeNo,  // 商户系统内部订单号，要求32个字符内，只能是数字、大小写字母_-|* 且在同一个商户号下唯一。
                    out_refund_no: 'PowerRefundWX_' + IdGen.id(),//intformat(flakeIdGen.next(), 'dec'), // 商户系统内部的退款单号，商户系统内部唯一，只能是数字、大小写字母_-|*@ ，同一退款单号多次请求只退一笔。
                    reason: '余额退回',
                    notify_url: notifyHost, //异步接收微信支付结果通知的回调地址，通知url必须为外网可访问的url，不能携带参数。公网域名必须为https，如果是走专线接入，使用专线NAT IP或者私有回调域名可使用http
                    // attach: attach,
                    amount: {
                        refund: refund,  //退款金额，币种的最小单位，只能为整数，不能超过原订单支付金额。
                        total: total, //原支付交易的订单总金额，币种的最小单位，只能为整数。
                        currency: 'CNY'
                    }
                }
                let body = JSON.stringify(params);
                let [timestamp, nonceStr, sign, authorization] = await this.buildAuthorization('POST', '/v3/refund/domestic/refunds', wxParams.mch_id, wxParams.serial_no, wxParams.private_key, body);
                let url = `https://api.mch.weixin.qq.com/v3/refund/domestic/refunds`;
                let data: any = await this.apiPost(url, body, {
                    "Content-Type": "application/json",
                    Authorization: authorization
                });

                Logger.trace(data, 'refunds');
                resolve({
                    status: data.status,
                    outTradeNo: outTradeNo,
                    outRefundNo: params.out_refund_no,
                    refund: refund
                })
            } catch (e: any) {
                Logger.debug(e);
                reject(Resp.gen_err(Resp.PaymentPlatformFail,  e.descr));
            }
        })
    }

    // 查询支付情况，文档：https://pay.weixin.qq.com/wiki/doc/apiv3/apis/chapter3_5_2.shtml
    queryUnifiedorder(appId: string, out_trade_no: string): Promise<any> {
        return new Promise<any>(async (resolve, reject) => {
            try {
                let wxParams = await this.getWxParams(appId);
                let url = `https://api.mch.weixin.qq.com/v3/pay/transactions/out-trade-no/${out_trade_no}?mchid=${wxParams.mch_id}`;
                let [timestamp, nonceStr, sign, authorization] =
                  await this.buildAuthorization('GET', `/v3/pay/transactions/out-trade-no/${out_trade_no}?mchid=${wxParams.mch_id}`,
                    wxParams.mch_id, wxParams.serial_no, wxParams.private_key);
                let data: any = await this.apiGet(url, {
                    Authorization: authorization
                });

                Logger.trace(data);
                resolve(data);
            }catch (e: any) {
                Logger.debug(e);
                reject(Resp.gen_err(Resp.PaymentPlatformFail,  e.descr));
            }
        });
    }

    // 查询退款情况，文档：https://pay.weixin.qq.com/wiki/doc/apiv3/apis/chapter3_5_10.shtml
    queryRefund(appId: string, out_refund_no: string): Promise<any> {
        return new Promise<any>(async (resolve, reject) => {
            try {
                let wxParams = await this.getWxParams(appId);
                let [timestamp, nonceStr, sign, authorization] =
                  await this.buildAuthorization('GET', '/v3/refund/domestic/refunds/'+ out_refund_no,
                  wxParams.mch_id, wxParams.serial_no, wxParams.private_key);
                let url = `https://api.mch.weixin.qq.com/v3/refund/domestic/refunds/` + out_refund_no;
                let data: any = await this.apiGet(url, {
                    Authorization: authorization
                });

                Logger.trace(data);
                resolve(data);
            }catch (e: any) {
                Logger.debug(e);
                reject(Resp.gen_err(Resp.PaymentPlatformFail,  e.descr));
            }
        });
    }

    /**
     * AEAD_AES_256_GCM 解密
     * @param key  apiv3 key
     * @param nonce
     * @param associatedData
     * @param ciphertext
     */
    public static aes256gcmDecrypt(key: string, nonce: string, associatedData: string, ciphertext: string): string {
        let ciphertextBuffer = Buffer.from(ciphertext, 'base64');
        let authTag = ciphertextBuffer.slice(ciphertextBuffer.length - 16);
        let data = ciphertextBuffer.slice(0, ciphertextBuffer.length - 16);
        let decipherIv = crypto.createDecipheriv('aes-256-gcm', key, nonce);
        decipherIv.setAuthTag(Buffer.from(authTag));
        decipherIv.setAAD(Buffer.from(associatedData));
        let decryptStr = decipherIv.update(data, undefined, 'utf8');
        decipherIv.final();
        return decryptStr
    }


    /**
     * 构建 v3 接口所需的 Authorization
     * https://pay.weixin.qq.com/wiki/doc/apiv3/wechatpay/wechatpay4_0.shtml
     * @param method     请求方法
     * @param urlSuffix  APi名称,
     * @param mchId     商户Id
     * @param serialNo  商户 API 证书序列号
     * @param key       key.pem 证书API密钥
     * @param body      接口请求参数
     */
    private async buildAuthorization(method: string, urlSuffix: string, mchId: string, serialNo: string, key: string, body?: string) {
        let timestamp: string = parseInt((Date.now() / 1000).toString()).toString(); // 时间戳
        let authType: string = 'WECHATPAY2-SHA256-RSA2048'; // 认证类型
        let nonceStr = 'payscore' + IdGen.id();//intformat(flakeIdGen.next(), 'dec'); // 随机字符串
        // 构建签名参数
        let buildSignMessage: string = this.buildReqSignMessage(method, urlSuffix, timestamp, nonceStr, body);
        // 生成签名
        let signature: string = this.sha256WithRsa(key, buildSignMessage);
        // 根据平台规则生成请求头 authorization
        return [timestamp, nonceStr, signature, this.getAuthorization(mchId, serialNo, nonceStr, timestamp, signature, authType)];
    }


    /**
     * 构建请求签名参数
     * https://pay.weixin.qq.com/wiki/doc/apiv3/wechatpay/wechatpay4_0.shtml
     * @param method  Http 请求方式
     * @param url 请求接口 /v3/certificates
     * @param timestamp 获取发起请求时的系统当前时间戳
     * @param nonceStr 随机字符串
     * @param body 请求报文主体
     */
    private buildReqSignMessage(method: string, url: string, timestamp: string, nonceStr: string, body?: string): string {
        if (body)
            return `${method}\n${url}\n${timestamp}\n${nonceStr}\n${body}\n`;
        else
            return `${method}\n${url}\n${timestamp}\n${nonceStr}\n\n`;
    }

    /**
     * SHA256withRSA
     * @param data 待加密字符
     * @param privatekey 私钥key  key.pem
     */
    private sha256WithRsa(privatekey: string, data: string): string {
        return crypto.createSign('RSA-SHA256').update(data).sign(privatekey, 'base64')
    }

    /**
     * 获取授权认证信息
     * https://pay.weixin.qq.com/wiki/doc/apiv3/wechatpay/wechatpay4_0.shtml
     * @param mchId     商户号
     * @param serialNo  商户API证书序列号
     * @param nonceStr  请求随机串
     * @param timestamp 时间戳
     * @param signature 签名值
     * @param authType  认证类型，目前为WECHATPAY2-SHA256-RSA2048
     */
    private getAuthorization(mchId: string, serialNo: string, nonceStr: string, timestamp: string, signature: string, authType: string): string {
        return `${authType} mchid="${mchId}",serial_no="${serialNo}",nonce_str="${nonceStr}",timestamp="${timestamp}",signature="${signature}"`;
    }
}
