/**
 * 用于微服务间内部交互消息的合法性校验
 * 微服务间交互，通过证书签名来验证，
 * 微服务服务端向其它微服务发放证书公钥
 * 请求时，消息中包含使用私钥加密的签名，接收端用对应的私钥验证签名
 * 请求消息的http头中，需要包含发送端的微服务名称，时间戳以及用这两个信息合并字符串的签名
 * 生成私钥：
 *   openssl genrsa -out rsa_private_key.pem 1024
 * 生成公钥
 *   openssl rsa -in rsa_private_key.pem -pubout -out rsa_public_key.pem
 * JAVA下，需要进行PKCS#8编码：
 *   openssl pkcs8 -topk8 -inform PEM -in rsa_private_key.pem -outform PEM -nocrypt > rsa_private_key_pkcs8.pem
 *   openssl rsa -in rsa_private_key_pkcs8.pem -pubout -out rsa_public_key_pkcs8.pem
 */
import {clone} from "@util/JsonToolkit";
import {Resp} from "@util/Resp";
import {Services} from "@util/Services";
import {TGuardFunction, ICachedData, IRequest} from "@core/Defined";

// 对api网关发来的签名进行验证的守卫
export const internal_guard: TGuardFunction = function (content: IRequest | unknown, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        if (process.env.TEST === 'true') { // 单元测试时直接跳过
            return resolve();
        }

        try {
            let raw_content: any = clone(content);
            let sign: string = raw_content.__sign;
            let service_name: string = raw_content.__serviceName;
            delete raw_content.__sign;
            delete raw_content.__serviceName;
            let service: Services = Services.get_instance(service_name);
            let is_valid: boolean = service.verify(raw_content, sign);
            if (!is_valid) {
                return reject(Resp.gen_err(Resp.Forbidden, `服务间通信签名错误, api: ${cached_data.api}`));
            }
            resolve();
        } catch (e) {
            reject(e);
        }
    });
};
