import Ajv, {ValidateFunction} from "ajv";
import pathToRegexp from 'path-to-regexp';
import {IApiProcessor, IGuard, IMethodModule, ICachedData, ISignInfo} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Logger} from "@util/Logger";

export class Auditor {
    private readonly avj = new Ajv({useDefaults: true, allowUnionTypes: true});
    private readonly guards: Array<IGuard>;

    constructor(guards: Array<IGuard>) {
        this.avj.addFormat('YYYY-MM-DD HH:NN:SS', /^[1-9]\d{3}-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])\s+(20|21|22|23|[0-1]\d):[0-5]\d:[0-5]\d$/);
        this.avj.addFormat('datetime', /^[1-9]\d{3}-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])\s+(20|21|22|23|[0-1]\d):[0-5]\d:[0-5]\d$/);
        this.avj.addFormat('YYYY-MM-DD', /^[1-9]\d{3}-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])$/);
        this.avj.addFormat('date-time', /^[1-9]\d{3}-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])\s+(20|21|22|23|[0-1]\d):[0-5]\d:[0-5]\d$/);
        this.avj.addFormat('date', /^[1-9]\d{3}-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])$/);
        this.avj.addFormat('MM-DD', /^(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])$/);
        this.avj.addFormat('YYYY-MM', /^[1-9]\d{3}-(0[1-9]|1[0-2])$/);
        this.avj.addKeyword('x-apifox-orders');
        this.guards = guards;
        for (let i = 0; i < this.guards.length; i++) {
            if (!this.guards[i].regexp) {
                let keys: any = [];
                this.guards[i].regexp = [];
                let paths: string[] = [];
                if (this.guards[i].path instanceof Array) {
                    paths = [...this.guards[i].path];
                } else {
                    paths = [<string>this.guards[i].path];
                }
                for (let j = 0; j < paths.length; j++) {
                    this.guards[i].regexp!.push(pathToRegexp(paths[j], keys, {
                        strict: false,
                        sensitive: false,
                        end: true
                    }));
                }

            }
        }
    }

    private compile_schema(schema: any, def_schema?: any): ValidateFunction {
        if (def_schema) return this.avj.addSchema(def_schema).compile(schema);
        else return this.avj.compile(schema);
    }

    // 校验api参数是否正确
    public check_params(processor: IApiProcessor, json: any): Promise<void> {
        return new Promise<void>((resolve, reject) => {
            try {
                if (!processor.validator) {
                    processor.validator = this.compile_schema(processor.schema, processor.def_schema);
                }
                let validator: ValidateFunction = processor.validator;
                let correct: boolean = validator(json);
                if (!correct) {
                    let msg: string = '';
                    Logger.debug(validator.errors);
                    if (validator.errors && validator.errors.length > 0)
                        msg = JSON.stringify(validator.errors);
                    reject(Resp.gen_err(Resp.ParamsError, msg));
                } else {
                    resolve();
                }
            }catch (e) {
                reject(Resp.throw_err(e));
            }

        })

    }

    public guard(path: string, json: any, cached_data: ICachedData, sign_info?: ISignInfo): Promise<void> {
        return new Promise<void>(async (resolve, reject) => {
            try {
                for (let i = 0; i < this.guards.length; i++) {
                    for (let j = 0; j < this.guards[i].regexp!.length; j++) {
                        if (this.guards[i].regexp![j].test(path)) {
                            for (let k = 0; k < this.guards![i].guards.length; k++)
                                await this.guards![i].guards[k](json, cached_data);
                            resolve();
                        }
                    }

                }
                return resolve();
            } catch (e) {
                reject(e);
            }

        });
    }


    public audit(path: string, json: any, module: IMethodModule, cached_data: ICachedData): Promise<void> {
        return new Promise<void>(async (resolve, reject) => {
            try {
                await this.guard(path, json, cached_data);
                await this.check_params(module.processor!, json);
                resolve();
            } catch (e) {
                reject(e);
            }
        })
    }

    public proxy_audit(path: string, content: any, cached_data: ICachedData, sign_info?: ISignInfo): Promise<void> {
        return new Promise<void>(async (resolve, reject) => {
            try{
                await this.guard(path, content, cached_data, sign_info);
                resolve();
            } catch (e) {
                reject(e);
            }

        })

    }
}
