import {Model, QueryTypes, Sequelize, Transaction, WhereOptions} from "sequelize";
import {Resp} from "@util/Resp";
import {clone, extend} from "@util/JsonToolkit";
import {IDataMap, IMethodParams, ICachedData, IRequest, IPageRequestData} from "@core/Defined";
import dayjs from "dayjs";
import {Logger} from "@util/Logger";
import {AcsDomain} from "@core-models/AcsDomain";
import {AcsUserDomain} from "@core-models/AcsUserDomain";
import {ModelStatic} from "sequelize/types/model";


export interface IPageListResponse<T> {
    page_count?: number;
    page_no: number;
    total: number;
    list: Array<T>
}

export interface ISQLReplacements {
    [x: string]: unknown;
}

export interface IPageSQLInfo {
    sequelize: Sequelize;
    page_no?: number;
    page_size?: number;
    count_sql: string;
    count_replacements: ISQLReplacements;
    select_sql: string;
    select_replacements: ISQLReplacements;
}

function build_input_data_values(json: any, input_data_map?: IDataMap, fixed_input_data?: any): any {
    let values: any = {};
    if (!input_data_map) return values;
    for (let field in input_data_map) {
        if (!input_data_map.hasOwnProperty(field)) continue;
        if (json[field] !== undefined) {
            values[<string>input_data_map[field]] = json[field];
        }
    }
    if (fixed_input_data) {
        for (let field in fixed_input_data) {
            if (!fixed_input_data.hasOwnProperty(field)) continue;
            values[field] = fixed_input_data[field];
        }
    }
    return values;
}

function build_returning(output_data_map?: IDataMap): Array<string> {
    let returning: Array<string> = [];
    for (let field in output_data_map) {
        if (!output_data_map.hasOwnProperty(field)) continue;
        returning.push(field);
    }
    return returning;
}

function build_where(content: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            if (params.where) {
                if (params.where instanceof Function) {
                    return resolve(await params.where(content, params, cached_data));
                } else {
                    return resolve(params.where);
                }
            }
            resolve(undefined);
        } catch (e) {
            Logger.error(e);
            reject(e);
        }
    })
}

function convert_output_values(field: string, value: any, output_data_map: IDataMap, model?: any): any {
    let result;
    if (model && model.attributes[field]?.type?.key === 'DATE') {
        if (!value) {
            result = undefined;
        } else {
            result = dayjs(value).format('YYYY-MM-DD HH:mm:ss');
        }
    } else if (model && (model.attributes[field]?.type?.key === 'DECIMAL' || model.attributes[field]?.type?.type?.key === 'DECIMAL')) {
        if (!value) {
            result = undefined;
        } else {
            result = parseFloat(value);
        }
    } else if (value === null) {
        result = undefined;
    } else {
        result = value;
    }
    return result;
}

function build_output_data_values(record: any, output_data_map?: IDataMap, model?: ModelStatic<Model>) {
    if (!output_data_map) return clone(record);
    let result: any = {};
    for (let field in output_data_map) {
        if (!output_data_map.hasOwnProperty(field)) continue;
        if (record[field] !== undefined) {
            if (record[field] instanceof Array) {
                if (output_data_map[field] instanceof Array) {
                    for (let i = 0; i < record[field].length; i++) {
                        result[output_data_map[field][i]] = convert_output_values(field, record[field][i], output_data_map, model);
                    }
                } else {
                    let values: any[] = [];
                    for (let i = 0; i < record[field].length; i++) {
                        values.push(convert_output_values(field, record[field][i], output_data_map, model));
                    }
                    result[<string>output_data_map[field]] = values;
                }
            } else {
                result[<string>output_data_map[field]] = convert_output_values(field, record[field], output_data_map, model);
            }
        }
    }
    return result;
}

function convert_list(list: Array<any>, output_data_map?: IDataMap, model?: any): Array<any> {
    let result: Array<any> = [];
    for (let i = 0; i < list.length; i++) {
        let item = build_output_data_values(list[i], output_data_map, model);
        result.push(item);
    }
    return result;
}

/**
 * 可在method_params中直接使用的函数，用于在数据模型中添加一行数据
 *
 * @param content       传入的json数据
 * @param params        定义在IApiProcess中的method_params参数
 * @param cached_data   从api网关传递过来的缓存数据
 * @param t             事务
 */
function internal_add_row(content: IRequest, params: IMethodParams, cached_data: any, t?: Transaction): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        if (!params || !params.model) {
            return reject(Resp.gen_err(Resp.InternalServerError));
        }
        let model = params.model;
        try {
            let data = content.data;
            if (params.fixed_input_data) {
                data = extend(data, params.fixed_input_data);
            }
            let result = await add_row(model, data, params.input_data_map, params.output_data_map, t);
            resolve(result);
        } catch (e) {
            Logger.error(e);
            return reject(Resp.throw_err(e));
        }
    });
}

/**
 * 往指定的数据模型中添加了行数据
 *
 * @param model             数据模型
 * @param input_data        输入的json数据
 * @param input_data_map    输入的json数据与数据模型字段的对应关系
 * @param returning_map     返回的数据与输出数据的对应关系
 * @param t
 */
function add_row<T extends ModelStatic<Model>>(model: T, input_data: any, input_data_map?: IDataMap, returning_map?: IDataMap, t?: Transaction): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            let values: any = build_input_data_values(input_data, input_data_map);
            let record = await model.create(values, {transaction: t});
            await record.save();
            let result: any = build_output_data_values(record, returning_map);
            return resolve(result);
        } catch (e) {
            Logger.error(e);
            return reject(e);
        }
    });
}


/**
 * 可在method_params中直接使用的函数，用于修改或添加一行
 *
 * @param content          传入的json数据
 * @param _params       定义在IApiProcess中的method_params参数
 * @param _cached_data  从api网关传递过来的缓存数据
 * @param t             事务
 */
function internal_upsert_row(content: IRequest, _params: IMethodParams, _cached_data: ICachedData, t?: Transaction): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        if (!_params || !_params.model || !_params.model.sequelize) {
            return reject(Resp.gen_err(Resp.InternalServerError));
        }
        try {
            let model = _params.model;
            let data = content.data;
            if (_params.fixed_input_data) {
                data = extend(data, _params.fixed_input_data);
            }
            let result = await add_row(model, data, _params.input_data_map, _params.output_data_map, t);
            resolve(result);
        } catch (e) {
            Logger.error(e);
            return reject(Resp.throw_err(e));
        }
    });
}

/**
 * 修改或添加一行，当主键不冲突时，添加一行，否则就更新
 *
 * @param model             数据模型
 * @param input_data        输入的json数据
 * @param input_data_map    输入的json数据与数据模型字段的对应关系
 * @param returning_map     返回的数据与输出数据的对应关系
 * @param t
 */
function upsert_row(model: any, input_data: any, input_data_map?: IDataMap, returning_map?: IDataMap, t?: Transaction): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            let values = build_input_data_values(input_data, input_data_map);
            let record = await model.upsert(values, {transaction: t});
            let result: any = build_output_data_values(record, returning_map);
            return resolve(result);
        } catch (e) {
            Logger.error(e);
            return reject(e);
        }
    });
}


/**
 * 可在method_params中直接使用的函数，用于修改一行数据
 *
 * @param content          传入的json数据
 * @param params        定义在IApiProcess中的method_params参数
 * @param cached_data   从api网关传递过来的缓存数据
 * @param t             事务
 */
function internal_update_row(content: IRequest, params: IMethodParams, cached_data: ICachedData, t?: Transaction): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        if (!params || !params.model) {
            return reject(Resp.gen_err(Resp.InternalServerError));
        }

        try {
            let model = params.model;
            let data = content.data;
            let where = await build_where(content, params, cached_data);
            if (where === undefined && params.force_no_where !== true) {
                return reject(Resp.gen_err(Resp.InternalServerError, '危险：修改数据时未提供where条件'))
            }
            if (params.fixed_input_data) {
                data = extend(data, params.fixed_input_data);
            }
            let result = await update_row(model, data, where, params.input_data_map, params.output_data_map, t);

            resolve(result);
        } catch (e) {
            Logger.error('internal update row');
            Logger.error(e);
            return reject(Resp.throw_err(e));
        }
    });
}

function update_row(model: any, input_data: any, where: WhereOptions, input_data_map?: IDataMap, returning_map?: IDataMap, t?: Transaction): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            if (where === undefined) {
                return reject(Resp.gen_err(Resp.InternalServerError, '危险：修改数据时未提供where条件'))
            }
            let values = build_input_data_values(input_data, input_data_map);
            let [count, rows] = await model.update(values, {
                where: where,
                returning: build_returning(returning_map),
                transaction: t
            });
            let result: any = {};
            if (count > 0) {
                result = build_output_data_values(rows[0], returning_map);
            } else {
                return reject(Resp.gen_err(Resp.ResourceNotFound, '更新失败'));
            }
            resolve(result);
        } catch (e) {
            Logger.error(e);
            return reject(e);
        }
    });
}


// 获取列表
function internal_get_rows(content: IRequest, params: IMethodParams, cached_data: ICachedData, t?: Transaction): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        if (!params || !params.model) {
            return reject(Resp.gen_err(Resp.InternalServerError));
        }

        let model = params.model;
        try {
            let data = <IPageRequestData>content.data;
            let where = await build_where(content, params, cached_data);

            let options: any = {
                order: params.order,
                include: params.include,
                raw: true,
                nest: false,
                transaction: t
            };
            if (where) {
                options.where = where;
            }
            if (data.page_no !== undefined && data.page_size !== undefined) {
                options.offset = (data.page_no - 1) * data.page_size;
                options.limit = data.page_size;
            }
            const {count, rows} = await model.findAndCountAll(options);
            let list = convert_list(rows, params.output_data_map, model);
            let listName = 'list';
            if (params.output_data_map && params.output_data_map['list']) {
                listName = <string>params.output_data_map['list'];
            }
            let result: any = {
                page_no: data.page_no,
                total: count,
                [listName]: list
            };
            if (data.page_size !== undefined) {
                result.page_count = Math.ceil(count / data.page_size);
            }

            return resolve(result);
        } catch (e: any) {
            Logger.error(e);
            return reject(Resp.throw_err(e));
        }
    })
}

function get_rows(model: any, input_data: any, where?: WhereOptions,
                  input_data_map?: IDataMap, output_data_map?: IDataMap,
                  page_no?: number, page_size?: number,
                  t?: Transaction): Promise<any> {
    return new Promise(async (resolve, reject) => {
        try {
            let options: any = {
                raw: true,
                nest: false,
                transaction: t
            };
            if (where) {
                options.where = where;
            }
            if (page_no !== undefined && page_size !== undefined) {
                options.offset = (page_no - 1) * page_size;
                options.limit = page_size;
            }
            const {count, rows} = await model.findAndCountAll(options);
            let list = convert_list(rows, output_data_map, model);
            let listName = 'list';
            if (output_data_map && output_data_map['list']) {
                listName = <string>output_data_map['list'];
            }
            let result: any = {

                total: count,
                [listName]: list
            };
            if (page_size !== undefined) {
                result.page_no = page_no;
                result.page_count = Math.ceil(count / page_size);
            }

            return resolve(result);
        } catch (e) {
            reject(e);
        }
    });
}

function internal_get_row(content: IRequest, params: IMethodParams, cached_data: ICachedData | any, t?: Transaction): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        if (!params || !params.model) {
            return reject(Resp.gen_err(Resp.InternalServerError));
        }

        try {
            let data = content.data;
            let where = await build_where(content, params, cached_data);

            let row = await params.model.findOne({
                where: where,
                include: params.include,
                raw: true,
                nest: false,
                transaction: t
            });
            if (row === null) {
                return reject(Resp.gen_err(Resp.ResourceNotFound))
            }
            let result: any = build_output_data_values(row, params.output_data_map, params.model);
            resolve(result);

        } catch (e) {
            Logger.error(e);
            return reject(Resp.throw_err(e));
        }
    });
}

// 删除一行数据
async function remove_row(model: any, where: WhereOptions, t?: Transaction): Promise<void> {
    if (!model) return Resp.gen_err(Resp.InternalServerError);
    try {
        let count = await model.destroy({where: where, transaction: t});
        if (count === 0) {
            return Resp.gen_err(Resp.ResourceNotFound, '删除时没有找到指定记录或不符合删除条件');
        }
    } catch (e) {
        Logger.error(e);
        return Resp.throw_err(e);
    }

}
// 删除一行数据（内部调用）
function internal_remove_row(content: IRequest, params: IMethodParams, cached_data: ICachedData, t?: Transaction): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        if (!params || !params.model) {
            return reject(Resp.gen_err(Resp.InternalServerError));
        }
        try {
            let data = content.data;
            let where = await build_where(content, params, cached_data);
            if (where === undefined && params.force_no_where !== true) {
                return reject(Resp.gen_err(Resp.InternalServerError, '危险：删除数据时未提供where条件'))
            }
            let count = await params.model.destroy({where: where, transaction: t});
            if (count === 0) {
                return reject(Resp.gen_err(Resp.ResourceNotFound, '删除时没有找到指定记录或不符合删除条件'))
            }
            resolve({});
        } catch (e) {
            Logger.error(e);
            return reject(Resp.throw_err(e));
        }
    });
}

async function internal_get_tree(content: IRequest, params: IMethodParams, cached_data: ICachedData, t?: Transaction): Promise<any> {
    return await get_tree(cached_data.user_id, (<any>content.data).pid || params.addition?.pid, (<any>content.data).category, (<any>content.data).tag, params.addition, t);
}

function get_tree(user_id: string, pid?: number, category?:string|Array<string>, tag?: string | Array<string>, addition?: any, t?: Transaction): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            let sql = `
                select id, name, tag, category, pid, path, nlevel(path) as depth, memo
                from ${AcsDomain.table_name} 
                where path <@ array(
                  select path from ${AcsDomain.table_name} domain, ${AcsUserDomain.table_name} user_domain
                  where domain.id = user_domain.domain_id and user_domain.user_id = :user_id
                )
            `;
            let replacements: ISQLReplacements = {
                user_id: user_id
            }
            if (pid !== undefined) {
                sql += ` and path <@ (select path from ${AcsDomain.table_name} where id = :pid) `;
                replacements.pid = pid;
            }
            if (tag) {
                if (typeof tag === 'string') {
                    sql += ` and tag = :tag `;
                    replacements.tag = tag;
                } else if (tag.length > 0) {
                    sql += ` and (`;
                    for (let i = 0; i < tag.length; i++) {
                        if (i > 0) sql += ' or ';
                        sql += ` tag = :tag${i} `;
                        replacements[`tag${i}`] = tag[i];
                    }
                    sql += ')';
                }
            }

            if (category) {
                if (typeof category === 'string') {
                    sql += ` and category = :category `;
                    replacements.category = category;
                } else if (category.length > 0) {
                    sql += ` and (`;
                    for (let i = 0; i < category.length; i++) {
                        if (i > 0) sql += ' or ';
                        sql += ` category = :category{i} `;
                        replacements[`category${i}`] = category[i];
                    }
                    sql += ')';
                }
            }

            if ((!addition?.with_pid || addition?.with_pid === false) && pid !== undefined) {
                sql += ` and id <> :pid  `;
                replacements.pid = pid;
            }
            if (addition && addition.category && addition.category.length) {
                sql += ' and ('
                for (let i = 0; i < addition.category.length; i++) {
                    if (i > 0) sql += ' or ';
                    sql += `category = '${addition.category[i]}' `
                }
                sql += ')';
            }

            if (addition && addition.tag && addition.tag.length) {
                sql += ' and ('
                for (let i = 0; i < addition.tag.length; i++) {
                    if (i > 0) sql += ' or ';
                    sql += `tag = '${addition.tag[i]}' `
                }
                sql += ')';
            }

            sql += ` order by path `;
            let rows = await AcsDomain.sequelize!.query<ITreeNodeRaw>(sql, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: replacements,
                transaction: t
            });
            rows = DataCURD.filter_null(rows);
            let tree: any = [];
            build_tree(rows, tree);
            let name = 'tree';
            if (addition && addition.output_name) {
                name = addition.output_name;
            }
            Logger.trace(JSON.stringify({[name]: tree}));
            resolve({[name]: tree});
        } catch (e) {
            Logger.error(e);
            return reject(e);
        }
    })
}

interface ITreeNodeRaw {
    id: number;
    pid: number;
    name: string;
    path: string;
    depth: number;
    tag?: string;
    category?: string;
    memo?: string;
}

interface ITreeNode {
    id: number;
    name: string;
    tag?: string;
    category?: string;
    memo?: string;
    children: Array<ITreeNode>;
}

function build_tree(rows: Array<ITreeNodeRaw>, tree: Array<ITreeNode>) {
    for (let row of rows) {
        if (!insert_into_tree(row, tree)) {
            let node = {
                id: row.id,
                name: row.name,
                tag: row.tag,
                category: row.category,
                memo: row.memo,
                children: []
            };
            tree.push(node);
        }
    }
}

function insert_into_tree(row: any, tree: any): boolean {
    let result: boolean = false;
    for (let tree_node of tree) {

        let node = {
            id: row.id,
            name: row.name,
            tag: row.tag,
            memo: row.memo,
            children: []
        };

        if (row.pid == tree_node.id) {
            tree_node.children.push(node);
            result = true;
            break;
        }

        if (tree_node.children != undefined) {
            result = insert_into_tree(row, tree_node.children);
            if (result) break;
        }
    }

    return result;
}

async function internal_add_tree_node(content: IRequest, params: IMethodParams, cached_data: ICachedData, t?: Transaction): Promise<any> {
    return await add_tree_node((<any>content.data).pid || params.addition?.pid, (<any>content.data).name,
    (<any>content.data).tag || params.addition?.tag,
        (<any>content.data).category || params.addition?.category,
        (<any>content.data).memo, t);
}

function add_tree_node(pid: number, name: string, tag?: string, category?: string, memo?: string, t?: Transaction): Promise<{ id: number }> {
    return new Promise<{ id: number }>(async (resolve, reject) => {
        let transaction: Transaction | undefined = t;
        if (!transaction) {
            transaction = await AcsDomain.sequelize!.transaction();
        }
        try {
            let parent = await AcsDomain.findOne({
                where: {id: pid},
                transaction: transaction,
                raw: true
            });
            if (!parent) {
                if (!t) await transaction.rollback();
                return reject(Resp.gen_err(Resp.ResourceNotFound, '找不到父域'));
            }
            let domain = await AcsDomain.create({
                name: name,
                pid: pid,
                tag: tag,
                category: category,
                memo: memo
            }, {transaction: transaction, raw: true});
            let domain_id = domain.id;
            await AcsDomain.update({
                path: parent.path + '.' + domain_id
            }, {where: {id: domain_id}, transaction: transaction});
            if (!t) await transaction.commit();
            resolve({id: domain_id});
        } catch (e) {
            Logger.error(e);
            if (!t) await transaction.rollback();
            reject(e);
        }
    })
}

/**
 * 移动一个树节点
 *
 * @param id    要移动的树节点
 * @param pid   目标节点
 * @param tag   目标节点tag，可以为undefined，如不为空，则表示只能移动到指定tag的目标节点
 * @param t     事务，可以为undefined
 */
function move_tree_node(id: number, pid: number, tag?: string, t?: Transaction): Promise<void> {
    return new Promise(async (resolve, reject) => {
        let transaction: Transaction | undefined = t;
        if (!transaction) {
            transaction = await AcsDomain.sequelize!.transaction();
        }
        try {
            let sql;
            // 如果父节点就是自己，直接返回错误
            if (pid === id) {
                throw Resp.gen_err(Resp.ParamsError, `目标父节点不能是自己。`)
            }
            let node = await AcsDomain.findOne({where: {id: id}, transaction: transaction, raw: true});
            if (!node) throw Resp.gen_err(Resp.ResourceNotFound, `节点（id: ${id}）不存在。`)
            let original_pid: number = node.pid;
            let original_path: string = node.path;

            // 如果要目标父节点已经是当前节点，就无需执行
            if (pid === original_pid) {
                if (!t) await transaction.rollback();
                return resolve();
            }

            // 获取新父节点的path
            let where: WhereOptions<any> = {id: pid};
            if (tag && tag.trim() !== '') where.tag = tag.trim();
            let new_parent_node = await AcsDomain.findOne({where: where, transaction: transaction});
            if (!new_parent_node) {
                throw Resp.gen_err(Resp.ResourceNotFound, `目标父节点（id：${pid}）不存在或类型不符。`);
            }
            let new_parent_path = new_parent_node.path;

            // 检查如果目标父节点是自己或自己的子节点，则不允许移入
            sql = `
                select id from ${AcsDomain.table_name}
                where path <@ (select path from ${AcsDomain.table_name}where id=:id) and id = :pid;
            `;
            let result = await AcsDomain.sequelize!.query(sql,
              {
                  type: QueryTypes.SELECT,
                  transaction: transaction,
                  replacements: {id: id, pid: pid}
              });
            if (result.length > 0) {
                throw Resp.gen_err(Resp.ParamsError, `目标父节点不能是自己的下属节点。`);
            }

            // 修改当前节点的父节点
            await AcsDomain.update({pid: pid, path: new_parent_path + '.' + id}, {
                where: {id: id}, transaction: transaction
            });

            // 递归修改子节点path
            let new_path_prefix = `${new_parent_path}.${node.id}.`;
            let old_path_prefix = `${original_path}.`;
            sql = `
                with RECURSIVE T(id, pid, path) as (
                    select id, pid, path from ${AcsDomain.table_name}where id=:id
                    union all
                    select D.id, D.pid, D.path
                    from ${AcsDomain.table_name} D
                    join T on D.pid = T.id
                )
                update ${AcsDomain.table_name}
                set path = replace(path::text, :old_path_prefix, :new_path_prefix)::ltree
                where id in (select distinct id from T )
            `;
            let replacements: ISQLReplacements = {
                id: id,
                old_path_prefix: old_path_prefix,
                new_path_prefix: new_path_prefix
            };
            await AcsDomain.sequelize!.query(sql, {
                type: QueryTypes.UPDATE,
                replacements: replacements,
                transaction: transaction
            });
            if (!t) await transaction.commit();
            resolve();
        } catch (e) {
            if (!t) await transaction.rollback();
            reject(e);
        }
    });
}

/**
 * 删除一个树节点
 * @param id        树节点id
 * @param force     是否强制删除，如果为true，则删除自己及所有子节点，但如果子节点有被引用，则仍无法删除
 * @param t         事务
 */
function remove_tree_node(id: number, force: boolean, t?: Transaction): Promise<void> {
    return new Promise(async (resolve, reject) => {
        try {
            let sql;
            if (force) {
                sql = `
                    with RECURSIVE T(id, pid, path) as (
                        select id, pid, path from ${AcsDomain.table_name}where id=:id
                        union all
                        select D.id, D.pid, D.path
                        from ${AcsDomain.table_name} D
                        join T on D.pid = T.id
                    )
                    delete from ${AcsDomain.table_name}
                    where id in (select distinct id from T )
                `;
            } else {
                sql = `
                    delete from ${AcsDomain.table_name}where id = :id
                `;
            }
            let replacements: ISQLReplacements = {
                id: id,
            };
            await AcsDomain.sequelize!.query(sql, {
                type: QueryTypes.DELETE,
                replacements: replacements,
                transaction: t
            });
            resolve();
        } catch (e) {
            reject(e);
        }
    });
}

function get_count(sequelize: Sequelize, sql: string, replacements?: ISQLReplacements, t?: Transaction): Promise<number> {
    return new Promise<number>(async (resolve, reject) => {
        try {
            let result = await sequelize.query<any>(sql, {
                type: QueryTypes.SELECT,
                replacements: replacements,
                transaction: t
            });
            let count = parseInt(result[0].count);
            resolve(count);
        } catch (e) {
            Logger.error(e);
            reject(Resp.throw_err(e));
        }
    })
}

function get_list<T extends object>(sequelize: Sequelize, page_no: number, page_size: number,
                                    count: number, sql: string, replacements?: ISQLReplacements,
                                    t?: Transaction): Promise<IPageListResponse<T>> {
    return new Promise<IPageListResponse<T>>(async (resolve, reject) => {
        if (count === 0) {
            return resolve({
                page_count: 0,
                page_no: page_no,
                total: 0,
                list: []
            })
        }
        if (page_size && page_no) {
            let offset_num = (page_no - 1) * page_size;
            sql += ' limit ' + page_size + ' offset ' + offset_num;
        }

        try {
            let result = await sequelize.query<T>(sql, {
                type: QueryTypes.SELECT,
                replacements: replacements,
                raw: true,
                transaction: t
            });
            resolve({
                page_count: Math.ceil(count/page_size),
                page_no: page_no,
                total: count,
                list: result
            });
        } catch (e: any) {
            Logger.error(e);
            reject(e);
        }

    })
}

function get_page_list<T extends object>(pagedSQLInfo: IPageSQLInfo, t?: Transaction): Promise<IPageListResponse<T>> {
    return new Promise<IPageListResponse<T>>(async (resolve, reject) => {
        try {
            let count: number = await get_count(pagedSQLInfo.sequelize, pagedSQLInfo.count_sql, pagedSQLInfo.count_replacements, t);
            let result: IPageListResponse<T> = await get_list<T>(
              pagedSQLInfo.sequelize,
              pagedSQLInfo.page_no ? pagedSQLInfo.page_no : 0,
              pagedSQLInfo.page_size ? pagedSQLInfo.page_size : 0,
              count,
              pagedSQLInfo.select_sql,
              pagedSQLInfo.select_replacements,
              t
            );
            resolve(result);
        } catch (e) {
            Logger.error(e);
            reject(e);
        }
    })
}

/// dateFields { 字段名1: 时间格式, 字段名2: 时间格式 }
function filter_null(record: any, dateFields?: Map<string, string>) {
    let result: any = {};
    if (record instanceof Array) {
        result = [];
        for (let i = 0; i < record.length; i++) {
            result.push(filter_null(record[i], dateFields));
        }
    } else if (typeof record === "object") {
        for (let field in record) {
            if (record[field] !== undefined && record[field] !== null) {
                if (typeof record[field] === "object" && (!dateFields || !dateFields.has(field)))
                    result[field] = filter_null(record[field], dateFields);
                else {
                    if (dateFields && dateFields.has(field)) {
                        result[field] = dayjs(record[field]).format(dateFields.get(field));
                    } else {
                        result[field] = record[field];
                    }
                }
            }
        }
    } else {
        result = record;
    }
    return result;
}

export default class DataCURD {
    // 这部分函数可直接在IApiProcessor配置中调用，也可在函数中调用
    static internal_add_row = internal_add_row;             // 添加一行
    static internal_update_row = internal_update_row;       // 更新一行
    static internal_upsert_row = internal_upsert_row;       // 添加一行，如主键已存在，则修改
    static internal_get_row = internal_get_row;             // 获取一行
    static internal_get_rows = internal_get_rows;           // 获取多行
    static internal_remove_row = internal_remove_row;       // 删除一行
    static internal_get_tree = internal_get_tree;           // 获取树
    static internal_add_tree_node = internal_add_tree_node;

    // 这部分函数可在其它函数中调用
    static add_tree_node = add_tree_node;                   // 添加一个树节点
    static get_tree = get_tree;                             // 获取树
    static move_tree_node = move_tree_node;                 // 移动树节点
    static remove_tree_node = remove_tree_node;             // 删除树节点


    static add_row = add_row;                               // 添加一行
    static upsert_row = upsert_row;                         // 添加一行，如主键已存在，则修改
    static update_row = update_row;                         // 修改一行
    static remove_row = remove_row;                         // 删除一行
    static get_rows = get_rows;                             // 获取列表
    static get_page_list = get_page_list;                   // 获取分页列表

    static convert_list = convert_list;
    static filter_null = filter_null;

}
