import {Model, Transaction, WhereOptions} from "sequelize";
import {ModelStatic} from "sequelize/types/model";
import {Orm} from "@util/Orm";

export const ADMINISTRATOR = 'administrator';

export interface IParamsMap {
    [x: string]: string;
}

export interface IDataMap {
    [x: string]: string | Array<string>;
}

export type TWhereFunction = (json: IRequest, params: IMethodParams, cached_data: ICachedData) => Promise<WhereOptions>;

export interface IDynamicMethodParams {
    data_src: Orm;          // 数据源
    sql: string;            // 获取数据的sql语句
    field_mapping?: any;    // 字段映射，{数据库字段名: 输出字段名, ...}
}

export interface IMethodParams {
    model?: any;     // 数据模型
    where?: object | TWhereFunction;      // where条件
    transaction?: Transaction;      // 数据库事务，跨多个函数时使用
    params_alias?: IParamsMap;      // 输入参数别名，为兼容老版本，用于将老版本名称转换成新版本名，key: value -> 老版本参数名：新版本参数名
    input_data_map?: IDataMap;      // 输入参数与表字段的映射关系，key: value  ->  json field: table field
    fixed_input_data?: any;         // 固定的输入参数，key: value，就是指除了input data中输入的key-value外，固定指定要输入的内容
    output_data_map?: IDataMap;     // 表字段与输出参数的映射关系，key: value  ->  table field: json field
    include?: any;                  // options中的include字段
    exclude?: Array<string>;        // 直接返回查表结果时，去除的字段名
    order?: Array<Array<string>>;   // 排序方式 [['field1', 'asc'], ['field2', 'desc']]
    force_no_where?: boolean;       // 删除或修改数据时，如果没有where条件，是否仍强制删除，默认false（危险！设为true时可能会删除表中所有数据）
    dynamic_method_params?: IDynamicMethodParams;   // 动态路由时使用的参数
    addition?: any;             // 附加参数，不同的接口可附加不同的参数
}

export interface IApiProcessor {
    def_schema?: any;                               // 外部引用的schema
    schema: any;                                    // json schema
    method: (content: IRequest, params: IMethodParams, cached_data: ICachedData | any) => Promise<unknown>;    // 处理api的方法
    validator?: any;                                // 编译后的json schema
    method_params: IMethodParams;
    guards?: Array<(content: IRequest, cached_data: ICachedData) => Promise<void>>;
    on_start?: (content: IRequest, params: IMethodParams, cached_data: ICachedData | any) => Promise<unknown>;  // 接口执行前，执行的操作
    on_before_respond?: (content: IRequest, params: IMethodParams, cached_data: ICachedData | any, result: any) => Promise<unknown>;  // 执行成功但还没有回复时，执行的操作
    on_completed?: (content: IRequest, params: IMethodParams, cached_data: ICachedData | any, result: any) => Promise<unknown>;  // 执行成功并已回复后，执行的操作
    on_failed?: (content: IRequest, e: any) => Promise<unknown>;     // 执行失败后，执行的操作
}

export interface IMethodModule {
    api: string;
    path: string;
    processor?: IApiProcessor;
}

export interface IGuard {
    path: string | Array<string>;// 含通配符的路径，用于匹配api接口
    guards: Array<(json: any, cached_data: ICachedData) => Promise<void>>;// 匹配成功的api要通过的守卫列表
    regexp?: Array<RegExp>;//将path编译后的正则表达式
}

export type TGuardFunction = (content: IRequest | unknown, cached_data: ICachedData) => Promise<void>;

export interface IDataModel {
    table_name: string;
    attributes: any;
    indexes: Array<any>;
    associate: () => void;
    init_sqls: Array<string>;
}

//api网关路由时用到的api信息
export interface IApiInfo {
    api: string;
}

// 登录时存在redis中的缓存数据
export interface ILoginCachedData {
    user_id: string;
    app_id: string;
    app_type: number;
}

// 从api网关发送来的缓存数据
export interface ICachedData {
    from?: string;
    api?: string;
    user_id: string;
    user_name: string;
    app_id: string;
    app_type: number;
    [property: string]: any;
}

// 签名相关数据
export interface ISignInfo {
    timestamp: number;
    sign: string;
}

export interface IRequest {
    // 公共参数
    ver: string;
    app_id: string;
    author?: string;
    timestamp: number;
    sign?: string;
    token?: string;
    data: unknown;          // 请求参数
}

export interface IPageRequestData {
    page_no?: number;
    page_size?: number;

    [x: string]: any;
}

export const Platform = {
    wx: 1,
    alipay: 2
}
