import {Sequelize} from "sequelize";
import {Logger} from "@util/Logger";
import {orm} from "@core/Infrastructure";

export interface IDataModelInfo {
    tag: string;
    model: any;
    timestamps?: boolean;
    created_at?: string | boolean; // undefined: 使用原名createdAt, string: 使用新名， false：不创建这个字段
    updated_at?: string | boolean; // undefined: 使用原名updatedAt, string: 使用新名， false：不创建这个字段
}

export async function init_data_models(id: string, models: Array<IDataModelInfo>, force_async_db?: boolean) {
    await init_models(id, models, force_async_db);
    await init_associate_and_default(id, models);
}

async function init_models(id: string, models: Array<IDataModelInfo>, force_async_db?: boolean) {
    let orm_instance = orm(id);
    let sequelize: Sequelize = orm_instance.sequelize;
    for (let i = 0; i < models.length; i++) {
        let info = models[i];

        // TODO: 自动化处理模型之间的reference依赖关系与模型初始化顺序之间的关系
        // 数据模型的tag不包含在此数据库连接实例里，则跳过
        if (!orm_instance.model_tags.includes(info.tag)) continue;
        try {
            info.model.init(info.model.attributes,
              {
                  indexes: info.model.indexes,
                  tableName: info.model.table_name,
                  timestamps: !!info.timestamps,
                  createdAt: info.created_at,
                  updatedAt: info.updated_at,
                  sequelize
              });
            await info.model.sync({force: force_async_db});
        } catch (e) {
            Logger.error(e);
        }
    }
}

async function init_associate_and_default(id: string, models: Array<IDataModelInfo>) {
    let sequelize: Sequelize = orm(id).sequelize;
    for (let i = 0; i < models.length; i++) {
        let info = models[i];
        // 数据模型的tag不包含在此数据库连接实例里，则跳过
        if (!orm(id).model_tags.includes(info.tag)) continue;
        if (info.model.associate) {
            try {
                info.model.associate();
            } catch (e) {
                Logger.error(e);
            }

        }
        if (info.model.init_sqls) {
            for (let i = 0; i < info.model.init_sqls.length; i++) {
                try {
                    await sequelize.query(info.model.init_sqls[i]);
                } catch (e) {
                    Logger.error(e);
                }
            }
        }
    }
}
