import express = require('express');
import compress = require('compression');
import * as bodyParser from 'body-parser';
import {Router} from "@core/Router";

import {Logger} from "@util/Logger";
import {Server} from "http";
import {Config} from "@util/Config";

let https = require('https');
const http = require('http');

export interface IMicroSvrParams {
    port?: number;           // 服务端口
    key?: string;            // key证书
    cert?: string;           // cert证书
    ca?: string,
    requestCert?: boolean,
    rejectUnauthorized?: boolean,
    router: any;
}

declare namespace Express {
    export interface Request {
        rawBody: any;
    }
}

export class MicroSvr {
    private readonly _app: express.Application;
    private readonly router: Router;
    private readonly port?: number;
    private https_server?: Server;
    private http_server?: Server;

    constructor(params: IMicroSvrParams) {
        this._app = express();
        this.router = params.router;
        this.port = params.port;
        this._app.use(compress());
        this._app.use(bodyParser.json({limit: '50mb'}));
        this._app.use(bodyParser.urlencoded({limit: '50mb', extended: false}));
        this._app.all('*', function (req, res, next) {
            res.header("Access-Control-Allow-Origin", "*");
            res.header("Access-Control-Allow-Headers", "X-Requested-With, Content-Type, Token, user-agent");
            res.header("Access-Control-Allow-Methods", "PUT,POST,GET,DELETE,OPTIONS");
            res.header("Access-Control-Max-Age", "3600");
            res.header("X-Powered-By", ' 3.2.1');
            res.header("Access-Control-Allow-Credentials", "true");
            next();
        });

        this._app.options('*', function (req, res) {
            res.sendStatus(200);
        });

        this._app.all("*", (req: express.Request, res: express.Response, next: express.NextFunction) => {
            if (req.method === 'GET') {
                req.body = req.query;
            }
            this.router.route(req, res, next).then(() => {
            });
        });

        this._app.all('*', function (req: express.Request, res: express.Response, _next: express.NextFunction) {
            res.status(404);
            res.render('error');
        });

        this.init_https_server();
        this.init_http_server();
    }

    init_https_server() {
        let config = Config.params();
        if (!config.https) return;
        let port = this.port ? this.port : config.https.port;

        this.https_server = https.Server({
            key: config.https.key,
            cert: config.https.cert
        }, this._app);

        this.https_server!.listen(port, () => {
            Logger.info(`HTTPS listening on port: ${port}`);
        });
    }

    init_http_server() {
        let config = Config.params();
        if (!config.http) return;
        let port = this.port ? this.port : config.http.port;

        this.http_server = http.createServer(this._app);
        this.http_server!.listen(port);
        this.http_server!.on('listening', () => {
            Logger.info(`HTTP listening on port: ${port}`);
        })

    }

    get app(): express.Application {
        return this._app;
    }

    stop() {
        if (this.http_server) {
            this.http_server.close();
        }
        if (this.https_server) {
            this.https_server.close();
        }
    }
}
