import https from 'https';
import express = require('express');
import * as http from "http";
import pathToRegexp from 'path-to-regexp';

import {Resp} from "@util/Resp";
// import {IProxyTarget} from "@src/proxyTargets";
import {Config} from "@util/Config";
import {Services} from "@util/Services";
import {Logger} from "@util/Logger";
//
// export interface ITarget {
//     serviceName: string;
//     host: string;
//     port: number;
//     protocol: 'https' |'http';
//     key?: string;
//     cert?: string;
// }

interface IProxyTarget {
    serviceName: string;
    path: Array<string>;
    regexp?: Array<RegExp>;
    protocol: 'http' | 'https';
    host: string;
    port: number;
    method?: string;
    key?: string;
    cert?: string;

}

export class Proxy {

    private readonly proxyTargets: Array<IProxyTarget>;
    constructor(gatewayInfo: Array<IProxyTarget>) {
        this.proxyTargets = gatewayInfo;
        for (let i = 0; i < this.proxyTargets.length; i++) {
            let info: IProxyTarget = this.proxyTargets[i];
            if (!info.regexp) info.regexp = [];
            for(let j = 0; j < info.path.length; j++) {
                info.regexp.push(pathToRegexp(info.path[j]));
            }
        }
    }

    private getTarget(api: string): IProxyTarget | null {
        for(let i = 0; i < this.proxyTargets.length; i++) {
            let info: IProxyTarget = this.proxyTargets[i];
            if (!info.regexp)  {
                throw new Error(`proxy target info error`);
            };
            for (let j = 0; j < info.path.length; j++) {
                if (info.regexp[j].test(api)) {
                    return info;
                }
            }
        }
        return null;
    }

    route(req: express.Request, res: express.Response, api: string): Promise<void> {
        return new Promise<void>((resolve, reject) => {
            let target = this.getTarget(req.path);
            if (!target) {
                // Resp.respond_err(api, res, Resp.gen_err(Resp.ProxyError, 'proxy target info error'));
                return reject(Resp.gen_err(Resp.ProxyError, 'proxy target info error'));
            }
            let options: any = {
                host: target.host,
                port: target.port,
                path: req.url,
                method: req.method,

                timeout: 3000,
            };

            let protocol = target.protocol === 'https'?https:http;
            if (target.protocol === 'https') {
                if (target.key) { // 如果配置了双向认证
                    options.key = target.key;
                    options.cert = target.cert;
                    options.requestCert = true;
                    options.rejectUnauthorized = true;
                } else {
                    options.rejectUnauthorized = false;
                }
            }
            let request = protocol.request(options, function (microSvrRes: http.IncomingMessage) {
                res.writeHead(microSvrRes.statusCode!, microSvrRes.headers);
                microSvrRes.on('data', function(buffer) {
                    res.write(buffer);
                })
                microSvrRes.on('end', function () {
                    res.end();
                })
                microSvrRes.on('error', function (err: Error) {
                    // Resp.respond_err(api, res, Resp.gen_err(Resp.ProxyError, err.message));
                    return reject(Resp.gen_err(Resp.ProxyError, err.message));
                })
            });
            request.on('error', function (err: Error) {
                // Resp.respond_err(api, res, Resp.gen_err(Resp.ProxyError, err.message));
                return reject(Resp.gen_err(Resp.ProxyError, err.message));
            })

            let service = Services.get_instance(target.serviceName);
            req.body.__sign = service.sign(req.body);
            req.body.__serviceName = Config.params().name;
            for(let key in req.headers) {
                if (key.toLowerCase() !== 'content-length') {
                    let value = <string>req.headers[key];
                    request.setHeader(key, value);
                }
            }
            request.setHeader('host', `${target.host}:${target.port}`);
            request.setHeader('Content-Type', 'application/json');
            request.write(JSON.stringify(req.body));
            request.end();
            return resolve();
        })

    }
}
