import express = require('express');
import {Logger} from "@util/Logger";
import {Resp} from "@util/Resp";
import {IApiProcessor, IGuard, IMethodModule, IMethodParams, ICachedData, IApiInfo} from "@core/Defined";
import {Auditor} from '@core/Auditor';
import {extend} from "@util/JsonToolkit";

export interface IRouteAliases {
    full_alias: any;
    path_alias: any;
    api_alias: any;
}

export class RouterBase {
    protected auditor: Auditor;
    protected readonly route_path: string;
    protected route_aliases?: IRouteAliases;

    // 是否是基于文件的module，true时通过调parser来获取路由模块，false调用重载的get_dynamic_module来得到路由模块
    protected file_based_modules: boolean = true;

    constructor(route_path: string, guards: Array<IGuard>, route_aliases?: IRouteAliases) {
        this.route_path = route_path;
        this.auditor = new Auditor(guards);
        this.route_aliases = route_aliases;
    }

    protected async get_dynamic_module(path: string, ver: string): Promise<IMethodModule> {
        return {
            api: '',
            path: path
        }
    }

    protected get_target_module(_require_path: string, _request_method: string, _ver: string): IApiProcessor {
        return {
            schema: {},
            method: (_content: unknown, _params: IMethodParams, _cached_data: ICachedData): Promise<unknown> => {
                return new Promise<unknown>(resolve => {
                    resolve({})
                })
            },
            method_params: {}
        };
    }

    protected parser(path: string, ver: string): IMethodModule {

        // 是否是全路径匹配的跳转接口，替换整个路径
        let full_match = false;
        if (this.route_aliases && this.route_aliases.full_alias[path]) {
            path = this.route_aliases.full_alias[path];
            full_match = true;
        }
        // 分析url路径，并生成路由路径
        let path_array: Array<string> = path.split('/');
        let api = path_array.pop() || '';
        let apiPath = path_array.join('/') + '/';
        if (this.route_aliases && !full_match) {  // 部分匹配的跳转接口，替换api或path
            api = this.route_aliases.api_alias[api] ? this.route_aliases.api_alias[api] : api;
            apiPath = this.route_aliases.path_alias[apiPath] ? this.route_aliases.path_alias[apiPath] : apiPath;
        }
        let api_array: Array<string> = api.split('.');
        let request_method = api_array.pop() || '';
        let require_path = apiPath + (api_array.length > 0 ? api_array.join('/') + '/' : '');

        try {
            let target_module: IApiProcessor;
            // 根据版本号获取对应的模块
            if (!ver) ver = 'default';
            else ver = 'v' + ver.replace('.', '_');
            if (process.env.NODE_ENV !== 'development') {
                target_module = this.get_target_module(require_path, request_method, ver);
            } else {
                target_module = require(this.route_path + require_path + request_method)[ver];
            }
            return {
                api: api,
                path: require_path + request_method,
                processor: target_module
            };
        } catch (e) {
            console.log(e);
            return {api: api, path: require_path + request_method};
        }
    }

    route(_req: express.Request, _res: express.Response, _next: any): Promise<void> {
        return new Promise<void>((resolve) => {
            resolve();
        });
    };
}

export class Router extends RouterBase {
    public async route(req: express.Request, res: express.Response, _next: any) {
        let content = req.body;
        let path = req.params[0];
        Logger.trace(path, 'request');
        let module: IMethodModule = this.file_based_modules ?
          this.parser(path, content.ver) :
          await this.get_dynamic_module(path, content.ver);
        Logger.trace(content, module.path);
        if (process.env.NODE_ENV === 'development') {
            if (!content.__cachedData) content.__cachedData = {};
            content.__cachedData.api = module.api;
        }
        if (module.processor) {
            try {
                let cached_data: ICachedData | any = {
                    from: content.__cachedData.from || '',
                    api: content.__cachedData.api || module.api,
                    user_id: content.__cachedData.user_id || '',
                    app_id: content.__cachedData.app_id || '',
                    app_type: content.__cachedData.app_type || 3
                };

                // 所有api接口的总守卫
                await this.auditor.audit(path, content, module, cached_data);
                // 为兼容老版本，将老版的参数名映射到新版
                if (module.processor.method_params && module.processor.method_params.params_alias) {
                    let aliases = module.processor.method_params.params_alias;
                    for (let key in aliases) {
                        if (!aliases.hasOwnProperty(key)) continue;
                        if (content[key] !== undefined)
                            content[aliases[key]] = content[key]; // 老版本参数名->新版本参数名
                    }
                }


                // 各个api接口自己的守卫
                if (module.processor.guards) {
                    for (let i = 0; i < module.processor.guards.length; i++) {
                        let guard = module.processor.guards[i];
                        await guard(content, cached_data);
                    }
                }
                cached_data = extend(cached_data, content.__cachedData, true);
                Logger.info(content);
                if (module.processor.on_start) {
                    await module.processor.on_start(content, module.processor.method_params, cached_data);
                }
                let result: any = await module.processor.method(content, module.processor.method_params, cached_data);
                if (module.processor.on_before_respond) {
                    let r = await module.processor.on_before_respond(content, module.processor.method_params, cached_data, result);
                    if (r) result = r;
                }
                Resp.respond(module.api, res, result);
                if (module.processor.on_completed) await module.processor.on_completed(content, module.processor.method_params, cached_data, result);
            } catch (e: any) {
                Logger.error(e);
                e = Resp.throw_err(e);
                Resp.respond_err(module.api, res, e, e.statusCode);
                if (module.processor.on_failed) await module.processor.on_failed(content, e);
            }

        } else {
            Resp.respond_err(module.api, res, Resp.gen_err(Resp.NonSupportedApi));
        }
    }
}
