import {IObjectStat, IOss} from "@util/Oss";
import {Resp} from "@util/Resp";
import {Stream} from "stream";
import {Logger} from "@util/Logger";
import {IOssConfig} from "@util/Config";

const OSS = require('ali-oss');

export type AliOssTag = string;


export class AliOss implements IOss {

    private client;
    private tag: string;
    private config: IOssConfig;

    constructor(tag: AliOssTag, config: IOssConfig) {
        this.tag = tag;
        this.config = config;
        this.client = new OSS({
            endpoint: config.end_point,
            accessKeyId: config.access_key,
            accessKeySecret: config.secret_key
        })
    }

    get bucket(): string {
        return this.config.bucket;
    }

    get url(): string {
        return `${this.config.use_ssl ? 'https' : 'http'}://${this.config.end_point}:${this.config.port}/`;
    }

    bucket_exists(bucket: string): Promise<boolean> {
        return new Promise<boolean>(async (resolve, reject) => {
            try {
                await this.client.getBucketInfo(bucket);
                resolve(true);
            } catch (error: any) {
                Logger.error(error);
                // 指定的存储空间不存在。
                if (error.name === 'NoSuchBucketError') {
                    resolve(false);
                } else {
                    reject(Resp.gen_err(Resp.InternalServerError, error.message));
                }
            }
        });
    }

    make_bucket(bucket: string): Promise<void> {
        return new Promise<void>(async (resolve, reject) => {
            try {
                const options = {
                    storageClass: 'Standard', // 存储空间的默认存储类型为标准存储，即Standard。如果需要设置存储空间的存储类型为归档存储，请替换为Archive。
                    acl: 'private', // 存储空间的默认读写权限为私有，即private。如果需要设置存储空间的读写权限为公共读，请替换为public-read。
                    dataRedundancyType: 'LRS' // 存储空间的默认数据容灾类型为本地冗余存储，即LRS。如果需要设置数据容灾类型为同城冗余存储，请替换为ZRS。
                };
                // 填写Bucket名称。
                await this.client.putBucket(bucket, options);
                resolve();
            } catch (err: any) {
                Logger.error(err);
                reject(Resp.gen_err(Resp.InternalServerError, err.message));
            }
        });
    }

    put_object(bucket: string, objectName: string, buffer: Stream | Buffer, size: number, contentType?: string): Promise<string> {
        return new Promise<string>(async (resolve, reject) => {
            try {
                let options;
                if (contentType) {
                    options = {
                        headers: {
                            'content-type': contentType
                        }
                    }
                }
                let client = new OSS({
                    bucket: bucket,
                    endpoint: this.config.end_point,
                    accessKeyId: this.config.access_key,
                    accessKeySecret: this.config.secret_key
                });
                Logger.info({
                    bucket: bucket,
                    endpoint: this.config.end_point,
                    accessKeyId: this.config.access_key,
                    accessKeySecret: this.config.secret_key
                });
                let etag = await client.put(objectName, buffer, options);
                resolve(etag);
            } catch (e: any) {
                Logger.error(e);
                reject(Resp.gen_err(Resp.InternalServerError, `上传图片失败， ${e.message}`));
            }
        });
    }

    remove_object(bucket: string, objectName: string): Promise<void> {
        return new Promise<void>(async (resolve, reject) => {
            try {
                let client = new OSS({
                    bucket: bucket,
                    endpoint: this.config.end_point,
                    accessKeyId: this.config.access_key,
                    accessKeySecret: this.config.secret_key
                });
                await client.delete(objectName);
                resolve();
            } catch (e: any) {
                reject(Resp.gen_err(Resp.InternalServerError, `删除对象出错， ${e.message}`));
            }
        })
    }

    presigned_url(bucket: string, objectName: string, expiry: number, filename?: string): Promise<string> {
        return new Promise<string>(async (resolve, reject) => {
            try {
                let client = new OSS({
                    bucket: bucket,
                    endpoint: this.config.end_point,
                    accessKeyId: this.config.access_key,
                    accessKeySecret: this.config.secret_key
                });
                let url = client.signatureUrl(objectName, {expires: expiry});
                resolve(url);
            } catch (e: any) {
                reject(Resp.gen_err(Resp.InternalServerError, `获取图片url出错， ${e.message}`));
            }
        })
    }

    presignedPostPolicy(bucket: string, objectName: string, expiry: number, prefix?: string): Promise<any> {
        return new Promise<string>(async (resolve, reject) => {
            try {
                let client = new OSS({
                    bucket: bucket,
                    endpoint: this.config.end_point,
                    accessKeyId: this.config.access_key,
                    accessKeySecret: this.config.secret_key
                });
                let url = client.signatureUrl(objectName, {expires: expiry, method: 'POST'});
                resolve(url);
            } catch (e: any) {
                reject(Resp.gen_err(Resp.InternalServerError, `获取图片url出错， ${e.message}`));
            }
        });
    }

    listenBucketNotification(bucket: string, prefix: string, suffix: string, events: Array<string>, cb: any) {
    }

    setBucketLifecycle(bucket: string, id: string, enabled: boolean, expiration: number): Promise<void>{
        return new Promise<void>(async (resolve, reject) => {
            return resolve();
        });
    }

    stat_object(bucket: string, object_name: string): Promise<IObjectStat | undefined> {
        return new Promise<IObjectStat | undefined>(async (resolve, reject) => {
            return resolve(undefined);
        });
    }
}
