import fs = require("fs");
import path = require('path');
import Ajv from "ajv";

interface IHttps {
    port: number;
    key: string;
    cert: string;
}

interface IHttp {
    port: number;
}

interface ILdtp{
    host: string;
    port: number;
}

interface IWebsocket {
    port: number;
}

export interface IRelationServiceConfig {
    name: string;
    urls: Array<string>;
    private_key: string;
    public_key: string;
}

export interface IDatabaseConfig {
    id: string;
    dialect: 'mysql' | 'postgres' | 'sqlite' | 'db2' | 'mariadb' | 'mssql';    //mysql, postgres, sqlite, db2, mariadb and mssql.
    tags: Array<string>;    // 这个数据库配置所适配的数据模型。在数据模型列表（Models.ts)中每个数据模型都有一个tag，这个tag在tags中时，表示这个数据模型使用此数据库连接
    database?: string;
    username?: string;
    password?: string;
    host: string;
    port?: number;
    storage?: string;       // Only used by sqlite. Defaults to ':memory:'
    schema?: string;
    logging?: boolean;
    force_sync?: boolean;
}

export interface IRedisConfig {
    id: string;
    host: string;
    port: number;
    connect_timeout?: number;
    no_delay?: boolean;
    keep_alive?: number;
    tls?: boolean;
    reject_unauthorized?: boolean;
    ca?: string;
    cert?: string;
    username?: string;
    password?: string;
    database: number;
    expires?: number;
}

export interface IKafkaConfig {
    id: string;
    host: string;               // ip:port
    group_id: string;           // 订阅时需要
    session_timeout?: number;   // 会话超时时间
}

export interface IMqttConfig {
    id: string;
    host: string;
    port: number;
    username: string;
    password: string;
}

export type OssType = 'Minio' | 'AliOss';

export interface IOssConfig {
    id: string;
    type: OssType;
    end_point: string;
    port?: number | undefined;
    use_ssl?: boolean;
    bucket: string;
    access_key: string;
    secret_key: string;
}

// websocket服务配置
export interface IWssConfig {
    id: string;
    host?: string;
    port: number;
    key?: string;
    cert?: string;
}

interface IInfrastructure {
    db?: Array<IDatabaseConfig>;
    redis?: Array<IRedisConfig>;
    kafka?: Array<IKafkaConfig>;
    mqtt?: Array<IMqttConfig>;
    oss?: Array<IOssConfig>;
    mongodb?: Array<IMongodbConfig>;
    wss?: Array<IWssConfig>;
}

export interface IMongodbConfig {
    tag: string;
    database: string;
    username: string;
    password: string;
    host: string;
    port: number;
}

export interface IConfig {
    id: string;
    name: string;
    log_level?: number;
    https?: IHttps;
    http?: IHttp;
    ldtp?: ILdtp;
    websocket?: IWebsocket;
    relations?: Array<IRelationServiceConfig>;
    infrastructure?: IInfrastructure;
}


const ConfigSchema = {
    "$schema": "http://json-schema.org/draft-07/schema#",
    "type": "object",
    "properties": {
        "name": {
            "type": "string",
            "title": "服务名",
            "maxLength": 256,
            "minLength": 2
        },
        "log_level": {
            "type": "integer",
            "title": "日志级别，0-5，0:无日志，1:ERROR，2:INFO，3:DEBUG, 4:TRACE",
            "minimum": 0,
            "maximum": 5,
        },
        "id": {
            "type": "string",
            "title": "服务id",
            "description": "服务分配的机器号,uuid"
        },
        "https": {
            "type": "object",
            "properties": {
                "port": {
                    "type": "integer",
                    "title": "https服务端口"
                },
                "key": {
                    "type": "string",
                    "title": "https私钥"
                },
                "cert": {
                    "type": "string",
                    "title": "https证书"
                }
            },
            "required": [
                "port",
                "key",
                "cert"
            ],
            "title": "https服务相关参数"
        },
        "http": {
            "type": "object",
            "title": "http服务相关参数",
            "properties": {
                "port": {
                    "type": "integer",
                    "title": "http服务端口"
                }
            },
            "required": [
                "port"
            ]
        },
        "ldtp": {
            "type": "object",
            "title": "ldtp(light device transport protocol)服务相关参数",
            "properties": {
                "host": {
                    "type": "string",
                    "title": "服务器地址"
                },
                "port": {
                    "type": "integer",
                    "title": "服务器端口"
                }
            },
            "required": [
                "host", "port"
            ]
        },
        "websocket": {
            "type": "object",
            "properties": {
                "port": {
                    "type": "integer",
                    "title": "服务端口"
                }
            },
            "title": "websocket服务相关参数",
            "required": [
                "port"
            ]
        },
        "relations": {
            "type": "array",
            "items": {
                "type": "object",
                "title": "相关服务信息",
                "properties": {
                    "name": {
                        "type": "string",
                        "title": "相关服务名"
                    },
                    "urls": {
                        "type": "array",
                        "items": {
                            "type": "string",
                            "title": "服务url"
                        },
                        "title": "相关服务的url，可以多个，负载均衡"
                    },
                    "private_key": {
                        "type": "string",
                        "title": "与该服务通信签名用的私钥"
                    },
                    "public_key": {
                        "type": "string",
                        "title": "验证该服务签名的公钥"
                    }
                },
                "required": [
                    "name",
                    "urls",
                    "private_key",
                    "public_key"
                ]
            },
            "title": "相关服务信息列表"
        },
        "infrastructure": {
            "type": "object",
            "title": "基础服务相关参数",
            "properties": {
                "db": {
                    "type": "array",
                    "title": "数据库连接相关参数",
                    "items": {
                        "type": "object",
                        "properties": {
                            "id": {
                                "type": "string",
                                "title": "数据库连接实例标记"
                            },
                            "dialect": {
                                "type": "string",
                                "title": "数据库类型：mysql,postgres,sqlite,mssql, db2, mariadb",
                                "enum": ["mysql", "postgres", "sqlite", "db2", "mssql", "mariadb"]
                            },
                            "database": {
                                "type": "string",
                                "title": "数据库名"
                            },
                            "username": {
                                "type": "string",
                                "title": "数据库账号"
                            },
                            "password": {
                                "type": "string",
                                "title": "数据库密码"
                            },
                            "host": {
                                "type": "string",
                                "title": "数据库主机名或IP"
                            },
                            "port": {
                                "type": "integer",
                                "title": "数据库端口"
                            },
                            "storage": {
                                "type": "string",
                                "title": "只用于sqlite，默认为\":memory:\""
                            },
                            "schema": {
                                "type": "string",
                                "title": "数据库schema"
                            },
                            "tags": {
                                "type": "array",
                                "title": "此数据库中包含的数据模型的tag清单",
                                "description": "如果数据模型的tag包含在此列表中，则此数据模型使用这个数据库连接",
                                "items": {
                                    "type": "string"
                                }
                            },
                            "force_sync": {
                                "type": "boolean",
                                "title": "是否强制同步数据库中的数据模型",
                                "description": "开启后，如果数据模型有修改，会强制同步到数据库中",
                                "default": false
                            }
                        },
                        "required": [
                            "id",
                            "dialect",
                            "host", "tags"
                        ]
                    }
                },
                "redis": {
                    "type": "array",
                    "title": "redis相关参数，参考：https://github.com/redis/node-redis/blob/HEAD/docs/client-configuration.md",
                    "items": {
                        "type": "object",
                        "properties": {
                            "id": {
                                "type": "string",
                                "title": "连接实例标记"
                            },
                            "host": {
                                "type": "string",
                                "title": "redis主机名"
                            },
                            "port": {
                                "type": "integer",
                                "title": "redis端口",
                                "default": 6379
                            },
                            "tls": {
                                "type": "boolean",
                                "title": "是否启用tls",
                                "default": false
                            },
                            "reject_unauthorized": {
                                "type": "boolean",
                                "title": "是否验证根书。自签名时，设置成false"
                            },
                            "ca": {
                                "type": "string",
                                "title": "tls连接的ca证书"
                            },
                            "cert": {
                                "type": "string",
                                "title": "tls连接证书"
                            },
                            "connect_timeout": {
                                "type": "integer",
                                "title": "连接超时时间，毫秒",
                                "default": 5000
                            },
                            "no_delay": {
                                "type": "boolean",
                                "title": "socket是否启用Nagle算法。https://nodejs.org/api/net.html#net_socket_setnodelay_nodelay",
                                "default": true
                            },
                            "keep_alive": {
                                "type": "integer",
                                "title": "socket是否启用keep-alive功能，0表示不启用，数字表示毫秒。https://nodejs.org/api/net.html#net_socket_setkeepalive_enable_initialdelay",
                                "default": 5000
                            },
                            "username": {
                                "type": "string",
                                "title": "ACL账号，https://redis.io/topics/acl"
                            },
                            "password": {
                                "type": "string",
                                "title": "密码，没有填空。ACL密码或旧的\"--requirepass\"密码"
                            },
                            "database": {
                                "type": "integer",
                                "title": "redis数据库号，即通道号",
                                "description": "redis数据库号，即通道号",
                                "maximum": 16,
                                "minimum": 1
                            },

                            "expires": {
                                "type": "integer",
                                "title": "过期时间（秒），默认不过期"
                            }
                        },
                        "required": [
                            "id",
                            "database",
                            "host",
                            "port",
                            "password"
                        ]
                    }
                },
                "kafka": {
                    "type": "array",
                    "items": {
                        "type": "object",
                        "properties": {
                            "id": {
                                "type": "string",
                                "title": "实例标记"
                            },
                            "host": {
                                "type": "string",
                                "title": "地址，host:port"
                            },
                            "group_id": {
                                "type": "string",
                                "title": "分组，分组订阅时需要。如果是生产者用空字串"
                            },
                            "session_timeout": {
                                "type": "number",
                                "title": "会话超时时间"
                            }
                        },
                        "required": [
                            "id",
                            "host",
                            "group_id"
                        ]
                    }
                }
            }
        }
    },
    "required": [
        "name",
        "id"
    ]
}

export class Config {
    private static instance: Config;
    private params: IConfig;
    private readonly avj = new Ajv({useDefaults: true});

    public static load(config_file?: string): boolean {
        if (!Config.instance) {
            Config.instance = new Config();
        }
        if (!config_file) {
            if (process.env.NODE_ENV === 'development') {
                config_file = path.join(process.cwd(), 'config_dev.json');
            } else {
                config_file = path.join(process.cwd(), 'config.json');
            }
        }
        console.log(config_file);
        Config.instance.params = JSON.parse(fs.readFileSync(config_file, "utf-8"));
        let validator = Config.instance.avj.compile(ConfigSchema);
        let correct: boolean = validator(Config.params());
        if (!correct) {
            let msg: string = '';
            if (validator.errors && validator.errors.length > 0)
                msg = validator.errors[0].message ? validator.errors[0].message : '';
            console.log(validator.errors);
            throw new Error(`配置文件格式错误：${msg}`);
        }

        return correct;
    }

    public static params(config_file?: string): IConfig {
        if (!Config.instance) {
            Config.load(config_file);
        }

        return Config.instance.params;
    }
}
