import {KafkaClient, HighLevelProducer, ConsumerGroup} from 'kafka-node';
import {Resp} from "@util/Resp";
import {Logger} from "@util/Logger";
import {IKafkaConfig} from "@util/Config";

export class Kafka {
    static instance_map: Map<string, Kafka> = new Map<string, Kafka>();

    static init_instance(id: string, config: IKafkaConfig): Kafka {
        let instance = Kafka.instance_map.get(id);
        if (!instance) {
            instance = new Kafka(id, config);
            Kafka.instance_map.set(id, instance);
        }
        return instance;
    }

    static get_instance(id: string): Kafka {
        let instance = Kafka.instance_map.get(id);
        if (!instance) {
            throw new Error(`kafka client ${id} should be init.`)
        }
        return instance;
    }

    private _config: IKafkaConfig;
    private producer_map: Map<string, HighLevelProducer>;
    private consumer_group_map: Map<string, ConsumerGroup>;
    constructor(id: string, config: IKafkaConfig) {
        this._config = config;
        this.producer_map = new Map<string, HighLevelProducer>();
        this.consumer_group_map = new Map<string, ConsumerGroup>();
    }

    send(topic: string, messages: Array<string>): Promise<any> {
        return new Promise<any>((resolve, reject) => {
            let producer = this.producer_map.get(topic);
            if (!producer) {
                let client: KafkaClient = new KafkaClient({
                    kafkaHost: this._config.host,
                    autoConnect: true,
                    connectRetryOptions: {
                        retries: 50000, factor: 0, minTimeout: 1000, maxTimeout: 5000, randomize: true
                    }
                });
                producer = new HighLevelProducer(client);

                producer.on('ready', () => {
                    this.producer_map.set(topic, producer!);
                    producer!.send([{topic: topic, messages: messages}], (error, data) => {
                        if (error) {
                            Logger.error(error, 'kafka');
                            return reject(Resp.gen_err(Resp.InternalServerError, 'kafka发送消息失败' + error))
                        }
                        resolve(data);
                    })
                });


                producer.on('error', (error: any) => {
                    Logger.error(error, 'kafka');
                    return resolve(Resp.gen_err(Resp.InternalServerError, 'Kafka生产者创建失败' + error));
                })

                // this.producer_map.set(topic, producer);

            } else {
                producer.send([{topic: topic, messages: messages}], (error, data) => {
                    if (error) {
                        Logger.error(error, 'kafka');
                        return reject(Resp.gen_err(Resp.InternalServerError, 'kafka发送消息失败' + error))
                    }
                    resolve(data);
                })
            }
        })
    }

    subscribe(topic: string, cb: (message: any) => void) {
        let consumer_group = this.consumer_group_map.get(topic);
        if (!consumer_group) {
            consumer_group = new ConsumerGroup({
                kafkaHost: this._config.host,
                batch: {noAckBatchSize: 1024, noAckBatchAge: 10},
                groupId: this._config.group_id,
                sessionTimeout: this._config.session_timeout,
                protocol: ['roundrobin'],
                fromOffset: 'earliest',
                outOfRangeOffset: 'earliest',
                ssl: true,
                migrateHLC: false,
                migrateRolling: true
            }, topic);
        }
        consumer_group.on('message', cb);
        consumer_group.on('error', (e) => {
            Logger.error(e, 'kafka');
        })
    }
}
