import {Resp} from "@util/Resp";
import {Readable as ReadableStream} from "stream";
import {IObjectStat, IOss} from "@util/Oss";
import {Logger} from "@util/Logger";
import {IOssConfig} from "@util/Config";
import {Client, UploadedObjectInfo} from "minio";

const Minio = require('minio');

export class MinioClient implements IOss {
    private client: Client;
    private id: string;
    private config: IOssConfig;

    constructor(id: string, config: IOssConfig) {
        this.id = id;
        this.config = config;
        this.client = new Minio.Client({
            endPoint: config.end_point,
            port: config.port,
            accessKey: config.access_key,
            secretKey: config.secret_key,
            useSSL: config.use_ssl,
        });
    }

    get url(): string {
        return `${this.config.use_ssl ? 'https' : 'http'}://${this.config.end_point}:${this.config.port}/`;
    }

    get bucket(): string {
        return this.config.bucket;
    }

    bucket_exists(bucket: string): Promise<boolean> {
        return new Promise<boolean>(async (resolve, reject) => {
            this.client.bucketExists(bucket, (err: Error | null, exists: boolean) => {
                if (err) {
                    Logger.error(err);
                    return reject(Resp.gen_err(Resp.InternalServerError, 'bucketExists error: ' + err.message))
                }

                return resolve(exists);
            })
        })
    }

    make_bucket(bucket: string): Promise<void> {
        return new Promise<void>(async (resolve, reject) => {
            this.client.makeBucket(bucket, 'us-east-1', (err: Error | null) => {
                if (err) {
                    return reject(Resp.gen_err(Resp.InternalServerError, 'makeBucket error: ' + err.message))
                }
                resolve();
            })
        })
    }

    put_object(bucket: string, objectName: string, buffer: ReadableStream | Buffer, size: number, content_type?: string): Promise<string> {
        return new Promise<string>(async (resolve, reject) => {
            try {
                let obj_info: UploadedObjectInfo = await this.client.putObject(bucket, objectName, buffer, size, {['Content-Type']: content_type});
                resolve(obj_info.etag);
            } catch (e: any) {
                reject(Resp.gen_err(Resp.InternalServerError, `上传图片失败， ${e.message}`));
            }

        });
    }

    remove_object(bucket: string, objectName: string): Promise<void> {
        return new Promise<void>(async (resolve, reject) => {
            try {
                await this.client.removeObject(bucket, objectName);
                resolve();
            } catch (e: any) {
                reject(Resp.gen_err(Resp.InternalServerError, `删除对象出错， ${e.message}`));
            }
        })
    }

    presigned_url(bucket: string, objectName: string, expiry: number, filename?: string): Promise<string> {
        return new Promise<string>(async (resolve, reject) => {
            try {
                let reqParams;
                let url;
                // if (filename) {
                //     reqParams = {
                //         "Content-Disposition": `attachment; filename="${filename}"`,
                //         "Content-Type": "application/octet-stream"
                //     }
                //     url = await this.client.presignedUrl('GET', bucket, objectName, expiry, reqParams);
                // }
                // else
                    url = await this.client.presignedUrl('GET', bucket, objectName, expiry);
                resolve(url);
            } catch (e: any) {
                reject(Resp.gen_err(Resp.InternalServerError, `获取图片url出错， ${e.message}`));
            }
        })
    }

    presignedPostPolicy(bucket: string, objectName: string, expiry: number): Promise<any> {
        return new Promise<any>(async (resolve, reject) => {
            let policy = this.client.newPostPolicy();
            policy.setBucket(bucket);
            policy.setKey(objectName);
            let expires = new Date;
            expires.setSeconds(expiry);
            policy.setExpires(expires);
            let result = await this.client.presignedPostPolicy(policy);
            console.log(result);
            resolve(result);
        })
    }


    listenBucketNotification(bucket: string, prefix: string, suffix: string, events: Array<string>, cb: any) {
        let listener = this.client.listenBucketNotification(bucket, prefix, suffix, events);
        listener.on('notification', function (record: any) {
            console.log(record);
            cb({
                event: record.eventName,
                bucket: record.s3.bucket.name,
                object: record.s3.object.key,
                size: record.s3.object.size,
            });
        })
    }

    setBucketLifecycle(bucket: string, id: string, enabled: boolean, expire_days: number): Promise<void> {
        return new Promise<void>(async (resolve, reject) => {
            try {
                const lifecycleConfig = {
                    Rule: [{
                        "ID": id,
                        "Status": enabled ? "Enabled" : "Disabled",
                        "Filter": {
                            "Prefix": "",
                        },
                        "Expiration": {
                            "Days": expire_days
                        }
                    }
                    ]
                }

                await this.client.setBucketLifecycle(bucket, lifecycleConfig);
                let result = await this.client.getBucketLifecycle(bucket);
                console.log(result);
                resolve();
            } catch (e) {
                Logger.error(e);
                reject(e);
            }
        });
    }

    stat_object(bucket: string, object_name: string): Promise<IObjectStat | undefined> {
        return new Promise<IObjectStat | undefined>(async (resolve, reject) => {
            try {
                let stat = await this.client.statObject(bucket, object_name);
                return resolve(stat);
            } catch (e) {
                return resolve(undefined);
            }

        });
    }
}
