import {Sequelize} from "sequelize";
import * as pg from 'pg';
import {IDatabaseConfig} from "./Config";
import {Logger} from "./Logger";

const {Client} = require("pg");

type dbDialect = "mysql" | "postgres" | "sqlite" | "mariadb" | "mssql" | undefined;
// bigint 处理bigint返回时变成string的问题，js bigint 最大值2^53
pg.types.setTypeParser(20, function (value) {
    return parseInt(value);
});
// decimal默认会处理成string,这里转成浮点数（注：数字大于范围时会有风险）
pg.types.setTypeParser(1700, parseFloat);


export class Orm {
    static instance_map: Map<string, Orm> = new Map<string, Orm>();

    static init_instance(id: string, config: IDatabaseConfig, force?: boolean): Promise<Orm> {
        return new Promise(async (resolve, reject) => {
            try {
                let instance = Orm.instance_map.get(id);
                if (!instance || force) {
                    instance = new Orm(id, config);
                    if (config.dialect === "postgres") {
                        let client = new Client({
                            user: config.username,
                            password: config.password,
                            host: config.host,
                            port: config.port,
                            database: 'postgres',

                        });
                        await client.connect();

                        try {
                            await client.query(`CREATE DATABASE  ${config.database}`);
                        } catch (e) {
                            // Logger.error(e);
                        }
                        try {
                            await instance._sequelize.query('CREATE EXTENSION IF NOT EXISTS ltree;');
                        } catch (e) {
                            // Logger.error(e);
                        }
                        try {
                            // 开启 two phase commit
                            await instance._sequelize.query('alter system set max_prepared_transactions = 100;');
                        } catch (e) {
                            // Logger.error(e);
                        }
                        await client.end();
                    }
                }
                resolve(instance);
            } catch (e) {
                reject(e);
            }
        });
    }

    static get_instance(id: string): Orm {
        let instance = Orm.instance_map.get(id);
        if (!instance) {
            throw new Error(`orm ${id} should be init.`);
        }

        return instance;
    }

    private readonly _sequelize: Sequelize;
    private readonly _model_tags: Array<string>;
    private readonly _force_sync: boolean;

    get sequelize(): Sequelize {
        return this._sequelize;
    }

    get model_tags(): Array<string> {
        return this._model_tags;
    }

    get force_sync(): boolean {
        return this._force_sync;
    }

    constructor(id: string, config: IDatabaseConfig) {
        try {
            this._sequelize = new Sequelize(
              config.database || '',
              config.username || '',
              config.password || '',
              {
                  dialect: <dbDialect>config.dialect,
                  dialectModule: pg,
                  host: config.host,
                  port: config.port,
                  logging: config.logging ? undefined : false,
                  timezone: '+08:00',
                  dialectOptions: {
                      useUTC: true,
                      dateStrings: true,
                      typeCast: true,
                      decimalNumbers: true
                  }
              }
            );
            Orm.instance_map.set(id, this);
            this._model_tags = config.tags;
            this._force_sync = !!config.force_sync;

        } catch (e) {
            Logger.error(e);
        }

    }
}
