import {Resp} from "@util/Resp";
import {Stream} from "stream";
import {MinioClient} from "@util/Minio";
import {AliOss} from "@util/AliOss";
import {IOssConfig} from "@util/Config";
import {Logger} from "@util/Logger";

export interface IObjectStat {
    size: number;
    etag: string;
}

export class Oss {
    static instance_map: Map<string, IOss> = new Map<string, IOss>();

    static async init_instance(id: string, config: IOssConfig): Promise<IOss> {
        try {
            let instance = Oss.instance_map.get(id);
            if (!instance) {
                switch (config.type) {
                    case "AliOss":
                        instance = new AliOss(id, config);
                        break;
                    case "Minio":
                    default:
                        instance = new MinioClient(id, config);
                        break;
                }
                Oss.instance_map.set(id, instance);
                let bucket = instance.bucket;
                let exists = await instance.bucket_exists(bucket);
                if (!exists) {
                    await instance.make_bucket(bucket);
                }
                return instance;
            }
            throw Resp.gen_err(Resp.InternalServerError,'oss初始化失败');
        } catch (e) {
            Logger.error(e);
            throw Resp.gen_err(Resp.InternalServerError,'oss初始化失败');
        }
    }

    static get_instance(id: string): IOss {
        let instance = Oss.instance_map.get(id);
        if (!instance) {
            throw Resp.gen_err(Resp.InternalServerError, `Oss Client ${id} should be init.`);
        }
        return instance;
    }
}

export interface IOss {
    bucket: string;
    url: string;
    bucket_exists(bucket: string): Promise<boolean>;
    make_bucket(bucket: string): Promise<void>;
    put_object(bucket: string, object_name: string, buffer: Stream | Buffer, size: number, content_type?: string): Promise<string>;
    remove_object(bucket: string, object_name: string): Promise<void>;
    presigned_url(bucket: string, object_name: string, expiry: number, filename?: string): Promise<string>;
    presignedPostPolicy(bucket: string, objectName: string, expiry: number): Promise<any>;
    listenBucketNotification(bucket: string, prefix: string, suffix: string, events: Array<string>, cb: any): void;
    setBucketLifecycle(bucket: string, id: string, enabled: boolean, expiration: number): Promise<void>;
    stat_object(bucket: string, object_name: string): Promise<IObjectStat | undefined>;
}
