import redis = require('redis');
import {IRedisConfig} from "@util/Config";
import {RedisClientType} from "redis";

/*
 如要支持JSON.GET/SET命令，需使用redis-stack-server镜像，docker-compose.yml片段示例如下：
redis:
    restart: always
    container_name: redis
    image: redis/redis-stack:latest
    environment:
      REDIS_ARGS: --requirepass 密码 --notify-keyspace-events Ex
    logging:
      driver: "json-file"
      options:
        max-size: "100m"
    ports:
      - 6379:6379
      - 8001:8001
*/

export class Redis {
    // redis client实例
    private readonly _client: RedisClientType;
    // redis 数据库号
    private readonly _database: number;
    // 默认的过期时间，调用具体接口时仍可指定不同的值
    private readonly _expires: number | undefined;

    // redis类实例map，用于通过id来找到相应的实例
    static instance_map: Map<string, Redis> = new Map<string, Redis>();
    // redis事件订阅者实例map，用于通过id来找到相应的订阅者
    static subscriber_map: Map<string, RedisClientType> = new Map<string, RedisClientType>();

    static init_instance(id: string, config: IRedisConfig): Promise<Redis> {
        return new Promise<Redis>(async (resolve, reject) => {
            try {
                let instance = Redis.instance_map.get(id);
                if (!instance) {
                    instance = new Redis(id, config);
                    await instance._client.connect();
                }
                return resolve(instance);
            } catch (e) {
                reject(e);
            }

        })

    }

    static get_instance(id: string): Redis {
        let instance = Redis.instance_map.get(id);
        if (!instance) {
            throw new Error(`Redis Client ${id} should be init.`);
        }
        return instance;
    }

    static get_subscriber(id: string): RedisClientType | undefined {
        return Redis.subscriber_map.get(id);
    }

    constructor(id: string, config: IRedisConfig) {
        let client_config: any = {
            socket: {
                host: config.host,
                port: config.port,
                connectTimeout: config.connect_timeout,
                noDelay: config.no_delay,
                keepAlive: config.keep_alive,
                tls: config.tls
            },
            database: config.database,
        };
        if (config.password) {
            client_config.password = config.password;
        }
        if (config.tls) {
            client_config.socket!.tls = true;
            if (config.reject_unauthorized) {
                client_config.socket!.rejectUnauthorized = false;
            } else {
                client_config.socket!.ca = config.ca;
            }
            client_config.socket!.cert = config.cert;
        }
        this._client = redis.createClient(client_config);
        Redis.instance_map.set(id, this);
        this._database = config.database;
        this._expires = config.expires;
    }

    get client(): RedisClientType {
        return this._client;
    }

    get database(): number {
        return this._database;
    }

    get expires(): number | undefined {
        return this._expires;
    }

    get(key: string, expires?: number): Promise<string | null> {
        let result = this._client.get(key);
        if (expires) {
            this._client.expire(key, expires).then();
        }
        return result;
    }

    set(key: string, value: string, expires?: number): Promise<string | null> {
        let result = this._client.set(key, value);
        if (expires) {
            this._client.expire(key, expires).then();
        }
        return result;
    }

    set_expires(key: string, expires: number): Promise<any> {
        return this._client.expire(key, expires);
    }

    hget(key: string, fields: any, expires?: number): Promise<string | undefined> {
        let result = this._client.hGet(key, fields);
        if (expires) {
            this._client.expire(key, expires).then();
        }
        return result;
    }

    hset(key: string, value: any, expires?: number): Promise<number> {
        let result = this._client.hSet(key, value);
        if (expires) {
            this._client.expire(key, expires).then();
        }
        return result;
    }

    hgetall(key: string, expires?: number): Promise<{ [x: string]: string; } | undefined> {
        let result = this._client.hGetAll(key);
        if (expires) {
            this._client.expire(key, expires).then();
        }
        return result;
    }

    del(key: string): Promise<number> {
        return this._client.del(key);
    }

    clear(pattern: string): Promise<void> {
        return new Promise<void>(async (resolve, reject) => {
            try {
                for await (const key of this.client.scanIterator({
                    MATCH: pattern,
                })) {
                    await this.client.del(key);
                }
                resolve();
            } catch (e) {
                reject(e);
            }
        });
    }

    subscribe(key: string, cb: (message: string, channel: string) => void): Promise<void> {
        return new Promise<void>(async (resolve, reject) => {
            try {
                let sub = Redis.get_subscriber(key);
                if (!sub) {
                    sub = this.client.duplicate();
                    await sub.connect();
                }
                await sub.subscribe(key, (message, channel) => {
                    cb(message, channel);
                });
                resolve();
            } catch (e) {
                reject(e);
            }
        })
    }

    subscribe_expired(cb: (message: string, channel: string) => void): Promise<void> {
        return new Promise<void>(async (resolve, reject) => {
            try {
                let key = `__keyevent@${this.database}__:expired`;
                await this.subscribe(key, cb);
                resolve();
            } catch (e) {
                reject(e);
            }
        })
    }

    publish(key: string, value: any): Promise<any> {
        return this.client.publish(key, value);
    }

    flush(): Promise<string> {
        return this.client.flushDb();
    }

    set_json(key: string, json: any, expires?: number): Promise<string | null> {
        let result = this.client.json.set(key, '$', json);
        if (expires) {
            this._client.expire(key, expires).then();
        }
        return result;
    }

    get_json(key: string, expires?: number): Promise<any> {
        let result = this.client.json.get(key);
        if (expires) {
            this._client.expire(key, expires).then();
        }
        return result;
    }

    // redis client的其它接口不一一封装，如需要使用，只需要通过调用Redis.client来得到实例，再参考node-redis模块（4.0以上）的接口来调用即可
}
