import {Response} from "express";
import {Logger} from "@util/Logger";
import {Config} from "@util/Config";

export interface IRespMsg {
    code: number,
    msg: string
}

export class Resp {
    static OK: IRespMsg = {code: 10000, msg: '成功。'};                                               //成功
    static DataExists: IRespMsg = {code: 10001, msg: '数据已存在。'};                                   //重复插入数据
    static KeyStillReferenced: IRespMsg = {code: 10002, msg: '此数据已被引用或所引用的外键无记录。'};                 //外键冲突
    static UniquenessConflicts: IRespMsg = {code: 10003, msg: '惟一性冲突。'};

    static NonSupportedApi: IRespMsg = {code: 10101, msg: '不支持的api或版本。'};
    static IIllegalRequest: IRespMsg = {code: 10102, msg: '非法请求。'};
    static ProxyError: IRespMsg = {code: 10109, msg: '服务器网关路由信息出错.'};
    static InternalCallFailed: IRespMsg = {code: 10110, msg: '微服务内部通信失败.'};

    static Unauthorized: IRespMsg = {code: 11000, msg: '没有登录。'};                  //没有登录
    static Forbidden: IRespMsg = {code: 11001, msg: '没有权限。'};                 //没有权限
    static LoginFailed: IRespMsg = {code: 11002, msg: '登录失败，账号或密码错误。'};    //无法登录
    static UserLocked: IRespMsg = {code: 11003, msg: '账号被锁定。'};                      //账号被锁定
    static DataLocked: IRespMsg = {code: 11004, msg: '数据已被后续流程使用，不允许删除或修改。'};  //数据已被后续流程使用，不允许删除或修改
    static InvalidFlow: IRespMsg = {code: 11005, msg: '业务流程异常.'};                                //业务流程错误
    static AtLeastOnDomain: IRespMsg = {code: 11006, msg: '至少为用户分配一个域。'};    //至少为用户分配一个域
    static ResourceLocked: IRespMsg = {code: 11007, msg: '资源被锁定'};     //资源被锁定，不允许修改
    static UserPauseLocked: IRespMsg = {code: 11008, msg: '多次登录失败，账号暂时锁定。'};           //账号暂时锁定
    static InternalServerError: IRespMsg = {code: 11100, msg: '内部错误。'};               //内部错误
    static PostgresqlServerError: IRespMsg = {code: 11101, msg: 'postgresql访问错误。'};                  //postgresql访问错误
    static RedisServerError: IRespMsg = {code: 11102, msg: 'Redis访问错误。'};                            //redis访问错误
    static ParamsError: IRespMsg = {code: 11103, msg: 'Api参数错误。'};                            //api参数错误
    static ResourceNotFound: IRespMsg = {code: 11104, msg: '资源未找到。'};                  //资源未找到
    static InterfaceDisabled: IRespMsg = {code: 11105, msg: '接口已废弃。'};                    //接口已废弃
    static HttpError: IRespMsg = {code: 11106, msg: '必须使用https协议。'};                          //必须https

    static MqttServerError: IRespMsg = {code: 21000, msg: 'Mqtt服务错误。'};
    static MqttAlreadyExistedDeviceName: IRespMsg = {code: 21001, msg: '设备已注册MQTT'};
    static MqttRegisterFailed: IRespMsg = {code: 21002, msg: 'MQTT注册失败'};
    static MqttPubFailed: IRespMsg = {code: 21003, msg: 'MQTT消息推送失败'};

    static PrjModifySelfRoleForbidden: IRespMsg = {code: 40000, msg: '不允许修改自己的项目组成员角色'};
    static PrjRemoveSelfForbidden: IRespMsg = {code: 40001, msg: '不允许在项目组成员中删除自己'};

    static MpPlatformFail: IRespMsg = {code: 50001, msg: '与小程序平台交互失败'};
    static PaymentPlatformFail: IRespMsg = {code: 50002, msg: '与支付平台交互失败'};
    static PaymentPrepayFail: IRespMsg = {code: 50003, msg: '生成预支付交易单失败。'};
    static NotifyUnifiedOrderFail: IRespMsg = {code: 50004, msg: '处理预支付交易回调消息失败。'}


    static track_str(stackDepth?: number): string {
        let info = Logger.stack_info(stackDepth);
        let file = info['file'];
        let line = info['line'];
        return `<${Config.params().name}:${file}:${line}> `;
    }

    static extend(des: any, src: any, override: boolean) {
        for (let i in src) {
            if (!src.hasOwnProperty(i)) continue;
            if (override || !(i in des)) {
                if (src[i] instanceof Array) {
                    des[i] = [];
                    for (let j = 0; j < src[i].length; j++) {
                        if (typeof src[i][j] === 'object') {
                            des[i].push({});
                            this.extend(des[i][j], src[i][j], override);
                        } else {
                            des[i].push(src[i][j]);
                        }

                    }
                } else {
                    if (src[i] !== undefined && src[i] !== null)
                        des[i] = src[i];
                }
            }
        }
        return des;
    }

    static respond(api: string, res: Response, result?: any, code?: number) {
        let json: any = {};
        json.api = api;
        json.code = code ? code : Resp.OK.code;
        json.msg = Resp.OK.msg;

        if (result != undefined) {
            json.data = result;
            // Resp.extend(json, result, true);
        }
        Logger.info(json);
        res.json(json);
    }

    static respond_json(res: Response, json: any) {
        Logger.info(json);
        res.json(json);
    }

    static respond_err(api: string, res: Response, error: any, status_code?: number) {
        let json: any = {};
        json.api = api;
        if (error.msg && error.msg.code) {
            json.code = error.msg.code;
        } else {
            json.code = Resp.InternalServerError.code;
        }
        if (error.msg && error.msg.msg) {
            json.msg = error.msg.msg;
        } else {
            json.msg = error;
        }
        if (error.msg && error.msg.err) {
            json.err = error.msg.err;
        }
        if (error.track) {
            json.track = error.track;
        } else {
            json.track = Resp.track_str();
        }

        if (error.descr) {
            json.info = json.msg;
            json.msg = ` ${error.descr}`;

        }

        Logger.error(json);
        if (status_code) {
            res.status(status_code).json(json);
        } else res.json(json);
    }

    static gen_err(msg: any, desc?: string, statusCode?: number) {
        let err: any = {
            msg: msg,
            track: Resp.track_str(),
            tag: 'CustomError'
        };
        if (desc !== undefined) {
            err.descr = desc;
        }
        if (statusCode !== undefined) {
            err.statusCode = statusCode;
        }
        return err;
    }

    static gen_db_err(error: any, msg?: string) {
        let json: any = {
            err: ''
        };
        error.code = error.code || (error.original && error.original.code);
        if (error.code) {

            switch (error.code) {
                case '23505':
                    json.code = Resp.DataExists.code;
                    json.msg = Resp.DataExists.msg;
                    break;
                case '23503':
                    json.code = Resp.KeyStillReferenced.code;
                    json.msg = Resp.KeyStillReferenced.msg;
                    break;
                default:
                    json.code = Resp.InternalServerError.code;
                    json.msg = Resp.InternalServerError.msg;
                    break;
            }
        } else {
            json.code = Resp.InternalServerError.code;
            json.msg = Resp.InternalServerError.msg;

        }
        if (error.code) {
            json.msg += ` <${error.code}>`;
        }
        if (error.errors) {
            json.msg += error.errors[0] ? error.errors[0].message : error.message;
        }
        if (error.message) {
            json.msg += ` ${error.message} `;
        }
        if (json.err == '') {
            delete json.err;
        }
        if (msg) json.msg += ` ${msg} `;
        return {
            msg: json,
            track: Resp.track_str(),
            tag: 'CustomError'
        }
    }

    static throw_err(err: any) {
        if (err.tag === 'CustomError') return err;
        else return this.gen_db_err(err, err.message);
    }

}
