import {obj_key_sort} from "@util/JsonToolkit";
import crypto from "crypto";

import axios from "axios";
import {Resp} from "@util/Resp";
import {Logger} from "@util/Logger";
import {IRelationServiceConfig} from "@util/Config";

export class Services {
    static instance_map: Map<string, Services> = new Map<string, Services>();

    static init_instance(name: string, config: IRelationServiceConfig): Services {
        let instance = Services.instance_map.get(name);
        if (!instance) {
            instance = new Services(name, config);
        }
        return instance;
    }

    static get_instance(name: string): Services {
        let instance = Services.instance_map.get(name);
        if (!instance) {
            throw Resp.gen_err(Resp.InternalServerError,`service ${name} should be init.`);
        }
        return instance;
    }

    private readonly _name: string;
    private readonly urls: Array<string>;
    private readonly public_key: string;
    private readonly private_key: string;

    constructor(name: string, config: IRelationServiceConfig) {
        this._name = name;
        this.urls = config.urls;
        this.public_key = config.public_key;
        this.private_key = config.private_key;
        Services.instance_map.set(name, this);
    }

    get name(): string {
        return this._name;
    }

    get url(): string {
        // TODO: 多地址时负载均衡
        return this.urls[0];
    }

    get key(): string {
        return this.public_key;
    }

    sign(json: any): string {
        let sorted = obj_key_sort(json);
        let data = JSON.stringify(sorted);
        let sign = crypto.createSign('RSA-SHA256');

        sign.write(data);
        sign.end();
        return sign.sign(this.private_key, 'base64');
    }

    verify(json: any, sign: string): boolean {
        try {
            let data = JSON.stringify(obj_key_sort(json));
            let verify = crypto.createVerify('RSA-SHA256');
            verify.write(data);
            verify.end();
            return verify.verify(this.public_key, sign, 'base64');
        } catch (e) {
            console.log(e);
            return false;
        }
    }

    private api_post(url: string, data: any, headers?: any): Promise<unknown> {
        return new Promise<unknown>(async (resolve, reject) => {
            try {
                let axiosResponse = await axios.post(url, data, {
                    headers: headers
                });
                resolve(JSON.parse(axiosResponse.data));
            } catch (e: any) {
                Logger.error(e);
                reject(Resp.gen_err(Resp.InternalCallFailed, e.message));
            }
        })
    }

    call(data: any, headers?: any): Promise<unknown> {
        return new Promise<unknown>(async (resolve, reject) => {
            try {
                data.sign = this.sign(data);
                data.service_name = this._name;
                let result = await this.api_post(this.url, data, headers);
                resolve(result);
            } catch (e) {
                reject(e);
            }

        });
    }
}
