import {Resp} from "@util/Resp";
import {Logger} from "@util/Logger";
import {IWssConfig} from "@util/Config";
import {createServer} from "https";
import {connection, server, request} from "websocket";
import {IncomingHttpHeaders} from "http";
import {Redis} from "@util/Redis";
import {EventEmitter} from "events";
import * as http from "http";

// websocket服务
export class WsService extends EventEmitter {
    static instance_map: Map<string, WsService> = new Map<string, WsService>();

    static async init_instance(id: string, config: IWssConfig): Promise<WsService> {
        try {
            let instance = WsService.instance_map.get(id);
            if (!instance) {
                instance = new WsService(id, config);
                WsService.instance_map.set(id, instance);
            }
            return instance;
        } catch (e) {
            Logger.error(e);
            throw Resp.gen_err(Resp.InternalServerError,'websocket service初始化失败');
        }
    }

    static get_instance(id: string): WsService {
        let instance = WsService.instance_map.get(id);
        if (!instance) {
            throw Resp.gen_err(Resp.InternalServerError, `websocket service ${id} should be init.`);
        }
        return instance;
    }

    private _config: IWssConfig;
    private wss: server;
    private token_conn_map: Map<string, connection>;
    private conn_token_map: Map<connection, string>;
    constructor(id: string, config: IWssConfig) {
        super();
        this._config = config;
        this.token_conn_map = new Map();
        this.conn_token_map = new Map();
        let https_server;
        if (process.env.NODE_ENV !== 'development') {
            https_server = createServer({
                key: config.key,
                cert: config.cert
            }, function (request, response) {
                Logger.trace((new Date()) + ' Received request for ' + request.url);
                response.writeHead(404);
                response.end();
            });
        } else {
          https_server = http.createServer( function (request, response) {
              Logger.trace((new Date()) + ' Received request for ' + request.url);
              response.writeHead(404);
              response.end();
          });
        }
        this.wss = new server({httpServer: https_server, autoAcceptConnections: false});
        this.wss.on('connect', this.on_connect);
        this.wss.on('request', this.on_request);
        // this.wss.on('close', this.on_close);
        https_server.listen(config.port, function () {
            console.log((new Date()) + ` Server is listening on port ${config.port}`);
        });
        https_server.on('tlsClientError', err => {
            Logger.error(err);
        });
    }

    private on_connect = async (conn: connection) => {
        Logger.debug('ws connected, socket: ' + conn.socket);
    }

    private on_request = async (request: request) => {
        let headers: IncomingHttpHeaders = request.httpRequest.headers;
        let query: any = request.resourceURL.query || {};
        let token: string = <string>headers.token || query['token'] || '';
        Logger.trace(`ws token ${token} `);
        if (token === '') {
            request.reject(404, '请先登录获取token');
            return request.socket.end();
        }

        try {
            let redis = await Redis.get_instance('client-token');
            let cached_info = await redis.hgetall(<string>token, redis.expires);


            if (cached_info === null) {
                request.reject(404, '请先登录获取token');
                return request.socket.end();
                Logger.debug('websocket invalid_token');
                // conn.send(JSON.stringify({
                //     topic: "response.invalid_token"
                // }));
                // return conn.close(connection.CLOSE_REASON_POLICY_VIOLATION, 'invalid token.');
            }
            let conn: connection = request.accept();
            this.token_conn_map.set(token, conn);
            this.conn_token_map.set(conn, token);
            conn.on('message', data => {
                if (data.type === 'utf8') {
                    Logger.trace('Received Message: ' + data.utf8Data);
                    this.emit('message', token, data.utf8Data);
                }
                else if (data.type === 'binary') {
                    Logger.trace('Received Message: ' + data.binaryData);
                    this.emit('message', token, data.binaryData);
                }

            });
            conn.on('close', (code: number, desc: string) => {
                Logger.debug(`ws disconnected, socket: ${conn.socket} reason(${code} token:${token}): ${desc}`);
                this.conn_token_map.delete(conn);
                this.token_conn_map.delete(token);
                this.emit('close', token, code, desc);
            });
            return conn;
        } catch (e) {
            Logger.error(e);
            throw Resp.throw_err(e);
        }
    }

    public send_json(token: string, data: any) {
        this.token_conn_map.get(token)?.send(JSON.stringify(data));
    }

    // private on_close = async (conn: connection, reason: number, desc: string) => {
    //
    //     let token = this.conn_token_map.get(conn);
    //     this.conn_token_map.delete(conn);
    //     Logger.debug(`ws disconnected, socket: ${conn.socket} reason(${reason} token:${token}): ${desc}`);
    //     if (token) {
    //         this.token_conn_map.delete(token);
    //     }
    // }
}
