import mongoose from 'mongoose';
import { IMongodbConfig } from "@util/Config";
import { Logger } from '@util/Logger';

/**
 * 使用 Node 自带 Promise 代替 mongoose 的 Promise,否则会报错
 */
// mongoose.Promise = global.Promise;

export default class MongoClient {
    public mongoClient: mongoose.Connection //& Promise<mongoose.Connection>

    constructor(databaseConf: IMongodbConfig) {
        /**
         * 创建 Mongo 连接，内部维护了一个连接池，全局共享
         */
        this.mongoClient = mongoose.createConnection(this.getMongoUrl(databaseConf), this.getMongodbConfig());

        /**
         * Mongo 连接成功回调
         */
        this.mongoClient.on('connected', () => {
            Logger.log(`Mongoose连接至 ：${this.getMongoUrl(databaseConf)}`);
        });

        /**
         * Mongo 连接失败回调
         */
        this.mongoClient.on('error', function (err) {
            console.log('Mongoose 连接失败，原因: ' + err);
        });
        /**
         * Mongo 关闭连接回调
         */
        this.mongoClient.on('disconnected', function () {
            console.log('Mongoose 连接关闭');
        });
    }

    /**
     * 配置 MongoDb options
     */
    private getMongodbConfig(): any {
        let options = {
            useNewUrlParser: true,
            maxPoolSize: 6, // 连接池中维护的连接数
            // reconnectTries: Number.MAX_VALUE,
            keepAlive: true,
            useUnifiedTopology:true
        }
        return options;
    }
    /**
     * 拼接 MongoDb Uri
     *
     */
    private getMongoUrl(databaseConf: IMongodbConfig): string {
        let mongodbConfig = databaseConf;
        let mongoUrl = 'mongodb://';
        let dbName = mongodbConfig.database;
        let upStr = "";
        if (mongodbConfig.username) {
            upStr += `${mongodbConfig.username}`;
            if (mongodbConfig.password) {
                upStr += `:${encodeURIComponent(mongodbConfig.password)}`;
            }
            upStr += "@";
        }
        mongoUrl += `${upStr}${mongodbConfig.host}:${mongodbConfig.port}`;
        mongoUrl += `/${dbName}?authSource=admin`;
        return mongoUrl;
    }

    /**
     * 关闭 Mongo 连接
     */
    public close() {
        this.mongoClient.close();
    }

}
