import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Op, WhereOptions} from "sequelize";
import DataCURD from "@core/DataCURD";
import {AcsApplication} from "@core-models/AcsApplication";

interface IData {
    keyword?: string;
    app_type?: number;
    auth_type?: number;
    status?: number;
}

function get_where(json: IRequest, params: IMethodParams, _cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        let data = <IData>json.data;
        try {
            let where: any = {};
            if (data.keyword) {
                where = {
                    [Op.or]: {
                        name: {[Op.like]: `%${data.keyword}%`},
                        app_id: {[Op.like]: `%${data.keyword}%`},
                        app_key: {[Op.like]: `%${data.keyword}%`}
                    }
                }
            }
            if (data.app_type !== undefined) where.app_type = data.app_type;
            if (data.auth_type !== undefined) where.auth_type = data.auth_type;
            if (data.status !== undefined) where.status = data.status;
            resolve(where);

        } catch (e) {
            reject(Resp.gen_err(Resp.InternalServerError));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "title": "获取应用列表",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "平台分配的app id"
            },
            "token": {
                "type": "string",
                "title": "登录后获得的令牌"
            },
            "timestamp": {
                "type": "integer",
                "title": "时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "data": {
                "type": "object",
                "title": "业务参数",
                "properties": {
                    "keyword": {
                        "type": "string",
                        "title": "关键词，匹配app_id、app_key和name"
                    },
                    "app_type": {
                        "type": "integer",
                        "title": "类型，0=内部微服务 1=微信小程序，2=支付宝小程序，3=web客户端，4=设备端，5=第三方平台",
                        "enum": [0, 1, 2, 3, 4, 5]
                    },
                    "auth_type": {
                        "type": "integer",
                        "title": "认证方式：1=仅账号密码 2=账号密码+签名 3=仅签名",
                        "enum": [1, 2, 3]
                    },
                    "status": {
                        "type": "integer",
                        "title": "应用状态，1=正常，2=锁定"
                    }
                }
            }
        },
        "required": ["ver", "app_id", "timestamp", "data"]
    },
    method: DataCURD.internal_get_rows,
    method_params: {
        model: AcsApplication,
        where: get_where,
        output_data_map: {
            app_id: 'app_id',
            app_key: 'app_key',
            name: 'name',
            app_type: 'app_type',
            auth_type: 'auth_type',
            status: 'status'
        }
    }
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
