import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {AcsDomain} from "@core-models/AcsDomain";
import {ISQLReplacements} from "@core/DataCURD";

interface IData {
    pid: number;
    name: string;
    tag?: string;
    memo?: string;
}

function add(json: IRequest, _param: IMethodParams, _cached_data: ICachedData): Promise<unknown> {
    return new Promise<unknown>(async (resolve, reject) => {
        let data: IData = <IData>json.data;
        try {
            let parent_path = await get_parent_path(data);
            let result = await do_add(data, parent_path);
            resolve(result);
        } catch (e) {
            reject(e);
        }
    });
}

function get_parent_path(params: IData): Promise<string> {
    return new Promise<string>(async (resolve, reject) => {
        try {
            let record = await AcsDomain.findOne({where: {id: params.pid},raw: true});
            if (!record)  return reject(Resp.gen_err(Resp.Forbidden));
            resolve(record.path);
        } catch (e) {
            reject(e);
        }
    });
}

function do_add(params: IData, parent_path: string): Promise<unknown> {
    return new Promise<unknown>(async (resolve, reject) => {
        let sql: string;
        let replacements: ISQLReplacements = {};
        if (params.pid === 0) {
            sql = `insert into ${AcsDomain.table_name} (name, pid, path, memo, tag) 
                values(:name, ${params.pid}, text2ltree(to_char(nextval('tb_acs_domain_id_seq')-1, 'FM999999999')), 
                :memo, :tag) returning id`;
            replacements = {name: params.name, memo: params.memo, tag: params.tag};
        } else {
            sql = `insert into ${AcsDomain.table_name} (name, pid, path, memo, tag) 
             values(:name, ${params.pid}, 
             text2ltree( :parent_path || '.' || to_char(nextval('tb_acs_domain_id_seq')-1, 'FM999999999')), 
             :memo, :tag) returning id`;
            replacements = {name: params.name, parent_path: parent_path, memo: params.memo, tag: params.tag};
        }
        try {
            const [results, metadata] = await AcsDomain.sequelize!.query(sql, {replacements: replacements});
            if (!results) return reject(Resp.gen_err(Resp.PostgresqlServerError));
            resolve({
                id: (<AcsDomain>results[0]).id
            });
        } catch (e) {
            reject(Resp.gen_db_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "平台分配的app id"
            },
            "token": {
                "type": "string",
                "title": "登录后获得的令牌"
            },
            "timestamp": {
                "type": "integer",
                "title": "时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "data": {
                "type": "object",
                "title": "业务参数",
                "properties": {
                    "pid": {
                        "type": "integer",
                        "title": "父域id，顶级为0"
                    },
                    "name": {
                        "type": "string",
                        "title": "域名称"
                    },
                    "tag": {
                        "type": "string",
                        "title": "标签",
                        "default": ""
                    },
                    "memo": {
                        "type": "string",
                        "title": "备注",
                        "default": ""
                    }
                },
                "required": [
                    "pid", "name"
                ]
            }
        },
        "required": [
            "ver", "app_id", "timestamp",
            "data"
        ]
    },
    method: add,
    method_params: {
        model: AcsDomain
    }
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
