import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {AcsModule} from "@core-models/AcsModule";
import {QueryTypes} from "sequelize";
import {Resp} from "@util/Resp";
import {AcsFunction} from "@core-models/AcsFunction";
import {AcsUserRole} from "@core-models/AcsUserRole";
import {AcsRoleFunction} from "@core-models/AcsRoleFunction";

interface IData {
    module_id: number;
    tag: string;
    name: string;
    check_auth: boolean;
    rw: number;
}

function add(json: IRequest, _params: IMethodParams, cached_data: ICachedData): Promise<unknown> {
    return new Promise<unknown>(async (resolve, reject) => {
        let params: IData = <IData>json.data;
        try {
            let user_id = cached_data.user_id;
            let parent_path = await get_parent_path(params);
            let result = await do_add(params, parent_path, user_id);
            resolve(result);
        } catch (e) {
            reject(e);
        }
    });
}

function get_parent_path(params: IData): Promise<string> {
    return new Promise<string>(async (resolve, reject) => {

        try {
            let sql = `select tag from tb_acs_module \
              where path @> (select path from tb_acs_module where id= ${params.module_id}) order by path`;

            let records: Array<AcsModule> = await AcsModule.sequelize!.query(sql, {type: QueryTypes.SELECT, raw: true});
            if (!records) return reject(Resp.gen_err(Resp.Forbidden));
            let parent_path = '';
            for (let i = 0; i < records.length; i++) {
                parent_path = parent_path + records[i].tag + '.';
            }
            resolve(parent_path);
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    });
}

function do_add(params: IData, parent_path: string, user_id: string): Promise<unknown> {
    return new Promise<unknown>(async (resolve, reject) => {
        let sql = `insert into tb_acs_function (tag, module_id, name, api, check_auth, rw) 
           values(:tag, :module_id, :name, 
                  :path, :check_auth, :rw) returning id`;
        let replacements = {tag: params.tag, module_id: params.module_id,
            name: params.name, path: parent_path + params.tag, check_auth: params.check_auth, rw: params.rw};
        let t = await AcsFunction.sequelize!.transaction();
        try {
            let [records, meta] = await AcsFunction.sequelize!.query(sql, {replacements: replacements, transaction: t})
            if (!records || records.length <= 0) {
                await t.rollback();
                return reject(Resp.gen_err(Resp.PostgresqlServerError));
            }
            let func_id = (<AcsFunction>records[0]).id;
            let role = await AcsUserRole.findOne({where: {user_id: user_id}, raw: true});
            if (!role) throw Resp.gen_err(Resp.ResourceNotFound, '用户未分配角色');
            await AcsRoleFunction.sequelize!.query(`
                insert into ${AcsRoleFunction.table_name} (role_id, func_id)
                values(:role_id, :func_id)
            `, {replacements: {role_id: role.role_id, func_id: func_id}, transaction: t});
            await t.commit();
            resolve({
                id: func_id
            });
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "平台分配的app id"
            },
            "token": {
                "type": "string",
                "title": "登录后获得的令牌"
            },
            "timestamp": {
                "type": "integer",
                "title": "时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "data": {
                "type": "object",
                "title": "业务参数",
                "properties": {
                    "module_id": {
                        "type": "integer"
                    },
                    "tag": {
                        "type": "string"
                    },
                    "name": {
                        "type": "string"
                    },
                    "check_auth": {
                        "type": "boolean"
                    },
                    "rw": {
                        "type": "integer",
                        "default": 1
                    }
                },
                "required": [
                    "module_id",
                    "tag",
                    "name",
                ]
            }
        },
        "required": [
            "ver", "app_id", "timestamp", "data"
        ]
    },
    method: add,
    method_params: {
    }
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
