import {Resp} from "@util/Resp";
import {AcsModule} from "@core-models/AcsModule";
import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {ISQLReplacements} from "@core/DataCURD";


interface IData {
    pid?: number;
    tag: string;
    name: string;
    memo?: string;
    ui_show?: boolean;
    order_id?: number;
}

interface IParentPathInfo {
    path: string;
    path_name: string;
}

function add(json: IRequest, _params: IMethodParams, _cached_data: ICachedData): Promise<unknown> {
    return new Promise<unknown>(async (resolve, reject) => {
        let data: IData = <IData>json.data;
        try {
            let info: IParentPathInfo = await get_parent_path(data);
            let parent_path = info.path === '' ? '' : info.path + '.';
            let current_tag = info.path === '' ? info.path_name : info.path_name + '.' + data.tag;
            let result = await do_add(data, parent_path, current_tag);
            resolve(result);
        } catch (e) {
            reject(e);
        }
    });
}

function get_parent_path(data: IData): Promise<IParentPathInfo> {
    return new Promise<IParentPathInfo>(async (resolve, reject) => {
        try {
            if (data.pid === undefined) { // 顶级
                return resolve( {path: '', path_name: data.tag});
            }
            let record = await AcsModule.findOne({where: {id: data.pid}, raw: true});
            if (!record)  return reject(Resp.gen_err(Resp.Forbidden));
            resolve({
                path: record.path,
                path_name: record.path_name
            });
        } catch (e) {
            reject(e);
        }
    });
}

function do_add(params: IData, parent_path: string, current_tag: string): Promise<unknown> {
    return new Promise<unknown>(async (resolve, reject) => {
        let sql = `insert into tb_acs_module (tag, name, pid, memo, path, path_name) 
                     values(:tag, :name, :pid, :memo,
                     text2ltree( :parent_path || to_char(nextval('tb_acs_module_id_seq')-1, 'FM999999999')), :current_tag) 
                     returning id;`;
        let replacements: ISQLReplacements = {
            tag: params.tag,
            name: params.name,
            pid: params.pid ? params.pid: null,
            memo: params.memo?params.memo: '',
            parent_path: parent_path,
            current_tag: current_tag
        }
        try {
            const [results, metadata] = await AcsModule.sequelize!.query(sql, {replacements: replacements});
            if (!results) return reject(Resp.gen_err(Resp.PostgresqlServerError));
            resolve({
                id: (<AcsModule>results[0]).id
            });
        } catch (e) {
            reject(e);
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "平台分配的app id"
            },
            "token": {
                "type": "string",
                "title": "登录后获得的令牌"
            },
            "timestamp": {
                "type": "integer",
                "title": "时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "data": {
                "type": "object",
                "title": "业务参数",
                "properties": {
                    "pid": {
                        "type": "integer"
                    },
                    "tag": {
                        "type": "string",
                    },
                    "name": {
                        "type": "string"
                    },
                    "memo": {
                        "type": "string",
                        "default": ""
                    }
                },
                "required": [
                    "tag", "name"
                ]
            }
        },
        "required": [
            "ver", "app_id", "timestamp", "data"
        ]
    },
    method: add,
    method_params: {}
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
