import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {AcsFunction} from "@core-models/AcsFunction";
import {AcsRoleFunction} from "@core-models/AcsRoleFunction";
import {AcsUserRole} from "@core-models/AcsUserRole";
import {QueryTypes} from "sequelize";
import {Resp} from "@util/Resp";
import {ISQLReplacements} from "@core/DataCURD";
import {escapeQuotes} from "@util/CryptTool";

interface IFunc {
    id: string;
    enabled: boolean;
}
interface IData {
    role_id: string;
    funcs: Array<IFunc>;
}
function set_functions(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<unknown>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            await check_functions(data, params, cached_data);
            let result = await do_set(data, params, cached_data);
            resolve(result);
        } catch (e) {
            reject(e);
        }
    });
}

function check_functions(data: IData, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        if (user === ADMINISTRATOR) return resolve();
        let funcs = '(';
        for (let i = 0; i < data.funcs.length; i++) {
            if (i > 0) {
                funcs += ','
            }
            funcs += data.funcs[i].id;
        }
        funcs += ')';
        let sql = `select id from ${AcsFunction.table_name} where id 
                in ${funcs} and id not in 
                (select func_id from ${AcsRoleFunction.table_name} 
                where role_id in (select role_id from ${AcsUserRole.table_name} where user_id= :user_id))`;
        let replacements: ISQLReplacements = {user_id: user}
        try {
            let result = await AcsFunction.sequelize!.query(sql, {type: QueryTypes.SELECT, replacements: replacements});
            if (result.length > 0) {
                return reject(Resp.throw_err(Resp.Forbidden));
            }
            resolve();
        } catch (e) {
            reject(e);
        }
    })
}

function do_set(data: IData, _params: IMethodParams, _cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        let new_funcs: any = [];
        let del_funcs: any = [];

        for (let i = 0; i < data.funcs.length; i++) {
            let func = data.funcs[i];
            let value = {
                role_id: data.role_id,
                func_id: func.id
            }
            if (func.enabled == true) {
                new_funcs.push(value);
            } else {
                del_funcs.push(value);
            }
        }
        let sql_list: any = [],
          values_list: any = [];
        let sql, values;

        if (new_funcs.length > 0) {
            sql = `insert into ${AcsRoleFunction.table_name} (role_id, func_id) values`;
            for (let i = 0; i < new_funcs.length; i++) {
                let func = new_funcs[i];
                if (i > 0) {
                    sql += ',';
                }
                sql += `('${escapeQuotes(func.role_id)}', ${func.func_id}) `;
            }
            sql += 'on conflict(role_id, func_id) do nothing';
            sql_list.push(sql);
            values_list.push([]);
        }
        if (del_funcs.length > 0) {
            sql = `delete from ${AcsRoleFunction.table_name} where role_id= '${escapeQuotes(data.role_id)}' and (`;
            for (let i = 0; i < del_funcs.length; i++) {
                let func = del_funcs[i];
                if (i > 0) {
                    sql += ' or ';
                }
                sql += 'func_id = ' + func.func_id;
            }
            sql += ')';
            sql_list.push(sql);
        }
        let t = await AcsRoleFunction.sequelize!.transaction();
        try {
            for(let i = 0; i < sql_list.length; i++) {
                let sql = sql_list[i];
                await AcsRoleFunction.sequelize!.query(sql, {transaction: t});
            }
            await t.commit();
            resolve({});
        } catch (e) {
            await t.rollback();
            reject(e);
        }

    })
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "平台分配的app id"
            },
            "token": {
                "type": "string",
                "title": "登录后获得的令牌"
            },
            "timestamp": {
                "type": "integer",
                "title": "时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "data": {
                "type": "object",
                "title": "业务参数",
                "properties": {
                    "role_id": {
                        "type": "string"
                    },
                    "funcs": {
                        "type": "array",
                        "items": {
                            "type": "object",
                            "properties": {
                                "id": {
                                    "type": "number"
                                },
                                "enabled": {
                                    "type": "boolean"
                                }
                            },
                            "required": [
                                "id",
                                "enabled"
                            ]
                        }
                    }
                },
                "required": [
                    "role_id",
                    "funcs"
                ]
            }
        },
        "required": [
            "ver", "app_id", "timestamp",
            "data",
        ]
    },
    method: set_functions,
    method_params: {},
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
