import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {domainGuard} from "@src/guards/domainGuard";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {AcsUserDomain} from "@core-models/AcsUserDomain";
import {Resp} from "@util/Resp";

interface IData {
    domain_id: number;
    user_id: string;
    name: string;
    pass: string;
    mobile: string;
    email: string;
    status: number;
}

function add(json: IRequest, _params: IMethodParams, _cached_data: ICachedData): Promise<any> {
    return new Promise<unknown>(async (resolve, reject) => {
        let data = <IData>json.data;
        data.user_id = data.user_id.toLowerCase();
        let t = await AcsUserInfo.sequelize!.transaction();
        try {
            if (data.mobile) {
                let userInfo = await AcsUserInfo.findOne({where: {mobile: data.mobile}});
                if (userInfo) {
                    await t.rollback();
                    return reject(Resp.gen_err(Resp.DataExists, `手机号为${data.mobile}的账号已存在，不允许重复。`))
                }
            }
            await AcsUserInfo.create({
                id: data.user_id,
                name: data.name,
                pass: data.pass,
                mobile: data.mobile,
                email: data.email,
                reg_time: new Date(),
                act_time: new Date(),
                status: data.status
            }, {transaction: t});

            await AcsUserDomain.create({
                user_id: data.user_id,
                domain_id: data.domain_id,
            }, {transaction: t});

            await AcsUserDomain.create({
                user_id: data.user_id,
                domain_id: 100,
            }, {transaction: t});

            await AcsUserDomain.create({
                user_id: data.user_id,
                domain_id: 4,
            }, {transaction: t});

            await t.commit();
            resolve({});
        } catch (e) {
            await t.rollback();
            reject(e);
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "平台分配的app id"
            },
            "token": {
                "type": "string",
                "title": "登录后获得的令牌"
            },
            "timestamp": {
                "type": "integer",
                "title": "时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "data": {
                "type": "object",
                "title": "业务参数",
                "properties": {
                    "domain_id": {
                        "type": "integer"
                    },
                    "user_id": {
                        "type": "string",
                        "maxLength": 15,
                        "minLength": 2
                    },
                    "name": {
                        "type": "string",
                        "default": ""
                    },
                    "pass": {
                        "type": "string"
                    },
                    "status": {
                        "type": "integer",
                        "title": "状态，0=锁定，1=正常",
                        "default": 1
                    },
                    "mobile": {
                        "type": "string",
                        "pattern": "^$|^(13[0-9]|14[01456879]|15[0-3,5-9]|16[2567]|17[0-8]|18[0-9]|19[0-3,5-9])\\d{8}$",
                        "default": ""
                    },
                    "email": {
                        "type": "string",
                        "pattern": "^$|^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\\.[a-zA-Z0-9_-]+)+$",
                        "default": ""
                    }
                },
                "required": [
                    "domain_id",
                    "user_id",
                    "name",
                    "pass",
                    "status"
                ]
            }
        },
        "required": [
            "ver", "app_id", "timestamp",
            "data"
        ]
    },
    method: add,
    method_params: {},
    guards: [domainGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
