import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {domainGuard} from "@src/guards/domainGuard";
import {AcsDomain} from "@core-models/AcsDomain";
import {AcsUserDomain} from "@core-models/AcsUserDomain";
import {QueryTypes} from "sequelize";
import {Resp} from "@util/Resp";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {ISQLReplacements} from "@core/DataCURD";
import {Logger} from "@util/Logger";
import {escapeQuotes} from "@util/CryptTool";

interface ISqlInfo {
    sql_count: string;
    sql: string;
    replacements: ISQLReplacements;
}
interface IData {
    page_no?: number;
    page_size?: number;
    domain_id: number;
    keyword?: string;
}
function get_list(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<unknown>(async (resolve, reject) => {
        let user = cached_data.user_id;
        try {
            let data = <IData>json.data;
            let domain_paths: Array<number> = [];
            if (user !== ADMINISTRATOR) {
                domain_paths = await get_domain_paths(json, params, cached_data);
            }
            let sqlInfo: ISqlInfo = await build_sql(data, params, cached_data, domain_paths);
            let count: number = await get_count(data, sqlInfo);
            let result = await do_get_list(data, count, sqlInfo);
            resolve(result);
        } catch (e) {
            reject(e);
        }
    })
}

function get_domain_paths(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    let user: string = cached_data.user_id;
    let values: any[] = [];

    return new Promise<Array<number>>(async (resolve, reject) => {
        let sql = `select ${AcsDomain.table_name}.path 
                    from ${AcsUserDomain.table_name},${AcsDomain.table_name}
                   where ${AcsDomain.table_name}.id = ${AcsUserDomain.table_name}.domain_id and 
                    ${AcsUserDomain.table_name}.user_id= :user`;
        try {
            let result = await AcsDomain.sequelize!.query(sql, {
                type: QueryTypes.SELECT,
                replacements: {user: user}
            });
            if (result.length <= 0) {
                return reject(Resp.gen_err(Resp.Forbidden));
            }
            let domain_paths = [];
            for (let i = 0; i < result.length; i++) {
                // @ts-ignore
                domain_paths.push(result[i].path);
            }
            return resolve(domain_paths);

        } catch (e) {
            reject(e);
        }
    })
};

function build_sql(data: IData, params: IMethodParams, cached_data: ICachedData, paths: Array<number>): Promise<ISqlInfo> {
    return new Promise<ISqlInfo>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let select_count_sql;
        let select_sql;
        let where_str;
        let values = [];
        let replacements: ISQLReplacements = {};
        if (data.domain_id === undefined) {

            select_count_sql = `select count(distinct ${AcsUserInfo.table_name}.id)from ${AcsUserInfo.table_name},${AcsUserDomain.table_name},${AcsDomain.table_name} `;
            select_sql = `select distinct ${AcsUserInfo.table_name}.id,${AcsUserInfo.tableName}.pass,${AcsUserInfo.tableName}.name,
                                          ${AcsUserInfo.tableName}.mobile,${AcsUserInfo.tableName}.email,
                                          to_char(${AcsUserInfo.tableName}.reg_time, 'yyyy-MM-dd HH24'||':'||'MI'||':'||'ss') as reg_time,
                                          to_char(${AcsUserInfo.tableName}.act_time, 'yyyy-MM-dd HH24'||':'||'MI'||':'||'ss') as act_time,
                                          ${AcsUserInfo.tableName}.status from ${AcsUserInfo.tableName},${AcsUserDomain.tableName},${AcsDomain.tableName} `;

            if (user == ADMINISTRATOR) {
                if (data.keyword != undefined && data.keyword != '') {
                    where_str = ` where 
                        ${AcsUserInfo.tableName}.id like :keyword or 
                        ${AcsUserInfo.tableName}.name like :keyword `;
                    replacements.keyword = `%${data.keyword}%`;
                }
            } else {
                where_str = `where  ${AcsUserDomain.tableName}.domain_id = ${AcsDomain.tableName}.id and 
                        ${AcsUserDomain.tableName}.user_id = ${AcsUserInfo.tableName}.id and (`;
                for (let i = 0; i < paths.length; i++) {
                    if (i > 0) {
                        where_str += ' or ';
                    }
                    where_str += ` (${AcsDomain.tableName}.path <@ '${escapeQuotes(paths[i])}' and ${AcsDomain.tableName}.path != '${escapeQuotes(paths[i])}') `;
                }

                where_str += ') ';
                if (data.keyword != undefined && data.keyword != '') {
                    where_str = ` and (${AcsUserInfo.tableName}.id like '%${escapeQuotes(data.keyword)}%' or ${AcsUserInfo.tableName}.name like '%${escapeQuotes(data.keyword)}%') `;
                }
            }

            select_count_sql += where_str;
            select_sql += where_str;


        } else {
            let where_sql;
            select_count_sql = `select count(distinct user_info.id) 
                from ${AcsUserInfo.tableName} user_info,${AcsUserDomain.tableName} user_domain,${AcsDomain.tableName} domain `;

            select_sql = `select distinct user_info.id,
                                          user_info.name, user_info.mobile, user_info.email, 
                to_char(user_info.reg_time AT TIME ZONE 'PRC', 'yyyy-MM-dd HH24'||':'||'MI'||':'||'ss') as reg_time, 
                to_char(user_info.act_time AT TIME ZONE 'PRC', 'yyyy-MM-dd HH24'||':'||'MI'||':'||'ss') as act_time,
                user_info.status,string_agg(domain.id::varchar, ',' order by domain.id) as domain_id, string_agg(domain.name, ',' order by domain.id) as domain_name
                from ${AcsUserInfo.tableName} user_info,${AcsUserDomain.tableName} user_domain,${AcsDomain.tableName} domain `;

            where_sql = ` where user_info.id = user_domain.user_id and 
                user_domain.domain_id = domain.id and 
                domain.path <@ (
                        select path from ${AcsDomain.tableName} where ${AcsDomain.tableName}.id =:domain_id ) and
                user_info.id != :user `;
            replacements.user = user;
            replacements.domain_id = data.domain_id;

            if (data.keyword != undefined && data.keyword != '') {
                where_sql += ` and (user_info.id like :keyword or 
                    user_info.name like :keyword) `;
                replacements.keyword = `%${data.keyword}%`
            }

            select_count_sql += where_sql;
            select_sql += where_sql + ' group by user_info.id, user_info.name, user_info.mobile, user_info.email, user_info.reg_time, user_info.act_time, user_info.status';
        }
        Logger.info(select_count_sql);
        Logger.info(select_sql);
        resolve({
            sql_count: select_count_sql,
            sql: select_sql,
            replacements: replacements
        });
    })
}

function get_count(data: IData, sqlInfo: ISqlInfo): Promise<number> {
    return new Promise<number>(async (resolve, reject) => {
        try {
            let result = await AcsUserInfo.sequelize!.query(sqlInfo.sql_count, {
                type: QueryTypes.SELECT,
                replacements: sqlInfo.replacements
            });
            // @ts-ignore
            let count = parseInt(result[0].count);
            resolve(count);
        } catch (e) {
            reject(e);
        }
    })
}

function do_get_list(data: IData,  count: number, sqlInfo: ISqlInfo): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        if (count === 0) {
            return resolve({
                page_no: data.page_no,
                page_count: 0,
                total: 0,
                users: []
            })
        }
        let page_count = 0;
        if (data.page_size && data.page_no) {

            page_count = Math.ceil(count / data.page_size);
            let offset_num = (data.page_no - 1) * data.page_size;
            sqlInfo.sql += ' order by reg_time desc, user_info.id limit ' + data.page_size + ' offset ' + offset_num;
        } else
            page_count = 1;
        Logger.info(sqlInfo.sql);
        try {
            let result = await AcsUserInfo.sequelize!.query(sqlInfo.sql,{
                type: QueryTypes.SELECT,
                replacements: sqlInfo.replacements,
                raw: true
            });
            resolve({
                page_no: data.page_no,
                page_count: page_count,
                total: count,
                users: result
            });
        } catch (e) {
            reject(e);
        }

    })
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "平台分配的app id"
            },
            "token": {
                "type": "string",
                "title": "登录后获得的令牌"
            },
            "timestamp": {
                "type": "integer",
                "title": "时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "data": {
                "type": "object",
                "title": "业务参数",
                "properties": {
                    "page_no": {
                        "type": "integer",
                        "title": "页码"
                    },
                    "page_size": {
                        "type": "integer",
                        "title": "每页条数"
                    },

                    "domain_id": {
                        "type": "integer"
                    },
                    "keyword": {
                        "type": "string",
                        "title": "搜索关键词，可匹配user id，user name"
                    }
                },
                "required": [
                    "domain_id"
                ]
            }

        },
        "required": [
            "ver", "app_id", "timestamp",
            "data"
        ]
    },
    method: get_list,
    method_params: {},
    guards: [domainGuard]

}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
