import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {Op} from "sequelize";
import {Resp} from "@util/Resp";
import DataCURD from "@core/DataCURD";
import {subAccountGuard} from "@src/guards/subAccountGuard";

interface IData {
    user_id: string;
    mobile?: string;
}

function get_where(json: IRequest, _params: IMethodParams, _cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            resolve({id: data.user_id});
        } catch (e) {
            reject(e);
        }
    })
}

function modify(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        let t = await AcsUserInfo.sequelize!.transaction();
        try {
            let data = <IData>json.data;
            if (data.mobile) {
                let userInfo = await AcsUserInfo.findOne({where: {mobile: data.mobile, id: {[Op.ne]:  data.user_id}}});
                if (userInfo) {
                    await t.rollback();
                    return reject(Resp.gen_err(Resp.DataExists, `手机号为${data.mobile}的账号已存在，不允许修改。`))
                }
            }

            await DataCURD.internal_update_row(json, {
                model: AcsUserInfo,
                where: get_where,
                input_data_map: {
                    name: 'name',
                    pass: 'pass',
                    mobile: 'mobile',
                    email: 'email',
                    status: 'status'
                }
            }, cached_data, t);

            await t.commit();
            resolve({id: data.user_id});
        } catch (e) {
            await t.rollback();
            reject(e);
        }
    })
}


const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "平台分配的app id"
            },
            "token": {
                "type": "string",
                "title": "登录后获得的令牌"
            },
            "timestamp": {
                "type": "integer",
                "title": "时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "data": {
                "type": "object",
                "title": "业务参数",
                "properties": {
                    "user_id": {
                        "type": "string",
                        "maxLength": 15,
                        "minLength": 2
                    },
                    "name": {
                        "type": "string",
                    },
                    "pass": {
                        "type": "string"
                    },
                    "status": {
                        "type": "integer",
                        "default": 1
                    },
                    "mobile": {
                        "anyOf": [
                            {
                                "type": "string",
                                "pattern": "^$|^(13[0-9]|14[01456879]|15[0-3,5-9]|16[2567]|17[0-8]|18[0-9]|19[0-3,5-9])\\d{8}$",
                            },
                            {
                                "type": "null"
                            }
                        ]


                    },
                    "email": {
                        "anyOf": [{
                            "type": "string",
                            "pattern": "^$|^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\\.[a-zA-Z0-9_-]+)+$",
                        },
                            {
                                "type": "null"
                            }]
                    }
                },
                "required": [
                    "user_id"
                ],
                "anyOf": [
                    {"required": ["name"]},
                    {"required": ["pass"]},
                    {"required": ["mobile"]},
                    {"required": ["email"]},
                    {"required": ["status"]}
                ]
            }
        },
        "required": [
            "ver", "app_id", "timestamp",
            "data"
        ]
    },
    method: modify,
    method_params: {
    },
    guards: [subAccountGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
