import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {AcsDomain} from "@core-models/AcsDomain";
import {AcsUserDomain} from "@core-models/AcsUserDomain";
import {domainsGuard} from "@src/guards/domainGuard";
import {Resp} from "@util/Resp";
import {QueryTypes} from "sequelize";
import {subAccountGuard} from "@src/guards/subAccountGuard";

interface IDomain {
    id: string;
    enabled: boolean;
}
interface IData {
    user_id: string;
    domains: Array<IDomain>;
}

function set_domains(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<unknown>(async (resolve, reject) => {
        let data = <IData>json.data;
        let t = await AcsDomain.sequelize!.transaction();
        try {
            if (data.user_id !== undefined) {
                data.user_id = data.user_id.toLowerCase();
            }

            await domainsGuard(json, cached_data);

            let new_domains: Array<IDomain> = [];
            let del_domains: Array<IDomain> = [];

            for (let i = 0; i < data.domains.length; i++){
                let domain = data.domains[i];
                if (domain.enabled == true){
                    new_domains.push(domain);
                }
                else{
                    del_domains.push(domain);
                }
            }

            if (new_domains.length <= 0){
                await t.rollback();
                return reject(Resp.gen_err(Resp.AtLeastOnDomain, '至少要有一个可用的域'));
            }

            let sql;
            if (new_domains.length >0){
                sql = `insert into ${AcsUserDomain.table_name} (user_id, domain_id)values`;
                for (let i = 0; i < new_domains.length; i++){
                    let domain = new_domains[i];
                    if (i > 0){
                        sql += ',';
                    }
                    sql += `('${data.user_id}', '${domain.id}')`;
                }
                sql += 'on conflict(user_id, domain_id) do nothing';
                await AcsUserDomain.sequelize!.query(sql, {type: QueryTypes.UPDATE});
            }

            if (del_domains.length > 0){
                sql =  `delete from ${AcsUserDomain.tableName} where user_id='${data.user_id}' and (`;
                for (let i = 0; i < del_domains.length; i++){
                    let domain = del_domains[i];
                    if (i > 0){
                        sql += ' or ';
                    }
                    sql += `domain_id = '${domain.id}' `;
                }
                sql += ')';
                await AcsUserDomain.sequelize!.query(sql, {type: QueryTypes.DELETE});
            }

            await t.commit();
            resolve({});
        } catch (e) {
            await t.rollback();
            reject(e);
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "平台分配的app id"
            },
            "token": {
                "type": "string",
                "title": "登录后获得的令牌"
            },
            "timestamp": {
                "type": "integer",
                "title": "时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "data": {
                "type": "object",
                "title": "业务参数",
                "properties": {
                    "user_id": {
                        "type": "string",
                        "maxLength": 15,
                        "minLength": 2
                    },
                    "domains": {
                        "type": "array",
                        "items": {
                            "type": "object",
                            "properties": {
                                "id": {
                                    "type": "integer"
                                },
                                "enabled": {
                                    "type": "boolean"
                                }
                            },
                            "required": [
                                "id",
                                "enabled"
                            ]
                        }
                    }
                },
                "required": [
                    "user_id", "domains"
                ]
            }
        },
        "required": [
            "ver", "app_id", "timestamp", "data"
        ]
    },
    method: set_domains,
    method_params: {},
    guards: [subAccountGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0,
}
