import {v1 as uuid} from 'uuid';

import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Redis} from "@util/Redis";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {AcsApplication} from "@core-models/AcsApplication";
import {AppType} from "@util/Const";

interface IData {
    app_id: string;
    username: string;
    password: string;
    session: string;
    vcode: string;
}

interface IResponse {
    username: string;
    token: string;
}

function login(content: IRequest, params: IMethodParams, _: ICachedData): Promise<IResponse> {
    return new Promise<IResponse>(async (resolve, reject) => {
        let data: IData = <IData>content.data;
        data.username = data.username.toLowerCase();
        try {
            if (data.vcode !== undefined && data.session && process.env.NODE_ENV !== 'development') {
                let redis = Redis.get_instance('client-token');
                let vcode = await redis.get(data.session);
                if (vcode !== data.vcode.toLowerCase()) {
                    throw Resp.gen_err(Resp.Forbidden, '验证码错误。');
                }
            }
            let record = await checkLock(data, params);
            record.act_time = new Date();
            await record.save();
            let result: IResponse = await do_login(content.app_id, data, record);
            return resolve(result);
        } catch (e) {
            return reject(e);
        }
    })
}

function checkLock(data: IData, params: IMethodParams): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            let record = await params.model.findOne({where: {id: data.username}});
            if (!record) {
                return reject(Resp.gen_err(Resp.LoginFailed));
            }
            if (record.getDataValue('status') === 0) {
                return reject(Resp.gen_err(Resp.UserLocked));
            }
            resolve(record);
        } catch (e) {
            reject(Resp.gen_db_err(e));
        }
    })
}

function do_login(app_id: string, data: IData, record: any): Promise<IResponse> {
    return new Promise<IResponse>(async (resolve, reject) => {
        try {
            let app_info = await AcsApplication.findOne({where: {app_id: app_id}, raw: true});
            let account = await AcsUserInfo.findOne({where: {id: data.username}, raw: true});
            if (!app_info || !account) {
                return reject(Resp.gen_err(Resp.Forbidden, 'app id不存在'));
            }
            if (app_info.status === 2) {
                return reject(Resp.gen_err(Resp.ResourceLocked, '应用已被锁定'));
            }
            if (account.status === 0) {
                return reject(Resp.gen_err(Resp.UserLocked, '用户被锁定'));
            }
            if (account.pass !== data.password) {
                return reject(Resp.gen_err(Resp.Forbidden, '账号或密码错误'));
            }

            let token = uuid();
            let cached_data: ICachedData = {
                user_id: account.id,
                user_name: account.name,
                app_id: app_id,
                app_type: AppType.WebClient
            };

            let redis = Redis.get_instance('client-token');
            await redis.hset(token, cached_data, redis.expires);
            resolve({
                token: token,
                username: record.name,
            });

        } catch (e) {
            reject(Resp.throw_err(e));
        }
    });
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "平台分配的app id"
            },
            "timestamp": {
                "type": "integer",
                "title": "时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "data": {
                "type": "object",
                "title": "业务参数",
                "properties": {
                    "username": {
                        "type": "string"
                    },
                    "password": {
                        "type": "string"
                    },
                    "session": {
                        "type": "string"
                    },
                    "vcode": {
                        "type": "string"
                    }
                },
                "required": ["username", "password"]
            }
        },
        "required": ["ver", "app_id", "timestamp"]
    },
    method: login,
    method_params:{
        model: AcsUserInfo
    }
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
