/**
 * 设置自身账户详情
 */
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {Resp} from "@util/Resp";
import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import DataCURD from "@core/DataCURD";
import {Logger} from "@util/Logger";
import {Op, WhereOptions} from "sequelize";

interface IData {
    name?: string;
    mobile?: string;
    email?: string;
    old_pass?: string;
    new_pass?: string;
}

function get_where(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        try {
            let where: WhereOptions = {id: cached_data.user_id};
            return resolve(where);
        } catch (e) {
            reject(Resp.gen_err(Resp.InternalServerError));
        }
    });
}
function set_self_info(content: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        let data = <IData>content.data;
        let t = await AcsUserInfo.sequelize!.transaction();
        try{
            if(data.new_pass) {
                let user_info = await AcsUserInfo.findOne({where: {id: cached_data.user_id}, raw: true, transaction: t});
                if (!user_info) return reject(Resp.gen_err(Resp.ResourceNotFound, '账号不存在'));
                if (user_info.pass !== data.old_pass) {
                    await t.rollback();
                    return reject(Resp.gen_err(Resp.Forbidden, '旧密码错误'))
                }
            }
            if (data.mobile) {
                let user_info = await AcsUserInfo.findOne({where: {mobile: data.mobile, id: {[Op.ne]:  cached_data.user_id}}, raw: true, transaction: t});
                if (user_info) {
                    await t.rollback();
                    return reject(Resp.gen_err(Resp.DataExists, `手机号为${data.mobile}的账号已存在，不允许修改。`))
                }
            }
            let result = await DataCURD.internal_update_row(content, params, cached_data, t);
            await t.commit();
            resolve(result);

        }catch (e) {
            Logger.error(e);
            await t.rollback();
            return reject(Resp.throw_err(e));
        }

    })
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "title": "设置自身账户详情",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "平台分配的app id"
            },
            "token": {
                "type": "string",
                "title": "登录后获得的令牌"
            },
            "timestamp": {
                "type": "integer",
                "title": "时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "data": {
                "type": "object",
                "title": "业务参数",
                "properties": {
                    "name": {
                        "type": "string",
                        "title": "实体卡用户名称"
                    },
                    "mobile": {
                        "anyOf": [
                            {
                                "type": "string",
                                "pattern": "^$|^(13[0-9]|14[01456879]|15[0-3,5-9]|16[2567]|17[0-8]|18[0-9]|19[0-3,5-9])\\d{8}$",
                            },
                            {
                                "type": "null"
                            }
                        ]
                    },
                    "email": {
                        "anyOf": [{
                            "type": "string",
                            "pattern": "^$|^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\\.[a-zA-Z0-9_-]+)+$",
                        },
                            {
                                "type": "null"
                            }]
                    },
                    "old_pass": {
                        "type": "string",
                        "title": "MD5 32位小写后的旧密码"
                    },
                    "new_pass": {
                        "type": "string",
                        "title": "MD5 32位小写后的新密码"
                    }
                },
                "anyOf": [
                    {"required": ["name"]},
                    {"required": ["mobile"]},
                    {"required": ["email"]},
                    {"required": ["old_pass", "new_pass"]},
                ]
            }
        },
        "required": [
            "ver", "app_id", "timestamp",
        ],

    },
    method: set_self_info,
    method_params: {
        model: AcsUserInfo,
        input_data_map: {
            name: 'name',
            mobile: 'mobile',
            email: 'email',
            new_pass: 'pass'
        },
        output_data_map: {

        },
        where: get_where
    },
    guards: []
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
