/**
 * 小程序登录系统
 */
import {v1 as uuid} from 'uuid';
import {IApiProcessor, ICachedData, IMethodParams, IRequest, Platform} from "@core/Defined";
import {Redis} from "@util/Redis";
import {Resp} from "@util/Resp";
import {MpPlatformFactory, PlatformType} from "@classes/MpPlatformFactory";
import {IMpOpenIdInfo} from "@classes/MpPlatformBase";
import {AcsApplication} from "@core-models/AcsApplication";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {AcsMpClient} from "@core-models/AcsMpClient";

interface IData {
    code: string;           // 用于从微信或支付宝平台换取open_id
    app_id: string;       // 小程序平台的app_id
    platform: number;       // 小程序平台，1=微信，2=支付宝
    type: number;           // 小程序类型，1=商户端，0或无为购物端
}

interface ISessionInfo {
    app_id: string;
    open_id: string;         // 小程序平台返回的open id
    platform: number;       // 小程序平台类型
}

// 用户信息获取状态
interface IInfoStatus {
    user_info: boolean;  // 是否已获得昵称及头像
    mobile: boolean;    // 是否已获得手机号
}

export interface IMpCachedInfo {
    timestamp: number;
    app_id: string;      // 小程序appId
    app_type: number;
    platform: number;
    open_id: string;
    // client_type: number;
    user_id?: string;
    user_name?: string;
}

interface IResponse {
    token: string;
    info_status: IInfoStatus;
}

function login(content: IRequest, _params: IMethodParams, _cached_data: ICachedData): Promise<IResponse> {
    let data = <IData>content.data;
    return new Promise<IResponse>(async (resolve, reject) => {
        try {
            let app_info = await AcsApplication.findOne({where: {app_id: content.app_id}, raw: true});
            if (!app_info) {
                return reject(Resp.gen_err(Resp.ResourceNotFound, 'app id不存在'));
            }
            // 从微信平台获取用户信息
            let platform: PlatformType = app_info.app_type === Platform.wx ? 'wx' : 'alipay';
            let mp_info: IMpOpenIdInfo = await MpPlatformFactory.getInstance(platform)
              .getOpenId(content.app_id, data.code);

            // 如无客户信息，添加，如已有，则获取信息
            let mp_client = await get_or_create_mp_client_info({
                app_id: content.app_id,
                open_id: mp_info.open_id,
                platform: mp_info.app_type,
            });

            // 生成本平台的token并更新缓存信息
            let token = await gen_token(mp_info.app_type, content.app_id, mp_info.app_type, mp_info.open_id, mp_client);
            resolve({
                token: token,
                // ws_url: `wss://${config.websocket.host}:${config.websocket.port}`,
                info_status: {
                    user_info: mp_client.name !== null && mp_client.name !== '' ? true : false,
                    mobile: mp_client.mobile !== null && mp_client.mobile !== '' ? true : false,
                }
            });
        } catch (e) {
            reject(e);
        }

    });
}

// 更新数据库中的数据并返回用户信息获得状态
function get_or_create_mp_client_info(sessionInfo: ISessionInfo): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            const [mp_client, created] = await AcsMpClient.findOrCreate({
                where: {open_id: sessionInfo.open_id},
                defaults: {
                    platform: sessionInfo.platform,
                    // client_type: sessionInfo.client_type
                }
            });
            if (!created) {
                if (mp_client.mobile !== null && mp_client.mobile !== '') {
                    let user_info = await AcsUserInfo.findOne({where: {mobile: mp_client.mobile}});
                    if (user_info) {
                        mp_client.user_id = user_info.id;
                        mp_client.name = user_info.name;
                    }
                }
                mp_client.platform = sessionInfo.platform;
                // mp_client.client_type = sessionInfo.client_type;
                await mp_client.save();
            }
            resolve(mp_client);
        } catch (e: any) {
            reject(Resp.gen_err(Resp.PostgresqlServerError, e.message));
        }
    });
}


// 生成本平台的session并更新缓存信息
function gen_token(platform: number, app_id: string, app_type: number, open_id: string,  mp_client: any): Promise<string> {
    return new Promise<string>(async (resolve, reject) => {
        try {
            // 生成本平台session
            let token = uuid();
            let user_id;

            if (mp_client.user_id && mp_client.user_id !== '') {
                user_id = mp_client.user_id;
            }

            //更新缓存数据
            let cached_data: IMpCachedInfo = {
                timestamp: (new Date()).getTime(),
                app_id: app_id,
                app_type: app_type,
                platform: platform,
                open_id: open_id,
                // client_type: client_type
            }
            if (user_id) {
                cached_data.user_id = user_id;
                cached_data.user_name = mp_client.name;
            }
            let redis: Redis = Redis.get_instance('client-token');
            await redis.hset(token, cached_data, redis.expires);
            resolve(token);
        } catch (e) {
            reject(e);
        }
    });
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "小程序app id"
            },
            "data": {
                "type": "object",
                "properties": {
                    "code": {
                        "type": "string",
                        "title": "小程序提供的code，用于换取openid"
                    },
                    "platform": {
                        "type": "integer",
                        "title": "小程序平台，1=微信，2=支付宝",
                        "enum": [1, 2],
                        "default": 1,

                    }
                },
                "required": [
                    "code",
                    "platform"
                ]
            }

        },
        "required": [
            "data",
            "app_id"
        ]
    },
    method: login,
    method_params: {}
};


module.exports = {
    default: v1_0,
    v1_0: v1_0
};
