/**
 * 小程序上报通过授权后获取到的用户信息，如手机号、昵称、头像等
 */
import {IApiProcessor, ICachedData, IMethodParams, Platform} from "@core/Defined";
import {Redis} from "@util/Redis";
import {Resp} from "@util/Resp";
import {IMpCachedInfo} from "@src/routes/api/mp/login";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {MpPlatformFactory, PlatformType} from "@classes/MpPlatformFactory";
import {IMpOpenIdInfo, MpPlatformBase} from "@classes/MpPlatformBase";
import {AcsMpClient} from "@core-models/AcsMpClient";

interface IEncryptedData {
    iv: string;
    encryptedData: string;
}

interface IData {
    code: string;
    userInfo: IEncryptedData;
    mobile: IEncryptedData;
}

interface IParams {
    token: string;
    // code: string;           // 小程序code
    openid: string;         // 小程序平台的openid
    platform: number;       // 小程序平台，1=微信，2=支付宝
    type: number;           // 小程序类型，1=商户端，0或无为购物端
    data: IData;
}

interface IInfo {
    name?: string;
    avatarUrl?: string;
    mobile?: string;
    unionId?: string;
    userId?: string;
}

function post_userinfo(json: unknown, param: IMethodParams, cachedData: any): Promise<unknown> {
    let params = <IParams>json;
    return new Promise<unknown>(async (resolve, reject) => {
        try {
            let mpCachedInfo: any = await getMpCachedInfo(params.token);
            let platformType: PlatformType = parseInt(mpCachedInfo.platform) === Platform.wx ? 'wx' : 'alipay';
            let mpPlatform: MpPlatformBase = await MpPlatformFactory.getInstance(platformType);
            let mpOpenIdInfo: IMpOpenIdInfo = await mpPlatform.getOpenId(mpCachedInfo.app_id, params.data.code);

            let info: IInfo = {};
            if (params.data && params.data.userInfo) {
                let result = mpPlatform.decryptUserInfo(mpOpenIdInfo.session_key, params.data.userInfo);
                if (result) {
                    info.name = result.nickname;
                    info.avatarUrl = result.avatarUrl;
                    info.unionId = result.unionId;
                }
            }
            if (params.data && params.data.mobile) {
                let result = mpPlatform.decryptMobile(mpOpenIdInfo.session_key, params.data.mobile);
                if (result) {
                    info.mobile = result.mobile;
                }
            }

            if (info.mobile) {
                info.userId = await getUserId(info.mobile);
            }
            if (info.userId) { // 更新token cache
                let redis = Redis.get_instance('client-token');
                try {
                    let result: any = await redis.hgetall(params.token, redis.expires);
                    if (result) {
                        result.user_id = info.userId;
                        await redis.hset(params.token, result, redis.expires);
                    }

                } catch (e: any) {
                    return reject(Resp.gen_err(Resp.RedisServerError, e.message));
                }
            }
            await updateInfo(mpCachedInfo.open_id, info);
            resolve({});
        } catch (e) {
            reject(e);
        }

    });
}

function getUserId(mobile: string): Promise<string> {
    return new Promise<string>(async (resolve, reject) => {
        try {
            let user = await AcsUserInfo.findOne({where: {mobile: mobile}, raw: true});
            if (!user) return resolve('');
            resolve(user.id);
        }catch (e) {
            reject(Resp.gen_db_err(e));
        }
    })
}

function getMpCachedInfo(token: string): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            let redis = Redis.get_instance('client-token');
            let result = await redis.hgetall(token, redis.expires);
            if (!result) {
                return reject(Resp.gen_err(Resp.Unauthorized));
            }
            return resolve(result);
        }catch (e) {
            reject(e);
        }

    });
}


function updateInfo(openId: string, info: IInfo): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        try {
            let values: any = {};
            if (info.unionId) values.union_id = info.unionId;
            if (info.name) values.nickname = info.name;
            if (info.avatarUrl) values.avatar_url = info.avatarUrl;
            if (info.mobile) values.mobile = info.mobile;
            if (info.userId) values.user_id = info.userId;
            await AcsMpClient.update(values, {where: {open_id: openId}, returning: false})
            resolve();
        }catch (e) {
            reject(e);
        }

    });
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "properties": {
            "ver": {
                "type": "string"
            },
            "app_id": {
                "type": "string"
            },
            "token": {
                "type": "string"
            },

            "data": {
                "type": "object",
                "properties": {
                    "code": {
                        "type": "string"
                    },
                    "userInfo": {
                        "type": "object",
                        "properties": {
                            "iv": {
                                "type": "string"
                            },
                            "encryptedData": {
                                "type": "string"
                            }
                        },
                        "required": [
                            "iv",
                            "encryptedData"
                        ]
                    },
                    "mobile": {
                        "type": "object",
                        "properties": {
                            "iv": {
                                "type": "string"
                            },
                            "encryptedData": {
                                "type": "string"
                            }
                        },
                        "required": [
                            "iv",
                            "encryptedData"
                        ]
                    }
                },
                "required": [
                    "code",
                ]
            }
        },
        "required": [
            // "code",
            "token",
            // "platform",
            // "type",
        ]
    },
    method: post_userinfo,
    method_params:{}
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
