import {Engine} from "bpmn-engine";
import {EventEmitter} from "node:events";
import dayjs from "dayjs";
import {IdGen} from "@util/IdGen";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjMembers} from "@core-models/PrjMembers";
import {AcsUserRole} from "@core-models/AcsUserRole";
import {Op, QueryTypes} from "sequelize";
import {AcsDomain} from "@core-models/AcsDomain";
import {AcsUserDomain} from "@core-models/AcsUserDomain";
import {BpmnCase} from "@core-models/BpmnCase";
import {BpmnWork} from "@core-models/BpmnWork";
import {Logger} from "@util/Logger";
import {BpmnForm} from "@core-models/BpmnForm";

const camunda = require('camunda-bpmn-moddle/resources/camunda.json');
import {resolveExpression} from '@aircall/expression-parser';
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {Checker} from "@src/routes/api/prj/plan/set_check_flow";
import {IChecker} from "@src/utils/define";
import {PrjLogger} from "@src/utils/prj_logger";

export interface IHandler {
    id: string;         // 处理人账号id
    name?: string;       // 处理人名称
    action?: string;    // 处理人动作，空时使用流程节点名
    due_day?: number;   // 到期天数, 超时后产生提醒, 空时使用流程节点超时时间
}

export interface IFlowArgs {
    id?: string;
    prj_id: string;
    owner: string;
    task_id?: string;
    outcome_id?: string;
}

export class FlowEngine extends EventEmitter {
    private engine: Engine;
    private readonly _id: string;
    private readonly _model_id: string;
    private readonly _prj_id: string;
    private readonly _task_id: string | undefined;
    private _owner: string;
    private readonly _args: IFlowArgs;
    private execution;
    private readonly _source: string;
    private readonly _callback?: Function;
    private _shared_values: Map<string, any> = new Map<string, any>();

    static execution_engine_map: Map<string, FlowEngine> = new Map<string, FlowEngine>();
    static case_engine_map: Map<string, FlowEngine> = new Map<string, FlowEngine>();


    constructor(model_id: string, source: string, args: IFlowArgs, callback?: Function) {
        super();
        this._id = args.id ? args.id : IdGen.id();
        this._prj_id = args['prj_id'];
        this._task_id = args.task_id;
        this._owner = args['owner'];
        this._args = args;
        this._source = source;
        this._model_id = model_id;
        this._callback = callback;
        this.engine = this.new_engine(source, args);
        FlowEngine.case_engine_map.set(this._id, this);
    }

    async run(state?: any) {
        const listener = new EventEmitter();
        listener.on('flow.take', this.on_flow_take);
        if (state) {
            this.engine = this.engine.recover(state);
            this.execution = await this.engine.resume({listener}, async (err, execution) => {
                if (err) console.log(err);
                console.log('Execution completed with id', this.id);
                await this.complete();
            });
        } else {
            this.execution = await this.engine.execute({listener}, async (err, execution) => {
                if (err) console.log(err);
                console.log('Execution completed with id', this.id);
                await this.complete();
            });
        }
        Logger.info(`flow engine started: ${this.engine.environment}`);
    }

    async stop() {
        if (this.execution.state === 'running')
            await this.engine.stop();
    }

    async complete() {
        await BpmnCase.update({completed_at: dayjs(), state: await this.state()}, {
            where: {id: this.id},
            returning: false
        })
        if (this._callback) {
            await this._callback(this.id, this._model_id, this._prj_id, this._owner);
        }
    }

    get id(): string {
        return this._id;
    }

    async state(): Promise<any> {
        return await this.engine.getState();
    }

    get model_id(): string {
        return this._model_id;
    }

    /// 项目负责人变动后，流程的负责人也相应变动
    set owner(value: string) {
        this._owner = value;
    }

    private new_engine(source: string, args: any): Engine {
        let self = this;
        // @ts-ignore
        return new Engine({
            name: this._id,
            source,
            disableDummyScript: true,
            moddleOptions: {
                camunda,
            },
            services: {
                get_value(key: string): any {
                    return self._shared_values.get(key);
                },
                set_value(key: string, value: any) {
                    self._shared_values.set(key, value);
                },
                get_handlers: (tag: string) => {
                    return this.get_handlers(tag);
                },
                gen_remind_task: (assigned_to: string, name: string, detail: string) => {
                    BpmnWork.create({
                        id: IdGen.id(),
                        name: name,
                        prj_id: this._prj_id,
                        task_id: this._task_id ? this._task_id : null,
                        case_id: this._id,
                        assigned_to: assigned_to,
                        started_at: dayjs(),
                        process_type: 2,
                        show_in_my_works: true,
                        desc: detail
                    })
                },
                set_prj_phase: (phase_id: string) => {
                    PrjInfo.update({phase_id: phase_id}, {
                        where: {id: this._prj_id},
                        returning: false
                    })
                }
            },
            variables: {
                ...args,
            },
            expressions: {
                resolveExpression,
            },
            extensions: {
                // @ts-ignore
                // resolveExpression,
                // @ts-ignore
                bpmnActivityHandler: (bpmnActivity: any, context) => {
                    // if (bpmnActivity.type !== 'bpmn:UserTask' && bpmnActivity.type !== 'bpmn:ManualTask') return;
                    if (bpmnActivity.type === 'bpmn:Process') return;
                    bpmnActivity.on('enter', this.on_enter);
                    bpmnActivity.on('wait', this.on_wait);
                    bpmnActivity.on('end', this.on_end);
                    bpmnActivity.on('leave', this.on_leave);

                    if (bpmnActivity.type !== 'bpmn:UserTask') return;
                    bpmnActivity.broker.subscribeTmp('execution', 'execute.iteration.batch', (routingKey, msg) => {
                        Logger.info('execute.iteration.batch');
                        Logger.info(msg);
                    });
                    bpmnActivity.broker.subscribeTmp('execution', 'execute.discard', (routingKey, msg) => {
                        Logger.trace('execute.discard');
                        Logger.info(msg);
                    });

                    bpmnActivity.broker.subscribeOnce('execution', 'execute.iteration.completed', self.on_loop_task_completed);
                    return {
                        activate: () => {
                            // bpmnActivity.on('end', async (elementApi, engineApi) => {
                            //     Logger.trace(`activity on end ${elementApi.id}`);
                            // },{consumerTag: `format-on-end_${IdGen.id()}`}),
                            //     bpmnActivity.on('execute.iteration.completed', async (elementApi, engineApi) => {
                            //             Logger.trace(`activity on execute.iteration.completed ${elementApi.id}`);
                            //         },{consumerTag: `format-on-execute.iteration.completed_${IdGen.id()}`});
                            // @ts-ignore
                            bpmnActivity.on('enter', async (elementApi, engineApi) => {
                                bpmnActivity.broker.publish('format', 'run.format.start', {endRoutingKey: 'run.format.complete'});
                                try {
                                    // await FlowActivityHandler.onStart(elementApi, engineApi);
                                    let handler_role = bpmnActivity.behaviour.assignee ||
                                        bpmnActivity.behaviour.candidateUsers ||
                                        bpmnActivity.behaviour.candidateGroups;
                                    let handlers = await this.get_handlers(handler_role);
                                    Logger.trace(handlers);
                                    // bpmnActivity.environment.variables.handlers = handlers;
                                    bpmnActivity.broker.publish('format', 'run.handlers', {handlers});
                                    if (bpmnActivity.behaviour.formKey) {
                                        let form = await BpmnForm.findOne({
                                            where: {id: bpmnActivity.behaviour.formKey},
                                            raw: true
                                        });
                                        if (form) {
                                            bpmnActivity.broker.publish('format', 'run.form', {form});
                                        }
                                    }
                                    if (bpmnActivity.behaviour.extensionElements.values) {
                                        for (const extn of bpmnActivity.behaviour.extensionElements.values) {
                                            // if (extn.$type === 'camunda:FormData' && extn.fields) {
                                            //     const form = {
                                            //         fields: extn.fields.map((f) => ({ ...f })),
                                            //     };
                                            //     bpmnActivity.broker.publish('format', 'run.form', { form });
                                            // } else
                                            if (extn.$type === 'camunda:InputOutput' && extn.inputParameters) {
                                                const input = extn.inputParameters.reduce((result, data) => {
                                                    result[data.name] = bpmnActivity.environment.resolveExpression(data.value);
                                                    return result;
                                                }, {});
                                                bpmnActivity.broker.publish('format', 'run.input', {input});
                                            }
                                        }
                                    }

                                } catch (err) {
                                    Logger.error(err);
                                    // end call
                                }
                                bpmnActivity.broker.publish('format', 'run.format.complete', {msg: "complete"});
                            }, {consumerTag: `format-on-enter_${IdGen.id()}`});

                            //
                            // bpmnActivity.on('activity.end', (elementApi) => {
                            //     Logger.info('activity.end');
                            //     Logger.info(elementApi.content.output);
                            //     // if (!elementApi.content.output) return;
                            //     // context.environment.output[elementApi.id] = elementApi.content.output;
                            // });
                        },
                        deactivate() {
                            bpmnActivity.broker.cancel('format-on-enter');
                        },
                    };
                },
                // @ts-ignore
                // camunda: this.camundaExt,
            },

        });
    }

    on_loop_task_completed = async (routingKey, msg, consumerCount) => {
        let activity = this.execution.getActivityById(msg.content.id);
        activity.broker.subscribeOnce('execution', 'execute.iteration.completed', this.on_loop_task_completed);
        // Logger.info(`execute.iteration.completed, count: `);
        // Logger.info(consumerCount);
        // Logger.info(msg);
        if (msg.content.input?.prj_phase) {
            await PrjInfo.update({phase_id: msg.content.input.prj_phase}, {
                where: {id: this._prj_id},
                returning: false
            })
        }
        if (msg.content.input?.task_status) {
            await PrjPlanTask.update({status: msg.content.input.task_status}, {
                where: {id: this._task_id},
                returning: false
            })
        }
        if (!msg.content || !msg.content.output || !msg.content.output[msg.content.index]) return;
        await BpmnWork.update({
                completed_at: dayjs(),
                handler: msg.content.output[msg.content.index].user_id,
                status: 2,
            },
            {
                returning: false,
                where: {
                    id: `${msg.content.executionId}_${msg.content.index}`,
                    assigned_to: msg.content.handler.id
                }
            });

    }


    on_wait = async (activity) => {
        console.log(`##### log state immediately in wait ${activity.id}  ${activity.executionId}`);

        if (activity.type === 'bpmn:ManualTask') {
            activity.signal();
            return;
        }
        // 非用户任务，不作处理
        if (activity.type !== 'bpmn:UserTask') return;
        let handler: IHandler | undefined;
        if (activity.content?.handlers) {
            if (activity.content.isMultiInstance === true) {
                handler = activity.content.handlers[activity.content.index ? activity.content.index : 0];

            } else {
                handler = activity.content.handlers[0];
            }
        }

        if (!handler) {
            return;
        }

        let process_type = 0;
        let show_in_my_works = false;
        let target_type;
        let params = {};
        if (activity.content.input) {
            process_type = activity.content.input.process_type ? activity.content.input.process_type : 0;
            show_in_my_works = activity.content.input.show_in_my_works ? activity.content.input.show_in_my_works : false;
            target_type = activity.content.input.target_type;
            // delete activity.content.input.process_type;
            // delete activity.content.input.show_in_my_works;
            // delete activity.content.input.target_type;
            let filter = ['process_type', 'show_in_my_works', 'target_type']
            params = Object.keys(activity.content.input).reduce((obj, key) => {
                if (!filter.includes(key)) {
                    obj[key] = activity.content.input[key];
                }
                return obj;
            }, {});
        }
        await BpmnWork.findOrCreate({
            where: {id: activity.executionId},
            defaults: {
                id: activity.executionId,
                execution_id: activity.content.isMultiInstance ? activity.content.parent.executionId : activity.executionId,
                name: handler.action ? handler.action : activity.name,
                activity_id: activity.id,
                assigned_to: handler.id,
                prj_id: this._prj_id,
                task_id: this._task_id ? this._task_id : null,
                started_at: dayjs(),
                completed_at: null,
                case_id: this._id,
                process_type: process_type,
                show_in_my_works: show_in_my_works,
                target_type: target_type,
                params: params,
                form: activity.content.form ? activity.content.form : null
            }
        });
        FlowEngine.execution_engine_map.set(activity.executionId, this);
        await BpmnCase.update({state: await this.state()}, {where: {id: this._id}, returning: false});
    }

    on_enter = async (activity) => {
        console.log(`##### log state immediately in enter ${activity.id} ${activity.executionId}`);
    }

    on_end = async (activity) => {
        if (activity.type !== 'bpmn:UserTask') return;
        console.log(`##### log state immediately in end ${activity.id} ${activity.executionId}`);

        if (activity.content.output) {
            activity.environment.output[activity.id] = activity.content.output;

            if (activity.content.isMultiInstance === true) {
                for (let key in activity.content.output[activity.content.index]) {
                    if (activity.content.output[activity.content.index].hasOwnProperty(key)) { // 确保属性是对象自身的而不是从原型链继承的
                        if (key === 'id' || key === 'executionId' || key === 'user_id') continue;
                        activity.environment.variables[key] = activity.content.output[activity.content.index][key];
                    }
                }
            } else {
                for (let key in activity.content.output) {
                    if (activity.content.output.hasOwnProperty(key)) { // 确保属性是对象自身的而不是从原型链继承的
                        if (key === 'id' || key === 'executionId' || key === 'user_id') continue;
                        activity.environment.variables[key] = activity.content.output[key];
                    }
                }
            }
        }

        let id;
        if (activity.content.isMultiInstance) {
            id = `${activity.executionId}_${activity.content.index}`;
            await BpmnWork.update({
                completed_at: dayjs(),
                handler: activity.content.output[activity.content.index].user_id,
                status: 2
            }, {where: {id: id}, returning: false});
        } else {
            id = activity.executionId;

            await BpmnWork.update({
                completed_at: dayjs(),
                handler: activity.content.output.user_id,
                status: 2
            }, {
                where: {id: id},
                returning: false
            });

        }
        FlowEngine.execution_engine_map.delete(id);
    }

    on_leave = async (activity) => {
        if (activity.type !== 'bpmn:UserTask') return;
        console.log(`##### log state immediately in leave ${activity.id} ${activity.executionId}`);
        await BpmnWork.update({completed_at: dayjs(), status: 2}, {
            where: {
                execution_id: activity.executionId
            },
            returning: false,
        });
    }

    on_flow_take = async (flow) => {
        console.log(`##### log state immediately in flow_take ${flow.id}`);

        if (flow.owner.behaviour.extensionElements && flow.owner.behaviour.extensionElements.values) {
            for (const extn of flow.owner.behaviour.extensionElements.values) {

                if (extn.$type === 'camunda:Properties' && extn.values) {
                    for (let value of extn.values) {
                        console.log(value);
                        switch (value.name) {
                            case 'prj_phase':
                                await PrjInfo.update({phase_id: value.value}, {
                                    where: {id: this._prj_id},
                                    returning: false
                                })
                                break;
                            case 'prj_phase_name':
                                await PrjLogger.status_changed({
                                    creator_id: 'system',
                                    creator_name: '系统',
                                    content: value.value,
                                    prj_id: this._prj_id
                                })
                                break;
                            case 'task_status':
                                await PrjPlanTask.update({status: value.value}, {
                                    where: {id: this._task_id}, returning: false
                                });
                                break;
                            case 'task_status_name':
                                await PrjLogger.status_changed({
                                    creator_id: 'system',
                                    creator_name: '系统',
                                    content: value.value,
                                    prj_id: this._prj_id,
                                    task_id: this._task_id
                                })
                                break;
                        }
                    }

                }
            }
        }
    }

    async signal(id: string, execution_id: string, user_id?: string, msg?: any) {
        const execution = this.execution;
        execution.signal({
            id: id,
            executionId: execution_id,
            user_id: user_id,
            ...msg
        })
    }

    /**
     * 获取任务处理人
     * @param handler_tag 处理人标记
     *      owner: 任务所有者
     *      checker: 审核组
     *      project_leader: 项目负责人
     *      project_team: 项目组成员
     *      all: 整个公司员工
     *
     * @private
     */
    private async get_handlers(handler_tag: string): Promise<Array<IHandler>> {
        let handlers: Array<IHandler> = new Array<IHandler>();
        let id: string;
        let name: string;
        if (!handler_tag) handler_tag = 'owner';
        switch (handler_tag) {
            case 'owner':   // 任务所有者
                id = this._owner;
                let user = await AcsUserInfo.findOne({where: {id: id}, raw: true});
                if (user) {
                    name = user.name;
                    handlers.push({id, name});
                }
                break;
            case 'project_leader': // 项目负责人
                let prj = await PrjInfo.findOne({
                    where: {id: this._prj_id},
                    include: {model: AcsUserInfo, as: 'PrjLeaderUserInfo'},
                    raw: true
                });
                if (prj) {
                    id = prj.leader_id;
                    name = prj['PrjLeaderUserInfo.name'];
                    handlers.push({id, name});
                }
                break;
            case 'project_team': // 项目组成员
                let team = await PrjMembers.findAll({
                    where: {prj_id: this._prj_id},
                    include: {model: AcsUserInfo, as: 'PrjMemberUserInfo'},
                    raw: true
                });
                for (let i = 0; team && i < team.length; i++) {
                    id = team[i].member_id;
                    name = team[i]['PrjMemberUserInfo.name'];
                    handlers.push({id, name});
                }
                break;
            case 'project_checker': // 项目审核组，处理项目立项审核、计划审核和计划变更审核
                let prj_info = await PrjInfo.findOne({
                    where: {id: this._prj_id},
                    raw: true
                });
                if (!prj_info) break;
                for (let i = 0; i < prj_info.checkers.length; i++) {
                    let checker = prj_info.checkers[i];
                    let user = await AcsUserInfo.findOne({where: {id: checker.id}, raw: true});
                    handlers.push({
                        id: checker.id,
                        name: user ? user.name : checker.id,
                        due_day: checker.due_day
                    });

                }
                break;
            case 'task_checker': // 项目任务审核组，处理任务成果的互校、二审和终审
                let task = await PrjPlanTask.findOne({
                    where: {id: this._args.task_id},
                });
                if (!task) break;
                // let outcome_checkers = outcome.
                for (let i = 0; i < task.checkers.length; i++) {
                    let checker: IChecker = task.checkers[i];
                    for (let j = 0; j < checker.handlers.length; j++) {
                        let handler = checker.handlers[j];
                        let user = await AcsUserInfo.findOne({where: {id: handler}, raw: true});
                        handlers.push({
                            id: handler,
                            name: user ? user.name : handler,
                            action: checker.name,
                            due_day: checker.due_day
                        })
                    }
                }
                break;
            case 'all':
                let all = await AcsUserRole.sequelize!.query<AcsUserInfo>(`
                    select 
                        staff.id, staff.name
                    from ${AcsUserInfo.table_name} staff,  ${AcsDomain.table_name} domain, ${AcsUserDomain.table_name} user_domain
                    where  staff.id = user_domain.user_id and 
                            domain.id = user_domain.domain_id and domain.path <@ '1.3.1000' 
                    order by COALESCE(staff.ext_info->'level', '0')::integer
                `, {type: QueryTypes.SELECT, raw: true});
                for (let i = 0; all && i < all.length; i++) {
                    id = all[i].id;
                    name = all[i].name;
                    handlers.push({id, name});
                }
                break;
            default:///其它标记，均通过角色获得
                let users = await AcsUserInfo.sequelize!.query<AcsUserInfo>(`
                    select u.id, u.name from
                        ${AcsUserInfo.table_name} u,
                        ${AcsUserRole.table_name} user_role
                    where
                        u.id = user_role.user_id and 
                        user_role.role_id = :role_id
                    order by COALESCE(u.ext_info->'level', '0')::integer
                `, {type: QueryTypes.SELECT, raw: true, replacements: {role_id: handler_tag}});
                for (let i = 0; users && i < users.length; i++) {
                    id = users[i].id;
                    name = users[i].name;
                    handlers.push({id, name});
                }
                break;

        }
        // if (handlers.length === 0) handlers.push({
        //     id: 'admin',
        //     name: '系统管理员'
        // })
        return handlers;
    }
}