import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {WhereOptions} from "sequelize";
import DataCURD from "@core/DataCURD";
import {BizCustomer} from "@core-models/BizCustomer";
import {IdGen} from "@util/IdGen";
import dayjs from "dayjs";

interface IData {
    /**
     * 客户实际经营地址
     */
    address?: string;
    /**
     * 客户开户行账号（开票资料）
     */
    bank_account?: string;
    /**
     * 客户开户行（开票资料）
     */
    bank_name?: string;
    /**
     * 商务负责人账号id
     */
    bizman_id?: string;
    /**
     * 所属行业id
     */
    industry_id: string;
    /**
     * 地址（开票资料）
     */
    invoice_addr?: string;
    /**
     * 电话（开票资料）
     */
    invoice_phone?: string;
    /**
     * 客户发票抬头（开票资料）
     */
    invoice_title?: string;
    /**
     * 客户级别id
     */
    level_id: string;
    /**
     * 客户联系人姓名
     */
    link_man?: string;
    /**
     * 客户联系人电话
     */
    link_phone?: string;
    /**
     * 备注
     */
    memo?: string;
    /**
     * 客户名称
     */
    name: string;
    /**
     * 所属地区id
     */
    region_id: number;
    /**
     * 客户税号（开票资料）
     */
    tax_no?: string;
    [property: string]: any;
}


function add(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let id = IdGen.id();
            let value: any = {
                id: id,
                created_at: dayjs().format('YYYY-MM-DD HH:mm:ss'),
                ...data
            }
            value = DataCURD.filter_null(value);
            await BizCustomer.create(value);
            resolve({id: id});
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                        "title": "客户名称"
                    },
                    "industry_id": {
                        "type": "string",
                        "title": "所属行业id"
                    },
                    "region_id": {
                        "type": "integer",
                        "title": "所属地区id"
                    },
                    "level_id": {
                        "type": "string",
                        "title": "客户级别id"
                    },
                    "link_man": {
                        "type": "string",
                        "title": "客户联系人姓名"
                    },
                    "link_phone": {
                        "type": "string",
                        "title": "客户联系人电话"
                    },
                    "address": {
                        "type": "string",
                        "title": "客户实际经营地址"
                    },
                    "invoice_title": {
                        "type": "string",
                        "title": "客户发票抬头（开票资料）"
                    },
                    "tax_no": {
                        "type": "string",
                        "title": "客户税号（开票资料）"
                    },
                    "bank_name": {
                        "type": "string",
                        "title": "客户开户行（开票资料）"
                    },
                    "bank_account": {
                        "type": "string",
                        "title": "客户开户行账号（开票资料）"
                    },
                    "invoice_addr": {
                        "type": "string",
                        "title": "地址（开票资料）"
                    },
                    "invoice_phone": {
                        "type": "string",
                        "title": "电话（开票资料）"
                    },
                    "memo": {
                        "type": "string",
                        "title": "备注"
                    },
                    "bizman_id": {
                        "type": "string",
                        "title": "商务负责人账号id"
                    }
                },
                "required": [
                    "name",
                    "region_id",
                    "industry_id",
                    "level_id"
                ],
                "title": "请求参数内容"
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "examples": [
                    "1.0"
                ],
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。",
                "default": "f2741721-1c07-430e-9f01-5529692340f9"
            },
            "timestamp": {
                "type": "integer",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理",
                "title": "UTC时间戳"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: add,
    method_params: {}
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
