import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Op, QueryTypes, WhereOptions} from "sequelize";
import DataCURD, {ISQLReplacements} from "@core/DataCURD";
import {BizCustomer} from "@core-models/BizCustomer";
import {BizCustomerLevel} from "@core-models/BizCustomerLevel";
import {BizCustomerIndustry} from "@core-models/BizCustomerIndustry";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {AcsDomain} from "@core-models/AcsDomain";

interface IData {
    /**
     * 所属行业id
     */
    industry_id?: string;
    /**
     * 客户级别id
     */
    level_id?: string;
    /**
     * 客户名称，用于模糊查询
     */
    name?: string;
    /**
     * 页码，从1开始
     */
    page_no: number;
    /**
     * 每页条目数
     */
    page_size: number;
    /**
     * 所属地区id
     */
    region_id?: number;

    /**
     * 市场负责人id
     */
    bizman_id?: string;
    [property: string]: any;
}


function get_list(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let count_sql = `select count(distinct c.id) `;
            let select_sql = `
                select c.id, c.name, c.region_id, r.name as region_name, 
                    c.level_id, l.name as level_name, c.industry_id, i.name as industry_name,
                    c.link_man, c.link_phone, c.bizman_id, u.name as bizman_name,
                    TO_CHAR(c.created_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as created_at,
                    TO_CHAR(c.updated_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as updated_at
            `;
            let condition = `
                from ${BizCustomer.table_name} c
                left join ${BizCustomerLevel.table_name} l on l.id = c.level_id
                left join ${BizCustomerIndustry.table_name} i on i.id = c.industry_id
                left join ${AcsUserInfo.table_name} u on u.id = c.bizman_id
                left join ${AcsDomain.table_name} r on r.id = c.region_id
                where 1=1 
            `;
            let replacements: ISQLReplacements = {};
            if (data.name !== undefined) {
                condition += ` and c.name like :keyword `;
                replacements.keyword = `%${data.name}%`;
            }
            if (data.level_id !== undefined) {
                condition += ` and c.level_id = :level_id `;
                replacements.level_id = data.level_id;
            }
            if (data.industry_id !== undefined) {
                condition += ` and c.industry_id = :industry_id `;
                replacements.industry_id = data.industry_id;
            }
            if (data.region_id !== undefined) {
                condition += `
                    and c.id in (
                        select customer.id as id from ${AcsDomain.table_name} domain, ${BizCustomer.table_name} customer
                        where domain.id = customer.region_id and  
                            domain.path <@(select path from ${AcsDomain.table_name} where id = :region_id)
                    )
                `;
                replacements.region_id = data.region_id;
            }
            if (data.bizman_id !== undefined) {
                condition += ` and c.bizman_id = :bizman_id `;
                replacements.bizman_id = data.bizman_id;
            }

            count_sql += condition;
            select_sql += condition;
            select_sql += ' order by c.created_at desc, c.name ';
            let result = await DataCURD.get_page_list({
                sequelize: BizCustomer.sequelize!,
                page_no: data.page_no,
                page_size: data.page_size,
                count_sql: count_sql,
                count_replacements: replacements,
                select_sql: select_sql,
                select_replacements: replacements
            });
            let list = DataCURD.filter_null(result);
            resolve(list);
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                        "title": "客户名称",
                        "description": "用于模糊查询"
                    },
                    "industry_id": {
                        "type": "string",
                        "title": "所属行业id"
                    },
                    "region_id": {
                        "type": "integer",
                        "title": "所属地区id"
                    },
                    "level_id": {
                        "type": "string",
                        "title": "客户级别id"
                    },
                    "page_no": {
                        "type": "integer",
                        "minimum": 1,
                        "title": "页码",
                        "description": "从1开始"
                    },
                    "page_size": {
                        "type": "integer",
                        "minimum": 1,
                        "title": "每页条目数"
                    }
                },
                "required": [
                    "page_no",
                    "page_size"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理",
                "title": "UTC时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]

    },
    method: get_list,
    method_params: {}
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
