import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {AcsUserDomain} from "@core-models/AcsUserDomain";
import {AcsUserRole} from "@core-models/AcsUserRole";

import {Resp} from "@util/Resp";

interface IData {
    /**
     * 部门id，为空时直接放在公司一级下
     */
    domain_id?: number;
    /**
     * 邮箱
     */
    email?: string;
    /**
     * 是否可访问所有项目，默认为false，false时，只能访问身为项目组成员的项目，否则可以访问公司内部的所有项目
     */
    full_range?: boolean;
    /**
     * 级别，默认0，越大级别越高
     */
    level?: number;
    /**
     * 手机号，必填，会与微信手机绑定
     */
    mobile: string;
    /**
     * 员工名称
     */
    name: string;
    /**
     * 初始密码，Md5 32位加密
     */
    pass: string;
    /**
     * 角色id
     */
    role_id?: string;
}



function add(json: IRequest, _params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<unknown>(async (resolve, reject) => {
        let data = <IData>json.data;
        if (data.domain_id === undefined) data.domain_id = 1000;

        let t = await AcsUserInfo.sequelize!.transaction();
        try {

            let userInfo = await AcsUserInfo.findOne({where: {mobile: data.mobile}});
            if (userInfo) {
                await t.rollback();
                return reject(Resp.gen_err(Resp.DataExists, `手机号为${data.mobile}的账号已存在，不允许重复。`))
            }
            if (data.full_range === undefined) data.full_range = false;
            await AcsUserInfo.create({
                id: data.mobile,
                name: data.name,
                pass: data.pass,
                mobile: data.mobile,
                email: data.email,
                reg_time: new Date(),
                act_time: new Date(),
                status: 1,
                ext_info: {full_range: data.full_range, level: data.level}
            }, {transaction: t});

            await AcsUserDomain.create({
                user_id: data.mobile,
                domain_id: data.domain_id,
            }, {transaction: t});
            await AcsUserDomain.create({
                user_id: data.mobile,
                domain_id: 100,
            }, {transaction: t});
            await AcsUserDomain.create({
                user_id: data.mobile,
                domain_id: 4,
            }, {transaction: t});

            await AcsUserRole.upsert({
                user_id: data.mobile,
                role_id: data.role_id
            }, {transaction: t});

            await t.commit();
            resolve({});
        } catch (e) {
            await t.rollback();
            reject(e);
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                        "title": "员工名称"
                    },
                    "domain_id": {
                        "type": "integer",
                        "title": "部门id",
                        "description": "为空时直接放在公司一级下"
                    },
                    "mobile": {
                        "type": "string",
                        "title": "手机号",
                        "description": "必填，会与微信手机绑定",
                        "pattern": "^$|^(13[0-9]|14[01456879]|15[0-3,5-9]|16[2567]|17[0-8]|18[0-9]|19[0-3,5-9])\\d{8}$"
                    },
                    "email": {
                        "type": "string",
                        "title": "邮箱",
                        "pattern": "^$|^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\\.[a-zA-Z0-9_-]+)+$"
                    },
                    "pass": {
                        "type": "string",
                        "title": "初始密码",
                        "description": "Md5 32位加密"
                    },
                    "full_range": {
                        "type": "boolean",
                        "title": "是否可访问所有项目",
                        "description": "默认为false，false时，只能访问身为项目组成员的项目，否则可以访问公司内部的所有项目",
                        "default": "false"
                    },
                    "level": {
                        "type": "integer",
                        "title": "级别",
                        "description": "默认0，越大级别越高",
                        "minimum": 0
                    },
                    "role_id": {
                        "type": "string",
                        "title": "角色id"
                    }
                },
                "x-apifox-orders": [
                    "name",
                    "domain_id",
                    "mobile",
                    "email",
                    "pass",
                    "full_range",
                    "level",
                    "role_id"
                ],
                "required": [
                    "name",
                    "mobile",
                    "pass",
                    "role_id"
                ],
                "title": "请求参数内容"
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "examples": [
                    "1.0"
                ],
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。",
                "default": "f2741721-1c07-430e-9f01-5529692340f9"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理",
                "default": null
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token",
                "default": "{{token}}"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: add,
    method_params: {},
    guards: []
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
