/**
 * 获取员工列表
 */
import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {AcsDomain} from "@core-models/AcsDomain";
import {AcsUserDomain} from "@core-models/AcsUserDomain";
import DataCURD, {ISQLReplacements} from "@core/DataCURD";
import {BizCustomer} from "@core-models/BizCustomer";
import {Resp} from "@util/Resp";
import {domainGuard} from "@src/guards/domainGuard";
import {AcsUserRole} from "@core-models/AcsUserRole";
import {AcsRole} from "@core-models/AcsRole";

interface IData {
    /**
     * 员工手机号，用于模糊查询
     */
    mobile?: string;
    /**
     * 员工姓名，用于模糊查询
     */
    name?: string;
    /**
     * 页码，从1开始
     */
    page_no: number;
    /**
     * 每页条目数
     */
    page_size: number;
    /**
     * 部门id，如果没有指定，表示获取钱公司员工
     */
    domain_id?: number;
}


export function get(json: IRequest, params: IMethodParams, cachedData: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            let data: IData = <IData>json.data;
            if (data.domain_id !== undefined)
                await domainGuard(json, cachedData);
            let count_sql = `
                select count(distinct staff.id) 
                from ${AcsUserInfo.table_name} staff, ${AcsUserDomain.table_name} user_domain, ${AcsDomain.table_name} domain
                where staff.id = user_domain.user_id and user_domain.domain_id = domain.id and domain.path <@ '1.3.1000'
                `;
            let select_sql = `
               select distinct staff.id,
                   staff.name,
                   staff.mobile,
                   staff.email,
                   staff.status,
                   COALESCE(staff.ext_info -> 'full_range', 'false')::boolean               as full_range,
                   COALESCE(staff.ext_info -> 'level', '0')::integer                        as level,
                   TO_CHAR(staff.reg_time, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as reg_time,
                   TO_CHAR(staff.act_time, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as act_time,
                   array_agg(distinct domain.id) as domain_id,
                   substring(array_agg(distinct domain.name)::text from 2 for char_length(array_agg(distinct domain.name)::text)-2) as domain_name,
                   array_agg(distinct role_id) as role_id,
                   substring(array_agg(distinct role.name)::text from 2 for char_length(array_agg(distinct role.name)::text)-2) as role_name
                from ${AcsUserInfo.table_name} staff
                left join tb_acs_user_domain user_domain on staff.id = user_domain.user_id
                left join tb_acs_domain domain on user_domain.domain_id = domain.id
                left join tb_acs_user_role user_role on staff.id = user_role.user_id
                left join tb_acs_role role on user_role.role_id = role.id
                where domain.path <@ '1.3.1000'
            `;
            let conditions = ` `;
            let replacements: ISQLReplacements = {};
            // if (data.domain_id === undefined) {
            //     conditions += ` and domain.path <@ array (
            //                 select path from ${AcsDomain.table_name} domain, ${AcsUserDomain.table_name} user_domain
            //                 where user_domain.user_id = :user_id and user_domain.domain_id = domain.id
            //               )`;
            //     replacements.user_id = cachedData.user_id;
            // } else
            if (data.domain_id !== undefined)
            {
                conditions += ` and domain.path <@ (select path from ${AcsDomain.table_name} domain where domain.id = :domain_id) `;
                replacements.domain_id = data.domain_id;
            }


            if (data.name && data.name.trim() !== '') {
                conditions += ` and staff.name like :name`;
                replacements.name = `%${data.name}%`;
            }

            if (data.mobile && data.mobile.trim() !== '') {
                conditions += ` and staff.mobile like :mobile`;
                replacements.name = `%${data.mobile}%`;
            }

            count_sql += conditions;
            select_sql += conditions;
            select_sql += `
                group by staff.id,
                   staff.name,
                   staff.mobile,
                   staff.email,
                   staff.status,
                   full_range,
                   level,
                   reg_time,
                   act_time
            `
            select_sql += ` order by name desc `;
            let result = await DataCURD.get_page_list({
                sequelize: BizCustomer.sequelize!,
                page_no: data.page_no,
                page_size: data.page_size,
                count_sql: count_sql,
                count_replacements: replacements,
                select_sql: select_sql,
                select_replacements: replacements
            });
            let list = DataCURD.filter_null(result);
            resolve(list);
        } catch (e) {
            reject(Resp.gen_db_err(e));
        }
    })
}


const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                        "title": "员工姓名",
                        "description": "用于模糊查询"
                    },
                    "page_no": {
                        "type": "integer",
                        "minimum": 1,
                        "title": "页码",
                        "description": "从1开始"
                    },
                    "page_size": {
                        "type": "integer",
                        "minimum": 1,
                        "title": "每页条目数"
                    },
                    "mobile": {
                        "type": "string",
                        "title": "员工手机号",
                        "description": "用于模糊查询"
                    },
                    "org_id": {
                        "type": "integer",
                        "title": "部门id",
                        "description": "如果没有指定，表示获取钱公司员工"
                    }
                },
                "x-apifox-orders": [
                    "page_no",
                    "page_size",
                    "name",
                    "mobile",
                    "org_id"
                ],
                "required": [
                    "page_no",
                    "page_size"
                ],
                "title": "请求参数内容"
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "examples": [
                    "1.0"
                ]
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理",
                "title": "UTC时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "sign",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get,
    method_params: {},
    guards: []

}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
