import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {WhereOptions} from "sequelize";
import DataCURD from "@core/DataCURD";
import {IdGen} from "@util/IdGen";
import dayjs from "dayjs";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjMembers} from "@core-models/PrjMembers";
import {BizContractInfo} from "@core-models/BizContractInfo";

interface IData {
    /**
     * 合同总金额（万元）
     */
    amount: number;
    /**
     * 合同起始日
     */
    begin_at: Date;
    /**
     * 合同摘要
     */
    comment?: string;
    /**
     * 客户id
     */
    customer_id: string;
    /**
     * 合同终止日
     */
    end_at?: Date;
    /**
     * 合同签订人id
     */
    handler_id: string;
    /**
     * 合同名称
     */
    name: string;
    /**
     * 收款约定
     */
    payments?: Payment[];
    /**
     * 项目id，合同允许不一定需要与项目绑定，可以在专门的合同管理模块中管理
     */
    prj_id?: string;
    /**
     * 合同签订日期（YYYY-MM-DD）
     */
    signed_at: Date;
    [property: string]: any;
}

export interface Payment {
    /**
     * 收款条件
     */
    condition: string;
    /**
     * 预计收款日
     */
    expect_pay_date: string;
    /**
     * 备注
     */
    memo?: string;
    /**
     * 收款比例
     */
    percent: number;
    [property: string]: any;
}



function add(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        let t = await BizContractInfo.sequelize!.transaction();
        try {
            let data = <IData>json.data;
            let id = IdGen.id();
            let value: any = {
                id: id,
                creator_id: cached_data.user_id,
                created_at: dayjs().format('YYYY-MM-DD HH:mm:ss'),
                ...data
            }
            value = DataCURD.filter_null(value);
            await BizContractInfo.create(value, {transaction: t});
            if (data.prj_id) {
                await PrjInfo.update({contract_id: id}, {where: {id: data.prj_id}, transaction: t, returning: false})
            }
            await t.commit();
            resolve({id: id});
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                        "title": "合同名称"
                    },
                    "prj_id": {
                        "type": "string",
                        "title": "项目id",
                        "description": "合同允许不一定需要与项目绑定，可以在专门的合同管理模块中管理"
                    },
                    "handler_id": {
                        "type": "string",
                        "title": "合同签订人id"
                    },
                    "signed_at": {
                        "type": "string",
                        "title": "合同签订日期（YYYY-MM-DD）",
                        "format": "date"
                    },
                    "amount": {
                        "type": "number",
                        "title": "合同总金额（万元）"
                    },
                    "customer_id": {
                        "type": "string",
                        "title": "客户id"
                    },
                    "begin_at": {
                        "type": "string",
                        "format": "date",
                        "title": "合同起始日"
                    },
                    "end_at": {
                        "type": "string",
                        "format": "date",
                        "title": "合同终止日"
                    },
                    "comment": {
                        "type": "string",
                        "title": "合同摘要"
                    },
                    "payments": {
                        "type": "array",
                        "items": {
                            "type": "object",
                            "properties": {
                                "condition": {
                                    "type": "string",
                                    "title": "收款条件"
                                },
                                "percent": {
                                    "type": "number",
                                    "title": "收款比例"
                                },
                                "expect_pay_date": {
                                    "type": "string",
                                    "title": "预计收款日"
                                },
                                "memo": {
                                    "type": "string",
                                    "title": "备注"
                                }
                            },
                            "x-apifox-orders": [
                                "condition",
                                "percent",
                                "expect_pay_date",
                                "memo"
                            ],
                            "required": [
                                "condition",
                                "percent",
                                "expect_pay_date"
                            ]
                        },
                        "title": "收款约定"
                    }
                },
                "x-apifox-orders": [
                    "prj_id",
                    "name",
                    "handler_id",
                    "signed_at",
                    "amount",
                    "customer_id",
                    "begin_at",
                    "end_at",
                    "comment",
                    "payments"
                ],
                "title": "请求参数内容",
                "required": [
                    "name",
                    "handler_id",
                    "signed_at",
                    "amount",
                    "customer_id",
                    "begin_at"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: add,
    method_params: {}
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
