import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Op, QueryTypes, WhereOptions} from "sequelize";
import DataCURD, {ISQLReplacements} from "@core/DataCURD";
import {BizCustomer} from "@core-models/BizCustomer";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {BizContractInfo} from "@core-models/BizContractInfo";
import {PrjInfo} from "@core-models/PrjInfo";


interface IData {
    /**
     * 合同签订时间（查询起始），YYYY-MM-DD HH:NN:SS
     */
    begin_from?: Date;
    /**
     * 根据客户名称过滤（模糊匹配）
     */
    customer_name?: string;
    /**
     * 合同签订时间（查询至），YYYY-MM-DD HH:NN:SS
     */
    end_to?: Date;
    /**
     * 根据合同签订人过滤
     */
    handler_id?: string;
    /**
     * 页码
     */
    page_no: number;
    /**
     * 每页条数
     */
    page_size: number;
    /**
     * 精确通过项目查找合同
     */
    prj_id?: string;
    /**
     * 根据项目名称过滤（模糊匹配）
     */
    prj_name?: string;
    /**
     * 合同名称模糊查询
     */
    name?: string;
}

function get_list(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let count_sql = `select count(distinct contract.id) `;
            let select_sql = `
                select contract.id, contract.name, 
                    contract.customer_id, customer.name as customer_name,
                    contract.prj_id, prj.name as prj_name,
                    prj.leader_id, leader.name as leader_name,
                    cast(contract.amount as float), contract.handler_id, handler.name as handler_name,
                   
                    TO_CHAR(contract.created_at, 'yyyy-MM-dd') as created_at,
                    TO_CHAR(contract.signed_at, 'yyyy-MM-dd') as signed_at,
                    TO_CHAR(contract.begin_at, 'yyyy-MM-dd') as begin_at,
                    TO_CHAR(contract.end_at, 'yyyy-MM-dd') as end_at
            `;
            let condition = `
                from ${BizContractInfo.table_name} contract
                left join ${PrjInfo.table_name} prj on prj.id = contract.prj_id
                left join ${AcsUserInfo.table_name} handler on handler.id = contract.handler_id
                left join ${BizCustomer.table_name} customer on customer.id = contract.customer_id
                left join ${AcsUserInfo.table_name} leader on leader.id = prj.leader_id
                where 1=1 
            `;
            let replacements: ISQLReplacements = {};

            if (data.name !== undefined) {
                condition += ` and contract.name like :name `;
                replacements.name = `%${data.name}%`;
            }
            if (data.prj_name !== undefined) {
                condition += ` and prj.name like :prj_name `;
                replacements.prj_name = `%${data.prj_name}%`;
            }
            if (data.prj_id !== undefined) {
                condition += ` and prj.id = :prj_id `;
                replacements.prj_id = data.prj_id;
            }
            if (data.customer_name !== undefined) {
                condition += ` and customer.name like :customer_name `;
                replacements.customer_name = `%${data.customer_name}%`;
            }
            if (data.handler_id !== undefined) {
                condition += ` and customer.handler_id = :handler_id `;
                replacements.handler_id = data.handler_id;
            }

            if (data.begin_from !== undefined) {
                condition += ' and contract.signed_at >= :begin_from ';
                replacements.begin_from = data.begin_from + ' 00:00:00';
            }
            if (data.end_to !== undefined) {
                condition += ' and contract.signed_at <= :end_to ';
                replacements.end_to = data.end_to + ' 23:59:59';
            }

            count_sql += condition;
            select_sql += condition;
            select_sql += ' order by contract.created_at desc, contract.name ';

            let result = await DataCURD.get_page_list({
                sequelize: BizContractInfo.sequelize!,
                page_no: data.page_no,
                page_size: data.page_size,
                count_sql: count_sql,
                count_replacements: replacements,
                select_sql: select_sql,
                select_replacements: replacements
            });
            let list = DataCURD.filter_null(result);
            resolve(list);
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "page_no": {
                        "type": "integer",
                        "title": "页码"
                    },
                    "page_size": {
                        "type": "integer",
                        "title": "每页条数"
                    },
                    "name": {
                        "type": "string",
                        "title": "合同名称模糊查询"
                    },
                    "prj_id": {
                        "type": "string",
                        "title": "精确通过项目查找合同"
                    },
                    "prj_name": {
                        "type": "string",
                        "title": "根据项目名称过滤（模糊匹配）"
                    },
                    "customer_name": {
                        "type": "string",
                        "title": "根据客户名称过滤（模糊匹配）"
                    },
                    "handler_id": {
                        "type": "string",
                        "title": "根据合同签订人过滤"
                    },
                    "begin_from": {
                        "type": "string",
                        "format": "date",
                        "title": "合同签订时间（查询起始）",
                        "description": "YYYY-MM-DD HH:NN:SS"
                    },
                    "end_to": {
                        "type": "string",
                        "format": "date",
                        "title": "合同签订时间（查询至）",
                        "description": "YYYY-MM-DD HH:NN:SS"
                    }
                },
                "x-apifox-orders": [
                    "page_no",
                    "page_size",
                    "prj_id",
                    "prj_name",
                    "customer_name",
                    "handler_id",
                    "begin_from",
                    "end_to"
                ],
                "title": "请求参数内容",
                "required": [
                    "page_no",
                    "page_size"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get_list,
    method_params: {}
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
