import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import DataCURD from "@core/DataCURD";
import {WhereOptions} from "sequelize";
import {BizContractInfo} from "@core-models/BizContractInfo";
import dayjs from "dayjs";

interface IData {
    /**
     * 合同总金额（万元）
     */
    amount?: number;
    /**
     * 合同摘要
     */
    comment?: string;
    /**
     * 客户id
     */
    customer_id?: string;
    /**
     * 合同终止日
     */
    end_at?: string;
    /**
     * 合同签订人id
     */
    handler_id?: string;
    /**
     * 合同id
     */
    id: string;
    /**
     * 合同名称
     */
    name?: string;
    /**
     * 收款约定
     */
    payments?: Payment[];
    /**
     * 合同签订日期（YYYY-MM-DD）
     */
    signed_at?: Date;
    /**
     * 合同起始日
     */
    begin_at?: string;
    description?: any;
    plan_time?: any;
}

export interface Payment {
    /**
     * 收款条件
     */
    condition: string;
    /**
     * 预计收款日
     */
    expect_pay_date: string;
    /**
     * 备注
     */
    memo?: string;
    /**
     * 收款比例
     */
    percent: number;
}


async function get_where(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    let data = <IData>json.data;
    return {id: data.id};
}

async function before_respond(content: IRequest, params: IMethodParams, cached_data: ICachedData | any, result: any): Promise<unknown> {
    result = DataCURD.filter_null(result, new Map(Object.entries({
        "updated_at": "YYYY-MM-DD HH:mm:ss",
        "created_at": "YYYY-MM-DD HH:mm:ss",
        "signed_at": "YYYY-MM-DD",
        "begin_at": "YYYY-MM-DD",
        "end_at": "YYYY-MM-DD"
    })));
    return result;
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                        "title": "合同名称"
                    },
                    "id": {
                        "type": "string",
                        "title": "合同id"
                    },
                    "signed_at": {
                        "type": "string",
                        "title": "合同签订日期（YYYY-MM-DD）",
                        "format": "date"
                    },
                    "handler_id": {
                        "type": "string",
                        "title": "合同签订人id"
                    },
                    "amount": {
                        "type": "number",
                        "title": "合同总金额（万元）"
                    },
                    "customer_id": {
                        "type": "string",
                        "title": "客户id"
                    },
                    "begin_at": {
                        "type": "string",
                        "title": "合同起始日"
                    },
                    "end_at": {
                        "type": "string",
                        "title": "合同终止日"
                    },
                    "comment": {
                        "type": "string",
                        "title": "合同摘要"
                    },
                    "payments": {
                        "type": "array",
                        "items": {
                            "type": "object",
                            "properties": {
                                "condition": {
                                    "type": "string",
                                    "title": "收款条件"
                                },
                                "percent": {
                                    "type": "number",
                                    "title": "收款比例"
                                },
                                "expect_pay_date": {
                                    "type": "string",
                                    "title": "预计收款日"
                                },
                                "memo": {
                                    "type": "string",
                                    "title": "备注"
                                }
                            },
                            "x-apifox-orders": [
                                "condition",
                                "percent",
                                "expect_pay_date",
                                "memo"
                            ],
                            "required": [
                                "condition",
                                "percent",
                                "expect_pay_date"
                            ]
                        },
                        "title": "收款约定"
                    }
                },
                "x-apifox-orders": [
                    "id",
                    "name",
                    "signed_at",
                    "handler_id",
                    "amount",
                    "customer_id",
                    "begin_at",
                    "end_at",
                    "comment",
                    "payments"
                ],
                "title": "请求参数内容",
                "required": [
                    "id"
                ],
                "anyOf": [
                    {
                        "required": [
                            "name"
                        ]
                    },
                    {
                        "required": [
                            "handler_id"
                        ]
                    },
                    {
                        "required": [
                            "signed_at"
                        ]
                    },
                    {
                        "required": [
                            "amount"
                        ]
                    },
                    {
                        "required": [
                            "customer_id"
                        ]
                    },
                    {
                        "required": [
                            "begin_at"
                        ]
                    },
                    {
                        "required": [
                            "end_at"
                        ]
                    },
                    {
                        "required": [
                            "comment"
                        ]
                    },
                    {
                        "required": [
                            "payments"
                        ]
                    }
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: DataCURD.internal_update_row,
    on_before_respond: before_respond,
    method_params: {
        model: BizContractInfo,
        where: get_where,
        fixed_input_data: {
            updated_at: dayjs().format('YYYY-MM-DD HH:mm:ss')
        },
        input_data_map: {
            name: "name",
            handler_id: "handler_id",
            signed_at: "signed_at",
            amount: "amount",
            customer_id: "customer_id",
            begin_at: "begin_at",
            end_at: "end_at",
            comment: "comment",
            payments: "payments",
            updated_at: 'updated_at'
        }
    }
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
