import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Oss} from "@util/Oss";
import {BizContractInfo} from "@core-models/BizContractInfo";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjFile} from "@core-models/PrjFile";
import dayjs from "dayjs";

interface IData {
    /**
     * 合同id
     */
    id: string;
    /**
     * 附件类型：10=打印件，20=扫描
     */
    type_id: number;
}

function remove_doc(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise(async (resolve, reject) => {
        let t = await BizContractInfo.sequelize!.transaction();
        try {
            let data = <IData>json.data;
            let contract = await BizContractInfo.findOne({where: {id: data.id}, raw: true, transaction: t});
            if (!contract) throw Resp.gen_err(Resp.ResourceNotFound, `合同（id：${data.id}）不存在。`);
            let uploaded = false;
            let object_name;
            let filename;
            switch (data.type_id) {
                case 10:
                    uploaded = contract.doc_uploaded;
                    object_name = contract.doc_id;
                    filename = contract.doc_name;
                    await BizContractInfo.update({doc_uploaded: false, doc_id: null, doc_name: null}, {
                        where: {id: data.id},
                        transaction: t
                    });
                    break;
                case 20:
                    uploaded = contract.pdf_uploaded;
                    object_name = contract.pdf_id;
                    filename = contract.pdf_name;
                    await BizContractInfo.update({pdf_uploaded: false, pdf_id: null, pdf_name: null}, {
                        where: {id: data.id},
                        transaction: t
                    })
                    break;
            }
            if (uploaded === false) throw Resp.gen_err(Resp.ResourceNotFound, '文档尚未上传。')
            let prj = await PrjInfo.findOne<PrjInfo>({where: {contract_id: data.id}, raw: true, transaction: t});
            if (prj) {
                await PrjFile.destroy({
                    where: {
                        id: object_name
                    },
                    transaction: t
                });
            }

            let oss = Oss.get_instance('pmr-doc');
            await oss.remove_object(oss.bucket, object_name);
            await t.commit();
            resolve({});
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    });
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "id": {
                        "type": "string",
                        "title": "合同id"
                    },
                    "type_id": {
                        "type": "integer",
                        "title": "附件类型：10=打印件，20=扫描"
                    }
                },
                "x-apifox-orders": [
                    "id",
                    "type_id"
                ],
                "title": "请求参数内容",
                "required": [
                    "id",
                    "type_id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: remove_doc,
    method_params: {},
    guards: []
};

module.exports = {
    default: v1_0,
    v1_0: v1_0
};
