import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Oss} from "@util/Oss";
import {BizContractInfo} from "@core-models/BizContractInfo";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjFile} from "@core-models/PrjFile";
import dayjs from "dayjs";


interface IData {
    /**
     * 文件名
     */
    filename: string;
    /**
     * 合同id
     */
    id: string;
    /**
     * 附件类型：10=打印件，20=扫描件
     */
    type_id: number;
}



function get_upload_url(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise(async (resolve, reject) => {
        let t = await BizContractInfo.sequelize!.transaction();
        try {
            let data = <IData>json.data;
            let object_name = `contract/${data.id}/${data.type_id}`;
            switch (data.type_id){
                case 10:
                    await BizContractInfo.update({doc_id: object_name, doc_name: data.filename}, {where: {id: data.id}, transaction: t});
                    break;
                case 20:
                    await BizContractInfo.update({pdf_id: object_name, pdf_name: data.filename}, {where: {id: data.id}, transaction: t});
                    break;
            }

            let prj = await PrjInfo.findOne<PrjInfo>({where: {contract_id: data.id}, raw: true, transaction: t});
            if (prj) {
                await PrjFile.upsert({
                    category_id: 'contract',
                    prj_id: prj.id,
                    req_upload_time: dayjs(),
                    id: object_name,
                    filename: data.filename,
                    dependent_id: data.id,
                    creator_id: cached_data.user_id

                }, {transaction: t});
            }

            let oss = Oss.get_instance('pmr-doc');
            let result = await oss.presignedPostPolicy(oss.bucket, object_name, 300);
            await t.commit();
            resolve({object_name: object_name, url: result.postURL, form_data: result.formData});
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    });
}

/// 检查是否存在
function existsGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let data = <IData>json.data;
        let contract = await BizContractInfo.findOne({where: {id: data.id}, raw: true});
        if (!contract) return reject(Resp.gen_err(Resp.ResourceNotFound, "合同不存在，id: "+ data.id));

        resolve();
    });
}


const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "id": {
                        "type": "string",
                        "title": "合同id"
                    },
                    "filename": {
                        "type": "string",
                        "title": "文件名"
                    },
                    "type_id": {
                        "type": "integer",
                        "title": "附件类型：10=打印件，20=扫描件"
                    }
                },
                "x-apifox-orders": [
                    "id",
                    "type_id",
                    "filename"
                ],
                "title": "请求参数内容",
                "required": [
                    "id",
                    "filename",
                    "type_id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get_upload_url,
    method_params: {},
    guards: [existsGuard]
};

module.exports = {
    default: v1_0,
    v1_0: v1_0
};
