import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Op, QueryTypes, WhereOptions} from "sequelize";
import DataCURD, {ISQLReplacements} from "@core/DataCURD";
import {BizCustomer} from "@core-models/BizCustomer";
import {BizCustomerLevel} from "@core-models/BizCustomerLevel";
import {BizCustomerIndustry} from "@core-models/BizCustomerIndustry";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {AcsDomain} from "@core-models/AcsDomain";
import {PrjInfo} from "@core-models/PrjInfo";
import {BizContractInfo} from "@core-models/BizContractInfo";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {PrjFile} from "@core-models/PrjFile";
import {PrjFileCategory} from "@core-models/PrjFileCategory";

interface IData {
    /**
     * 查询起始，按上传时间查询，YYYY-MM-DD
     */
    begin_from?: Date;
    /**
     * 通过文档类型查找
     */
    category_id?: string;
    /**
     * 根据创建人账号查询
     */
    creator_id?: string;
    /**
     * 根据创建人名称模糊查询
     */
    creator_name?: string;
    /**
     * 查询至，按上传时间查询，YYYY-MM-DD
     */
    end_to?: Date;
    /**
     * 根据文档名模糊查询
     */
    filename?: string;
    /**
     * 页码
     */
    page_no: number;
    /**
     * 每页条数
     */
    page_size: number;
    /**
     * 精确通过项目编号查找文档
     */
    prj_id?: string;
}


function get_list(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let count_sql = `select count(distinct prj_file.id) `;
            let select_sql = `
                select prj_file.id, prj_file.filename,
                    SPLIT_PART(prj_file.filename, '.', -1) AS extension,

                    prj_file.category_id, category.name as category_name,
                    prj_file.prj_id, prj.name as prj_name,
                    prj_file.creator_id, creator.name as creator_name,
                    prj_file.size,
                    prj_file.uploaded,
                    TO_CHAR(prj_file.uploaded_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as uploaded_at
            `;
            let condition = `
                from ${PrjFile.table_name} prj_file
                left join ${PrjFileCategory.table_name} category on prj_file.category_id = category.id
                left join ${PrjInfo.table_name} prj on prj_file.prj_id = prj.id
                left join ${AcsUserInfo.table_name} creator on creator.id = prj_file.creator_id
                where 1=1 
            `;
            let replacements: ISQLReplacements = {};
            if (data.filename !== undefined) {
                condition += ` and prj_file.filename like :keyword `;
                replacements.keyword = `%${data.filename}%`;
            }
            if (data.prj_id !== undefined) {
                condition += ` and prj_file.prj_id = :prj_id `;
                replacements.prj_id = data.prj_id;
            }
            if (data.category_id !== undefined) {
                condition += ` and prj_file.category_id = :category_id `;
                replacements.category_id = data.category_id;
            }
            if (data.creator_id !== undefined) {
                condition += ` and prj_file.creator_id = :creator_id `;
                replacements.creator_id = data.creator_id;
            }
            if (data.creator_name !== undefined) {
                condition += ` and creator.name like :creator_name `;
                replacements.creator_name = `%${data.creator_name}%`;
            }
            if (data.begin_from !== undefined) {
                condition += ' and prj_file.uploaded_at >= :begin_from ';
                replacements.begin_from = data.begin_from + ' 00:00:00';
            }
            if (data.end_to !== undefined) {
                condition += ' and prj_file.uploaded_at <= :end_to ';
                replacements.end_to = data.end_to + ' 23:59:59';
            }

            count_sql += condition;
            select_sql += condition;
            select_sql += ' order by prj_file.filename ';
            let result = await DataCURD.get_page_list({
                sequelize: BizCustomer.sequelize!,
                page_no: data.page_no,
                page_size: data.page_size,
                count_sql: count_sql,
                count_replacements: replacements,
                select_sql: select_sql,
                select_replacements: replacements
            });
            let list = DataCURD.filter_null(result);
            resolve(list);
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "page_no": {
                        "type": "integer",
                        "title": "页码"
                    },
                    "page_size": {
                        "type": "integer",
                        "title": "每页条数"
                    },
                    "prj_id": {
                        "type": "string",
                        "title": "精确通过项目编号查找文档"
                    },
                    "begin_from": {
                        "type": "string",
                        "title": "查询起始",
                        "description": "按上传时间查询，YYYY-MM-DD",
                        "format": "date"
                    },
                    "end_to": {
                        "type": "string",
                        "title": "查询至",
                        "description": "按上传时间查询，YYYY-MM-DD",
                        "format": "date"
                    },
                    "creator_id": {
                        "type": "string",
                        "title": "根据创建人账号查询"
                    },
                    "creator_name": {
                        "type": "string",
                        "title": "根据创建人名称模糊查询 "
                    },
                    "filename": {
                        "type": "string",
                        "title": "根据文档名模糊查询"
                    },
                    "category_id": {
                        "type": "string",
                        "title": "通过文档类型查找"
                    }
                },
                "x-apifox-orders": [
                    "page_no",
                    "page_size",
                    "prj_id",
                    "category_id",
                    "filename",
                    "creator_id",
                    "creator_name",
                    "begin_from",
                    "end_to"
                ],
                "title": "请求参数内容",
                "required": [
                    "page_no",
                    "page_size"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get_list,
    method_params: {}
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
