import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {WhereOptions} from "sequelize";
import DataCURD from "@core/DataCURD";
import {IdGen} from "@util/IdGen";
import dayjs from "dayjs";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjMembers} from "@core-models/PrjMembers";
import {BpmnModel} from "@core-models/BpmnModel";
import {FlowEngine} from "@src/bpmn/flow_engine";
import {BpmnCase} from "@core-models/BpmnCase";
import {bpmn_flow_on_end} from "@src/utils/bpmn_work_helper";
import {PrjLogger} from "@src/utils/prj_logger";

interface IData {
    /**
     * 商务负责人，商务负责人id
     */
    bizman_id: string;
    /**
     * 合同id，合同id
     */
    contract_id?: string;
    /**
     * 客户名称，客户id
     */
    customer_id?: string;
    /**
     * 预计交付时间，预计交付时间（YYYY-MM-DD）
     */
    deliver_at?: string;
    /**
     * 项目简介，项目简介
     */
    intro?: string;
    /**
     * 项目负责人，项目负责人账号id
     */
    leader_id: string;
    /**
     * 项目名称，项目名称
     */
    name: string;
    /**
     * 所在地区，所在地区id
     */
    region_id: number;
    /**
     * 项目模板，通过模板可以自动生成项目任务计划
     */
    template_id?: string;
    /**
     * 项目类型，项目类型id
     */
    type_id: number;
}



function add(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        let t = await PrjInfo.sequelize!.transaction();
        try {
            let data = <IData>json.data;
            let id = IdGen.id();
            let flow_case_id = IdGen.id();
            if (data.deliver_at) {
                data.deliver_at += ' 23:59:59';
            }
            let value: any = {
                id: id,
                phase_id: 'new',
                created_at: dayjs().format('YYYY-MM-DD HH:mm:ss'),
                flow_case_id: flow_case_id,
                ...data
            }
            value = DataCURD.filter_null(value);
            // 创建项目
            await PrjInfo.create(value, {transaction: t});
            // 创建项目leader
            await PrjMembers.create({
                prj_id: id,
                member_id: data.leader_id,
                role_id: 'prj_leader',
                created_at: dayjs().format('YYYY-MM-DD HH:mm:ss'),

            }, {transaction: t});

            // 启动工作流
            let bpmn = await BpmnModel.findOne({where: {id: 'prj'}, transaction: t});
            if (!bpmn) throw Resp.gen_err(Resp.InternalServerError, '项目流程模型数据缺失！');
            let flow = new FlowEngine(bpmn.id, bpmn.content, {prj_id: id, owner: cached_data.user_id}, bpmn_flow_on_end);
            await flow.run();
            await BpmnCase.create({
                id: flow.id,
                model_id: bpmn.id,
                model: bpmn.content,
                started_at: dayjs(),
                creator_id: cached_data.user_id,
                prj_id: id,
                // state: await flow.state(),
            }, {transaction: t});
            await PrjLogger.created({
                creator_id: cached_data.user_id,
                creator_name: cached_data.user_name,
                content: `项目`,
                prj_id: id,
                t: t
            });
            await t.commit();
            resolve({id: id});
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                        "description": "项目名称",
                        "title": "项目名称"
                    },
                    "type_id": {
                        "type": "integer",
                        "title": "项目类型",
                        "description": "项目类型id"
                    },
                    "leader_id": {
                        "type": "string",
                        "title": "项目负责人",
                        "description": "项目负责人账号id"
                    },
                    "region_id": {
                        "type": "integer",
                        "title": "所在地区",
                        "description": "所在地区id"
                    },
                    "customer_id": {
                        "type": "string",
                        "title": "客户名称",
                        "description": "客户id"
                    },
                    "template_id": {
                        "type": "string",
                        "title": "项目模板",
                        "description": "通过模板可以自动生成项目任务计划"
                    },
                    "deliver_at": {
                        "type": "string",
                        "description": "预计交付时间（YYYY-MM-DD）",
                        "title": "预计交付时间"
                    },
                    "bizman_id": {
                        "type": "string",
                        "description": "商务负责人id",
                        "title": "商务负责人"
                    },
                    "intro": {
                        "type": "string",
                        "description": "项目简介",
                        "title": "项目简介"
                    },
                    "contract_id": {
                        "type": "string",
                        "title": "合同id",
                        "description": "合同id"
                    }
                },
                "title": "请求参数内容",
                "required": [
                    "name",
                    "region_id",
                    "leader_id",
                    "bizman_id",
                    "type_id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: add,
    method_params: {}
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
