import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Op, QueryTypes, WhereOptions} from "sequelize";
import DataCURD, {ISQLReplacements} from "@core/DataCURD";
import {BizCustomer} from "@core-models/BizCustomer";
import {BizCustomerLevel} from "@core-models/BizCustomerLevel";
import {BizCustomerIndustry} from "@core-models/BizCustomerIndustry";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {AcsDomain} from "@core-models/AcsDomain";
import {PrjInfo} from "@core-models/PrjInfo";
import {BizContractInfo} from "@core-models/BizContractInfo";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {BpmnWork} from "@core-models/BpmnWork";
import {BpmnCase} from "@core-models/BpmnCase";
import {PrjMembers} from "@core-models/PrjMembers";

interface IData {
    /**
     * 商务负责人id
     */
    bizman_id?: string;
    /**
     * 客户名称，根据客户名称过滤
     */
    customer_name?: string;
    /**
     * 项目交付时间（查询起始），项目交付时间（查询起始）YYYY-MM-DD HH:NN:SS
     */
    deliver_at_from?: string;
    /**
     * 项目交付时间（查询至），项目交付时间（查询至）YYYY-MM-DD HH:NN:SS
     */
    deliver_at_to?: string;
    /**
     * 项目负责人，根据项目负责人账号id过滤
     */
    leader_id?: string;
    /**
     * 项目名称，根据项目名称过滤（模糊匹配）
     */
    name?: string;
    /**
     * 页码，页码，从1开始
     */
    page_no: number;
    /**
     * 每页条数
     */
    page_size: number;
    /**
     * 项目阶段，根据项目阶段过滤
     */
    phase_id?: string;
    /**
     * 所在地区，根据所在地区id过滤
     */
    region_id?: number;
    /**
     * 项目启动时间查询起始，项目启动时间（查询起始）YYYY-MM-DD HH:NN:SS
     */
    start_at_from?: string;
    /**
     * 项目启动时间查询至，项目启动时间（查询至）YYYY-MM-DD HH:NN:SS
     */
    start_at_to?: string;
    /**
     * 项目类型，根据项目子类型id过滤
     */
    type_id?: number;
}

function get_list(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let count_sql = `select count(distinct prj.id) `;
            let select_sql = `
                select prj.id, prj.name, prj.phase_id, phase.name as phase_name, 
                    prj.region_id, region.name as region_name, 
                    prj.type_id, 
                    case when pidtype.id = 300 then 
                        ptype.name
                        else CONCAT_WS('/', pidtype.name, ptype.name)
                    end as type_name,
                    prj.customer_id, customer.name as customer_name, 
                    customer.industry_id, industry.name as industry_name,
                    cast(contract.amount as float) as contract_amount,
                    prj.leader_id, leader.name as leader_name,
                    prj.bizman_id, bizman.name as bizman_name,
                    prj.progress,
                    
                    TO_CHAR(prj.created_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as created_at,
                    TO_CHAR(prj.deliver_at, 'yyyy-MM-dd') as deliver_at,
                    TO_CHAR(prj.updated_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as updated_at,
                    TO_CHAR(prj.started_at, 'yyyy-MM-dd') as started_at,
                    
                    --case when NULLIF(array_agg(work.id), '{NULL}') is null then '{}' else
                    --   array_agg(json_build_object('id', work.id, 'name', work.name, 'form', work.form))
                    --       end as works
                    array_agg(
                        case when work.id is null then null
                            else json_build_object('id', work.id, 'name', work.name,'form', work.form)
                        end
                    ) FILTER (WHERE case when work.id is null then null
                            else json_build_object('id', work.id, 'name', work.name,'form', work.form)
                        end IS NOT NULL)  as works
            `;
            let condition = `
                from ${PrjInfo.table_name} prj
                left join ${AcsDomain.table_name} ptype on prj.type_id = ptype.id
                left join ${AcsDomain.table_name} pidtype on ptype.pid = pidtype.id
                left join ${PrjPhaseDefine.table_name} phase on phase.id = prj.phase_id
                left join ${AcsDomain.table_name} region on region.id = prj.region_id
                left join ${BizCustomer.table_name} customer on customer.id = prj.customer_id
                left join ${BizCustomerIndustry.table_name} industry on industry.id = customer.industry_id
                left join ${BizContractInfo.table_name} contract on prj.contract_id = contract.id
                left join ${AcsUserInfo.table_name} bizman on bizman.id = prj.bizman_id
                left join ${AcsUserInfo.table_name} leader on leader.id = prj.leader_id
                left join ${BpmnCase.table_name} flow on flow.prj_id = prj.id 
                LEFT JOIN ${BpmnWork.table_name} work ON work.prj_id = prj.id and work.case_id = flow.id and
                    work.assigned_to = '${cached_data.user_id}' and 
                    work.status <> 2 and work.process_type = 1 and
                    flow.model_id = 'prj'
                where (
                    -- 项目负责人可以取到自己的项目
                    prj.leader_id = :user_id or
                    
                    -- 项目商务负责人可以取到自己的项目
                    prj.bizman_id = :user_id or
                    
                    -- 项目组成员可以取到自己的项目
                    EXISTS (
                        select 1 from ${PrjMembers.table_name} member where member.member_id = :user_id and member.prj_id = prj.id
                    ) or
                  
                    -- 有全部项目查看权限的人(特权人员)可以取到全部项目
                    EXISTS (
                        select 1 from tb_acs_user
                        where ext_info->>'full_range' = 'true'
                         and id = :user_id
                    ) or
                    
                    -- 项目审核人可以取到相关项目
                    EXISTS (
                        select 1 from tb_prj_info prj_info, jsonb_array_elements(prj.checkers) AS checker_obj
                        where checker_obj->>'id' = :user_id and prj_info.id = prj.id
                    ) or
                    --prj.id in (
                    --   select prj_id from (
                    --    select * from (SELECT distinct jsonb_array_elements_text((value ->> 'handlers')::jsonb) AS handler, prj_id
                    --                   FROM tb_prj_plan_task_draft,
                    --                        jsonb_array_elements(checkers) AS checker_obj) a
                    --    ) b) or
                    

                    -- 任务审核人可以取到相关项目
                    EXISTS (
                        SELECT 1
                        FROM tb_prj_plan_task prj_task,
                             jsonb_array_elements(prj_task.checkers) AS checker_obj
                        WHERE  (checker_obj->>'handlers')::jsonb ? :user_id
                          and prj_task.prj_id = prj.id
                    
                       ) 
                    --prj.id in (select prj_id from (
                    --    select * from (SELECT  distinct jsonb_array_elements_text((value ->> 'handlers')::jsonb) AS handler, prj_id
                    --    FROM tb_prj_plan_task_draft,
                    --          jsonb_array_elements(checkers) AS checker_obj) a where a.handler = :user_id
                    --) b)
                     
                )    

            `;
            let replacements: ISQLReplacements = {user_id: cached_data.user_id};
            if (data.name !== undefined) {
                condition += ` and prj.name like :name `;
                replacements.name = `%${data.name}%`;
            }
            if (data.customer_name !== undefined) {
                condition += ` and customer.name like :customer_name `;
                replacements.name = `%${data.customer_name}%`;
            }
            if (data.type_id !== undefined) {
                condition += ` and prj.type_id = :type_id `;
                replacements.type_id = data.type_id;
            }
            if (data.leader_id !== undefined) {
                condition += ` and prj.leader_id = :leader_id `;
                replacements.leader_id = data.leader_id;
            }
            if (data.region_id !== undefined) {
                condition += `
                    and prj.id in (
                        select prj.id as id from ${AcsDomain.table_name} domain, ${PrjInfo.table_name} prj
                        where domain.id = prj.region_id and  
                            domain.path <@(select path from ${AcsDomain.table_name} where id = :region_id)
                    )
                `;
                replacements.region_id = data.region_id;
            }
            if (data.bizman_id !== undefined) {
                condition += ` and prj.bizman_id = :bizman_id `;
                replacements.bizman_id = data.bizman_id;
            }
            if (data.phase_id !== undefined) {
                condition += ` and prj.phase_id = :phase_id `;
                replacements.phase_id = data.phase_id;
            }
            if (data.start_at_from !== undefined) {
                condition += ' and prj.started_at >= :start_at_from ';
                replacements.start_at_from = data.start_at_from;
            }
            if (data.start_at_to !== undefined) {
                condition += ' and prj.started_at <= :start_at_to ';
                replacements.start_at_to = data.start_at_to;
            }
            if (data.deliver_at_from !== undefined) {
                condition += ' and prj.deliver_at >= :deliver_at_from ';
                replacements.deliver_at_from = data.deliver_at_from;
            }
            if (data.deliver_at_to !== undefined) {
                condition += ' and prj.deliver_at <= :deliver_at_to ';
                replacements.deliver_at_to = data.deliver_at_to;
            }

            count_sql += condition;
            select_sql += condition;
            select_sql += `group by prj.id, phase.name, region.name, pidtype.name, ptype.name,customer.name, 
                customer.industry_id, industry.name, contract_amount, prj.leader_id, leader.name,
                    prj.bizman_id, bizman.name, pidtype.id
                order by prj.created_at desc, prj.name `;
            let result = await DataCURD.get_page_list({
                sequelize: BizCustomer.sequelize!,
                page_no: data.page_no,
                page_size: data.page_size,
                count_sql: count_sql,
                count_replacements: replacements,
                select_sql: select_sql,
                select_replacements: replacements
            });
            let list = DataCURD.filter_null(result);
            resolve(list);
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "page_no": {
                        "type": "integer",
                        "title": "页码",
                        "description": "页码，从1开始"
                    },
                    "page_size": {
                        "type": "integer",
                        "title": "每页条数"
                    },
                    "name": {
                        "type": "string",
                        "description": "根据项目名称过滤（模糊匹配）",
                        "title": "项目名称"
                    },
                    "type_id": {
                        "type": "integer",
                        "title": "项目类型",
                        "description": "根据项目子类型id过滤"
                    },
                    "leader_id": {
                        "type": "string",
                        "title": "项目负责人",
                        "description": "根据项目负责人账号id过滤"
                    },
                    "region_id": {
                        "type": "integer",
                        "title": "所在地区",
                        "description": "根据所在地区id过滤"
                    },
                    "start_at_from": {
                        "type": "string",
                        "description": "项目启动时间（查询起始）YYYY-MM-DD HH:NN:SS",
                        "title": "项目启动时间查询起始"
                    },
                    "start_at_to": {
                        "type": "string",
                        "title": "项目启动时间查询至",
                        "description": "项目启动时间（查询至）YYYY-MM-DD HH:NN:SS"
                    },
                    "deliver_at_from": {
                        "type": "string",
                        "description": "项目交付时间（查询起始）YYYY-MM-DD HH:NN:SS",
                        "title": "项目交付时间（查询起始）"
                    },
                    "deliver_at_to": {
                        "type": "string",
                        "description": "项目交付时间（查询至）YYYY-MM-DD HH:NN:SS",
                        "title": "项目交付时间（查询至）"
                    },
                    "phase_id": {
                        "type": "string",
                        "title": "项目阶段",
                        "description": "根据项目阶段过滤"
                    },
                    "customer_name": {
                        "type": "string",
                        "title": "客户名称",
                        "description": "根据客户名称过滤"
                    },
                    "bizman_id": {
                        "type": "string",
                        "title": "商务负责人id"
                    }
                },
                "x-apifox-orders": [
                    "page_no",
                    "page_size",
                    "name",
                    "type_id",
                    "leader_id",
                    "region_id",
                    "customer_name",
                    "phase_id",
                    "start_at_from",
                    "start_at_to",
                    "deliver_at_from",
                    "deliver_at_to",
                    "bizman_id"
                ],
                "title": "请求参数内容",
                "required": [
                    "page_no",
                    "page_size"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理",
                "default": null
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "app_id",
            "timestamp",
            "token",
            "ver"
        ]
    },
    method: get_list,
    method_params: {}
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
