import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import DataCURD from "@core/DataCURD";
import {Op, Transaction, WhereOptions} from "sequelize";
import {PrjInfo} from "@core-models/PrjInfo";
import dayjs from "dayjs";
import {PrjLogger} from "@src/utils/prj_logger";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {Resp} from "@util/Resp";
import {Logger} from "@util/Logger";
import {ChangeMarker} from "@src/utils/define";
import {start_plan_alt_flow, update_parent_task_info} from "@src/utils/plan_task_helper";
import {PrjFile} from "@core-models/PrjFile";
import {BizContractInfo} from "@core-models/BizContractInfo";
import {IdGen} from "@util/IdGen";
import {is_project_modifiable, is_project_privileged_account} from "@src/utils/prj_premission_helper";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {BpmnCase} from "@core-models/BpmnCase";
import {BpmnWork} from "@core-models/BpmnWork";
import {FlowEngine} from "@src/bpmn/flow_engine";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";

interface IData {
    /**
     * 商务负责人，商务负责人id
     */
    bizman_id?: string;
    /**
     * 合同id，合同id
     */
    contract_id?: string;
    /**
     * 客户名称，客户id
     */
    customer_id?: string;
    /**
     * 预计交付时间，预计交付时间（YYYY-MM-DD）
     */
    deliver_at?: string;
    /**
     * 项目id，项目id
     */
    id: string;
    /**
     * 项目简介，项目简介
     */
    intro?: string;
    /**
     * 项目负责人，项目负责人账号id
     */
    leader_id?: string;
    /**
     * 项目名称，项目名称
     */
    name?: string;
    /**
     * 所在地区，所在地区id
     */
    region_id?: number;
    /**
     * 项目类型，项目类型id
     */
    type_id?: number;
}

function permission_guard(content: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let data = <IData>content.data;
        let prj = await PrjInfo.findOne({where: {id: data.id}, raw: true});
        if (!prj)
            return reject(Resp.gen_err(Resp.ResourceNotFound, '项目不存在。'));
        let phase = await PrjPhaseDefine.findOne({where: {id: prj.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj.id));
        if (phase.order_index >= 20 ) {
            if (data.type_id !== undefined) {
                return reject(Resp.gen_err(Resp.Forbidden, '项目立项后不允许修改项目类型。'))
            }
            if (!await is_project_privileged_account(cached_data.user_id)) {
                if (data.deliver_at)
                    return reject(Resp.gen_err(Resp.Forbidden, '项目已立项，您没有权限修改项目交付时间，请确认您是特权人员。'));
            }

        }

        /// 只有项目特权人员才可以修改项目负责人
        if (data.leader_id && data.leader_id !== prj.leader_id) {
            if (!await is_project_privileged_account(cached_data.user_id)) {
                return  reject(Resp.gen_err(Resp.Forbidden, '您没有权限修改项目负责人，请确认您是特权人员。'));
            }
        }
        if (!await is_project_modifiable(cached_data.user_id, prj.id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限修改该项目信息，请确认您是项目负责人或特权人员。'));
        if (data.deliver_at) {
            data.deliver_at += ' 23:59:59';
        }
        let task = await PrjPlanTask.findOne({where: {prj_id: prj.id, end_at: {[Op.gt]: data.deliver_at}}, raw: true});
        if (task) {
            return reject(Resp.gen_err(Resp.Forbidden, '项目交付时间不能早于项目计划中任务的结束时间。'));
        }
        task = await PrjPlanTaskDraft.findOne({where: {prj_id: prj.id, end_at: {[Op.gt]: data.deliver_at}}, raw: true});
        if (task) {
            return reject(Resp.gen_err(Resp.Forbidden, '项目交付时间不能早于项目计划草稿中任务的结束时间。'));
        }
        resolve();

    });
}

async function get_where(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    let data = <IData>json.data;

    return {id: data.id};
}

async function before_respond(content: IRequest, params: IMethodParams, cached_data: ICachedData | any, result: any): Promise<unknown> {
    let data = <IData>content.data;
    result = DataCURD.filter_null(result, new Map(Object.entries({
        "updated_at": "YYYY-MM-DD HH:mm:ss",
        "created_at": "YYYY-MM-DD HH:mm:ss",
        "deliver_at": "YYYY-MM-DD HH:mm:ss"
    })));
    await PrjLogger.updated({
        creator_id: cached_data.user_id,
        creator_name: cached_data.user_name,
        content: '项目信息',
        prj_id: data.id
    });
    return result;
}

function modify(content: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        let data = <IData>content.data;

        let t = await PrjInfo.sequelize!.transaction();
        try {
            let prj = await PrjInfo.findOne({where: {id: data.id}, raw: true, transaction: t});
            if (!prj) {
                throw Resp.gen_err(Resp.ResourceNotFound, '项目不存在.');
            }

            // 修改了项目负责人，这个问题就严重了，要改很多东西
            // 要将该项目的项目流程全部移交给新负责人，
            // 涉及bpmn case和bpmn work中与项目相关部分, 以及flow实例中的owner
            // task相关的work不变
            if (data.leader_id && data.leader_id !== prj.leader_id) {
                let new_leader = await AcsUserInfo.findOne({where: {id: data.leader_id}, raw: true, transaction: t});
                if (!new_leader) {
                    throw Resp.gen_err(Resp.ResourceNotFound, '新选中的项目负责人不存在.');
                }
                let [count, flow_cases] = await BpmnCase.update({creator_id: data.leader_id},
                    {where: {prj_id: prj.id, task_id: {[Op.is]: null}, completed_at: {[Op.is]: null}}, transaction: t, returning: true});
                await BpmnWork.update({assigned_to: data.leader_id},
                    {where: {prj_id: prj.id, task_id: {[Op.is]: null}, status: {[Op.ne]: 2}}, transaction: t});
                if (flow_cases && flow_cases.length > 0) {
                    for (let flow_case of flow_cases) {
                        let engine = FlowEngine.case_engine_map.get(flow_case.id);
                        if (engine) engine.owner = data.leader_id;
                    }
                }
            }

            let contract_id = prj.contract_id;
            // 修改合同id，需要在项目文档中移除原合同附件，移入新合同附件
            Logger.trace(`new contract_id: ${data.contract_id} old contract id: ${contract_id}`);
            if (data.contract_id && data.contract_id !== contract_id) {
                if (contract_id) {
                    // 如果有原合同，则移除原合同附件（只是从项目文档中移出，并不删除附件，附件仍挂在合同上）
                    await PrjFile.destroy({where: {prj_id: data.id, category_id: 'contract'}, transaction: t});
                }
                /// 将新合同附件移入项目文档
                let contract = await BizContractInfo.findOne({
                    where: {id: data.contract_id},
                    raw: true,
                    transaction: t
                });
                if (!contract) throw Resp.gen_err(Resp.ResourceNotFound, '合同不存在.');
                Logger.trace(contract);
                if (contract.doc_uploaded) {
                    await PrjFile.create({
                        id: contract.doc_id,
                        category_id: 'contract',
                        prj_id: prj.id,
                        req_upload_time: contract.doc_uploaded_at,
                        filename: contract.doc_name,
                        dependent_id: data.contract_id,
                        creator_id: contract.handler_id,
                        uploaded: true,
                        uploaded_at: contract.doc_uploaded_at,
                        size: contract.doc_size

                    }, {transaction: t});
                }
                if (contract.pdf_uploaded) {
                    await PrjFile.upsert({
                        category_id: 'contract',
                        prj_id: prj.id,
                        req_upload_time: contract.pdf_uploaded_at,
                        id: contract.pdf_id,
                        filename: contract.pdf_name,
                        dependent_id: data.contract_id,
                        creator_id: contract.handler_id,
                        uploaded: true,
                        uploaded_at: contract.pdf_uploaded_at,
                        size: contract.pdf_size
                    }, {transaction: t});
                }
            }
            let result = await DataCURD.internal_update_row(content, {
                model: PrjInfo,
                where: get_where,
                fixed_input_data: {
                    updated_at: dayjs().format('YYYY-MM-DD HH:mm:ss')
                },
                input_data_map: {
                    name: "name",
                    customer_id: "customer_id",
                    region_id: "region_id",
                    type_id: "type_id",
                    contract_id: "contract_id",
                    deliver_at: "deliver_at",
                    leader_id: "leader_id",
                    bizman_id: "bizman_id",
                    intro: "intro",
                }
            }, cached_data, t);


            await t.commit();
            resolve(result);
        } catch (e) {
            await t.rollback();
            reject(e);
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                        "description": "项目名称",
                        "title": "项目名称"
                    },
                    "type_id": {
                        "type": "integer",
                        "title": "项目类型",
                        "description": "项目类型id"
                    },
                    "leader_id": {
                        "type": "string",
                        "title": "项目负责人",
                        "description": "项目负责人账号id"
                    },
                    "region_id": {
                        "type": "integer",
                        "title": "所在地区",
                        "description": "所在地区id"
                    },
                    "customer_id": {
                        "type": "string",
                        "title": "客户名称",
                        "description": "客户id"
                    },
                    "deliver_at": {
                        "type": "string",
                        "description": "预计交付时间（YYYY-MM-DD）",
                        "title": "预计交付时间"
                    },
                    "bizman_id": {
                        "type": "string",
                        "description": "商务负责人id",
                        "title": "商务负责人"
                    },
                    "intro": {
                        "type": "string",
                        "description": "项目简介",
                        "title": "项目简介"
                    },
                    "contract_id": {
                        "type": "string",
                        "title": "合同id",
                        "description": "合同id"
                    },
                    "id": {
                        "type": "string",
                        "title": "项目id",
                        "description": "项目id"
                    }
                },
                "title": "请求参数内容",
                "required": [
                    "id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: modify,//DataCURD.internal_update_row,
    on_before_respond: before_respond,
    method_params: {
        model: PrjInfo,
        where: get_where,
        fixed_input_data: {
            updated_at: dayjs().format('YYYY-MM-DD HH:mm:ss')
        },
        input_data_map: {
            name: "name",
            customer_id: "customer_id",
            region_id: "region_id",
            type_id: "type_id",
            contract_id: "contract_id",
            deliver_at: "deliver_at",
            leader_id: "leader_id",
            bizman_id: "bizman_id",
            intro: "intro",
            updated_at: "updated_at"
        }
    },
    guards: [permission_guard],
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
