import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {QueryTypes, WhereOptions} from "sequelize";
import {PrjInfo} from "@core-models/PrjInfo";
import {AcsDomain} from "@core-models/AcsDomain";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {BizCustomer} from "@core-models/BizCustomer";
import {BizCustomerIndustry} from "@core-models/BizCustomerIndustry";
import {BizCustomerLevel} from "@core-models/BizCustomerLevel";
import {Resp} from "@util/Resp";
import {PrjMembers} from "@core-models/PrjMembers";
import DataCURD, {ISQLReplacements} from "@core/DataCURD";
import {AcsRole} from "@core-models/AcsRole";
import {BpmnWork} from "@core-models/BpmnWork";
import {BpmnCase} from "@core-models/BpmnCase";
import {is_project_accessible} from "@src/utils/prj_premission_helper";
import {BpmnModel} from "@core-models/BpmnModel";
import {FlowEngine} from "@src/bpmn/flow_engine";
import {bpmn_flow_on_end} from "@src/utils/bpmn_work_helper";
import dayjs from "dayjs";

interface IData {
    id: string;
}

function guard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let data = <IData>json.data;
        if (!await is_project_accessible(cached_data.user_id, data.id))
            return reject(Resp.gen_err(Resp.Forbidden, '无权访问该项目'));
        resolve();
    });
}


async function overview(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    let data = <IData>json.data;
    let sql = `
        select prj.id, prj.name, 
            prj.type_id, CONCAT_WS('/', pidtype.name, ptype.name) as type_name,
            prj.region_id, region.name as region_name,
            prj.phase_id, phase.name as phase_name, 
            prj.progress,
            prj.leader_id, leader.name as leader_name,
            prj.bizman_id, bizman.name as bizman_name,
            prj.customer_id, customer.name as customer_name, 
            prj.region_id, region.name as region_name,
            customer.industry_id as industry_id, industry.name as industry_name,
            customer.level_id as level_id, level.name as level_name,
            prj.contract_id,
            prj.intro, 
            customer_region.id as customer_region_id, customer_region.name as customer_region_name,
            customer_bizman.id as customer_bizman_id, customer_bizman.name as customer_bizman_name,
            
            TO_CHAR(prj.created_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as created_at,
            TO_CHAR(prj.deliver_at, 'yyyy-MM-dd') as deliver_at,
            TO_CHAR(prj.updated_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as updated_at,
            TO_CHAR(prj.started_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as started_at,
            -- case when NULLIF(array_agg(work.id), '{NULL}') is null then '{}' else
  
           array_agg(
                case when work.id is null then null
                    else json_build_object('id', work.id, 'name', work.name,'form', work.form)
                end
            ) FILTER (WHERE case when work.id is null then null
                    else json_build_object('id', work.id, 'name', work.name,'form', work.form)
                end IS NOT NULL) as works
        from ${PrjInfo.table_name} prj
        
        left join ${AcsDomain.table_name} ptype on prj.type_id = ptype.id
        left join ${AcsDomain.table_name} pidtype on ptype.pid = pidtype.id
        left join ${PrjPhaseDefine.table_name} phase on phase.id = prj.phase_id
        left join ${AcsUserInfo.table_name} bizman on bizman.id = prj.bizman_id
        left join ${AcsUserInfo.table_name} leader on leader.id = prj.leader_id
        left join ${BizCustomer.table_name} customer on customer.id = prj.customer_id
        left join ${AcsDomain.table_name} region on prj.region_id = region.id
        left join ${BizCustomerIndustry.table_name} industry on customer.industry_id = industry.id
        left join ${BizCustomerLevel.table_name} level on customer.level_id = level.id
        left join ${AcsDomain.table_name} customer_region on customer.region_id = customer_region.id
        left join ${AcsUserInfo.table_name} customer_bizman on customer.bizman_id = customer_bizman.id
        left join ${BpmnCase.table_name} flow on flow.prj_id = prj.id 
        LEFT JOIN ${BpmnWork.table_name} work ON work.prj_id = prj.id and work.case_id = flow.id and
            work.assigned_to = '${cached_data.user_id}' and 
            work.status <> 2 and work.process_type = 1 and 
            (flow.model_id = 'prj')
                
        where prj.id = :prj_id
        
        group by prj.id,
            phase.name, pidtype.name, ptype.name,
            leader.name,
            bizman.name,
            customer.name, 
            region.name,
            customer.industry_id , industry.name,
            customer.level_id, level.name,
            customer_region.id, customer_region.name,
            customer_bizman.id, customer_bizman.name
    `;
    let replacements = {prj_id: data.id}
    let result: any = await PrjInfo.sequelize!.query(sql, {
        type: QueryTypes.SELECT,
        replacements: replacements,
        raw: true
    });
    if (!result || !result[0]) throw Resp.gen_err(Resp.ResourceNotFound);
    result = DataCURD.filter_null(result[0]);
    // let members = await PrjMembers.findAll({where: {prj_id: data.id}, raw: true});
    // if (!members) members = [];

    sql = `select member.member_id as account_id, userinfo.name as name,
               member.role_id, role.name as role_name,
               member.memo
        from ${PrjMembers.table_name} member
        left join ${AcsUserInfo.table_name} userinfo on member.member_id = userinfo.id 
        left join ${AcsRole.table_name} role on member.role_id = role.id
        where prj_id = :prj_id and draft = false
            `
    replacements = {prj_id: data.id};
    let members = await PrjMembers.sequelize!.query(sql, {
        type: QueryTypes.SELECT,
        replacements: replacements,
        raw: true
    });
    if (!members) members = [];
    members = DataCURD.filter_null(members);
    result.members = members;

    /// 如果创建项目时，流程没有正确启动，则在这里补上（通常不会发生）
    let flow = await BpmnCase.findOne({where: {prj_id: data.id, model_id: 'prj'}, raw: true});
    if (!flow) {
        let t = await PrjInfo.sequelize!.transaction();
        try {
            // 启动工作流
            let bpmn = await BpmnModel.findOne({where: {id: 'prj'}, transaction: t});
            if (!bpmn) throw Resp.gen_err(Resp.InternalServerError, '项目流程模型数据缺失！');
            let flow = new FlowEngine(bpmn.id, bpmn.content, {
                prj_id: data.id,
                owner: result.leader_id
            }, bpmn_flow_on_end);
            await flow.run();
            await PrjInfo.update({flow_case_id: flow.id}, {where: {id: data.id}, transaction: t});
            await BpmnCase.create({
                id: flow.id,
                model_id: bpmn.id,
                model: bpmn.content,
                started_at: dayjs(),
                creator_id: result.leader_id,
                prj_id: data.id,
            }, {transaction: t});

            await t.commit();
        } catch (err) {
            await t.rollback();
            throw err;
        }
    }

    return result;
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "title": "获取项目概览",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "平台分配的app id"
            },
            "token": {
                "type": "string",
                "title": "登录后获得的令牌"
            },
            "timestamp": {
                "type": "integer",
                "title": "时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "data": {
                "type": "object",
                "title": "业务参数",
                "properties": {
                    "id": {
                        "type": "string",
                        "title": "项目编号"
                    }
                },
                "required": [
                    "id"
                ]
            }
        },
        "required": [
            "ver", "app_id", "timestamp", "data"
        ]
    },
    method: overview,
    method_params: {},
    guards: [guard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
