import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjMembers} from "@core-models/PrjMembers";
import {Resp} from "@util/Resp";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {BpmnCase} from "@core-models/BpmnCase";
import {BpmnWork} from "@core-models/BpmnWork";
import {FlowEngine} from "@src/bpmn/flow_engine";
import {PrjLogger} from "@src/utils/prj_logger";
import {is_project_modifiable} from "@src/utils/prj_premission_helper";

interface IData {
    id: string;//项目id
}

/// 项目提交立项后，不允许删除
export function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();
        let prj_info = await PrjInfo.findOne({where: {id: data.id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound));
        if (!await is_project_modifiable(cached_data.user_id, prj_info.id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限删除该项目，请确认您是项目负责人或特权人员。'));
        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));
        if (phase.order_index > 0) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许删除项目，可使用作废功能使项目作废。'));

        resolve();
    });

}

async function remove(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    let data = <IData>json.data;
    let t = await PrjInfo.sequelize!.transaction();
    try {
        let flows = await BpmnCase.findAll({where: {prj_id: data.id}, transaction: t, raw: true});
        for (let flow of flows) {
            let to_stop = FlowEngine.case_engine_map.get(flow.id);
            await to_stop?.stop();
        }
        await PrjMembers.destroy({where: {prj_id: data.id}, transaction: t});
        await BpmnWork.destroy({where: {prj_id: data.id}, transaction: t});
        await BpmnCase.destroy({where: {prj_id: data.id}, transaction: t});
        await PrjInfo.destroy({where: {id: data.id}, transaction: t});
        await t.commit();
    } catch (e) {
        await t.rollback();
        throw Resp.throw_err(e);
    }
}

const v1_0: IApiProcessor = {
    schema: {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "title": "删除项目记录",
        "properties": {
            "ver": {
                "type": "string",
                "title": "版本号，默认1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "平台分配的app id"
            },
            "token": {
                "type": "string",
                "title": "登录后获得的令牌"
            },
            "timestamp": {
                "type": "integer",
                "title": "时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "data": {
                "type": "object",
                "title": "业务参数",
                "properties": {
                    "id": {
                        "type": "string",
                        "title": "项目编号"
                    }
                },
                "required": [
                    "id"
                ]
            }
        },
        "required": [
            "ver", "app_id", "timestamp", "data"
        ]
    },
    method: remove,
    method_params: {},
    guards: [statusGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
};
