import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {IHandler} from "@src/bpmn/flow_engine";
import {PrjLogger} from "../../../../utils/prj_logger";
import {is_project_modifiable, is_project_privileged_account} from "../../../../utils/prj_premission_helper";

interface IData {
    /**
     * 项目审核组成员
     */
    checkers: IHandler[];
    /**
     * 项目id
     */
    id: string;
}

/// 项目任务提交申请后，不允许修改
function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();

        let prj_info = await PrjInfo.findOne({where: {id: data.id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound));

        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));
        if (phase.id !== 'new' && phase.id !== 'doing' && !await is_project_privileged_account(user))
            return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许修改审核人，请确认您是特权人员。。'));
        if (!await is_project_modifiable(cached_data.user_id, prj_info.id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限修改该项目，请确认您是项目负责人或特权人员。'));
        resolve();
    });
}

function set_checkers(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        let data = <IData>json.data;
        let t = await PrjInfo.sequelize!.transaction();
        try {

            await PrjInfo.update({checkers: data.checkers},
                {where: {id: data.id}, transaction: t});
            await PrjLogger.updated({
                creator_id: cached_data.user_id,
                creator_name: cached_data.user_name,
                content: '项目审核组成员',
                prj_id: data.id,
                t
            });
            await t.commit();
            resolve({});
        } catch (e) {
            await t.rollback();
            reject(e);
        }
    })
}


const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "id": {
                        "type": "string",
                        "title": "项目id"
                    },
                    "checkers": {
                        "type": "array",
                        "items": {
                            "type": "object",
                            "properties": {
                                "id": {
                                    "type": "string",
                                    "title": "账号"
                                },
                                "due_day": {
                                    "type": "integer",
                                    "title": "超时时限（天）",
                                    "description": "超时后产生提醒"
                                }
                            },
                            "x-apifox-orders": [
                                "id",
                                "due_day"
                            ],
                            "required": [
                                "id",
                                "due_day"
                            ]
                        },
                        "title": "项目审核组成员"
                    }
                },
                "x-apifox-orders": [
                    "id",
                    "checkers"
                ],
                "required": [
                    "id",
                    "checkers"
                ],
                "title": "请求参数内容"
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "examples": [
                    "1.0"
                ],
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。",
                "examples": [
                    "f2741721-1c07-430e-9f01-5529692340f9"
                ]
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理",
                "default": null
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: set_checkers,
    method_params: {
    },
    guards: [statusGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
