import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {QueryTypes, WhereOptions} from "sequelize";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {PrjInfo} from "@core-models/PrjInfo";
import {BpmnWork} from "@core-models/BpmnWork";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import DataCURD from "@core/DataCURD";
import {FlowEngine} from "../../../../bpmn/flow_engine";

interface IData {
    /**
     * 项目id
     */
    prj_id: string;
    [property: string]: any;
}

function milestone(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let engine = FlowEngine.execution_engine_map.get(data.id);
            if (engine) {
                let work = await BpmnWork.findOne({where: {id: data.id}, raw: true});
                if (work)
                    await engine.signal(work.activity_id, data.id, cached_data.user_id, data.msg);
            }

            let prj = await PrjInfo.findOne({where: {id: data.prj_id}, raw: true});
            if (!prj) return reject(Resp.gen_err(Resp.DataExists, `项目不存在，id：${data.prj_id}`));
            let phase = await PrjPhaseDefine.findOne({where: {id: prj.phase_id}, raw: true});
            let result = {
                progress: prj.progress,
                tasks: <any>[]
            }
            let tasks = await PrjPlanTask.sequelize!.query(`
                select task.id as task_id, task.name, task.progress, task.status, 
                    task.handler_id, handler.name as handler_name, task.pid,
                    TO_CHAR(task.begin_at, 'yyyy-MM-dd') as begin_at,
                    TO_CHAR(task.end_at, 'yyyy-MM-dd') as end_at,
                    TO_CHAR(task.completed_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as completed_at
                from ${PrjPlanTask.table_name} task
                left join ${AcsUserInfo.table_name} handler on handler.id = task.handler_id
                where task.prj_id = :prj_id and task.pid isnull
                order by task.order_id asc
            `, {type: QueryTypes.SELECT, raw: true, replacements: {
                    prj_id: data.prj_id
                }})
            if (!tasks) return resolve(result);
            tasks = DataCURD.filter_null(tasks);
            result.tasks = tasks;
            resolve(result);
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "prj_id": {
                        "type": "string",
                        "title": "项目id"
                    }
                },
                "title": "请求参数内容",
                "required": [
                    "prj_id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: milestone,
    method_params: {},
    guards: []
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
