import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {WhereOptions} from "sequelize";
import DataCURD from "@core/DataCURD";
import {IdGen} from "@util/IdGen";
import dayjs from "dayjs";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjMembers} from "@core-models/PrjMembers";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {is_project_privileged_account, is_project_modifiable} from "@src/utils/prj_premission_helper";

interface IData {
    /**
     * 项目组成员账号id
     */
    account_id: string;
    /**
     * 说明
     */
    memo?: string;
    /**
     * 项目id
     */
    prj_id: string;
    /**
     * 项目组成员角色id
     */
    role_id: string;
    [property: string]: any;
}


/// 项目提交立项申请后，不允许再修改项目组成员
export function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();
        let prj_info = await PrjInfo.findOne({where: {id: data.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound));


        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));

        // 当前项目阶段不允许添加新成员, 但特权人员除外
        if (phase.order_index !== 0 && !await is_project_privileged_account(user))
            return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许添加新成员。'));

        // 项目负责人或特权人员可以修改项目组成员
        if (!await is_project_modifiable(user, prj_info.id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限添加项目组成员，请确认您是项目负责人或特权人员。'));

        resolve();
    });
}

function add(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        // let t = await PrjInfo.sequelize!.transaction();
        try {
            let data = <IData>json.data;
            if (await PrjMembers.findOne({where: {prj_id: data.prj_id, member_id: data.account_id}})) {
                return reject(Resp.gen_err(Resp.DataExists));
            }
            let value: any = {
                prj_id: data.prj_id,
                member_id: data.account_id,
                role_id: data.role_id,
                draft: false,
                created_at: dayjs().format('YYYY-MM-DD HH:mm:ss')
            }

            await PrjMembers.create(value);

            resolve({});
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "prj_id": {
                        "type": "string",
                        "title": "项目id"
                    },
                    "account_id": {
                        "type": "string",
                        "title": "项目组成员账号id"
                    },
                    "role_id": {
                        "type": "string",
                        "title": "项目组成员角色id"
                    },
                    "memo": {
                        "type": "string",
                        "title": "说明"
                    }
                },
                "title": "请求参数内容",
                "required": [
                    "prj_id",
                    "account_id",
                    "role_id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: add,
    method_params: {},
    guards: [statusGuard]
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
