import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjMembers} from "@core-models/PrjMembers";
import {Resp} from "@util/Resp";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {is_project_modifiable, is_project_privileged_account} from "../../../../utils/prj_premission_helper";

interface IData {
    /**
     * 成员账号id
     */
    account_id: string;
    /**
     * 项目id
     */
    prj_id: string;
    [property: string]: any;
}

async function remove(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    let data = <IData>json.data;
    // let t = await PrjInfo.sequelize!.transaction();
    try {
        await PrjMembers.destroy({where: {prj_id: data.prj_id, member_id: data.account_id, draft: false}});
        // await t.commit();
    } catch (e) {
        // await t.rollback();
        throw Resp.throw_err(e);
    }
}

/// 不允许删除自己
export function accountGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();
        if (!await is_project_modifiable(user, data.prj_id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限删除项目组成员，请确认您是项目负责人或特权人员。'));
        if (user === data.account_id) return reject(Resp.gen_err(Resp.PrjRemoveSelfForbidden));

        resolve();
    });
}

/// 项目提交立项申请后，不允许再删除项目组成员
export function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();
        let prj_info = await PrjInfo.findOne({where: {id: data.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound));
        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));
        if (phase.order_index !== 0 && !await is_project_privileged_account(user))
            return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许删除成员，请确认您是特权人员。'));
        // 项目负责人或特权人员可以修改项目组成员
        if (!await is_project_modifiable(user, prj_info.id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限添加项目组成员，请确认您是项目负责人或特权人员。'));

        resolve();
    });
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "prj_id": {
                        "type": "string",
                        "title": "项目id"
                    },
                    "account_id": {
                        "type": "string",
                        "title": "成员账号id"
                    }
                },
                "required": [
                    "prj_id",
                    "account_id"
                ],
                "title": "请求参数内容"
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "examples": [
                    "1.0"
                ],
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。",
                "examples": [
                    "f2741721-1c07-430e-9f01-5529692340f9"
                ]
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理",
                "default": null
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: remove,
    method_params: {},
    guards: [accountGuard, statusGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
