import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {WhereOptions} from "sequelize";
import DataCURD from "@core/DataCURD";
import {IdGen} from "@util/IdGen";
import dayjs from "dayjs";
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {PrjTaskOutcomeDraft} from "@core-models/PrjTaskOutcomeDraft";
import {start_plan_alt_flow} from "../../../../utils/plan_task_helper";
import {ChangeMarker, TaskStatus} from "../../../../utils/define";

interface IData {
    /**
     * 交付物描述
     */
    description?: string;
    /**
     * 是否是副本
     */
    draft: boolean;
    /**
     * 交付物名称
     */
    name: string;
    /**
     * 计划交付时间（YYYY-MM-DD）
     */
    plan_time: Date;
    /**
     * 任务id
     */
    task_id: string;
}


function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();

        let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
        let task = await model.findOne({where: {id: data.task_id}, raw: true});
        if (!task) return reject(Resp.gen_err(Resp.ResourceNotFound, '任务不存在。'));


        let prj_info = await PrjInfo.findOne({where: {id: task.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目不存在。'));

        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));

        if (phase.order_index < 20) return reject(Resp.gen_err(Resp.InvalidFlow, '需要先对项目进行立项，立项后才可制订计划。'));
        if (phase.order_index >= 80) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许添加交付物。'));

        if (data.draft == true) {
            if (phase.order_index >= 60 && phase.order_index <= 65) {
                return reject(Resp.gen_err(Resp.InvalidFlow, '计划变更审核中，现阶段不允许添加交付物。'));
            }
        } else {
            if (phase.order_index >=30 && phase.order_index <= 40)
                return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划审核中，当前阶段不允许添加。'));
            if (phase.order_index === 70)
                return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许直接添加交付物，应在草稿中添加后重新提交计划变更审核。'));
        }
        if (task.status >= TaskStatus.ApplyForReview) {
            return reject(Resp.gen_err(Resp.InvalidFlow, '当前任务阶段不允许添加交付物。'));
        }


        resolve();
    });
}

function add(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        let t = await PrjPlanTaskDraft.sequelize!.transaction();
        try {
            let data = <IData>json.data;
            let id = IdGen.id();
            let value: any = {
                id: id,
                creator_id: cached_data.user_id,
                created_at: dayjs().format('YYYY-MM-DD HH:mm:ss'),
                order_id: IdGen.id(),
                ...data
            }
            if (data.draft) {
                value.change_marker = ChangeMarker.Added;
            }
            value = DataCURD.filter_null(value);
            let model = data.draft ? PrjTaskOutcomeDraft : PrjTaskOutcome;
            await model.create(value, {transaction: t});
            /// 如果计划任务是草稿，且还没有生成计划变更任务，则生成任务
            if (data.draft === true ) {
                let task = await PrjPlanTaskDraft.findOne({where: {id: data.task_id}, raw: true, transaction: t});
                if (task) {
                    await start_plan_alt_flow(task.prj_id, cached_data.user_id, t);
                    await PrjPlanTaskDraft.update({change_marker: ChangeMarker.Changed}, {where: {id: data.task_id}, transaction: t});
                }
            }
            await t.commit();
            resolve({id: id});
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "draft": {
                        "type": "boolean",
                        "title": "是否是副本",
                        "default": "false"
                    },
                    "name": {
                        "type": "string",
                        "title": "交付物名称"
                    },
                    "description": {
                        "type": "string",
                        "title": "交付物描述"
                    },
                    "task_id": {
                        "type": "string",
                        "title": "任务id"
                    },
                    "plan_time": {
                        "type": "string",
                        "title": "计划交付时间（YYYY-MM-DD）",
                        "format": "date"
                    }
                },
                "x-apifox-orders": [
                    "draft",
                    "task_id",
                    "name",
                    "plan_time",
                    "description"
                ],
                "title": "请求参数内容",
                "required": [
                    "draft",
                    "name",
                    "plan_time",
                    "task_id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: add,
    method_params: {},
    guards: [statusGuard]
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
