import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {QueryTypes, WhereOptions} from "sequelize";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {PrjInfo} from "@core-models/PrjInfo";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import DataCURD from "@core/DataCURD";
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import {PrjTaskOutcomeDraft} from "@core-models/PrjTaskOutcomeDraft";
import {PrjFile} from "@core-models/PrjFile";


interface IData {
    /**
     * 是否草稿
     */
    draft: boolean;
    /**
     * 交付物id
     */
    outcome_id: string;
}

interface IResult {
    /**
     * 创建时间（YYYY-MM-DD)
     */
    created_at: Date;
    /**
     * 创建人id
     */
    creator_id: string;
    /**
     * 创建人姓名
     */
    creator_name: string;
    /**
     * 任务描述
     */
    description?: string;
    /**
     * 实际交付时间（YYYY-MM-DD HH:NN:SS）
     */
    end_at: Date;
    /**
     * 负责人id
     */
    handler_id: string;
    /**
     * 负责人名称
     */
    handler_name: string;
    /**
     * 交付物id
     */
    id: string;
    /**
     * 交付物名称
     */
    name: string;
    /**
     * 计划交付时间（YYYY-MM-DD）
     */
    plan_time: Date;
    /**
     * 项目id
     */
    prj_id: string;
    /**
     * 项目名称
     */
    prj_name: string;
    /**
     * 进度百分比（%）
     */
    progress: number;
    /**
     * 状态，状态：0=未交付、10=申请审核中，20=审核中、30=审核未通过、100=已完成交付。默认为0。
     */
    status: number;
    /**
     * 任务id
     */
    task_id: string;
    /**
     * 任务名称
     */
    task_name?: string;
    /**
     * 修改时间（YYYY-MM-DD)
     */
    updated_at?: Date;
    [property: string]: any;
}


function detail(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<IResult> {
    return new Promise<IResult>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let model = data.draft ? PrjTaskOutcomeDraft : PrjTaskOutcome;
            let outcomes = await model.sequelize!.query(`
                select outcome.id , outcome.name,
                    task.prj_id, prj.name as prj_name,
                    task.id as task_id, task.name as task_name,
                    outcome.status, 
                    task.handler_id, handler.name as handler_name, 
                    outcome.creator_id, creator.name as creator_name,
                    TO_CHAR(outcome.plan_time, 'yyyy-MM-dd HH24') as plan_time,
                    TO_CHAR(outcome.updated_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as updated_at,
                    TO_CHAR(outcome.created_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as created_at,
                    TO_CHAR(outcome.completed_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as completed_at,
                    outcome.description,
                    outcome.uploaded,
                    file.id as object_name, file.filename as filename, file.size as file_size
                from ${model.table_name} outcome
                left join ${PrjPlanTask.table_name} task on task.id = outcome.task_id
                left join ${PrjInfo.table_name} prj on prj.id = task.prj_id
                left join ${AcsUserInfo.table_name} handler on handler.id = task.handler_id
                left join ${AcsUserInfo.table_name} creator on creator.id = outcome.creator_id
                left join ${PrjFile.table_name} file on file.dependent_id = outcome.id
                where outcome.id = :outcome_id
            `, {type: QueryTypes.SELECT, raw: true, replacements: {
                    outcome_id: data.outcome_id,
                }})
            if(!outcomes) return reject(Resp.gen_err(Resp.ResourceNotFound));
            resolve(DataCURD.filter_null(outcomes[0]));
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "outcome_id": {
                        "type": "string",
                        "title": "交付物id"
                    },
                    "draft": {
                        "type": "boolean",
                        "title": "是否草稿"
                    }
                },
                "x-apifox-orders": [
                    "outcome_id",
                    "draft"
                ],
                "title": "请求参数内容",
                "required": [
                    "outcome_id",
                    "draft"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: detail,
    method_params: {},
    guards: []
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
