import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Op, QueryTypes, WhereOptions} from "sequelize";
import DataCURD, {ISQLReplacements} from "@core/DataCURD";
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import {PrjTaskOutcomeDraft} from "@core-models/PrjTaskOutcomeDraft";
import {PrjFile} from "@core-models/PrjFile";
interface IData {
    /**
     * 是否是副本
     */
    draft: boolean;
    /**
     * 任务项id
     */
    task_id: string;
}

interface IResult {
    /**
     * 列表
     */
    list: List[];
    [property: string]: any;
}

export interface List {
    /**
     * 实际交付时间（YYYY-MM-DD）
     */
    complete_at?: Date;
    /**
     * 交付物id
     */
    id: string;
    /**
     * 交付物名称
     */
    name: string;
    /**
     * 计划交付时间（YYYY-MM-DD）
     */
    plan_time: Date;
    /**
     * 状态，状态：0=未交付、10=申请审核中，20=审核中、30=审核未通过、100=已完成交付。默认为0。
     */
    status: number;
    [property: string]: any;
}

function get_list(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<IResult> {
    return new Promise<IResult>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let model = data.draft ? PrjTaskOutcomeDraft : PrjTaskOutcome;
            let sql = `
                select outcome.id, outcome.name as name,
                       outcome.status, 
                       TO_CHAR(outcome.plan_time, 'yyyy-MM-dd') as plan_time,
                       TO_CHAR(outcome.completed_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as completed_at,
                       outcome.uploaded, file.id as object_name, file.filename, file.size  `
            if (data.draft) sql += ', outcome.change_marker '
            sql += `
                from ${model.table_name} outcome
                left join ${PrjFile.table_name} file on file.dependent_id = outcome.id
                where outcome.task_id = :task_id
            `
            let replacements: ISQLReplacements = {task_id: data.task_id};
            let result = await model.sequelize!.query(sql, {type: QueryTypes.SELECT, replacements: replacements, raw: true});
            let list = DataCURD.filter_null(result);
            resolve({list: list});
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "draft": {
                        "type": "boolean",
                        "title": "是否是副本",
                        "default": "false"
                    },
                    "task_id": {
                        "type": "string",
                        "title": "任务项id"
                    }
                },
                "x-apifox-orders": [
                    "task_id",
                    "draft"
                ],
                "title": "请求参数内容",
                "required": [
                    "task_id",
                    "draft"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get_list,
    method_params: {}
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
