import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import DataCURD from "@core/DataCURD";
import {WhereOptions} from "sequelize";
import dayjs from "dayjs";
import {Resp} from "@util/Resp";
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import {PrjTaskOutcomeDraft} from "@core-models/PrjTaskOutcomeDraft";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {start_plan_alt_flow} from "@src/utils/plan_task_helper";
import {ChangeMarker, TaskStatus} from "@src/utils/define";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";

interface IData {
    /**
     * 交付物描述
     */
    description?: string;
    /**
     * 是否草稿
     */
    draft: boolean;
    /**
     * 交互物id
     */
    id: string;
    /**
     * 交付物名称
     */
    name?: string;
    /**
     * 计划交付时间（YYYY-MM-DD）
     */
    plan_time?: Date;

    change_marker?: number;
    // [property: string]: any;
}

async function get_where(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    let data = <IData>json.data;
    return {id: data.id};
}

// /// 检查是否存在
// function existsGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
//     return new Promise<void>(async (resolve, reject) => {
//         let data = <IData>json.data;
//         let model = data.draft ? PrjTaskOutcomeDraft : PrjTaskOutcome;
//         let outcome = await model.findOne({where: {id: data.id}, raw: true});
//         if (!outcome) return reject(Resp.gen_err(Resp.ResourceNotFound, "交付物不存在，id: " + data.id));
//         if (data.draft) {
//             if ((<PrjTaskOutcomeDraft>outcome).change_marker !== ChangeMarker.Added) {
//                 data.change_marker = ChangeMarker.Changed;
//             }
//         }
//
//         resolve();
//     });
// }

function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));

        let outcome_model = data.draft ? PrjTaskOutcomeDraft : PrjTaskOutcome;
        let outcome = await outcome_model.findOne({where: {id: data.id}, raw: true});
        if (!outcome) return reject(Resp.gen_err(Resp.ResourceNotFound, "交付物不存在，id: " + data.id));
        if (data.draft) {
            if ((<PrjTaskOutcomeDraft>outcome).change_marker !== ChangeMarker.Added) {
                data.change_marker = ChangeMarker.Changed;
            }
        }

        let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
        let task = await model.findOne({where: {id: outcome.task_id}, raw: true});
        if (!task) return reject(Resp.gen_err(Resp.ResourceNotFound, '任务不存在。'));


        let prj_info = await PrjInfo.findOne({where: {id: task.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目不存在。'));

        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));

        if (phase.order_index < 20) return reject(Resp.gen_err(Resp.InvalidFlow, '需要先对项目进行立项，立项后才可制订计划。'));
        if (phase.order_index >= 80) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许修改交付物。'));

        if (data.draft == true) {
            if (phase.order_index >= 60 && phase.order_index <= 65) {
                return reject(Resp.gen_err(Resp.InvalidFlow, '计划变更审核中，现阶段不允许修改交付物。'));
            }
        } else {
            if (phase.order_index >=30 && phase.order_index <= 40)
                return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划审核中，当前阶段不允许修改。'));
            if (phase.order_index === 70)
                return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许直接修改交付物，应在草稿中修改后重新提交计划变更审核。'));
        }
        if (task.status >= TaskStatus.ApplyForReview) {
            return reject(Resp.gen_err(Resp.InvalidFlow, '当前任务阶段不允许修改交付物。'));
        }


        resolve();
    });
}

async function before_respond(content: IRequest, params: IMethodParams, cached_data: ICachedData | any, result: any): Promise<unknown> {
    let data = <IData>content.data;
    result = DataCURD.filter_null(result, new Map(Object.entries({
        "updated_at": "YYYY-MM-DD HH:mm:ss",
        "created_at": "YYYY-MM-DD HH:mm:ss",
        "plan_time": "YYYY-MM-DD HH:mm:ss",
        "completed_at": "YYYY-MM-DD HH:mm:ss"
    })));

    let t = await PrjTaskOutcome.sequelize!.transaction();
    try {
        /// 如果是草稿，且还没有生成计划变更任务，则生成任务
        if (data.draft === true ) {
            let task = await PrjPlanTaskDraft.findOne({where: {id: result.task_id}, transaction: t});
            if (task){
                await start_plan_alt_flow(task.prj_id, cached_data.user_id, t);
                await PrjPlanTaskDraft.update({change_marker: ChangeMarker.Changed},
                    {where: {id: result.task_id}, transaction: t, returning: false});
            }
        }
        await t.commit();
    } catch (e) {
        await t.rollback();
        throw Resp.gen_err(Resp.InternalServerError, '数据库操作失败');
    }

    return result;
}

async function on_start(content: IRequest, params: IMethodParams, cached_data: ICachedData | any): Promise<unknown> {
    let data = <IData>content.data;
    let model = data.draft ? PrjTaskOutcomeDraft : PrjTaskOutcome;
    params.model = model;

    return {};
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                        "title": "交付物名称"
                    },
                    "description": {
                        "type": "string",
                        "title": "交付物描述"
                    },
                    "plan_time": {
                        "type": "string",
                        "title": "计划交付时间（YYYY-MM-DD）",
                        "format": "date"
                    },
                    "id": {
                        "type": "string",
                        "title": "交互物id"
                    },
                    "draft": {
                        "type": "boolean",
                        "title": "是否草稿"
                    }
                },
                "x-apifox-orders": [
                    "id",
                    "draft",
                    "name",
                    "plan_time",
                    "description"
                ],
                "title": "请求参数内容",
                "required": [
                    "id",
                    "draft"
                ],
                "anyOf": [
                    {
                        "required": [
                            "name"
                        ]
                    },
                    {
                        "required": [
                            "description"
                        ]
                    },
                    {
                        "required": [
                            "plan_time"
                        ]
                    }
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: DataCURD.internal_update_row,
    on_start: on_start,
    on_before_respond: before_respond,
    method_params: {
        // model: PrjTaskOutcome,
        where: get_where,
        fixed_input_data: {
            updated_at: dayjs().format('YYYY-MM-DD HH:mm:ss')
        },
        input_data_map: {
            name: 'name',
            description: 'description',
            plan_time: 'plan_time',
            updated_at: 'updated_at',
            change_marker: 'change_marker'
        }
    },
    guards: [statusGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
