import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjMembers} from "@core-models/PrjMembers";
import {Resp} from "@util/Resp";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import {PrjTaskOutcomeDraft} from "@core-models/PrjTaskOutcomeDraft";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {start_plan_alt_flow} from "@src/utils/plan_task_helper";
import {Logger} from "@util/Logger";
import {ChangeMarker, TaskStatus} from "@src/utils/define";
import {PrjPlanTask} from "@core-models/PrjPlanTask";

interface IData {
    /**
     * 是否草稿
     */
    draft: boolean;
    /**
     * 交互物id
     */
    id: string;
    change_marker?: number;
}


async function remove(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    let data = <IData>json.data;
    let t = await PrjInfo.sequelize!.transaction();
    try {
        if (data.draft) {
            let outcome = await PrjTaskOutcomeDraft.update({change_marker: ChangeMarker.Deleted}, {
                where: {id: data.id},
                transaction: t,
                returning: true
            });
            Logger.trace(outcome);
            if (!outcome[1][0]) throw Resp.gen_err(Resp.ResourceNotFound);
            let task = await PrjPlanTaskDraft.findOne({where: {id: outcome[1][0].task_id}, transaction: t});
            if (task) {
                await start_plan_alt_flow(task.prj_id, cached_data.user_id, t);
                await PrjPlanTaskDraft.update({change_marker: ChangeMarker.Changed},
                    {where: {id: task.id}, transaction: t, returning: false});
            }

        } else {
            await PrjTaskOutcome.destroy({where: {id: data.id}, transaction: t});
        }

        await t.commit();
    } catch (e) {
        await t.rollback();
        throw Resp.throw_err(e);
    }
}

// /// 检查是否存在
// function existsGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
//     return new Promise<void>(async (resolve, reject) => {
//         let data = <IData>json.data;
//         let model = data.draft ? PrjTaskOutcomeDraft : PrjTaskOutcome;
//         let outcome = await model.findOne({where: {id: data.id}, raw: true});
//         if (!outcome) return reject(Resp.gen_err(Resp.ResourceNotFound, "交付物不存在，id: " + data.id));
//
//         resolve();
//     });
// }

function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));

        let outcome_model = data.draft ? PrjTaskOutcomeDraft : PrjTaskOutcome;
        let outcome = await outcome_model.findOne({where: {id: data.id}, raw: true});
        if (!outcome) return reject(Resp.gen_err(Resp.ResourceNotFound, "交付物不存在，id: " + data.id));
        // if (data.draft) {
        //     if ((<PrjTaskOutcomeDraft>outcome).change_marker !== ChangeMarker.Added) {
        //         data.change_marker = ChangeMarker.Changed;
        //     }
        // }

        let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
        let task = await model.findOne({where: {id: outcome.task_id}, raw: true});
        if (!task) return reject(Resp.gen_err(Resp.ResourceNotFound, '任务不存在。'));


        let prj_info = await PrjInfo.findOne({where: {id: task.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目不存在。'));

        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));

        if (phase.order_index < 20) return reject(Resp.gen_err(Resp.InvalidFlow, '需要先对项目进行立项，立项后才可制订计划。'));
        if (phase.order_index >= 80) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许删除交付物。'));

        if (data.draft == true) {
            if (phase.order_index >= 60 && phase.order_index <= 65) {
                return reject(Resp.gen_err(Resp.InvalidFlow, '计划变更审核中，现阶段不允许删除交付物。'));
            }
        } else {
            if (phase.order_index >=30 && phase.order_index <= 40)
                return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划审核中，当前阶段不允许删除。'));
            if (phase.order_index === 70)
                return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许直接删除交付物，应在草稿中修改后重新提交计划变更审核。'));
        }
        if (task.status >= TaskStatus.ApplyForReview) {
            return reject(Resp.gen_err(Resp.InvalidFlow, '当前任务阶段不允许删除交付物。'));
        }


        resolve();
    });
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "id": {
                        "type": "string",
                        "title": "交互物id"
                    },
                    "draft": {
                        "type": "boolean",
                        "title": "是否草稿"
                    }
                },
                "x-apifox-orders": [
                    "id",
                    "draft"
                ],
                "title": "请求参数内容",
                "required": [
                    "id",
                    "draft"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: remove,
    method_params: {},
    guards: [statusGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
