import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Oss} from "@util/Oss";
import {BizContractInfo} from "@core-models/BizContractInfo";
import {PrjFile} from "@core-models/PrjFile";
import {OutcomeStatus, PrjFileType, TaskStatus} from "@src/utils/define";
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";

interface IData {
    /**
     * 交付物id
     */
    id: string;
}
function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));


        let outcome = await PrjTaskOutcome.findOne({where: {id: data.id}, raw: true});
        if (!outcome) return reject(Resp.gen_err(Resp.ResourceNotFound, "交付物不存在，id: " + data.id));

        let task = await PrjPlanTask.findOne({where: {id: outcome.task_id}, raw: true});
        if (!task) return reject(Resp.gen_err(Resp.ResourceNotFound, '任务不存在。'));


        let prj_info = await PrjInfo.findOne({where: {id: task.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目不存在。'));

        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));

        if (phase.order_index < 20) return reject(Resp.gen_err(Resp.InvalidFlow, '需要先对项目进行立项。'));
        if (phase.order_index >= 80) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许删除交付物。'));


        if (phase.order_index >=30 && phase.order_index <= 40)
            return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划审核中，当前阶段不允许删除交付物。'));


        if (task.status >= TaskStatus.ApplyForReview) {
            return reject(Resp.gen_err(Resp.InvalidFlow, '当前任务阶段不允许删除交付物。'));
        }


        resolve();
    });
}



function remove_file(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise(async (resolve, reject) => {
        let t = await BizContractInfo.sequelize!.transaction();
        try {
            let data = <IData>json.data;
            let file = await PrjFile.findOne({where: {dependent_id: data.id, category_id: PrjFileType.Outcome}, raw: true, transaction: t});
            if (!file) throw Resp.gen_err(Resp.ResourceNotFound, `交付物文件（id：${data.id}）不存在。`);
            let prj_id = file.prj_id;
            // TODO: 检查是否是项目组成员或有访问项目的权限

            if (file.uploaded === false) throw Resp.gen_err(Resp.ResourceNotFound, '附件尚未上传。')


            let oss = Oss.get_instance('pmr-doc');
            await oss.remove_object(oss.bucket, file.id);
            await PrjFile.destroy({where: {id: data.id}, transaction: t});
            await PrjTaskOutcome.update({uploaded: false, status: OutcomeStatus.New},{
                where: {id: file.dependent_id},
                returning: false,
                transaction: t
            });
            await t.commit();
            resolve({});
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    });
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "id": {
                        "type": "string",
                        "title": "交付物id（不是交付物object name)"
                    }
                },
                "x-apifox-orders": [
                    "id"
                ],
                "title": "请求参数内容",
                "required": [
                    "id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: remove_file,
    method_params: {},
    guards: [statusGuard]
};

module.exports = {
    default: v1_0,
    v1_0: v1_0
};
