import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Oss} from "@util/Oss";
import {IdGen} from "@util/IdGen";
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {QueryTypes} from "sequelize";
import {PrjFile} from "@core-models/PrjFile";
import dayjs from "dayjs";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {TaskStatus} from "@src/utils/define";

interface IData {
    /**
     * 文件名
     */
    filename: string;
    /**
     * 交互物id
     */
    id: string;
}

function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));

        let outcome = await PrjTaskOutcome.findOne({where: {id: data.id}, raw: true});
        if (!outcome) return reject(Resp.gen_err(Resp.ResourceNotFound, "交付物不存在，id: " + data.id));

        let task = await PrjPlanTask.findOne({where: {id: outcome.task_id}, raw: true});
        if (!task) return reject(Resp.gen_err(Resp.ResourceNotFound, '任务不存在。'));


        let prj_info = await PrjInfo.findOne({where: {id: task.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目不存在。'));

        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));

        if (phase.order_index < 20) return reject(Resp.gen_err(Resp.InvalidFlow, '需要先对项目进行立项。'));
        if (phase.order_index >= 20 && phase.order_index <= 40)
            return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划制订或审核中，当前阶段不允许上传交付物。'));
        if (phase.order_index >= 60 && phase.order_index <= 65)
            return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划变更正在审核中，当前阶段不允许上传交付物。'));
        if (phase.order_index >= 80) return reject(Resp.gen_err(Resp.InvalidFlow, '项目收尾阶段不允许上传交付物。'));


        if (phase.order_index >=30 && phase.order_index <= 40)
            return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划审核中，当前阶段不允许上传交付物。'));


        if (task.status >= TaskStatus.ApplyForReview) {
            return reject(Resp.gen_err(Resp.InvalidFlow, '当前任务阶段不允许上传交付物。'));
        }


        resolve();
    });
}


function get_upload_url(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise(async (resolve, reject) => {
        let t = await PrjTaskOutcome.sequelize!.transaction();
        try {
            let data = <IData>json.data;
            let sql = `
                select prj.id from
                ${PrjInfo.table_name} prj, ${PrjPlanTask.table_name} task, ${PrjTaskOutcome.table_name} outcome
                where outcome.task_id = task.id and task.prj_id = prj.id and outcome.id = :id
            `;
            let replacements = {id: data.id}
            let prj: any = await PrjInfo.sequelize!.query(sql, {type: QueryTypes.SELECT, raw: true, replacements: replacements, transaction: t});
            if (!prj || prj.length <= 0) throw Resp.gen_err(Resp.ResourceNotFound, '交付物记录不存在。');
            let prj_id = prj[0].id;
            let object_name = `project/${prj_id}/outcome/${data.id}`;
            // await PrjTaskOutcome.update({object_name: object_name, filename: data.filename}, {where: {id: data.id}, transaction: t});


            await PrjFile.upsert({
                category_id: 'outcome',
                prj_id: prj_id,
                req_upload_time: dayjs(),
                id: object_name,
                filename: data.filename,
                dependent_id: data.id,
                creator_id: cached_data.user_id

            }, {transaction: t});

            let oss = Oss.get_instance('pmr-doc');
            let result = await oss.presignedPostPolicy(oss.bucket, object_name, 300);
            await t.commit();
            resolve({object_name: object_name, url: result.postURL, form_data: result.formData});
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    });
}

/// 检查是否存在
function existsGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let data = <IData>json.data;
        let outcome = await PrjTaskOutcome.findOne({where: {id: data.id}, raw: true});
        if (!outcome) return reject(Resp.gen_err(Resp.ResourceNotFound, "交付物不存在，id: "+ data.id));

        resolve();
    });
}


const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "id": {
                        "type": "string",
                        "title": "交互物id"
                    },
                    "filename": {
                        "type": "string",
                        "title": "文件名"
                    }
                },
                "x-apifox-orders": [
                    "id",
                    "filename"
                ],
                "title": "请求参数内容",
                "required": [
                    "id",
                    "filename"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get_upload_url,
    method_params: {},
    guards: [existsGuard, statusGuard]
};

module.exports = {
    default: v1_0,
    v1_0: v1_0
};
