import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {WhereOptions} from "sequelize";
import DataCURD from "@core/DataCURD";
import {BizCustomer} from "@core-models/BizCustomer";
import {IdGen} from "@util/IdGen";
import dayjs from "dayjs";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {start_plan_alt_flow, update_parent_task_info} from "@src/utils/plan_task_helper";
import {FlowEngine} from "@src/bpmn/flow_engine";
import {BpmnCase} from "@core-models/BpmnCase";
import {BpmnModel} from "@core-models/BpmnModel";
import {bpmn_flow_on_end} from "@src/utils/bpmn_work_helper";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {ChangeMarker} from "@src/utils/define";
import {is_project_modifiable, is_project_task_addable} from "@src/utils/prj_premission_helper";

interface IData {
    /**
     * 任务描述
     */
    description?: string;
    /**
     * 是否是副本
     */
    draft: boolean;
    /**
     * 计划结束时间（YYYY-MM-DD）
     */
    end_at: string;
    /**
     * 负责人id
     */
    handler_id: string;
    /**
     * 任务名称
     */
    name: string;
    /**
     * 项目id
     */
    prj_id: string;
    /**
     * 计划开始时间（YYYY-MM-DD)
     */
    begin_at: string;
    /**
     * 父级任务id，父级任务id，为空时表示本身是一级任务
     */
    task_pid?: string | null;
}

function guard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();

        let prj_info = await PrjInfo.findOne({where: {id: data.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound));

        data.begin_at += ' 00:00:00';
        data.end_at += ' 23:59:59';

        // 检查计划开始时间不能早于项目立项时间
        if (dayjs(data.begin_at).diff(dayjs(prj_info.started_at)) < 0) {
            return reject(Resp.gen_err(Resp.ParamsError, `计划开始时间不能早于项目立项时间 ${dayjs(prj_info.started_at).format('YYYY-MM-DD')}。`));
        }
        // 检查计划结束时间不能晚于项目交付时间
        if (dayjs(data.end_at).diff(dayjs(prj_info.deliver_at)) > 0) {
            return reject(Resp.gen_err(Resp.ParamsError, `计划结束时间不能晚于项目交付时间 ${dayjs(prj_info.deliver_at).format('YYYY-MM-DD')}。`));
        }
        // 检查计划结束时间是否晚于计划开始时间

        if (dayjs(data.end_at).diff(dayjs(data.begin_at)) <= 0)
            return reject(Resp.gen_err(Resp.ParamsError, '计划结束时间必须晚于计划开始时间。'));

        // 只有项目负责人或特权人员才可以添加任务
        if (!await is_project_task_addable(user, prj_info.id)) {
            return reject(Resp.gen_err(Resp.Forbidden, '您不是项目负责人或特权人员，不能添加任务。'));
        }
        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));
        if (phase.order_index < 20) return reject(Resp.gen_err(Resp.InvalidFlow, '需要先对项目进行立项，立项后才可制订计划。'));
        if (phase.order_index >= 30 && phase.order_index <= 40 && data.draft === false)
            return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划审核中，当前阶段不允许添加。'));
        if (phase.order_index >= 60 && phase.order_index <= 70 && data.draft === false)
            return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许直接添加计划，应在草稿中修改后重新提交审核。'));
        if (phase.order_index >= 80) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许添加计划。'));

        resolve();
    });
}

// 如添加的是子任务，查找父任务是否存在
// 如任务已经执行，不能再作为主任务添加子任务
function check_parent_guard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let data = <IData>json.data;
        if (data.task_pid === undefined) return resolve();
        let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
        let task = await model.findOne({where: {id: data.task_pid}, raw: true});
        if (!task) return reject(Resp.gen_err(Resp.ResourceNotFound, '没有找到指定的父任务'));
        if (task.status > 0) return reject(Resp.gen_err(Resp.InvalidFlow, '任务已开始执行，此任务下不允许添加子任务。'));

        resolve();
    });
}


function add(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        let t = await PrjPlanTask.sequelize!.transaction();
        try {
            let user = cached_data.user_id;
            let data = <IData>json.data;
            // if (data.begin_at && data.end_at) {
            //     if (dayjs(data.end_at).diff(data.begin_at) < 0)
            //         throw Resp.gen_err(Resp.IIllegalRequest, '任务预计完成时间不能早于或等于任务开始时间。')
            // }

            let id = IdGen.id();

            let value: any = {
                id: id,
                pid: data.task_pid,
                prj_id: data.prj_id,
                name: data.name,
                handler_id: data.handler_id,
                begin_at: data.begin_at,
                end_at:  data.end_at,
                creator_id: user,
                description: data.description,

                created_at: dayjs().format('YYYY-MM-DD HH:mm:ss'),
                order_id: IdGen.id(),
                change_marker: data.draft ? ChangeMarker.Added : undefined
            }
            value = DataCURD.filter_null(value);

            let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
            await model.create(value, {transaction: t});
            if (data.task_pid === undefined) data.task_pid = null;
            await update_parent_task_info(data.draft, data.prj_id, data.task_pid, t);

            // 检查计划审核工作流程是否已经启动，没有的话就建一个
            let flow = await BpmnCase.findOne({
                where: {prj_id: data.prj_id, model_id: 'plan'},
                transaction: t,
                raw: true
            });
            if (!flow && data.draft === false) {
                let bpmn = await BpmnModel.findOne({where: {id: 'plan'}, transaction: t});
                if (!bpmn) throw Resp.gen_err(Resp.InternalServerError, '项目流程模型数据缺失！');
                let flow = new FlowEngine(bpmn.id, bpmn.content, {
                    prj_id: data.prj_id,
                    owner: cached_data.user_id
                }, bpmn_flow_on_end);
                await flow.run();
                await BpmnCase.create({
                    id: flow.id,
                    model_id: bpmn.id,
                    model: bpmn.content,
                    started_at: dayjs().format('YYYY-MM-DD HH:mm:ss'),
                    creator_id: cached_data.user_id,
                    prj_id: data.prj_id,
                    // state: await flow.state(),
                }, {transaction: t});
            }

            /// 如果计划任务是草稿，且还没有生成计划变更任务，则生成任务
            if (data.draft === true) {
                await start_plan_alt_flow(data.prj_id, cached_data.user_id, t);
            }

            await t.commit();
            resolve({id: id});
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "prj_id": {
                        "type": "string",
                        "title": "项目id"
                    },
                    "draft": {
                        "type": "boolean",
                        "title": "是否是副本",
                        "default": "false"
                    },
                    "task_pid": {
                        "type": "string",
                        "title": "父级任务id",
                        "description": "父级任务id，为空时表示本身是一级任务"
                    },
                    "name": {
                        "type": "string",
                        "title": "任务名称"
                    },
                    "handler_id": {
                        "type": "string",
                        "title": "负责人id"
                    },
                    "begin_at": {
                        "type": "string",
                        "title": "计划开始时间（YYYY-MM-DD)",
                        "format": "date"
                    },
                    "end_at": {
                        "type": "string",
                        "title": "计划结束时间（YYYY-MM-DD）",
                        "format": "date"
                    },
                    "description": {
                        "type": "string",
                        "title": "任务描述"
                    }
                },
                "title": "请求参数内容",
                "required": [
                    "prj_id",
                    "draft",
                    "name",
                    "handler_id",
                    "begin_at",
                    "end_at"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: add,
    method_params: {},
    guards: [guard, check_parent_guard]
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
