import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {is_project_accessible} from "@src/utils/prj_premission_helper";

interface IData {
    /**
     * 是否是副本
     */
    draft: boolean;
    /**
     * 任务id
     */
    task_id: string;
}

// 接口访问守卫
function guard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
        let task = await model.findOne({where: {id: data.task_id}, raw: true});
        if (!task) return reject(Resp.gen_err(Resp.ResourceNotFound, '任务不存在。'));
        if (!await is_project_accessible(user, task.prj_id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限访问该项目。'));
        resolve();
    });
}

function get_check_flow(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
            let task = await model.findOne({
                where: {
                    id: data.task_id
                }
            })
            if(!task) return reject(Resp.gen_err(Resp.ResourceNotFound));
            resolve({
                checkers: task.checkers
            });
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "draft": {
                        "type": "boolean",
                        "title": "是否是副本",
                        "default": "false"
                    },
                    "task_id": {
                        "type": "string",
                        "title": "任务id"
                    }
                },
                "title": "请求参数内容",
                "required": [
                    "task_id",
                    "draft"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get_check_flow,
    method_params: {},
    guards: [guard]
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
