import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import { WhereOptions} from "sequelize";
import DataCURD, {ISQLReplacements} from "@core/DataCURD";
import {BizCustomer} from "@core-models/BizCustomer";
import {PrjLogs} from "@core-models/PrjLogs";
import {is_project_accessible} from "../../../../utils/prj_premission_helper";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";

interface IData  {
    /**
     * 查询起始时间，YYYY-MM-DD格式
     */
    begin_at?: string;
    /**
     * 查询结束时间，YYYY-MM-DD格式
     */
    end_at?: string;
    /**
     * 项目id
     */
    prj_id: string;
    /**
     * 任务id
     */
    task_id: string;
    /**
     * 页码，从1开始
     */
    page_no: number;
    /**
     * 每页条数
     */
    page_size: number;
}

// 接口访问守卫
function guard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!await is_project_accessible(user, data.prj_id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限访问该项目。'));
        resolve();
    });
}

function get_logs(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let count_sql = `select count(distinct log.id) `;
            let select_sql = `
                select log.id, log.creator_id, log.creator_name,
                    log.prj_id, log.task_id, log.content,
                    TO_CHAR(log.ts, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as ts
            `;
            let condition = `
                from ${PrjLogs.table_name} log
                where log.prj_id = :prj_id and log.task_id = :task_id
            `;
            let replacements: ISQLReplacements = {prj_id: data.prj_id, task_id: data.task_id};

            if (data.begin_at !== undefined) {
                condition += ' and log.ts >= :begin_at ';
                replacements.begin_at = data.begin_at + ' 00:00:00';
            }
            if (data.end_at !== undefined) {
                condition += ' and log.ts <= :end_at ';
                replacements.end_at = data.end_at + ' 23:59:59';
            }

            count_sql += condition;
            select_sql += condition;
            select_sql += `
                order by log.ts desc`;
            let result = await DataCURD.get_page_list({
                sequelize: BizCustomer.sequelize!,
                page_no: data.page_no,
                page_size: data.page_size,
                count_sql: count_sql,
                count_replacements: replacements,
                select_sql: select_sql,
                select_replacements: replacements
            });
            let list = DataCURD.filter_null(result);
            resolve(list);
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "page_no": {
                        "type": "integer",
                        "title": "页码",
                        "description": "从1开始"
                    },
                    "page_size": {
                        "type": "integer",
                        "title": "每页条数"
                    },
                    "begin_at": {
                        "type": "string",
                        "title": "查询起始时间",
                        "description": "YYYY-MM-DD格式",
                        "format": "date"
                    },
                    "end_at": {
                        "type": "string",
                        "title": "查询结束时间",
                        "description": "YYYY-MM-DD格式",
                        "format": "date"
                    },
                    "prj_id": {
                        "type": "string",
                        "title": "项目id"
                    },
                    "task_id": {
                        "type": "string",
                        "title": "任务id"
                    }
                },
                "x-apifox-orders": [
                    "page_no",
                    "page_size",
                    "prj_id",
                    "task_id",
                    "begin_at",
                    "end_at"
                ],
                "title": "请求参数内容",
                "required": [
                    "prj_id",
                    "page_no",
                    "page_size",
                    "task_id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "examples": [
                    "1.0"
                ]
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理",
                "title": "UTC时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "sign",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get_logs,
    method_params: {},
    guards: [guard]
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
