import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {QueryTypes} from "sequelize";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import DataCURD from "@core/DataCURD";
import {BpmnWork} from "@core-models/BpmnWork";
import {BpmnCase} from "@core-models/BpmnCase";
import {BpmnModel} from "@core-models/BpmnModel";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import {PrjTaskOutcomeDraft} from "@core-models/PrjTaskOutcomeDraft";
import {ChangeMarker} from "@src/utils/define";
import {Logger} from "@util/Logger";
import {clone} from "@util/JsonToolkit";
import {is_project_accessible} from "@src/utils/prj_premission_helper";

interface IData {
    /**
     * 是否是副本
     */
    draft: boolean;
    /**
     * 项目id
     */
    prj_id: string;
}

const ui_status = {
    created: {
        name: "rw",
        progress: "r",
        status: "r",
        handler_name: "rw",
        begin_at: "rw",
        end_at: "rw",
        alter: false
    },
    reject_plan: {
        name: "rw",
        progress: "r",
        status: "r",
        handler_name: "rw",
        begin_at: "rw",
        end_at: "rw",
        alter: false
    },
    apply_plan: {
        name: "r",
        progress: "r",
        status: "r",
        handler_name: "r",
        begin_at: "r",
        end_at: "r",
        alter: false
    },
    overview_plan: {
        name: "r",
        progress: "r",
        status: "r",
        handler_name: "r",
        begin_at: "r",
        end_at: "r",
        alter: false
    },
    doing: {
        name: "r",
        progress: "rw",
        status: "r",
        handler_name: "r",
        begin_at: "r",
        end_at: "r",
        alter: true
    },
    reject_plan_alt: {
        name: "r",
        progress: "rw",
        status: "r",
        handler_name: "r",
        begin_at: "r",
        end_at: "r",
        alter: true
    },
    apply_plan_alt: {
        name: "r",
        progress: "rw",
        status: "r",
        handler_name: "r",
        begin_at: "r",
        end_at: "r",
        alter: true
    },
    review_plan_alt: {
        name: "r",
        progress: "r",
        status: "r",
        handler_name: "r",
        begin_at: "r",
        end_at: "r",
        alter: true
    },
    checking: {
        name: "r",
        progress: "r",
        status: "r",
        handler_name: "r",
        begin_at: "r",
        end_at: "r",
        alter: false
    },
    deprecated: {
        name: "r",
        progress: "r",
        status: "r",
        handler_name: "r",
        begin_at: "r",
        end_at: "r",
        alter: false
    },
    done: {
        name: "r",
        progress: "r",
        status: "r",
        handler_name: "r",
        begin_at: "r",
        end_at: "r",
        alter: false
    }
}

function build_tasks(rows: Array<any>): Array<any> {
    const treeMap = {};
    const root: Array<any> = new Array();
    rows.forEach((item) => {
        const {task_id, pid} = item;
        treeMap[task_id] = {...item, sub_tasks: treeMap[task_id]?.sub_tasks || []};
        if (pid) {
            if (!treeMap[pid]) {
                treeMap[pid] = {sub_tasks: []};
            }
            treeMap[pid].sub_tasks.push(treeMap[task_id]);
        } else {
            root.push(treeMap[task_id]);
        }
    })

    return root;
}

/// 计划草稿只能在计划开始实施后才能获取
function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();

        let prj_info = await PrjInfo.findOne({where: {id: data.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound));
        if (!await is_project_accessible(user, data.prj_id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限访问该项目。'));

        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));
        if (phase.order_index < 20) return reject(Resp.gen_err(Resp.InvalidFlow, '需要先对项目进行立项，立项后才可制订计划。'));
        // if (phase.order_index >= 30 && phase.order_index <= 40 && data.draft === true)
        //     return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划审核中，当前阶段不允许变更计划。'));
        // if (phase.order_index >= 30 && phase.order_index <= 40 && data.draft === true)
        //     return reject(Resp.gen_err(Resp.InvalidFlow, '项目进入收尾阶段，当前阶段不允许变更计划。'));
        // if (phase.order_index >= 45 && data.draft === false) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许直接修改计划，应在草稿中修改后重新提交审核。'));

        // 项目执行阶段，可修改计划，但只能修改草稿，如草稿还没有生成，则自动生成，如果已经生成，则更新草稿中的进度和状态
        if (phase.order_index >= 60 && phase.order_index <= 70 && data.draft === true) {
            let t = await PrjPlanTaskDraft.sequelize!.transaction();
            try {
                let count = await PrjPlanTaskDraft.count({where: {prj_id: data.prj_id}, transaction: t});
                if (count === 0) {
                    await PrjPlanTaskDraft.sequelize!.query(`
                        INSERT INTO ${PrjPlanTaskDraft.table_name} 
                            (id, name, pid, prj_id, progress, status, handler_id, creator_id, begin_at, end_at, completed_at,created_at,
                            updated_at, description, order_id, version, change_marker, checkers)
                        SELECT id, name, pid, prj_id, progress, status, handler_id, creator_id, begin_at, end_at, completed_at,created_at,
                            updated_at, description, order_id, version, 0 as change_marker, checkers
                        FROM ${PrjPlanTask.table_name}
                        WHERE prj_id = :prj_id;
                    `, {type: QueryTypes.INSERT, transaction: t, replacements: {prj_id: data.prj_id}});

                    await PrjTaskOutcome.sequelize!.query(`
                        INSERT INTO ${PrjTaskOutcomeDraft.table_name}
                            (id, version, name, task_id, status, creator_id, plan_time,
                            completed_at, created_at, updated_at, description, order_id, uploaded)
                        SELECT outcome.id, outcome.version, outcome.name, 
                            outcome.task_id, outcome.status, outcome.creator_id, outcome.plan_time,
                            outcome.completed_at, outcome.created_at, outcome.updated_at, outcome.description, 
                            outcome.order_id, outcome.uploaded
                            FROM ${PrjTaskOutcome.table_name} outcome, ${PrjPlanTask.table_name} task
                            WHERE task.prj_id = :prj_id and outcome.task_id = task.id;
                        `, {type: QueryTypes.INSERT, transaction: t, replacements: {prj_id: data.prj_id}});
                } else {
                    // 修改草稿中的状态
                    await PrjPlanTask.sequelize!.query(`
                        UPDATE ${PrjPlanTaskDraft.table_name} 
                        SET progress = B.progress, status = B.status
                        FROM ${PrjPlanTask.table_name} B
                        WHERE ${PrjPlanTaskDraft.table_name}.id = B.id and B.prj_id = :prj_id;
                    `, {type: QueryTypes.UPDATE, transaction: t, replacements: {prj_id: data.prj_id}});
                    await PrjTaskOutcome.sequelize!.query(`
                        UPDATE ${PrjTaskOutcomeDraft.table_name} 
                        SET  status = B.status, updated_at = B.updated_at, uploaded = B.uploaded, completed_at = B.completed_at
                        FROM ${PrjTaskOutcome.table_name} B, ${PrjPlanTask.table_name} C
                        WHERE ${PrjTaskOutcomeDraft.table_name}.id = B.id and 
                            B.task_id = C.id and C.prj_id = :prj_id;
                        `, {type: QueryTypes.UPDATE, transaction: t, replacements: {prj_id: data.prj_id}});
                }
                await t.commit();
            } catch (e) {
                await t.rollback();
                return reject(Resp.throw_err(e));
            }
        }
        resolve();
    });
}

function get_tasks(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let prj = await PrjInfo.findOne({where: {id: data.prj_id}, raw: true});
            if (!prj) return reject(Resp.gen_err(Resp.DataExists, `项目不存在，id：${data.prj_id}`));
            let phase = await PrjPhaseDefine.findOne({where: {id: prj.phase_id}, raw: true});
            let result: any = {
                status: phase!.order_index,
                tasks: new Array<any>(),
                ui_status: data.draft ? {
                    name: "rw",
                    progress: "r",
                    status: "r",
                    handler_name: "rw",
                    begin_at: "rw",
                    end_at: "rw"
                } : clone(ui_status[prj.phase_id]),
            }

            if (cached_data.user_id !== prj.leader_id && prj.phase_id === 'doing') {
                result.ui_status.alter = false;
            }
            let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
            let sql = `
                select task.id as task_id, task.name, task.progress, task.status, 
                    task.handler_id, handler.name as handler_name, task.pid,
                    TO_CHAR(task.begin_at, 'yyyy-MM-dd') as begin_at,
                    TO_CHAR(task.end_at, 'yyyy-MM-dd') as end_at,
                    TO_CHAR(task.completed_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as completed_at
                    `
            if (data.draft) {
                sql += `, task.change_marker
                    from ${model.table_name} task
                    left join ${AcsUserInfo.table_name} handler on handler.id = task.handler_id
                    where task.prj_id = :prj_id 
                     order by task.order_id asc
                `;
            } else {
                sql += `,
                    --case when NULLIF(array_agg(work.id), '{NULL}') is null then '{}' else
                    --   array_agg(json_build_object('id', work.id, 'name', work.name, 'form', work.form))
                    --       end as works
                    array_agg(
                        case when work.id is null then null
                            else json_build_object('id', work.id, 'name', work.name,'form', work.form)
                        end
                    ) FILTER (WHERE case when work.id is null then null
                            else json_build_object('id', work.id, 'name', work.name,'form', work.form)
                        end IS NOT NULL) as works
                           
                    from ${model.table_name} task
                    left join ${AcsUserInfo.table_name} handler on handler.id = task.handler_id
                    left join ${BpmnCase.table_name} flow on flow.prj_id = :prj_id
                    LEFT JOIN ${BpmnWork.table_name} work ON work.case_id = flow.id and
                        work.assigned_to = '${cached_data.user_id}' and 
                        work.status <> 2 and 
                        flow.model_id = 'task' and
                        work.task_id = task.id
                    where task.prj_id = :prj_id 
                    group by task.id, task.name, task.progress, task.status, 
                    task.handler_id, handler.name, task.pid,
                    task.begin_at,
                    task.end_at,
                    task.completed_at
                    order by task.order_id asc
                `
            }
            // sql += `
            //     from ${model.table_name} task
            //     left join ${AcsUserInfo.table_name} handler on handler.id = task.handler_id
            //     where task.prj_id = :prj_id
            //      order by task.order_id asc`;
            let tasks = await model.sequelize!.query(sql, {
                type: QueryTypes.SELECT, raw: true, replacements: {
                    prj_id: data.prj_id
                }
            })
            let works = await BpmnWork.sequelize!.query(`
                select work.id, work.name, work.form 
                from ${BpmnWork.table_name} work, ${BpmnCase.table_name} flow, ${BpmnModel.table_name} bpmn
                where work.case_id = flow.id and flow.model_id = bpmn.id and 
                    work.prj_id = :prj_id and work.status <> 2 and work.process_type = 1 and
                    bpmn.id = :bpmn_id and work.assigned_to = :assigned_to
                `, {
                type: QueryTypes.SELECT,
                raw: true,
                replacements: {prj_id: data.prj_id, bpmn_id: data.draft? 'plan_alter' : 'plan', assigned_to: cached_data.user_id}
            });
            result.works = works;

            if (!tasks) return resolve(result);
            tasks = DataCURD.filter_null(tasks);
            result.tasks = build_tasks(tasks);


            resolve(result);
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "prj_id": {
                        "type": "string",
                        "title": "项目id"
                    },
                    "draft": {
                        "type": "boolean",
                        "title": "是否是副本",
                        "default": "false"
                    }
                },
                "title": "请求参数内容",
                "required": [
                    "prj_id",
                    "draft"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get_tasks,
    method_params: {},
    guards: [statusGuard]
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
