import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import DataCURD from "@core/DataCURD";
import {Op, Transaction, WhereOptions} from "sequelize";
import {PrjInfo} from "@core-models/PrjInfo";
import dayjs, {Dayjs} from "dayjs";
import {Resp} from "@util/Resp";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {start_plan_alt_flow, update_parent_task_info} from "../../../../utils/plan_task_helper";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {Logger} from "@util/Logger";
import {ChangeMarker, TaskStatus} from "@src/utils/define";
import {is_project_task_modifiable} from "@src/utils/prj_premission_helper";

interface IData {
    /**
     * 计划开始时间（YYYY-MM-DD)
     */
    begin_at?: string;
    /**
     * 任务描述
     */
    description?: string;
    /**
     * 是否是副本
     */
    draft: boolean;
    /**
     * 计划结束时间（YYYY-MM-DD）
     */
    end_at?: string;
    /**
     * 负责人id
     */
    handler_id?: string;
    /**
     * 任务名称
     */
    name?: string;
    /**
     * 任务id，任务id，为空时表示本身是一级任务
     */
    task_id: string;

    [property: string]: any;
}

function time_guard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let data = <IData>json.data;

        let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
        let task = await model.findOne({where: {id: data.task_id}, raw: true});
        if (!task) return reject(Resp.gen_err(Resp.ResourceNotFound));
        let begin_at = data.begin_at ? data.begin_at : task.begin_at;
        let end_at = data.end_at ? data.end_at : task.end_at;
        if (dayjs(end_at).diff(begin_at) < 0) {
            return reject(Resp.gen_err(Resp.IIllegalRequest, '任务预计完成时间不能早于或等于任务开始时间。'));
        }

        resolve();
    });
}

function guard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();
        let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
        let task = await model.findOne({where: {id: data.task_id}, raw: true});
        if (!task)
            return reject(Resp.gen_err(Resp.ResourceNotFound, '计划任务项不存在。'));
        if (!await is_project_task_modifiable(model, user, task.id))
            return reject(Resp.gen_err(Resp.Forbidden, `您没有权限修改任务 ${task.id}。`));

        let prj_info = await PrjInfo.findOne({where: {id: task.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound));
        cached_data.started_at = prj_info.started_at;   // 缓存项目的开始时间
        cached_data.deliver_at = prj_info.deliver_at; // 缓存项目的交付时间

        // 检查用户是否是项目负责人，只有项目负责人才可以修改任务
        if (cached_data.user_id !== prj_info.leader_id) {
            return reject(Resp.gen_err(Resp.Forbidden, '您不是项目负责人，不能修改任务。'));
        }
        if (task.status === TaskStatus.Completed || task.status === TaskStatus.Cancelled) {
            return reject(Resp.gen_err(Resp.InvalidFlow, '任务已完成或已取消，不允许修改。'));
        }
        if (data.draft) return resolve();/// 草稿允许随意修改

        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));
        if (phase.order_index < 20) return reject(Resp.gen_err(Resp.InvalidFlow, '需要先对项目进行立项，立项后才可制订计划。'));
        if (phase.order_index >= 30 && phase.order_index <= 40 && data.draft === false)
            return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划审核中，当前阶段不允许修改。'));
        if (phase.order_index >= 60 && phase.order_index <= 70 && data.draft === false) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许直接修改计划，应在草稿中修改后重新提交审核。'));
        if (phase.order_index >= 80) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许修改计划。'));

        resolve();
    });
}

async function get_where(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    let data = <IData>json.data;
    return {id: data.task_id};
}

function modify(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        let data = <IData>json.data;
        let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
        let t = await model.sequelize!.transaction();
        try {
            let task = await model.findOne({where: {id: data.task_id}, raw: true, transaction: t});
            if (!task) throw Resp.gen_err(Resp.ResourceNotFound);
            // 检查是否是最后一级
            let sub_task = await model.findOne({where: {pid: data.task_id}, transaction: t});
            if (sub_task && (data.handler_id || data.begin_at || data.end_at))
                throw Resp.gen_err(Resp.IIllegalRequest, '只有最后一级任务可以修改主责人和时间。');

            if (data.begin_at) data.begin_at += ' 00:00:00';
            if (data.end_at) data.end_at += ' 23:59:59';
            // 检查起止时间是否合法
            let begin_at = data.begin_at || task.begin_at;
            let end_at = data.end_at || task.end_at;
            // 检查计划开始时间不能早于项目立项时间
            if (dayjs(begin_at).diff(dayjs(cached_data.started_at)) < 0) {
                return reject(Resp.gen_err(Resp.ParamsError, `计划开始时间不能早于项目立项时间 ${dayjs(cached_data.started_at).format('YYYY-MM-DD')}。`));
            }
            // 检查计划结束时间不能晚于项目交付时间
            if (dayjs(data.end_at).diff(dayjs(cached_data.deliver_at)) > 0) {
                return reject(Resp.gen_err(Resp.ParamsError, `计划结束时间不能晚于项目交付时间 ${dayjs(cached_data.deliver_at).format('YYYY-MM-DD')}。`));
            }

            if (dayjs(end_at).diff(dayjs(begin_at)) <= 0)
                throw Resp.gen_err(Resp.ParamsError, '任务结束时间必须晚于开始时间。');

            if (data.draft) {
                Logger.trace(`[prj-plan-task-modify] draft task:  ${data.task_id}, change_marker: ${(<PrjPlanTaskDraft>task).change_marker}`);
                if ((<PrjPlanTaskDraft>task).change_marker !== ChangeMarker.Added) {
                    data.change_marker = ChangeMarker.Changed;
                }
            }
            let result = await DataCURD.internal_update_row(json, {
                model: model,
                where: get_where,
                fixed_input_data: {
                    updated_at: dayjs().format('YYYY-MM-DD HH:mm:ss')
                },
                input_data_map: {
                    name: "name",
                    handler_id: "handler_id",
                    begin_at: "begin_at",
                    end_at: "end_at",
                    description: "description",
                    change_marker: "change_marker",
                    updated_at: "updated_at"
                }
            }, cached_data, t);

            await update_parent_task_info(data.draft, result.prj_id, result.pid, t as Transaction);

            /// 如果计划任务是草稿，且还没有生成计划变更任务，则生成任务
            if (data.draft === true ) {
                await start_plan_alt_flow(result.prj_id, cached_data.user_id, t);
            }

            await t.commit();
            resolve(result);
        } catch (e) {
            await t.rollback();
            reject(e);
        }
    })
}

async function before_respond(content: IRequest, params: IMethodParams, cached_data: ICachedData | any, result: any): Promise<unknown> {
    result = DataCURD.filter_null(result, new Map(Object.entries({
        "updated_at": "YYYY-MM-DD HH:mm:ss",
        "created_at": "YYYY-MM-DD HH:mm:ss",
        "begin_at": "YYYY-MM-DD HH:mm:ss",
        "end_at": "YYYY-MM-DD HH:mm:ss"
    })));
    return result;
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "draft": {
                        "type": "boolean",
                        "title": "是否是副本",
                        "default": "false"
                    },
                    "name": {
                        "type": "string",
                        "title": "任务名称"
                    },
                    "handler_id": {
                        "type": "string",
                        "title": "负责人id"
                    },
                    "end_at": {
                        "type": "string",
                        "title": "计划结束时间（YYYY-MM-DD）"
                    },
                    "description": {
                        "type": "string",
                        "title": "任务描述"
                    },
                    "begin_at": {
                        "type": "string",
                        "title": "计划开始时间（YYYY-MM-DD)"
                    },
                    "task_id": {
                        "type": "string",
                        "title": "任务id",
                        "description": "任务id，为空时表示本身是一级任务"
                    }
                },
                "title": "请求参数内容",
                "required": [
                    "draft", "task_id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: modify,
    on_before_respond: before_respond,
    method_params: {
        model: PrjPlanTask,
        where: get_where,
        fixed_input_data: {
            updated_at: dayjs().format('YYYY-MM-DD HH:mm:ss')
        },
        input_data_map: {
            name: "name",
            handler_id: "handler_id",
            begin_at: "begin_at",
            end_at: "end_at",
            description: "description"
        }
    },
    guards: [time_guard, guard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
