/// 取消删除操作
import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {PrjInfo} from "@core-models/PrjInfo";
import {Resp} from "@util/Resp";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {update_parent_task_info} from "@src/utils/plan_task_helper";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {ChangeMarker} from "@src/utils/define";
import {is_project_task_modifiable} from "@src/utils/prj_premission_helper";

interface IData {
    /**
     * 任务id
     */
    task_id: string;
}


/// 项目任务提交申请后，不允许修改
function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();

        let task = await PrjPlanTaskDraft.findOne({where: {id: data.task_id}, raw: true});
        if (!task)
            return reject(Resp.gen_err(Resp.ResourceNotFound, '计划任务项不存在。'));

        let prj_info = await PrjInfo.findOne({where: {id: task.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound));
        // 检查用户是否是项目负责人，只有项目负责人才可以恢复任务
        // if (cached_data.user_id !== prj_info.leader_id) {
        //     return reject(Resp.gen_err(Resp.Forbidden, '您不是项目负责人，不能恢复任务。'));
        // }

        if (!await is_project_task_modifiable(PrjPlanTask, user, task.id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限恢复任务。'));

        if (task.change_marker !== ChangeMarker.Deleted) {
            return reject(Resp.gen_err(Resp.InvalidFlow, '任务没有被删除，不用恢复。'));
        }
        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));


        if (phase.order_index >= 60 && phase.order_index <= 65) return reject(Resp.gen_err(Resp.InvalidFlow, '计划变更已提交审核，当前阶段不允许删除。'));
        return resolve();/// 其它情况下草稿允许随意删除

        resolve();
    });
}

async function recover_task(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    let data = <IData>json.data;
    let t = await PrjInfo.sequelize!.transaction();
    try {
        let task = await PrjPlanTaskDraft.findOne({where: {id: data.task_id}, raw: true, transaction: t});
        if (!task) throw Resp.gen_err(Resp.ResourceNotFound);

        await PrjPlanTaskDraft.update({change_marker: ChangeMarker.NoChange}, {
            where: {id: data.task_id},
            transaction: t,
            returning: false
        });

        await update_parent_task_info(true, task.prj_id, task.pid, t);
        await t.commit();
    } catch (e) {
        await t.rollback();
        throw Resp.throw_err(e);
    }
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "task_id": {
                        "type": "string",
                        "title": "任务id"
                    }
                },
                "x-apifox-orders": [
                    "task_id"
                ],
                "title": "请求参数内容",
                "required": [
                    "task_id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: recover_task,
    method_params: {},
    guards: [statusGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
