import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjMembers} from "@core-models/PrjMembers";
import {Resp} from "@util/Resp";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {start_plan_alt_flow, update_parent_task_info} from "@src/utils/plan_task_helper";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import {Op} from "sequelize";
import {PrjTaskOutcomeDraft} from "@core-models/PrjTaskOutcomeDraft";
import {ChangeMarker, TaskStatus} from "@src/utils/define";
import {is_project_task_modifiable} from "@src/utils/prj_premission_helper";

interface IData {
    /**
     * 是否是副本
     */
    draft: boolean;
    /**
     * 任务id
     */
    task_id: string;

    [property: string]: any;
}


/// 项目任务提交申请后，不允许修改
function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();
        let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
        let task = await model.findOne({where: {id: data.task_id}, raw: true});
        if (!task)
            return reject(Resp.gen_err(Resp.ResourceNotFound, '计划任务项不存在。'));

        let prj_info = await PrjInfo.findOne({where: {id: task.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound));
        if (!await is_project_task_modifiable(model, user, task.id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限删除任务。'));

        if (task.status === TaskStatus.Completed || task.status === TaskStatus.Cancelled) {
            return reject(Resp.gen_err(Resp.InvalidFlow, '任务已完成或已取消，不允许删除。'));
        }
        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));

        if (data.draft) {
            if (phase.order_index >= 60 && phase.order_index <= 65) return reject(Resp.gen_err(Resp.InvalidFlow, '计划变更已提交审核，当前阶段不允许删除。'));
            return resolve();/// 其它情况下草稿允许随意删除
        }

        // 任务已启动，有进度或已提交了交付物，不允许删除
        let outcome_model = data.draft ? PrjTaskOutcomeDraft : PrjTaskOutcome;
        let outcome = await outcome_model.findOne({where: {uploaded: true, task_id: data.task_id}, raw: true});
        if (task.progress > 0 || outcome) return reject(Resp.gen_err(Resp.InvalidFlow, '任务已启动，不允许删除。'));



        if (phase.order_index < 20) return reject(Resp.gen_err(Resp.InvalidFlow, '需要先对项目进行立项，立项后才可制订计划。'));
        if (phase.order_index >= 30 && phase.order_index <= 40 && data.draft === false)
            return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划审核中，当前阶段不允许删除。'));
        if (phase.order_index >= 60 && phase.order_index <= 70 && data.draft === false)
            return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许直接删除计划，应在草稿中删除后重新提交审核。'));
        if (phase.order_index >= 80) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许删除计划。'));
        resolve();
    });
}

async function remove(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    let data = <IData>json.data;
    let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
    let t = await PrjInfo.sequelize!.transaction();
    try {
        let task = await model.findOne({where: {id: data.task_id}, raw: true, transaction: t});
        if (!task) throw Resp.gen_err(Resp.ResourceNotFound);
        let child = await model.findOne({where: {pid: data.task_id}, raw: true, transaction: t});
        if (child) throw Resp.gen_err(Resp.IIllegalRequest, '计划任务项有子任务，无法删除。');

        if (data.draft) {
            await model.update({change_marker: ChangeMarker.Deleted}, {
                where: {id: data.task_id},
                transaction: t,
                returning: false
            });
            /// 如果计划任务是草稿，且还没有生成计划变更任务，则生成任务
            await start_plan_alt_flow(task.prj_id, cached_data.user_id, t);

        } else {
            await model.destroy({where: {id: data.task_id}, transaction: t});
        }
        await update_parent_task_info(data.draft, task.prj_id, task.pid, t);
        await t.commit();
    } catch (e) {
        await t.rollback();
        throw Resp.throw_err(e);
    }
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "draft": {
                        "type": "boolean",
                        "title": "是否是副本",
                        "default": "false"
                    },
                    "task_id": {
                        "type": "string",
                        "title": "任务id"
                    }
                },
                "x-apifox-orders": [
                    "task_id",
                    "draft"
                ],
                "title": "请求参数内容",
                "required": [
                    "task_id",
                    "draft"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: remove,
    method_params: {},
    guards: [statusGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
