import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {ChangeMarker, TaskStatus} from "@src/utils/define";
import {start_plan_alt_flow} from "@src/utils/plan_task_helper";
import {is_project_task_modifiable} from "@src/utils/prj_premission_helper";

interface IData {
    /**
     * 审核流程
     */
    checkers: Checker[];
    /**
     * 是否是副本
     */
    draft: boolean;
    /**
     * 任务id
     */
    task_id: string;
}

export interface Checker {
    /**
     * 超时时限（天），超时后产生提醒
     */
    due_day: number;
    /**
     * 审核人账号列表
     */
    handlers: string[];
    /**
     * 审核名，互校、二审、终审
     */
    name: string;
}

/// 项目任务提交申请后，不允许修改
function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();
        let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
        let task = await model.findOne({where: {id: data.task_id}, raw: true});
        if (!task)
            return reject(Resp.gen_err(Resp.ResourceNotFound, '计划任务项不存在。'));


        if (data.draft) return resolve();/// 草稿允许随意修改
        let prj_info = await PrjInfo.findOne({where: {id: task.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound));

        if (!await is_project_task_modifiable(model, user, task.id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限设置任务审核流程。'));

        if (task.status >= TaskStatus.ApplyForReview)
            return reject(Resp.gen_err(Resp.InvalidFlow, '计划任务项已提交审核，不允许修改。'));

        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));
        if (phase.order_index < 20) return reject(Resp.gen_err(Resp.InvalidFlow, '需要先对项目进行立项，立项后才可制订计划。'));
        if (phase.order_index >= 30 && phase.order_index <= 40 && data.draft === false)
            return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划审核中，当前阶段不允许修改。'));
        if (phase.order_index >= 60 && phase.order_index <= 70 && data.draft === false) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许直接修改计划，应在草稿中修改后重新提交审核。'));
        if (phase.order_index >= 80) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许修改计划。'));

        resolve();
    });
}

function set_check_flow(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        let data = <IData>json.data;
        let t = await PrjPlanTaskDraft.sequelize!.transaction();
        try {
            /// 如果计划任务是草稿，且还没有生成计划变更任务，则生成任务
            if (data.draft === true ) {
                await PrjPlanTaskDraft.update({
                    checkers: data.checkers,
                    change_marker: ChangeMarker.Changed
                }, {where: {id: data.task_id}, transaction: t});
                let task = await PrjPlanTaskDraft.findOne({where: {id: data.task_id}, transaction: t, raw: true});
                if (task)
                    await start_plan_alt_flow(task.prj_id, cached_data.user_id, t);
            } else {
                await PrjPlanTask.update({checkers: data.checkers},
                    {where: {id: data.task_id}});
            }
            await t.commit();
            resolve({});
        } catch (e) {
            await t.rollback();
            reject(e);
        }
    })
}


const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "draft": {
                        "type": "boolean",
                        "title": "是否是副本",
                        "default": "false"
                    },
                    "checkers": {
                        "type": "array",
                        "items": {
                            "type": "object",
                            "properties": {
                                "name": {
                                    "type": "string",
                                    "title": "审核名",
                                    "description": "互校、二审、终审"
                                },
                                "handlers": {
                                    "type": "array",
                                    "items": {
                                        "type": "string"
                                    },
                                    "title": "审核人账号列表"
                                },
                                "due_day": {
                                    "type": "integer",
                                    "minimum": 1,
                                    "title": "超时时限（天）",
                                    "description": "超时后产生提醒"
                                }
                            },
                            "x-apifox-orders": [
                                "name",
                                "handlers",
                                "due_day"
                            ],
                            "required": [
                                "name",
                                "handlers",
                                "due_day"
                            ]
                        },
                        "title": "审核流程"
                    },
                    "task_id": {
                        "type": "string",
                        "title": "任务id"
                    }
                },
                "x-apifox-orders": [
                    "draft",
                    "task_id",
                    "checkers"
                ],
                "title": "请求参数内容",
                "required": [
                    "draft",
                    "task_id",
                    "checkers"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: set_check_flow,
    method_params: {
    },
    guards: [statusGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
