import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {PrjInfo} from "@core-models/PrjInfo";
import {update_parent_task_info} from "@src/utils/plan_task_helper";
import {TaskStatus} from "@src/utils/define";
import {create_task_approve_flow_if_not_exists} from "@src/utils/bpmn_work_helper";
import {is_project_task_progress_modifiable} from "@src/utils/prj_premission_helper";

interface IData {
    progress: number;
    /**
     * 任务id
     */
    task_id: string;
}

/// 计划审核通过前，进度不允许修改。计划审核通过后，在计划变更审核过程中，进度不允许修改。
function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();
        let task = await PrjPlanTask.findOne({where: {id: data.task_id}, raw: true});
        if (!task)
            return reject(Resp.gen_err(Resp.ResourceNotFound, '计划任务项不存在。'));

        // 检查用户是否是项目负责人或任务主责人，只有他们才可以对设置任务进度
        if (!await is_project_task_progress_modifiable(user, task.id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限修改任务进度。'));

        if (task.status === TaskStatus.ApplyForReview || task.status === TaskStatus.Reviewing) {
            return reject(Resp.gen_err(Resp.InvalidFlow, '计划任务项审核中，当前阶段不允许修改进度。'));
        }
        if (task.status === TaskStatus.DueDone || task.status === TaskStatus.Completed) {
            return reject(Resp.gen_err(Resp.InvalidFlow, '计划任务项已完成，当前阶段不允许修改进度。'));
        }

        let prj_info = await PrjInfo.findOne({where: {id: task.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound));

        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));
        if (phase.order_index < 20) return reject(Resp.gen_err(Resp.InvalidFlow, '需要先对项目进行立项，立项后才可制订计划。'));
        if (phase.order_index >= 30 && phase.order_index <= 40)
            return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划审核中，当前阶段不允许修改。'));
        if (phase.order_index >= 60 && phase.order_index <= 65) return reject(Resp.gen_err(Resp.InvalidFlow, '计划变更正在审核中，当前阶段不能修改进度。'));
        if (phase.order_index >= 80) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不能修改进度。'));
        resolve();
    });
}

function set_progress(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        let t = await PrjPlanTask.sequelize!.transaction();
        try {
            let data = <IData>json.data;

            // 检查是否是最后一级
            let sub_task = await PrjPlanTask.findOne({where: {pid: data.task_id}, transaction: t});
            if (sub_task) throw Resp.gen_err(Resp.IIllegalRequest, '只有最后一级任务可以修改进度。');
            // // 检查是否存在并更新
            let task = await PrjPlanTask.findOne({where: {id: data.task_id}, transaction: t, raw: true});
            if (!task) throw Resp.gen_err(Resp.ResourceNotFound, '任务不存在');
            let status = task.status;
            if (data.progress > 0 && status === TaskStatus.New) status = TaskStatus.Doing;
            else if (data.progress === 0) status = TaskStatus.New;

            if (data.progress === 100) {    // 不能直接设置任务完成度为100%，因为还有一个审核流程
                data.progress = 99;
            }
            await create_task_approve_flow_if_not_exists(task.prj_id, cached_data.user_id, data.task_id, t);

            let result = await PrjPlanTask.update({progress: data.progress, status: status}, {
                where: {id: data.task_id},
                transaction: t,
                returning: true
            });

            // let task = result[1][0];
            // await update_parent_task_progress(data.task_id, t);
            await update_parent_task_info(false, result[1][0].prj_id, result[1][0].pid, t);
            await t.commit();
            resolve({});
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "task_id": {
                        "type": "string",
                        "title": "任务id"
                    },
                    "progress": {
                        "type": "integer",
                        "minimum": 0,
                        "maximum": 100
                    }
                },
                "x-apifox-orders": [
                    "task_id",
                    "progress"
                ],
                "title": "请求参数内容",
                "required": [
                    "task_id",
                    "progress"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: set_progress,
    method_params: {},
    guards: [statusGuard]
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
