import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {QueryTypes} from "sequelize";
import {PrjInfo} from "@core-models/PrjInfo";
import {Resp} from "@util/Resp";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {IdGen} from "@util/IdGen";
import {start_plan_alt_flow, update_parent_task_info} from "@src/utils/plan_task_helper";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {ChangeMarker} from "@src/utils/define";
import {is_project_modifiable, is_project_task_modifiable} from "@src/utils/prj_premission_helper";

interface IData {
    /**
     * 是否是副本
     */
    draft: boolean;
    /**
     * 项目id
     */
    prj_id: string;
    /**
     * 调整后的任务id数组
     */
    task_ids: string[];
    /**
     * 父任务id，需要调整顺序的父任务id，为空时表示调整的是一级任务
     */
    task_pid?: string;
    [property: string]: any;
}


/// 项目任务提交申请后，只允许在副本中添加
function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        if (user === ADMINISTRATOR) return resolve();
        let prj_info = await PrjInfo.findOne({where: {id: data.prj_id}, raw: true});
        if (!prj_info) return reject(Resp.gen_err(Resp.ResourceNotFound));
        // 检查用户是否是项目负责人，只有项目负责人才可以对任务进行排序
        if (!await is_project_modifiable(user, prj_info.id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限对任务进行排序。'));

        let phase = await PrjPhaseDefine.findOne({where: {id: prj_info.phase_id}, raw: true});
        if (!phase) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目状态信息出错，id: ' + prj_info.id));
        // if (phase.order_index >= 30 && data.draft === false) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许直接修改计划，应在草稿中修改后重新提交审核。'));
        // if (phase.order_index < 20) return reject(Resp.gen_err(Resp.InvalidFlow, '需要先对项目进行立项，立项后才可制订计划。'));
        if (phase.order_index >=30 && phase.order_index <= 40 && data.draft === false)
            return reject(Resp.gen_err(Resp.InvalidFlow, '项目计划审核中，当前阶段不允许修改。'));
        if (phase.order_index >= 60 && phase.order_index <=70 && data.draft === false) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许直接修改计划，应在草稿中修改后重新提交审核。'));
        if (phase.order_index >= 80) return reject(Resp.gen_err(Resp.InvalidFlow, '当前项目阶段不允许修改计划。'));
        resolve();
    });
}

function sort_task(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        let data = <IData>json.data;
        let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
        let t = await PrjPlanTask.sequelize!.transaction();
        try {

            if (data.task_ids.length <= 0) {
                await t.rollback();
                return resolve({});
            }

            let pid: string|undefined|null = data.task_pid;
            if (!pid) pid = null;

            for (const id of data.task_ids) {
                let task = await model.findOne({where: {id: id}, transaction: t});
                if (!task) continue;
                let old_pid = task.pid;

                let sql = `
                    update ${model.table_name} 
                    set order_id = :order_id, pid = :pid `;
                if (data.draft) {
                    if ((<PrjPlanTaskDraft>task).change_marker === ChangeMarker.NoChange) {
                        sql += `, change_marker = ${ChangeMarker.Changed} `
                    }
                }
                sql += `
                    where id = :id
                `;
                let replacements = {
                    order_id: IdGen.id(),
                    id: id,
                    pid: pid
                }
                await model.sequelize!.query(sql, {
                    type: QueryTypes.UPDATE,
                    transaction: t,
                    replacements: replacements});
                if (old_pid !== pid) {
                    await update_parent_task_info(data.draft, data.prj_id, task.pid, t);
                }
            }
            await update_parent_task_info(data.draft, data.prj_id, pid, t);
            /// 如果计划任务是草稿，且还没有生成计划变更任务，则生成任务
            if (data.draft === true ) {
                await start_plan_alt_flow(data.prj_id, cached_data.user_id, t);
            }
            await t.commit();
            resolve({});
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "draft": {
                        "type": "boolean",
                        "title": "是否是副本",
                        "default": "false"
                    },
                    "prj_id": {
                        "type": "string",
                        "title": "项目id"
                    },
                    "task_pid": {
                        "type": "string",
                        "title": "父任务id",
                        "description": "需要调整顺序的父任务id，为空时表示调整的是一级任务"
                    },
                    "task_ids": {
                        "type": "array",
                        "items": {
                            "type": "string",
                            "description": "任务id"
                        },
                        "title": "调整后的任务id数组"
                    }
                },
                "x-apifox-orders": [
                    "prj_id",
                    "draft",
                    "task_pid",
                    "task_ids"
                ],
                "title": "请求参数内容",
                "required": [
                    "draft",
                    "prj_id",
                    "task_ids"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: sort_task,
    method_params: {},
    guards: [statusGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
