import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {QueryTypes, WhereOptions} from "sequelize";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {PrjInfo} from "@core-models/PrjInfo";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {BpmnWork} from "@core-models/BpmnWork";
import DataCURD from "@core/DataCURD";
import {is_project_task_accessible} from "@src/utils/prj_premission_helper";

interface IData {
    /**
     * 是否是副本
     */
    draft: boolean;
    /**
     * 任务id
     */
    task_id: string;
}

// 接口访问守卫
function guard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!await is_project_task_accessible(user, data.task_id))
            return reject(Resp.gen_err(Resp.Forbidden, '您没有权限访问该项目。'));
        resolve();
    });
}

function task_detail(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let model = data.draft ? PrjPlanTaskDraft : PrjPlanTask;
            let sql = `
                select task.id as task_id, task.pid,
                    task.prj_id, prj.name as prj_name,
                    task.name, task.progress, task.status, 
                    task.handler_id, handler.name as handler_name, 
                    task.creator_id, creator.name as creator_name,
                    TO_CHAR(task.begin_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as begin_at,
                    TO_CHAR(task.end_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as end_at,
                    TO_CHAR(task.updated_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as updated_at,
                    TO_CHAR(task.created_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as created_at,
                    TO_CHAR(task.completed_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as completed_at,
                    task.description, task.checkers 
                    `;
            if (data.draft) {
                sql += `
                    from ${model.table_name} task
                    left join ${PrjInfo.table_name} prj on prj.id = task.prj_id
                    left join ${AcsUserInfo.table_name} handler on handler.id = task.handler_id
                    left join ${AcsUserInfo.table_name} creator on creator.id = task.creator_id
                    where task.id = :task_id 
                `;
            } else {
                sql += `
                    , array_agg(
                            case when work.id is null then null
                                else json_build_object('id', work.id, 'name', work.name,'form', work.form)
                            end
                        ) FILTER (WHERE case when work.id is null then null
                                else json_build_object('id', work.id, 'name', work.name,'form', work.form)
                            end IS NOT NULL) as works
                    from ${model.table_name} task
                    left join ${PrjInfo.table_name} prj on prj.id = task.prj_id
                    left join ${AcsUserInfo.table_name} handler on handler.id = task.handler_id
                    left join ${AcsUserInfo.table_name} creator on creator.id = task.creator_id
                    LEFT JOIN ${BpmnWork.table_name} work ON 
                        work.assigned_to = '${cached_data.user_id}' and 
                        work.status <> 2 and 
                        work.task_id = task.id
                    where task.id = :task_id 
                    group by task.id, task.pid,
                    task.prj_id, prj.name,
                    task.name, task.progress, task.status, 
                    task.handler_id, handler.name, 
                    task.creator_id, creator.name,
                    task.begin_at,
                    task.end_at,
                    task.updated_at,
                    task.created_at, 
                    task.completed_at, 
                    task.description, task.checkers 
                    `;
            }

            let tasks = await model.sequelize!.query(sql, {type: QueryTypes.SELECT, raw: true, replacements: {
                    task_id: data.task_id
                }})
            if(!tasks) return reject(Resp.gen_err(Resp.ResourceNotFound));
            resolve(DataCURD.filter_null(tasks[0]));
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "draft": {
                        "type": "boolean",
                        "title": "是否是副本",
                        "default": "false"
                    },
                    "task_id": {
                        "type": "string",
                        "title": "任务id"
                    }
                },
                "title": "请求参数内容",
                "required": [
                    "task_id",
                    "draft"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: task_detail,
    method_params: {},
    guards: [guard]
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
