import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {WhereOptions} from "sequelize";
import {IdGen} from "@util/IdGen";
import dayjs from "dayjs";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjWeekReport} from "@core-models/PrjWeekReport";
import "dayjs/locale/zh-cn";
import weekOfYear from 'dayjs/plugin/weekOfYear';
import {PrjFile} from "@core-models/PrjFile";
dayjs.locale("zh-cn");
dayjs.extend(weekOfYear);
interface IData {
    /**
     * 多个附件的object name数组
     */
    attachments: string[];
    /**
     * issues
     */
    issue?: string;
    /**
     * 下周计划
     */
    plan: string;
    /**
     * 项目id
     */
    prj_id?: string;
    /**
     * 本周总结
     */
    summary: string;
}

/// 检查项目是否存在
function existsGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let data = <IData>json.data;
        let prj = await PrjInfo.findOne({where: {id: data.prj_id}, raw: true});
        if (!prj) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目不存在'));

        resolve();
    });
}

function add(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        let t = await PrjInfo.sequelize!.transaction();
        try {
            let data = <IData>json.data;
            let week = dayjs().week();
            let report = await PrjWeekReport.findOne({where: {prj_id: data.prj_id, week: week}, transaction: t});
            if (report) throw Resp.gen_err(Resp.DataExists, '本周周报已创建，不可重复创建。');
            let id = IdGen.id();
            let value: any = {
                id: id,
                prj_id: data.prj_id,
                summary: data.summary,
                plan: data.plan,
                issue: data.issue,
                created_at: dayjs().format('YYYY-MM-DD HH:mm:ss'),
                reporter_id: cached_data.user_id,
                year: dayjs().year(),
                month: dayjs().month()+1,
                week: week,
                begin_from: dayjs().day(1).format('YYYY-MM-DD'),
                end_to: dayjs().day(7).format('YYYY-MM-DD')
            }

            await PrjWeekReport.create(value, {transaction: t});
            for (let attachment of data.attachments) {
                await PrjFile.update({dependent_id: id}, {where: {id: attachment}, transaction: t})
            }
            await t.commit();
            resolve({id: id});
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "prj_id": {
                        "type": "string",
                        "title": "项目id"
                    },
                    "summary": {
                        "type": "string",
                        "title": "本周总结"
                    },
                    "plan": {
                        "type": "string",
                        "title": "下周计划"
                    },
                    "issue": {
                        "type": "string",
                        "title": "issues"
                    },
                    "attachments": {
                        "type": "array",
                        "items": {
                            "type": "string",
                            "description": "附件object name"
                        },
                        "description": "多个附件的object name数组"
                    }
                },
                "x-apifox-orders": [
                    "prj_id",
                    "summary",
                    "plan",
                    "issue",
                    "attachments"
                ],
                "title": "请求参数内容",
                "required": [
                    "summary",
                    "plan",
                    "attachments"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: add,
    method_params: {},
    guards: [existsGuard]
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
