import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {QueryTypes, WhereOptions} from "sequelize";
import {PrjInfo} from "@core-models/PrjInfo";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import DataCURD from "@core/DataCURD";
import {BizContractInfo} from "@core-models/BizContractInfo";
import {BizCustomer} from "@core-models/BizCustomer";
import {PrjWeekReport} from "@core-models/PrjWeekReport";
import {PrjWeekReportAttachment} from "@core-models/PrjWeekReportAttachment";
import dayjs from "dayjs";
import {PrjFile} from "@core-models/PrjFile";

interface IData {
    /**
     * 周报id
     */
    id: string;
}


function detail(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let report = await PrjWeekReport.sequelize!.query(`
                select report.id, report.summary, report.plan, report.issue, report.status,
                    report.reporter_id, reporter.name as reporter_name, 
                    report.prj_id, prj.name as prj_name,
                    report.year, report.month, report.week,
    
                    TO_CHAR(report.begin_from, 'yyyy-MM-dd') as begin_from,
                    TO_CHAR(report.end_to, 'yyyy-MM-dd') as end_to,
                    TO_CHAR(report.created_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as created_at
                    
                from ${PrjWeekReport.table_name} report
                left join ${PrjInfo.table_name} prj on prj.id = report.prj_id
                left join ${AcsUserInfo.table_name} reporter on reporter.id = report.reporter_id
                where report.id = :report_id 
            `, {type: QueryTypes.SELECT, raw: true, replacements: {
                    report_id: data.id
                }})
            if(!report || report.length <= 0) return reject(Resp.gen_err(Resp.ResourceNotFound));
            let result = {
                ...DataCURD.filter_null(report[0]),
                attachments: []
            };
            let attachments = await PrjFile.findAll({
                where: {dependent_id: data.id, category_id: 'week_report', uploaded: true},
                order: [["uploaded_at", "asc"]],
            });
            for (let i = 0; i < attachments.length; i++) {
                result.attachments.push({
                    obj_id: attachments[i].id,
                    filename: attachments[i].filename,
                    uploaded_at: dayjs(attachments[i].uploaded_at).format('YYYY-MM-DD HH:mm:ss'),
                    size: attachments[i].size
                })
            }
            resolve(result);
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "id": {
                        "type": "string",
                        "title": "周报id"
                    }
                },
                "x-apifox-orders": [
                    "id"
                ],
                "title": "请求参数内容",
                "required": [
                    "id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: detail,
    method_params: {},
    guards: []
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
