import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Op, QueryTypes, WhereOptions} from "sequelize";
import DataCURD, {ISQLReplacements} from "@core/DataCURD";
import {BizCustomer} from "@core-models/BizCustomer";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {BizContractInfo} from "@core-models/BizContractInfo";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjWeekReport} from "@core-models/PrjWeekReport";


interface IData {
    /**
     * 查询起始，YYYY-MM-DD
     */
    begin_from?: Date;
    /**
     * 查询至，YYYY-MM-DD
     */
    end_to?: Date;
    /**
     * 页码
     */
    page_no: number;
    /**
     * 每页条数
     */
    page_size: number;
    /**
     * 精确通过项目查找合同
     */
    prj_id?: string;
    /**
     * 根据项目名称过滤（模糊匹配）
     */
    prj_name?: string;
    /**
     * 根据填报人过滤
     */
    reporter_id?: string;
    /**
     * 周数，一年中的第几周
     */
    week?: number;
    /**
     * 年份
     */
    year?: number;
}


function get_list(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let count_sql = `select count(distinct report.id) `;
            let select_sql = `
                select report.id, 
                    report.prj_id, prj.name as prj_name,
                    report.reporter_id, reporter.name as reporter_name,
                    report.summary, report.plan, report.issue,
                    report.year, report.month, report.week,
                    report.status,
                   
                    TO_CHAR(report.created_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as created_at,
                    TO_CHAR(report.begin_from, 'yyyy-MM-dd') as begin_from,
                    TO_CHAR(report.end_to, 'yyyy-MM-dd') as end_to
            `;
            let condition = `
                from ${PrjWeekReport.table_name} report
                left join ${PrjInfo.table_name} prj on prj.id = report.prj_id
                left join ${AcsUserInfo.table_name} reporter on reporter.id = report.reporter_id
                where 1=1 
            `;
            let replacements: ISQLReplacements = {};

            if (data.prj_name !== undefined) {
                condition += ` and prj.name like :keyword `;
                replacements.keyword = `%${data.prj_name}%`;
            }
            if (data.prj_id !== undefined) {
                condition += ` and prj.id = :prj_id `;
                replacements.prj_id = data.prj_id;
            }
            if (data.reporter_id !== undefined) {
                condition += ` and reporter.id = :reporter_id `;
                replacements.reporter_id = data.reporter_id;
            }

            if (data.begin_from !== undefined) {
                condition += ' and report.created_at >= :begin_from ';
                replacements.begin_from = data.begin_from + ' 00:00:00';
            }
            if (data.end_to !== undefined) {
                condition += ' and report.created_at <= :end_to ';
                replacements.end_to = data.end_to + ' 23:59:59';
            }
            if (data.year !== undefined) {
                condition += ` and report.year = :year`;
                replacements.year = data.year;
            }
            if (data.week !== undefined) {
                condition += ` and report.week = :week`;
                replacements.week = data.week;
            }

            count_sql += condition;
            select_sql += condition;
            select_sql += ' order by report.year desc, report.week desc, report.created_at desc  ';

            let result = await DataCURD.get_page_list({
                sequelize: PrjWeekReport.sequelize!,
                page_no: data.page_no,
                page_size: data.page_size,
                count_sql: count_sql,
                count_replacements: replacements,
                select_sql: select_sql,
                select_replacements: replacements
            });
            let list = DataCURD.filter_null(result);
            resolve(list);
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "page_no": {
                        "type": "integer",
                        "title": "页码"
                    },
                    "page_size": {
                        "type": "integer",
                        "title": "每页条数"
                    },
                    "prj_id": {
                        "type": "string",
                        "title": "精确通过项目查找合同"
                    },
                    "prj_name": {
                        "type": "string",
                        "title": "根据项目名称过滤（模糊匹配）"
                    },
                    "begin_from": {
                        "type": "string",
                        "title": "查询起始",
                        "description": "YYYY-MM-DD",
                        "format": "date"
                    },
                    "end_to": {
                        "type": "string",
                        "title": "查询至",
                        "description": "YYYY-MM-DD",
                        "format": "date"
                    },
                    "reporter_id": {
                        "type": "string",
                        "title": "根据填报人过滤"
                    },
                    "week": {
                        "type": "integer",
                        "title": "周数",
                        "description": "一年中的第几周"
                    },
                    "year": {
                        "type": "integer",
                        "title": "年份"
                    }
                },
                "x-apifox-orders": [
                    "page_no",
                    "page_size",
                    "prj_id",
                    "prj_name",
                    "reporter_id",
                    "year",
                    "week",
                    "begin_from",
                    "end_to"
                ],
                "title": "请求参数内容",
                "required": [
                    "page_no",
                    "page_size"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get_list,
    method_params: {}
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
