import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import DataCURD from "@core/DataCURD";
import {WhereOptions} from "sequelize";
import {PrjInfo} from "@core-models/PrjInfo";
import dayjs from "dayjs";
import {Resp} from "@util/Resp";
import {PrjPhaseDefine} from "@core-models/PrjPhaseDefine";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {PrjWeekReport} from "@core-models/PrjWeekReport";
import {PrjFile} from "@core-models/PrjFile";

interface IData {
    /**
     * 周报id
     */
    id: string;
    /**
     * 需要协调的问题
     */
    issue?: string;
    /**
     * 下周计划
     */
    plan?: string;
    /**
     * 本周总结
     */
    summary?: string;
    /**
     * 多个附件的object name数组
     */
    attachments: string[];
}

/// 周报提交后，不允许修改，只允许修改自己的周报
function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        let report = await PrjWeekReport.findOne({where: {id: data.id}, raw: true});
        if (!report) return reject(Resp.gen_err(Resp.ResourceNotFound));
        if (report.reporter_id !== user) return reject(Resp.gen_err(Resp.Forbidden, '只允许修改自己的周报。'));

        if (report.status > 0 ) return reject(Resp.gen_err(Resp.InvalidFlow, '周报已提交，不允许修改，在未审阅前可撤回后再修改。'));

        resolve();
    });
}

async function get_where(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    let data = <IData>json.data;
    return {id: data.id};
}

async function before_respond(content: IRequest, params: IMethodParams, cached_data: ICachedData | any, result: any): Promise<unknown> {
    result = DataCURD.filter_null(result, new Map(Object.entries({
        "created_at": "YYYY-MM-DD HH:mm:ss",
        "begin_from": "YYYY-MM-DD",
        "end_to": "YYYY-MM-DD"
    })));

    // let data = <IData>content.data;
    // if (!data.attachments) return result;
    // for (let attachment of data.attachments) {
    //     await PrjFile.update({dependent_id: data.id}, {where: {id: attachment}})
    // }
    return result;
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "id": {
                        "type": "string",
                        "title": "周报id"
                    },
                    "summary": {
                        "type": "string",
                        "title": "本周总结"
                    },
                    "plan": {
                        "type": "string",
                        "title": "下周计划"
                    },
                    "issue": {
                        "type": "string",
                        "title": "需要协调的问题"
                    }
                },
                "x-apifox-orders": [
                    "id",
                    "summary",
                    "plan",
                    "issue"
                ],
                "title": "请求参数内容",
                "required": [
                    "id"
                ],
                "anyOf": [
                    {
                        "required": [
                            "summary"
                        ]
                    },
                    {
                        "required": [
                            "plan"
                        ]
                    },
                    {
                        "required": [
                            "issue"
                        ]
                    }
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: DataCURD.internal_update_row,
    on_before_respond: before_respond,
    method_params: {
        model: PrjWeekReport,
        where: get_where,
        input_data_map: {
            summary: "summary",
            plan: "plan",
            issue: "issue"
        }
    },
    guards: [statusGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
