import {ADMINISTRATOR, IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {BizContractInfo} from "@core-models/BizContractInfo";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjWeekReport} from "@core-models/PrjWeekReport";

interface IData {
    /**
     * 周报id
     */
    id: string;
}

/// 周报提交后，不允许删除，只允许删除自己的周报
function statusGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let user = cached_data.user_id;
        let data = <IData>json.data;
        if (!user) return reject(Resp.gen_err(Resp.Forbidden));
        let report = await PrjWeekReport.findOne({where: {id: data.id}, raw: true});
        if (!report) return reject(Resp.gen_err(Resp.ResourceNotFound));
        if (report.reporter_id !== user) return reject(Resp.gen_err(Resp.Forbidden, '只允许删除自己的周报。'));

        if (report.status > 0 ) return reject(Resp.gen_err(Resp.InvalidFlow, '周报已提交，不允许删除，在未审阅前可撤回后再删除。'));

        resolve();
    });
}

async function remove(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    let data = <IData>json.data;
    let t = await PrjWeekReport.sequelize!.transaction();
    try {
        //TODO: 移除所有该周报的附件
        await PrjWeekReport.destroy({where: {id: data.id}, transaction: t});
        await t.commit();
    } catch (e) {
        await t.rollback();
        throw Resp.throw_err(e);
    }
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "id": {
                        "type": "string",
                        "title": "周报id"
                    }
                },
                "x-apifox-orders": [
                    "id"
                ],
                "title": "请求参数内容",
                "required": [
                    "id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: remove,
    method_params: {},
    guards: [statusGuard]
}

module.exports = {
    default: v1_0,
    v1_0: v1_0
}
