import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Oss} from "@util/Oss";
import {IdGen} from "@util/IdGen";
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import dayjs from "dayjs";
import {PrjFile} from "@core-models/PrjFile";
import {PrjInfo} from "@core-models/PrjInfo";

interface IData {
    /**
     * 文件名
     */
    filename: string;
    /**
     * 项目id
     */
    prj_id: string;
    /**
     * 周报id，当周报id为空时，表示周报还未创建时上传附件，用于新建周报时上传附件用，周报已创建后，修改时可直接上传附件，此时必须有周报id
     */
    report_id?: string;
}


// TODO: 所有文档都放在一个表中，记录项目id和文档类型

function get_upload_url(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise(async (resolve, reject) => {
        let t = await PrjTaskOutcome.sequelize!.transaction();
        try {
            let id = IdGen.id();
            let data = <IData>json.data;
            let object_name = `project/${data.prj_id}/week_report/${id}`;

            await PrjFile.create({
                category_id: 'week_report',
                prj_id: data.prj_id,
                req_upload_time: dayjs(),
                id: object_name,
                filename: data.filename,
                dependent_id: data.report_id ? data.report_id : null,
                creator_id: cached_data.user_id

            }, {transaction: t});

            let oss = Oss.get_instance('pmr-doc');
            let result = await oss.presignedPostPolicy(oss.bucket, object_name, 300);
            await t.commit();
            resolve({object_name: object_name, url: result.postURL, form_data: result.formData});
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    });
}

/// 检查项目是否存在
function existsGuard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let data = <IData>json.data;
        let prj = await PrjInfo.findOne({where: {id: data.prj_id}, raw: true});
        if (!prj) return reject(Resp.gen_err(Resp.ResourceNotFound, '项目不存在'));

        resolve();
    });
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "filename": {
                        "type": "string",
                        "title": "文件名"
                    },
                    "prj_id": {
                        "type": "string",
                        "title": "项目id"
                    },
                    "report_id": {
                        "type": "string",
                        "title": "周报id",
                        "description": "当周报id为空时，表示周报还未创建时上传附件，用于新建周报时上传附件用，周报已创建后，修改时可直接上传附件，此时必须有周报id"
                    }
                },
                "x-apifox-orders": [
                    "filename",
                    "prj_id",
                    "report_id"
                ],
                "title": "请求参数内容",
                "required": [
                    "filename",
                    "prj_id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get_upload_url,
    method_params: {},
    guards: [existsGuard]
};

module.exports = {
    default: v1_0,
    v1_0: v1_0
};
