import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {QueryTypes, WhereOptions} from "sequelize";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {PrjInfo} from "@core-models/PrjInfo";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import DataCURD from "@core/DataCURD";
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import {BpmnWork} from "@core-models/BpmnWork";
import dayjs from "dayjs";

interface IData  {
    /**
     * 工作id
     */
    id: string;
}


interface IResult {
    /**
     * 任务完成时间
     */
    completed_at?: string;
    /**
     * 待阅类任务的文本，待阅任务的详情待阅文本
     */
    desc?: string;
    /**
     * 过期时间
     */
    due_at?: string;
    /**
     * 任务表单，如果任务有相关表单要填写，这个用于定义表单的格式
     */
    form?: { [key: string]: any };
    /**
     * 任务id
     */
    id?: string;
    /**
     * 项目负责人id
     */
    leader_id?: string;
    /**
     * 项目负责人名称
     */
    leader_name?: string;
    /**
     * 任务名称
     */
    name?: string;
    /**
     * 任务相关id，相关参数，
     * * project-detail：项目详情预览
     * - prj_id: 项目id
     * * project-plan：任务清单预览
     * - prj_id: 项目id
     * * project-plan-draft：任务草稿预览
     * - prj_id: 项目id
     * * project-outcome：任务交付物详情
     * - prj_id: 项目id
     * - outcome_id：交付物id
     * * project-week-report：项目周报详情
     * - prj_id: 项目id
     * - report_id: 周报id
     */
    params?: { [key: string]: any };
    /**
     * 项目id
     */
    prj_id?: string;
    /**
     * 项目名称
     */
    prj_name?: string;
    /**
     * 处理类型，1=待办，2=待阅
     */
    process_type?: number;
    /**
     * 任务创建时间
     */
    started_at?: string;
    /**
     * 任务目标，project-detail：项目详情预览
     * project-plan：任务清单预览
     * project-plan-draft：任务草稿预览
     * project-outcome：任务交付物详情
     * project-week-report：项目周报详情
     */
    target_type?: string;
}


function detail(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<IResult> {
    return new Promise<IResult>(async (resolve, reject) => {
        let t = await BpmnWork.sequelize!.transaction();
        try {
            let data = <IData>json.data;
            let works = await BpmnWork.sequelize!.query(`
                select work.id, work.name, 
                    work.prj_id, prj.name as prj_name, prj.leader_id as leader_id, leader.name as leader_name,
                    work.process_type, work.status,
                    work.target_type, work.params, work.form,
                    
                    TO_CHAR(work.started_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as started_at,
                    TO_CHAR(work.completed_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as completed_at,
                    TO_CHAR(work.due_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as due_at,
                    work.desc
                from ${BpmnWork.table_name} work
                left join ${PrjInfo.table_name} prj on prj.id = work.prj_id
                left join ${AcsUserInfo.table_name} leader on leader.id = prj.leader_id
                where work.id = :work_id
            `, {type: QueryTypes.SELECT, raw: true, replacements: {
                    work_id: data.id,
                }, transaction: t});
            await BpmnWork.update({status: 2, completed_at: dayjs(), handler: cached_data.user_id}, {
                where: {id: data.id, process_type: 2},
                returning: false,
                transaction: t
            });
            if(!works) throw Resp.gen_err(Resp.ResourceNotFound);
            await t.commit();
            resolve(DataCURD.filter_null(works[0]));
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "id": {
                        "type": "string",
                        "title": "任务id"
                    }
                },
                "x-apifox-orders": [
                    "id"
                ],
                "title": "请求参数内容",
                "required": [
                    "id"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: detail,
    method_params: {},
    guards: []
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
