import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {QueryTypes, WhereOptions} from "sequelize";
import DataCURD, {ISQLReplacements} from "@core/DataCURD";
import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {BpmnWork} from "@core-models/BpmnWork";
import {PrjInfo} from "@core-models/PrjInfo";
interface IData {
    /**
     * 查询起始，按创建时间查询，YYYY-MM-DD
     */
    begin_from?: Date;
    /**
     * 查询至，按创建时间查询，YYYY-MM-DD
     */
    end_to?: Date;
    /**
     * 页码
     */
    page_no: number;
    /**
     * 每页条数
     */
    page_size: number;
    /**
     * 精确通过项目编号查找文档
     */
    prj_id?: string;
    /**
     * 工作状态，0=进行中，1=已过期，2=已完成
     */
    status?: number;
}


function get_list(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<WhereOptions> {
    return new Promise<WhereOptions>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let count_sql = `select count(distinct work.id) `;
            let select_sql = `
                select work.id, work.name, 
                    work.prj_id, prj.name as prj_name, prj.leader_id as leader_id, leader.name as leader_name,
                    work.process_type, work.status,
                    work.target_type, work.params, work.form,
                    
                    TO_CHAR(work.started_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as started_at,
                    TO_CHAR(work.completed_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as completed_at,
                    TO_CHAR(work.due_at, 'yyyy-MM-dd HH24' || ':' || 'MI' || ':' || 'ss') as due_at
            `;
            let condition = `
                from ${BpmnWork.table_name} work
                left join ${PrjInfo.table_name} prj on work.prj_id = prj.id
                left join ${AcsUserInfo.table_name} leader on leader.id = prj.leader_id
                where work.show_in_my_works = true and work.assigned_to = :user_id
            `;
            let replacements: ISQLReplacements = {user_id: cached_data.user_id};
            if (data.prj_id !== undefined) {
                condition += ` and work.prj_id like :prj_id `;
                replacements.prj_id = `%${data.prj_id}%`;
            }

            if (data.status !== undefined) {

                if (data.status === 3) {
                    condition += ` and work.status <> :status `;
                    replacements.status = 2;
                } else {
                    condition += ` and work.status = :status `;
                    replacements.status = data.status;
                }

            }

            if (data.begin_from !== undefined) {
                condition += ' and work.started_at >= :begin_from ';
                replacements.begin_from = data.begin_from + ' 00:00:00';
            }
            if (data.end_to !== undefined) {
                condition += ' and work.started_at <= :end_to ';
                replacements.end_to = data.end_to + ' 23:59:59';
            }

            count_sql += condition;
            select_sql += condition;
            select_sql += ' order by work.started_at desc ';
            let result = await DataCURD.get_page_list({
                sequelize: BpmnWork.sequelize!,
                page_no: data.page_no,
                page_size: data.page_size,
                count_sql: count_sql,
                count_replacements: replacements,
                select_sql: select_sql,
                select_replacements: replacements
            });
            let list = DataCURD.filter_null(result);
            resolve(list);
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "page_no": {
                        "type": "integer",
                        "title": "页码"
                    },
                    "page_size": {
                        "type": "integer",
                        "title": "每页条数"
                    },
                    "prj_id": {
                        "type": "string",
                        "title": "精确通过项目编号查找文档"
                    },
                    "begin_from": {
                        "type": "string",
                        "title": "查询起始",
                        "description": "按创建时间查询，YYYY-MM-DD",
                        "format": "date"
                    },
                    "end_to": {
                        "type": "string",
                        "title": "查询至",
                        "description": "按创建时间查询，YYYY-MM-DD",
                        "format": "date"
                    },
                    "status": {
                        "type": "integer",
                        "title": "任务状态",
                        "description": "0=进行中，1=已过期，2=已完成"
                    }
                },
                "x-apifox-orders": [
                    "page_no",
                    "page_size",
                    "prj_id",
                    "begin_from",
                    "end_to",
                    "status"
                ],
                "title": "请求参数内容",
                "required": [
                    "page_no",
                    "page_size"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get_list,
    method_params: {}
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
