import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {Oss} from "@util/Oss";
import {IdGen} from "@util/IdGen";
import {PrjInfo} from "@core-models/PrjInfo";
import dayjs from "dayjs";
import {PrjFile} from "@core-models/PrjFile";
import {PrjFileCategory} from "@core-models/PrjFileCategory";
import {BpmnWork} from "@core-models/BpmnWork";

interface IData {
    /**
     * 文件名
     */
    filename: string;
    /**
     * 工作项id
     */
    work_id: string;
    // [property: string]: any;
}

// TODO: 所有文档都放在一个表中，记录项目id和文档类型

function get_upload_url(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise(async (resolve, reject) => {
        let t = await PrjFile.sequelize!.transaction();
        try {
            let id = IdGen.id();
            let data = <IData>json.data;

            /// 检查工作项是否存在
            let work = await BpmnWork.findOne({where: {id: data.work_id}, raw: true});
            if (!work) return reject(Resp.gen_err(Resp.ResourceNotFound, "工作项不存在，id: "+ data.work_id));

            let object_name = `project/${work.prj_id}/intermediate/${id}`;

            await PrjFile.create({
                category_id: 'intermediate',
                prj_id: work.prj_id,
                req_upload_time: dayjs(),
                id: object_name,
                filename: data.filename,
                creator_id: cached_data.user_id,
                dependent_id: data.work_id
            }, {transaction: t});

            let oss = Oss.get_instance('pmr-doc');
            let result = await oss.presignedPostPolicy(oss.bucket, object_name, 300);
            await t.commit();
            resolve({object_name: object_name, url: result.postURL, form_data: result.formData});
        } catch (e) {
            await t.rollback();
            reject(Resp.throw_err(e));
        }
    });
}

/// 检查工作项是否存在
function exists_guard(json: IRequest, cached_data: ICachedData): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
        let data = <IData>json.data;
        let work = await BpmnWork.findOne({where: {id: data.work_id}, raw: true});
        if (!work) return reject(Resp.gen_err(Resp.ResourceNotFound, "工作项不存在，id: "+ data.work_id));
        resolve();
    });
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "work_id": {
                        "type": "string",
                        "title": "工作项id"
                    },
                    "filename": {
                        "type": "string",
                        "title": "文件名"
                    }
                },
                "x-apifox-orders": [
                    "work_id",
                    "filename"
                ],
                "title": "请求参数内容",
                "required": [
                    "work_id",
                    "filename"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "default": "1.0"
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "title": "UTC时间戳",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get_upload_url,
    method_params: {},
    guards: [exists_guard]
};

module.exports = {
    default: v1_0,
    v1_0: v1_0
};
