import {IApiProcessor, ICachedData, IMethodParams, IRequest} from "@core/Defined";
import {Resp} from "@util/Resp";
import {QueryTypes} from "sequelize";
import DataCURD, {ISQLReplacements} from "@core/DataCURD";
import {PrjMembers} from "@core-models/PrjMembers";
import {ReportPrjStat} from "@core-models/ReportPrjStat";
import dayjs from "dayjs";

interface IData {
    /**
     * 起始月份，YYYY-MM格式
     */
    begin_at: string;
    /**
     * 结束月份，YYYY-MM格式
     */
    end_at: string;
}

function get_list(json: IRequest, params: IMethodParams, cached_data: ICachedData): Promise<any> {
    return new Promise<any>(async (resolve, reject) => {
        try {
            let data = <IData>json.data;
            let sql = `
                select 
                    TO_CHAR(ts, 'yyyy-MM') as ts,
                    amount, undone
                from ${ReportPrjStat.table_name} 
                where ts >= :begin_at and ts <= :end_at
                order by ts asc
            `
            let replacements: ISQLReplacements = {
                begin_at: dayjs(data.begin_at).startOf('month').startOf('day').format('YYYY-MM-DD'),
                end_at: dayjs(data.end_at).endOf('month').endOf('day').format('YYYY-MM-DD'),
            };
            let result = await PrjMembers.sequelize!.query(sql, {type: QueryTypes.SELECT, replacements: replacements, raw: true});
            let list = DataCURD.filter_null(result);
            resolve({list: list});
        } catch (e) {
            reject(Resp.throw_err(e));
        }
    })
}

const v1_0: IApiProcessor = {
    schema: {
        "type": "object",
        "properties": {
            "data": {
                "type": "object",
                "properties": {
                    "begin_at": {
                        "type": "string",
                        "title": "起始月份",
                        "description": "YYYY-MM格式",
                        "format": "YYYY-MM"
                    },
                    "end_at": {
                        "type": "string",
                        "title": "结束月份",
                        "description": "YYYY-MM格式",
                        "format": "YYYY-MM"
                    }
                },
                "x-apifox-orders": [
                    "begin_at",
                    "end_at"
                ],
                "title": "请求参数内容",
                "required": [
                    "begin_at",
                    "end_at"
                ]
            },
            "ver": {
                "type": "string",
                "title": "版本号",
                "description": "文档中没有说明时，默认为1.0",
                "examples": [
                    "1.0"
                ]
            },
            "app_id": {
                "type": "string",
                "title": "应用id",
                "description": "平台分配给客户端的app id。一个应用一个id。"
            },
            "timestamp": {
                "type": "integer",
                "description": "UTC时间戳，精确到毫秒。平台对超过五分钟的消息，将做忽略处理",
                "title": "UTC时间戳"
            },
            "sign": {
                "type": "string",
                "title": "签名"
            },
            "token": {
                "type": "string",
                "title": "身份认证token",
                "description": "平台登录后分配的token"
            }
        },
        "x-apifox-orders": [
            "ver",
            "app_id",
            "timestamp",
            "token",
            "sign",
            "data"
        ],
        "required": [
            "data",
            "ver",
            "app_id",
            "timestamp",
            "token"
        ]
    },
    method: get_list,
    method_params: {}
}


module.exports = {
    default: v1_0,
    v1_0: v1_0
}
