import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {Resp} from "@util/Resp";
import {Logger} from "@util/Logger";
import {PrjInfo} from "@core-models/PrjInfo";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import {PrjTaskOutcomeDraft} from "@core-models/PrjTaskOutcomeDraft";
import {QueryTypes, Sequelize, Transaction} from "sequelize";
import {ChangeMarker, TaskStatus} from "@src/utils/define";
import {BpmnModel} from "@core-models/BpmnModel";
import {FlowEngine} from "@src/bpmn/flow_engine";
import {BpmnCase} from "@core-models/BpmnCase";
import dayjs from "dayjs";
import {PrjLogger} from "@src/utils/prj_logger";
import {AcsUserInfo} from "@core-models/AcsUserInfo";


export async function bpmn_flow_on_end(id: string, model_id: string, prj_id: string, owner: string): Promise<void> {
    let user = await AcsUserInfo.findOne({where: {id: owner}, raw: true});
    let user_name = user ? user.name : '系统';
    switch (model_id) {
        case 'prj': // 立项成功
            await PrjInfo.update({
                phase_id: 'created',
                started_at: dayjs().format('YYYY-MM-DD 00:00:00')
            }, {where: {id: prj_id}});
            await PrjLogger.log({
                creator_id: owner,
                creator_name: user_name,
                prj_id: prj_id,
                content: `项目立项成功`,
            });
            break;
        case 'plan':
            await PrjInfo.update({phase_id: 'doing'}, {where: {id: prj_id}});
            await PrjPlanTask.increment({version: 1}, {where: {prj_id: prj_id}});
            await PrjLogger.log({
                creator_id: owner,
                creator_name: user_name,
                prj_id: prj_id,
                content: `项目计划制定成功`,
            });
            break;
        case 'plan_alter':
            let t = await PrjPlanTask.sequelize!.transaction();
            try {
                await PrjInfo.update({phase_id: 'doing'}, {where: {id: prj_id}, transaction: t});
                await PrjLogger.log({
                    creator_id: owner,
                    creator_name: user_name,
                    prj_id: prj_id,
                    content: `项目计划变更成功`,
                    t
                });
                // 更新计划变更版本
                await PrjPlanTaskDraft.increment({version: 1}, {
                    where: {prj_id: prj_id},
                    transaction: t
                });

                // 依据change_marker字段中的从草稿表向正本表同步信息，包括新增、修改和删除
                // 1. 修改
                await PrjPlanTask.sequelize!.query(`
                    UPDATE ${PrjPlanTask.table_name} 
                    SET name = B.name, status = B.status, handler_id = B.handler_id,
                        creator_id = B.creator_id, begin_at = B.begin_at, end_at = B.end_at, completed_at = B.completed_at,
                        created_at = B.created_at, updated_at = B.updated_at, description = B.description, order_id = B.order_id,
                        version = B.version, checkers = B.checkers
                    FROM ${PrjPlanTaskDraft.table_name} B
                    WHERE B.change_marker = ${ChangeMarker.Changed} and ${PrjPlanTask.table_name}.id = B.id and B.prj_id = :prj_id;
                `, {type: QueryTypes.UPDATE, transaction: t, replacements: {prj_id: prj_id}});
                // 2. 新增
                await PrjPlanTask.sequelize!.query(`
                    INSERT INTO ${PrjPlanTask.table_name} 
                        (id, name, pid, prj_id, progress, status, handler_id, creator_id, begin_at, end_at, completed_at,
                        created_at, updated_at, description, order_id, version, checkers)
                    SELECT id, name, pid, prj_id, progress, status, handler_id, creator_id, begin_at, end_at, completed_at,
                        created_at, updated_at, description, order_id, version, checkers
                    FROM ${PrjPlanTaskDraft.table_name}
                    WHERE change_marker = ${ChangeMarker.Added} and prj_id = :prj_id;
                `, {type: QueryTypes.INSERT, transaction: t, replacements: {prj_id: prj_id}});
                // 3. 删除
                await PrjPlanTask.sequelize!.query(`
                    DELETE FROM ${PrjPlanTask.table_name}
                    WHERE id IN (SELECT id FROM ${PrjPlanTaskDraft.table_name} WHERE change_marker = ${ChangeMarker.Deleted} and prj_id = :prj_id);
                `, {type: QueryTypes.DELETE, transaction: t, replacements: {prj_id: prj_id}});

                /// 同步交付物记录
                // 1. 修改
                await PrjTaskOutcome.sequelize!.query(`
                    UPDATE ${PrjTaskOutcome.table_name} 
                    SET name = B.name, task_id = B.task_id, status = B.status, creator_id = B.creator_id,
                        plan_time = B.plan_time, completed_at = B.completed_at,
                        created_at = B.created_at, updated_at = B.updated_at, description = B.description, order_id = B.order_id,
                        uploaded = B.uploaded,
                        version = B.version
                    FROM ${PrjTaskOutcomeDraft.table_name} B, ${PrjPlanTaskDraft.table_name} C
                    WHERE B.change_marker = ${ChangeMarker.Changed} and 
                        ${PrjTaskOutcome.table_name}.id = B.id and 
                        B.task_id = C.id and C.prj_id = :prj_id;
                    `, {type: QueryTypes.UPDATE, transaction: t, replacements: {prj_id: prj_id}});

                // 2. 新增
                await PrjTaskOutcome.sequelize!.query(`
                    INSERT INTO ${PrjTaskOutcome.table_name} 
                        (id, name, task_id, status, creator_id, plan_time, completed_at,
                        created_at, updated_at, description, order_id, uploaded, version)
                    SELECT outcome.id, outcome.name, outcome.task_id, outcome.status, outcome.creator_id, 
                        outcome.plan_time, outcome.completed_at,
                        outcome.created_at, outcome.updated_at, outcome.description, outcome.order_id, 
                        outcome.uploaded, outcome.version
                    FROM ${PrjTaskOutcomeDraft.table_name} outcome, ${PrjPlanTaskDraft.table_name} task
                    WHERE outcome.change_marker = ${ChangeMarker.Added} and outcome.task_id = task.id and task.prj_id = :prj_id;
                `, {type: QueryTypes.INSERT, transaction: t, replacements: {prj_id: prj_id}});
                // 3. 删除
                await PrjTaskOutcome.sequelize!.query(`
                    DELETE FROM ${PrjTaskOutcome.table_name}
                    WHERE id IN (
                        SELECT outcome.id FROM ${PrjTaskOutcomeDraft.table_name} outcome, ${PrjPlanTaskDraft.table_name} task
                        WHERE outcome.change_marker = ${ChangeMarker.Deleted} and 
                            outcome.task_id = task.id and 
                            task.prj_id = :prj_id);
                `, {type: QueryTypes.DELETE, transaction: t, replacements: {prj_id: prj_id}});

                await PrjTaskOutcomeDraft.sequelize!.query(`
                    delete from ${PrjTaskOutcomeDraft.table_name}
                        where task_id in (select id from ${PrjPlanTaskDraft.table_name} where prj_id = :prj_id)
                `, {type: QueryTypes.DELETE, transaction: t, replacements: {prj_id: prj_id}})
                await PrjPlanTaskDraft.destroy({
                    where: {prj_id: prj_id}, transaction: t
                });

                await t.commit();
            } catch (e) {
                Logger.error(e);
                return Resp.gen_err(Resp.InternalServerError, '计划变更失败');
            }


            break;
        case 'task':
            let flow_case = await BpmnCase.findOne({where: {id: id, model_id: 'task'}, raw: true});
            if (flow_case) {
                await PrjPlanTask.update({
                    progress: 100,
                    status: TaskStatus.Completed,
                    completed_at: dayjs().format('YYYY-MM-DD HH:mm:ss'),
                }, {
                    where: {id: flow_case.task_id}
                });
                await PrjLogger.log({
                    creator_id: owner,
                    creator_name: user_name,
                    content: `任务审核通过，任务完成`,
                    prj_id: prj_id,
                    task_id: flow_case.task_id,
                })
            }
            break;
    }
}

export async function create_task_approve_flow_if_not_exists(prj_id: string, owner: string, task_id: string, t?: Transaction): Promise<void> {
    // 查询流程是否已存在，存在的话直接返回，不重复建流程
    let flow = await BpmnCase.findOne({
        where: {prj_id: prj_id, task_id: task_id, model_id: 'task'},
        transaction: t,
        raw: true
    });
    if (flow) return;

    // 如果审核人不存在，不开启流程
    let task = await PrjPlanTask.findOne({where: {id: task_id}, transaction: t, raw: true});
    if (!task) return;
    let checkers = task.checkers;
    if (!checkers || checkers.length === 0) return;

    let bpmn = await BpmnModel.findOne({where: {id: 'task'}, raw: true});
    if (bpmn) {
        let flow = new FlowEngine('task', bpmn.content, {
                prj_id: prj_id,
                owner: owner,
                task_id: task_id
            },
            bpmn_flow_on_end
        );
        await BpmnCase.create({
            id: flow.id,
            model_id: bpmn.id,
            model: bpmn.content,
            started_at: dayjs(),
            creator_id: owner,
            prj_id: prj_id,
            task_id: task_id
            // state: await flow.state(),
        }, {});
        await flow.run();
    }

}