import {Op, Transaction, WhereOptions} from "sequelize";
import dayjs, {Dayjs} from "dayjs";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {PrjInfo} from "@core-models/PrjInfo";
import {Logger} from "@util/Logger";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";
import {BpmnModel} from "@core-models/BpmnModel";
import {Resp} from "@util/Resp";
import {FlowEngine} from "@src/bpmn/flow_engine";
import {bpmn_flow_on_end} from "@src/utils/bpmn_work_helper";
import {BpmnCase} from "@core-models/BpmnCase";
import {ChangeMarker} from "@src/utils/define";
import {PrjTaskOutcome} from "@core-models/PrjTaskOutcome";
import {PrjTaskOutcomeDraft} from "@core-models/PrjTaskOutcomeDraft";

export async function update_parent_task_info(draft: boolean, prj_id: string, task_pid: string | null, t: Transaction, max?: Dayjs, min?: Dayjs): Promise<void> {
    Logger.trace(`prj_id: ${prj_id} task_id: ${task_pid}`)

    let model = draft ? PrjPlanTaskDraft : PrjPlanTask;
    let where: WhereOptions = {pid: task_pid, prj_id: prj_id, status: {[Op.ne]: 90}};  // 已废除的任务不参与计算
    if (draft) where.change_marker = {[Op.ne]: ChangeMarker.Deleted};
    let child_tasks = await model.findAll({
        where: where,
        transaction: t
    });
    let total_duration = 0;
    let completed = 0;
    for (let i = 0; i < child_tasks.length; i++) {
        let child_task = child_tasks[i];
        Logger.trace(child_task);
        let begin = dayjs(child_task.begin_at);
        let end = dayjs(child_task.end_at);
        if (max === undefined) max = end;
        else max = end.diff(max) > 0 ? end : max;
        if (min === undefined) min = begin;
        else min = begin.diff(min) < 0 ? begin : min;
        let duration = dayjs(child_task.end_at).diff(child_task.begin_at, 'hours');
        total_duration += duration;
        completed += duration * child_task.progress / 100;
        Logger.trace(`completed: ${completed} total: ${total_duration} hours`)
    }
    let progress = total_duration !== 0 ? (completed * 100 / total_duration) : 0;
    if (progress > 0 && progress < 1) progress = 1;
    else if (progress < 100 && progress > 99) progress = 99;
    else progress = Math.round(progress);
    Logger.trace(`progress: ${progress} %`);
    if (task_pid !== null) {
        let task = await model.findOne({where: {id: task_pid}, raw: true, transaction: t});
        if (!task) return;
        let status = task.status;
        if (progress > 0 && status === 0) status = 10;
        else if (progress === 0) status = 0;
        let parent = await model.update({
            begin_at: min!,
            end_at: max!,
            progress: progress,
            status: status
        }, {where: {id: task_pid}, transaction: t, returning: true});
        if (parent[0] === 0) return;
        await update_parent_task_info(draft, prj_id, parent[1][0].pid, t);
    } else {
        await PrjInfo.update({progress: progress}, {where: {id: prj_id}, transaction: t});
    }

    // }
}

export async function start_plan_alt_flow(prj_id: string, user_id: string, t?: Transaction): Promise<void> {
    let flow_case = await BpmnCase.findOne({
        where: {
            prj_id: prj_id,
            model_id: 'plan_alter',
            completed_at: {[Op.is]: null}
        }, transaction: t
    });
    if (flow_case) return;

    // 启动计划变更工作流
    let bpmn = await BpmnModel.findOne({where: {id: 'plan_alter'}, transaction: t});
    if (!bpmn) throw Resp.gen_err(Resp.InternalServerError, '项目流程模型数据缺失！');
    let flow = new FlowEngine(bpmn.id, bpmn.content, {
        prj_id: prj_id,
        owner: user_id
    }, bpmn_flow_on_end);
    await flow.run();
    // await PrjInfo.update({flow_case_id: flow.id}, {where: {id: id}, transaction: t});
    await BpmnCase.create({
        id: flow.id,
        model_id: bpmn.id,
        model: bpmn.content,
        started_at: dayjs(),
        creator_id: user_id,
        prj_id: prj_id,
        // state: await flow.state(),
    }, {transaction: t});
}

export interface TaskInfo {
    prj_id: string;
    task_id: string;
    outcome_status: number;
}

export async function get_task_info_by_outcome_id(draft: boolean, outcome_id: string, t?: Transaction): Promise<TaskInfo | undefined> {
    let outcome_model = draft ? PrjTaskOutcomeDraft : PrjTaskOutcome;
    let outcome = await outcome_model.findOne({
        where: {outcome_id: outcome_id},
        raw: true,
        transaction: t
    });
    if (!outcome) return undefined;
    let model = draft ? PrjPlanTaskDraft : PrjPlanTask;
    let task = await model.findOne({
        where: {id: outcome.task_id},
        raw: true,
        transaction: t
    });
    if (!task) return undefined;
    return {
        prj_id: task.prj_id,
        task_id: task.id,
        outcome_status: outcome.status
    };
}

