import {AcsUserInfo} from "@core-models/AcsUserInfo";
import {PrjMembers} from "@core-models/PrjMembers";
import {PrjInfo} from "@core-models/PrjInfo";
import {QueryTypes} from "sequelize";
import {PrjPlanTask} from "@core-models/PrjPlanTask";
import {Logger} from "@util/Logger";
import {PrjPlanTaskDraft} from "@core-models/PrjPlanTaskDraft";

/// 是否是项目特权账户
export async function is_project_privileged_account(account_id: string): Promise<boolean> {
    // 检查账户是否为特权账户
    let result = await AcsUserInfo.findOne({where: {id: account_id}, raw: true});
    if (result && result.ext_info.full_range === true) {
        Logger.trace(`${account_id} 是项目特权人员。`);
        return true;
    }
    Logger.trace(`${account_id} 不是项目特权人员。`);
    return false;
}

/// 是否是项目组成员
export async function is_project_member(account_id: string, project_id: string): Promise<boolean> {
    // 检查账户是否为项目组成员
    let result = await PrjMembers.findOne({
        where: {member_id: account_id, prj_id: project_id, draft: false},
        raw: true
    });
    if (result) {
        Logger.trace(`${account_id} 是项目 ${project_id} 的项目组成员。`);
        return true;
    }
    Logger.trace(`${account_id} 不是项目 ${project_id} 的项目组成员。`);
    return false;
}

/// 是否是项目负责人
export async function is_project_leader(account_id: string, project_id: string): Promise<boolean> {
    let prj = await PrjInfo.findOne({where: {id: project_id}, raw: true});
    if (prj && prj.leader_id === account_id) {
        Logger.trace(`${account_id} 是项目 ${project_id} 的项目负责人。`);
        return true;
    }
    Logger.trace(`${account_id} 不是项目 ${project_id} 的项目负责人。`);
    return false;
}

/// 是否是项目商务负责人
export async function is_project_business_leader(account_id: string, project_id: string): Promise<boolean> {
    let prj = await PrjInfo.findOne({where: {id: project_id}, raw: true});
    if (prj && prj.bizman_id === account_id) {
        Logger.trace(`${account_id} 是项目 ${project_id} 的商务负责人。`);
        return true;
    }
    Logger.trace(`${account_id} 不是项目 ${project_id} 的商务负责人。`);
    return false;
}

/// 是否是项目审核组成员
export async function is_project_checker(account_id: string, project_id: string): Promise<boolean> {
    let result = await PrjInfo.sequelize!.query(`
        select 1 
        from tb_prj_info prj_info, jsonb_array_elements(prj_info.checkers) AS checker_obj
        where checker_obj->>'id' = :user_id and prj_info.id = :prj_id
    `, {replacements: {user_id: account_id, prj_id: project_id}, type: QueryTypes.SELECT});

    if (result && result.length > 0) {
        Logger.trace(`${account_id} 是项目 ${project_id} 的审核组成员。`);
        return true;
    }
    Logger.trace(`${account_id} 不是项目 ${project_id} 的审核组成员。`);
    return false;
}

/// 是否是项目任务审核组成员
export async function is_project_task_checker(account_id: string, project_id: string): Promise<boolean> {

    let result = await PrjInfo.sequelize!.query(`
        SELECT 1
        FROM tb_prj_plan_task prj_task, jsonb_array_elements(prj_task.checkers) AS checker_obj
        WHERE  (checker_obj->>'handlers')::jsonb ? :user_id
          and prj_task.prj_id = :prj_id
    `, {replacements: {user_id: account_id, prj_id: project_id}, type: QueryTypes.SELECT});

    if (result && result.length > 0) {
        Logger.trace(`${account_id} 是项目 ${project_id} 的任务审核组成员。`);
        return true;
    }
    Logger.trace(`${account_id} 不是项目 ${project_id} 的任务审核组成员。`);
    return false;
}

/// 是否有权访问项目
export async function is_project_accessible(account_id: string, project_id: string): Promise<boolean> {
    let prj = await PrjInfo.findOne({where: {id: project_id}, raw: true});
    if (prj && (
        prj.leader_id === account_id ||
        prj.bizman_id === account_id ||
        await is_project_privileged_account(account_id) ||
        await is_project_member(account_id, project_id) ||
        await is_project_checker(account_id, project_id) ||
        await is_project_task_checker(account_id, project_id)
    )) {
        Logger.trace(`${account_id} 有权访问项目 ${project_id}。`);
        return true;
    }

    Logger.trace(`${account_id} 没有权限访问项目 ${project_id}。`);
    return false;
}

/// 是否有修改项目信息的权限
export async function is_project_modifiable(account_id: string, project_id: string): Promise<boolean> {
    if (
        await is_project_leader(account_id, project_id) ||
        await is_project_privileged_account(account_id)
    ) {
        Logger.trace(`${account_id} 是权修改项目 ${project_id} 的信息。`);
        return true;
    }

    Logger.trace(`${account_id} 没有权限修改项目 ${project_id} 的信息。`);
    return false;
}

/// 是否有访问项目任务的权限
export async function is_project_task_accessible(account_id: string, task_id: string): Promise<boolean> {
    let task = await PrjPlanTask.findOne({where: {id: task_id}, raw: true});
    if (task && (
        task.handler_id === account_id ||
        await is_project_accessible(account_id, task.prj_id)
    )) {
        Logger.trace(`${account_id} 有权访问项目 ${task.prj_id} 的任务 ${task_id}。`);
        return true;
    }

    Logger.trace(`${account_id} 没有权限访问任务 ${task_id}。`);
    return false;
}

/// 是否有权限添加任务
export async function is_project_task_addable(account_id: string, project_id: string): Promise<boolean> {
    if ( await is_project_modifiable(account_id, project_id)) {
        Logger.trace(`${account_id} 有权添加项目 ${project_id} 的任务。`);
        return true;
    }

    Logger.trace(`${account_id} 没有权限添加项目 ${project_id} 的任务。`);
    return false;
}

/// 是否有修改项目任务的权限
export async function is_project_task_modifiable(model: any, account_id: string, task_id: string): Promise<boolean> {
    let task = await model.findOne({where: {id: task_id}, raw: true});
    if (task &&  await is_project_modifiable(account_id, task.prj_id)) {
        Logger.trace(`${account_id} 有权修改项目 ${task.prj_id} 的任务 ${task_id}。`);
        return true;
    }

    Logger.trace(`${account_id} 没有权限修改任务 ${task_id}。`);
    return false;
}

/// 是否有修改项目任务进度的权限
export async function is_project_task_progress_modifiable(account_id: string, task_id: string): Promise<boolean> {
    let task = await PrjPlanTask.findOne({where: {id: task_id}, raw: true});
    if (task && (task.handler_id === account_id || await is_project_modifiable(account_id, task.prj_id))) {
        Logger.trace(`${account_id} 有权修改项目 ${task.prj_id} 的任务 ${task_id}的进度。`);
        return true;
    }

    Logger.trace(`${account_id} 没有权限修改任务 ${task_id}的进度。`);
    return false;
}